	parameter(pi=3.1415926536)
c--does cross-sections through global block models parametrized as 
c--van der hilst's (97) one or grand's (97).

c--2*nodes is the number of points of the great circle used to
c--evaluate the cross-section.
	parameter(nodes=360)
	parameter(nodrad=50)

	dimension xla(2*nodes),xlo(2*nodes)
c==================================================================
c--describe parametrization of the model to be read
	parameter(nx=180,ny=90)
	parameter(nlayers=22)
	parameter(ngrid=nx*ny*nlayers)
c--number of blocks corresponding to one record of the model file
	parameter(nel_rec=10)
c==================================================================
	real*4 model(ngrid),db(0:nlayers+1),depths(nlayers),temp(nel_rec)
	character name*50

c	name='3Dglobal_P_999'
c	name='mk12wm13.p.2x2'
	name='grand_model_gsat'

	open(21,file='cross.dat')

c==================================================================
c--define the cross-section
	print*,'pole of the great circle: colatitude(deg)?'
	read*,capth
	capth=(capth/180.)*pi

	print*,'pole of the great circle: longitude(deg)?'
	read*,capph
	capph=(capph/180.)*pi

c	print*,'azimuth(deg)?'
c	read*,azm
c	azm=(azm/180.)*pi

c	print*,'increment along the great circle(deg)?'
c	read*,delta

c==================================================================
c--find a number of equidistant points on a great circle defined by
c--its pole.
c--the formulae employed here can be found by means of a double rotation
c--from the reference frame whose pole is the geographical north to
c--the one whose pole is the pole of the great circle. 
	open(1,file='gc.dat')

	sf=sin(capph)
	cf=cos(capph)
	ctsf=cos(capth)*sf
	ctcf=cos(capth)*cf
	st=sin(capth)


	rincr=360./nodes
	print*,'incremento = ',rincr
	do i=1,nodes
c--by summing 90. to i*rincr in the next line I set the azimuth to 90.,
c--i.e. in the output file the 0-point on the great circle corresponds
c--to longitude 90 in the rotated system (the system whose pole is the
c--pole of the great circle).
	   delta=((90.+i*rincr)/180.)*pi
	   theta=abs(acos(-cos(delta)*st))
	   sphi=(ctsf*cos(delta)+cf*sin(delta))/sin(theta)
	   cphi=(ctcf*cos(delta)-sf*sin(delta))/sin(theta)
	   phi=atan2(sphi,cphi)
	   xla(i)=90.-((theta/pi)*180.)
	   xlo(i)=(phi/pi)*180.
	   if(xlo(i).lt.0.)xlo(i)=xlo(i)+360.
	   xla(nodes+i)=-xla(i)
	   xlo(nodes+i)=xlo(i)+180.
	   if(xlo(nodes+i).lt.0.)xlo(nodes+i)=xlo(nodes+i)+360.
	enddo

c--store the great circle in a file
	do i=1,nodes
	   write(1,*)xlo(i),xla(i)
	enddo
	close(1)
c==================================================================

c--first read the whole model and store it in memory
	call getmodel(model,name,ngrid,1,depths,nlayers,nel_rec,temp)
c--compute radii of boundaries between layers
	db(0)=0.
	do idb=1,nlayers
	   db(idb)=db(idb-1)+2.*(depths(idb)-db(idb-1))
	enddo
	do ik=0,nlayers
	   print*,db(ik)
	enddo

c==================================================================
c==================================================================
c--loop over depth 
	incre=50
	ibot=db(nlayers)
	print*,'inner boundary of mantle is ',ibot,' km deep'
	DO II=0,ibot,incre

c==================================================================
c--figure out the layer number
	depth=1.*ii
	print*,'depth',depth
	if(depth.lt.0.)stop
	if(depth.ge.db(nlayers))then
	   if((depth-db(nlayers)).lt.incre)then
	      depth=db(nlayers)
	      il=nlayers
	      goto221
	   else
	      print*,'too deep!'
	      goto441
	   endif
	endif

	do idb=1,nlayers
	if(depth.ge.db(idb-1).and.depth.lt.db(idb))then
	   il=idb
	   goto221
	endif
	enddo
221	print*,'layer n. ',il,', incremento=',rincr
c==================================================================
c--for this depth, evaluate the model along the whole great circle
	do i=1,nodes*2
	   xlon=xlo(i)
	   colat=90.-xla(i)
	   call readmodel(model,xlon,colat,IL,value,
     &	                  nx,ny,ngrid)
	   write(21,*)i*rincr,6371.-depth,value
	enddo

	ENDDO
c==================================================================
c==================================================================
441	close(21)
	end


C..................................................................
C..................................................................
	subroutine getmodel(model,name,n,io,depths,
     &	                    nlayers,nel_rec,a)
c--reads a block model from file and stores it in memory.
c--the parametrization of the model is describes by input variables.
	character name*50
	real*4 model(n),a(nel_rec),depths(nlayers)

	open(unit=io,file=name,status='old')
	print*,'reading from model',name

	read(io,*)
	read(io,*)(depths(jl),jl=1,nlayers)
	write(*,*)(depths(jl),jl=1,nlayers)
	nrec=n/nel_rec
	do i=1,nrec
	   read(io,'(10e7.3)')(a(k),k=1,nel_rec)
	   do kk=1,nel_rec
	      nel=((i-1)*10)+kk
	      model(nel)=a(kk)
	   enddo
	enddo
	close(io)
	return
	end
c==================================================================
	subroutine readmodel(model,lon,colat,layern,value,
     &	                     nx,ny,n)
	real*4 model(n),lon

	nhmax=nx*ny

	n1=lon*((nx*1.)/360.)+1.
	IF(COLAT.EQ.180.)COLAT=COLAT-1.
	n2=colat*((ny*1.)/180.)+1.
	nh=((n2-1)*nx)+n1

	ntot=((layern-1)*(nx*ny))+nh

	value=model(ntot)
	return
	end
c==================================================================
