#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <unistd.h>
#include <limits.h>
#include <assert.h>
#include "build_ata.h"
//  The following routines calculate the ATA matrix for a given
//  A matrix.  They were originally implemented by Weijia Su,
//  then later re-written by Yu Gu, 1998.
//
//  List Of Functions:
//---------------------------------------------------------------
//  CheckMenu()
//  PrintMenu()
//  SumArow()
//  RdAmatModPara()
//  AddArow()
//  ReadArow()
//  CapReadArow()
//  ReadArow_Eqinfo()
//  BSModel::ReadModel()
//  BSModel::ReadModel_split()
//  BSModel::ReadModel_multisplit()
//  BSModel::InitKnots()
//  BSModel::CalAvgDist()
//  BSModel::DistBt2()
//  SysOpen()
//  WriteIpm()
//
//---------------------------------------------------------------

void CheckMenu(int argc, char *argv[], char &model, char &modeltype)
{
// This checks the menu options

      void PrintMenu();
      int i, j=0;
      char ch;

      model = 'W';
      modeltype = 'C';
      cout << "**This program calculates an ATA Matrix from a given A Matrix**" << endl;
      cout << "-----------------Chosen Options-----------------"<< endl;
      if(argc > 1) { //  options
	 for(i=1; i< argc;i++) {
		if(argv[i][0] == 'W' || argv[i][0] == 'w') {
			model = 'W';
			cout << "  Global Model";
		} else if(argv[i][0] == 'R' || argv[i][0] == 'r'){
			model = 'R';
			cout << "  Regional Model";
		} else if(argv[i][0] == 'C' || argv[i][0] == 'c'){
			modeltype = 'C';
			cout << "  Continuous";
		} else if(argv[i][0] == 'S' || argv[i][0] == 's'){
			modeltype = 'S';
			cout << "  Split";
		} else if(argv[i][0] == 'M' || argv[i][0] == 'm'){
			modeltype = 'M';
			cout << "  Multiply Split";
		} else {
			cout << "option(s) not found !!" << endl << endl;
	 		PrintMenu();
		}
	 }
	 cout << endl;
      } else {
	 cout << "Global Model " << "  Continuous"  <<endl;
      }
      cout << "------------------------------------------------"<< endl;
      
}

void PrintMenu() 
{
      cout << endl<<endl;
      cout << "Options: " << endl;
      cout << "-------------------------------------------------------------------------" << endl;
      cout << "W             --Global model (default)" << endl;
      cout << "R             --Regional model" << endl;
      cout << "C             --Continuous model across 670 (default)" << endl;
      cout << "S             --Split model across 670" << endl;
      cout << "S             --Split model across 670" << endl;
      cout << "M             --Multiply Split model" << endl;
      cout << "D rec1 rec2   --Read record from rec1 to rec2 (default 1--1000000)" << endl;
      cout << "-------------------------------------------------------------------------" << endl;
      cout << endl << endl;
      exit(-1);
}

real  SumArow(BSModel &bsmodel,float *arow,short *irow,int &ind)
{
int i,j;
real res=0.0;

      for(i=0;i<ind;i++){
          j=irow[i];
	  cout << "arow[100]=" << arow[100] <<  "arow[1000]=" << arow[1000] <<" coef 10=" << bsmodel.coeff[10] << endl;
          res+=arow[i]*bsmodel.coeff[j];
      }
      return res;
}

// The following subroutines are specifically designed for multiply split
// earth models, Jeffrey Gu, 1999.

void BSModel::ReadModel_multisplit(int choice, char *flname)
{
fstream inFile;
int i,j,k,m,nr, nc;
real dum,lat,lon;

 cout << " model = " <<flname<<endl;
 inFile.open(flname,ios::in);		// open prem file for read
 
 if(!inFile) { 				// open failed?
     cerr << "Cannot open "<<flname<<" for input" << endl;
     exit(-1);
 }
 Read1stLine(inFile,choice);
 cout << "maxk_lm=" << maxk_lm << " maxk_um=" << maxk_um << " total=" << maxk << endl;
 maxk_um--;				// to be consistent with other rad functions
 maxk_lm--;				// to be consistent with other rad functions
 maxk--;				// to be consistent with other rad functions
 InitKnots(choice);		// allocate space
 cout << "gpt = " << gpt << endl;
 for(m=i=0;i<gpt;i++){
   inFile >> j >> lat >> lon;
   theta[i]=(90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=360.0;
   phi[i] *= RAD;
   if(choice!=REGIONAL && choice !=SPLIT_REGIONAL){	// also read adjacent knots if it is not a regional model
      for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
      }
   }
 }
 cout << "maxk =" << maxk << endl;
 for(k=i=0;i<=maxk;i++){  // Model coefficients from CMB (lm) to Surface
     inFile >> m >>rknt[i];
     for(j=0;j<gpt;j++){
        inFile >> dum;
        coeff[k++]=dum;
     }
 }
 inFile.close();
 if(choice==REGIONAL || choice==SPLIT_REGIONAL){// read in average distance between knots
    cout <<"input the average distance (in degrees) between knots\n";
    cin >> avgdist;
    avgdist*=RAD;
 }else{
    CalAvgDist();
 }
 k=0;
 for(i=0;i<(nsplit_um+nsplit_lm);i++){
	for(j=0;j<nbsperlayer[i];j++){
		rknt_mult[i][j]=rknt[k];
     		cout <<"layer= " << i<<" rknt =" <<j << " "<<rknt[k]<<endl;
		k++;	
	}
	// add two more boundary points to make sure
	// spline calculation works.
	rknt_mult[i][j]=rknt_mult[i][j+1]=rknt_mult[i][j-1];
 }
 
 region=UPLOWMANTLE;
}


void BSModel::Read1stLine(fstream &inFile, int choice)
{
 int i,j;
 int nsplit; 
 char mod_desc[50];

 if(choice==SPLIT || choice==SPLIT_REGIONAL){
 	inFile >> mod_desc >> gpt >> maxk_lm >> maxk_um;
 	cout << mod_desc << " " << gpt << " " << maxk_lm
		 << " " << maxk_um << endl;
 }else if(choice==MULTISPLIT){
	inFile >> mod_desc  >> nsplit_lm >> nsplit_um
		>> gpt;
	nsplit=nsplit_um+nsplit_lm;
	cout << "nsplit = " << nsplit << endl;
	maxk_lm=0; maxk_um=0;
	if(nsplit>9) cout << "too many splits, not implemented!" <<endl;
	for(i=0;i<nsplit;i++){
		inFile >> nbsperlayer[i];
		cout << "i=" << i << "  layer =" <<  nbsperlayer[i] << endl;
		if(i<nsplit_lm) {
			maxk_lm += nbsperlayer[i];
		}else{
			maxk_um += nbsperlayer[i];
		}
	}
	maxk=maxk_lm+maxk_um;
 }
}

void RdAmatModPara(int &fd, int buildopt, int &htps,int &maxk, int &opt)
{
int htps0,maxk0;

    read(fd,&htps0,sizeof(int));
    read(fd,&maxk0,sizeof(int));
    if(buildopt != 1) {     // NEW FORMAT 
       read(fd,&opt, sizeof(int));
       cout << "Input A Matrix format = " << opt << endl;
    }else{
       opt =1;              // OLD FORMAT Xianfeng's data or ISC
    }
    if(htps0!=htps || maxk0!=maxk){
       cout << "Inconsistent model and matrix\n";
       cout << "Model  htps = " << htps << endl;
       cout << "Matrix htps = " << htps0 << endl;
       cout << "Model  maxk = " << maxk << endl;
       cout << "Matrix maxk = " << maxk0 << endl;
       exit(-1);
    }else{
       cout << " htps = " << htps << endl;
       cout << " maxk = " << maxk << endl;
    }
}


void  AddArow(EqSt &eqst,int &ind,float *arow, short *irow, float *ata, 
		float *atd, float dc400, float dc670, float weight, 
		int num3d, int hpts,
		int resopt, int buildopt)
{
float res,arowii;
float corr400, corr670;
int ii,jj,kk,ia, isurf, nbody;

    if(resopt == 1 || resopt == 5) {
	res = eqst.res1 + eqst.res2 ;
    }else if(resopt == 2 || resopt == 6) {
	res = eqst.res1 - eqst.res2 ;	// observed - predicted by initial model
    }else if(resopt == 3) {
	res = eqst.res1 ;
    }else if(resopt == 4) {
	res = eqst.res2 ;
    }
//
//  the following removes perturbation to the average of discontinuities
//  at 400 and 670 km
//
    if(resopt > 4){
// assumes at least 1 discontinuity layer

    	corr400 = 0.0;
    	corr670 = 0.0;

//	cout << "num3d = " << num3d << endl;
//	cout << "hpts = " << hpts << endl;
//	cout << "dc400 = " << dc400 << endl;
//	cout << "dc670 = " << dc670 << endl;
    	for(ii=0;ii<ind;ii++){
        	ia=(int)irow[ii];
		if((ia >= num3d) && (ia < num3d+hpts)){
			corr400 += arow[ii]*dc400;
		}else if(ia >= num3d+hpts){
			corr670 += arow[ii]*dc670;
		}
	}
//	cout << "correction due to 400 shift =" << corr400 << endl;
//	cout << "correction due to 670 shift =" << corr670 << endl;
//	cout << "last nonzero index =" << (int)irow[ind-1] << endl;
	res = res - corr400 - corr670; 
    }
    res *= weight;
//        cout << "eqst.res1=" << eqst.res1 
//		<< " eqst.res2=" << eqst.res2  
//		<< "  res=" << res << endl;
    for(ii=0;ii<ind;ii++){
        ia=(int)irow[ii];
        kk=((ia+1)*ia)/2;
        arowii=arow[ii];
        atd[ia]+=arowii*res;
        for(jj=0;jj<ind;jj++){
            if(jj<=ii){
               ata[kk+(int)irow[jj]]+=arowii*arow[jj]*weight;
            }
        }
    }
}

void  AddArow_Resolution(EqSt &eqst,int &ind,float *arow, short *irow, float *ata, float *atd, float weight, int iop)
{  // for resolution test, use the synthetic data `res2' as the residual
float res,arowii;
int ii,jj,kk,ia, isurf, nbody;

    res=eqst.res2;		// predicted by initial model
    res *= weight;
    for(ii=0;ii<ind;ii++){
        ia=(int)irow[ii];
        kk=((ia+1)*ia)/2;
        arowii=arow[ii];
        atd[ia]+=arowii*res;
        for(jj=0;jj<ind;jj++){
            if(jj<=ii){
               ata[kk+(int)irow[jj]]+=arowii*arow[jj]*weight;
            }
        }
    }

}

void ReadArow(int fd,class EqSt *eqst, float *arow,short *irow,int *ind, int *numrd)
{
float buf[9];
int ibuf[3];

    *numrd=read(fd,buf,sizeof(buf));
    if(*numrd==0)return;		// end of file
    read(fd,ibuf,sizeof(ibuf));
    *ind=ibuf[2];
    read(fd,irow,sizeof(short)*(*ind));
    read(fd,arow,sizeof(float)*(*ind));

    eqst->res1=(real)buf[6];		// observed residual
    eqst->res2=(real)buf[7];		// predicted residual
    eqst->time = (real)buf[8];          // PREM travel time	
//	printf("%d %f %f %f %f\n", *ind, eqst->res1, eqst->res2, arow[100], arow[300]);
//    cout << eqst->res1 << "  " << eqst->res2 << endl;
//    int i=0;
//    for(i=0; i< *ind; i++) {
//      cout << irow[i] << " " << arow[i] << "  " << eqst->res1 << " " 
//      << eqst->res2 << " " << eqst->time << endl;
//    }
}

void CapReadArow(int fd, EqSt *eqst, float *arow,short *irow,int *ind, int *numrd)
{
float buf[9];
int ibuf[3],i;

    *numrd=read(fd,buf,sizeof(buf));
    if(*numrd==0)return;		// end of file
    read(fd,ibuf,sizeof(ibuf));
    *ind=ibuf[2];
    read(fd,irow,sizeof(short)*(*ind));
    read(fd,arow,sizeof(float)*(*ind));
    eqst->res1=(real)buf[6];	// observed Precursor time
    eqst->res2=(real)buf[7];	// predicted Precursor heterogeneity
//    cout << "  observed precursor time = " << buf[6] << "  Model prediction = " << buf[7] << endl;
}

void ReadArow_Eqinfo(int fd, float *arow,short *irow, int &ind, int &numrd, int opt, float &eqlat, float &eqlon, float &stlat, float &stlon, float &res, int outopt)
{  // returns also info about the geographical coordinates.
float buf[9];
int ibuf[3],i,n1;
float dep;

    numrd=read(fd,buf,sizeof(buf));
    if(numrd==0)return;		// end of file
    eqlat = buf[0];
    eqlon = buf[1];
     dep   = buf[2];
    stlat = buf[3];
    stlon = buf[4];
    if(outopt ==1) {
	res = buf[6];
    }else if(outopt ==2){
	res = buf[7];
    }else if(outopt == 3){
    	res = buf[6]-buf[8];	// observed subtract PREM time (Scripps)
    }else if(outopt == 4){
    	res = buf[7]-buf[8];	// predicted subtract PREM time (Scripps)
    }

    read(fd,ibuf,sizeof(ibuf));
    ind=ibuf[2];
    read(fd,irow,sizeof(short)*(ind));
    read(fd,arow,sizeof(float)*(ind));
}


void BSModel::ReadModel(int choice)
{
fstream inFile;
int i,j,k,m,nr;
real dum,lat,lon;

 cout << "name of the CONTINUOUS B-spline model-->" << endl;
 cin >> modelname;
 cout << " choice = " << choice << endl;
 inFile.open(modelname,ios::in);		// open prem file for read
 
 if(!inFile) { 				// open failed?
     cerr << "Cannot open "<< modelname<<" for input\n";
     exit(-1);
 }
 inFile >> gpt >> maxk;
 maxk--;				// to be consistent with other rad functions
 InitKnots(choice);		// allocate space
 
 for(m=i=0;i<gpt;i++){
   inFile >> j >> lat >> lon;
   theta[i]=(90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=360.0;
   phi[i] *= RAD;
   for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
   }
 }
 
 for(k=i=0;i<=maxk;i++){
     inFile >> m >>rknt[i];
     for(j=0;j<gpt;j++){
        inFile >> dum;
        if(choice==MODALL)coeff[k++]=dum;
     }
 }
 
 if(choice==MODALL){
    for(k=i=0;k<=maxk;k++){
        cout <<" k= " <<k<<" rknt = " <<rknt[k]<<endl;
 /*       for(j=0;j<gpt;j++){
            cout <<" "<< coeff[i++];
            if((j+1)%5==0)cout << endl;
        }
        cout << endl;
 */
    }
 }
 
 inFile.close();
 CalAvgDist();
 
 if(rknt[0]<RADIUS-DEPCMB+1.0 && rknt[maxk]>RADIUS-DEPMOHO-1.0)
    region=WHOLEMANTLE;
 else {
    cerr << " Error 3, not implemented yet\n";
    exit(-1);
 }
}

void BSModel::ReadModel_split(int choice)
{

fstream inFile;
int i,j,k,m,nr;
char mod_desc[50];
real dum,lat,lon;

 cout << "name of the SPLIT B-spline model-->" << endl;
 cin >> modelname;
 cout << " choice = " << choice << endl;
 inFile.open(modelname,ios::in);		// open prem file for read
 
 if(!inFile) { 				// open failed?
     cerr << "Cannot open "<< modelname<<" for input\n";
     exit(-1);
 }
 inFile >> mod_desc >> gpt >> maxk_lm >> maxk_um;
 cout << mod_desc << " " << gpt << " " << maxk_lm << " " << maxk_um << endl;
 maxk = maxk_lm+maxk_um-1;
 maxk_lm--;			// to be consistent with other rad functions
 maxk_um--;
 InitKnots(choice);		// allocate space
 
 for(m=i=0;i<gpt;i++){
   inFile >> j >> lat >> lon;
   theta[i]=(90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=360.0;
   phi[i] *= RAD;
   for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
   }
 }
 
 for(k=i=0;i<=maxk;i++){
     inFile >> m >>rknt[i];
     for(j=0;j<gpt;j++){
        inFile >> dum;
        if(choice==MODALL)coeff[k++]=dum;
     }
 }
 
 if(choice==MODALL){
    for(k=i=0;k<=maxk;k++){
        cout <<" k= " <<k<<" rknt = " <<rknt[k]<<endl;
 /*       for(j=0;j<gpt;j++){
            cout <<" "<< coeff[i++];
            if((j+1)%5==0)cout << endl;
        }
        cout << endl;
 */
    }
 }
 
 inFile.close();
 CalAvgDist();
 
 if(rknt[0]<RADIUS-DEPCMB+1.0 && rknt[maxk]>RADIUS-DEPMOHO-1.0)
    region=UPLOWMANTLE;
 else {
    cerr << " Error 3, not implemented yet\n";
    exit(-1);
 }
}


void BSModel::InitKnots(int choice)
{
  int i, j;
	theta = new real[gpt];
	phi = new real[gpt];
	ajknts = new int[gpt*7];
	rknt = new real[maxk+1];
        if(choice==MULTISPLIT){
                rknt_mult=new real* [nsplit_lm+nsplit_um];
                for(i=0; i<nsplit_lm+nsplit_um; i++){
                        rknt_mult[i] = new real[maxk];
                }
                for(i=0;i<nsplit_lm+nsplit_um;i++){
                        for(j=0;j<maxk;j++) {
                                rknt_mult[i][j]=-99.0;
                        }
                }                                                
	}
	if(choice==MODALL || choice==MULTISPLIT){
	   coeff = new real[(maxk+2)*gpt]; //make room for topography
	}
}

void BSModel::CalAvgDist()
{
int i,j,m,n,nb;

  for(i=m=n=0, avgdist=0.0;i<gpt;i++){	/* cal avg distance between nodes */
      m++;						/* 1st element is # of ajacent points, skip it */
	for(j=0;j<6;j++){
	    nb=ajknts[m++];
	    if(nb>i){				/* calculate only once, skip if ==0 */
		n++;
		avgdist+=DistBt2(i,nb);
	    }
	}
  }
  
  avgdist/=(real)n;
}


real BSModel::DistBt2(const int &pt1, const int &pt2)
{
real theta1,theta2,phi1,phi2,dist;

  theta1=theta[pt1];
  phi1  =phi[pt1];
  theta2=theta[pt2];
  phi2  =phi[pt2];

  dist = cos(theta1)*cos(theta2)+sin(theta1)*sin(theta2)*cos(phi2-phi1);
  dist = acos(dist);
  
  return dist;
	
}

/*
 * mode = 0 --- for read, 1 for write and 2 for read and write
 */

int SysOpen(char *filenm,int mode)
{
int fd;

  if(mode==1){
     if(creat(filenm,PERMS)==-1){
        printf("error, cannot creat file\n");
        return -1;
     }
  }
  if((fd=open(filenm,mode))==-1){
      printf(" erorr in  opening  file %s\n",filenm);
      return -1;
  }
  return fd;
}

void  WriteIpm(BSModel &bsmodel,float *ata, float *atd, int &numatd,
int format, int ntopo)
{
int i,j,k,fd,numata,n_written,npt,maxk,numrknt;
float theta_phi[2],*rknt;
char filename[132];

  cout<<"number of atd="<< numatd<<endl;

  cout <<"file to save ATA:\n";
  cin >> filename;
  cout <<"output ipm file="<<filename<<endl;

  fd=SysOpen(filename,1);
  cout << "output format = " << format << endl; 
  n_written=write(fd,&format,sizeof(int));
// write out model knots (horizontal)
  npt=bsmodel.gpt;
  n_written=write(fd,&npt,sizeof(int));
  for(i=0;i<npt;i++){
      theta_phi[0]=bsmodel.theta[i];
      theta_phi[1]=bsmodel.phi[i];
      n_written=write(fd,theta_phi,2*sizeof(float));
  }
// write out model knots (radial)
  maxk=bsmodel.maxk;
  numrknt=maxk+1;
  rknt=new float[maxk+1];
  for(i=0;i<=maxk;i++)rknt[i]=bsmodel.rknt[i];
  n_written=write(fd,&numrknt,sizeof(int));
  n_written=write(fd,rknt,(maxk+1)*sizeof(float));
  delete [] rknt;
  
// write out matrices
  n_written=write(fd,&numatd,sizeof(int));
  if(format > 19 || format == 6) {  
  // numbers between 1-20 are reserved for 3D only
     n_written=write(fd,&ntopo, sizeof(int));  // topography b-spline
     cout << "output number of 2D topo = " << ntopo << endl;
  }
  cout << "number of topography b-splines:" << ntopo << endl;
  numata=(numatd*(numatd+1))/2;
  n_written=write(fd,ata,numata*sizeof(float));
  cout<<"number of bytes for ata="<<n_written<<endl;
  n_written=write(fd,atd,numatd*sizeof(float));
  cout<<"number of bytes for atd="<<n_written<<endl;
  close(fd);
  
  for(i=0;i<5;i++)cout <<"atd["<<i<<"]="<<atd[i]<< endl;
//  for(i=5068;i<5430;i++)cout << i << "  " << atd[i]<< endl;
  
  cout << "ata=\n";
  
  for(k=0,i=0;i<5;i++){
      for(j=0;j<=i;j++,k++){
          cout<<ata[k]<<" ";
      }
      cout <<endl;
  }
}
