#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>


void getstat_double(double *xarr, double *yarr, double wtmin, double wtmax, 
	int num, double *ywinmin, double *ywinmax, double *xmin, 
      	double *xmax, double *ymin, double *ymax)
{
   int i;

   *xmin = 99999999999.9;
   *xmax = -99999999999.9;
   *ymin = 99999999999.9;
   *ymax = -99999999999.9;
   *ywinmin = 99999999999.9;
   *ywinmax = -99999999999.9;
   for(i=0; i<num; i++){
	if(*xmin > xarr[i]) *xmin=xarr[i];
	if(*xmax < xarr[i]) *xmax=xarr[i];
	if(*ymin > yarr[i]) *ymin=yarr[i];
	if(*ymax > yarr[i]) *ymax=yarr[i];
/*  below compute min max in a selected window */
	if(xarr[i] >= wtmin && xarr[i] <= wtmax) {
		if(*ywinmin > yarr[i]) *ywinmin=yarr[i];
		if(*ywinmax < yarr[i]) {
			 *ywinmax=yarr[i];
		}
	}
   }
}

void collon2euler(float scol,float slon,float rcol,float rlon,
	   float *delta,float *alpha,float *beta,float *gamma) 
{
  float az,sina,cosa,eta,mu,ep,tiny=1e-6;
  
 
  *delta=acos(cos(scol)*cos(rcol)+sin(scol)*sin(rcol)*cos(rlon-slon));
  sina=sin(rlon-slon)*sin(rcol)/sin(*delta);
  cosa=(cos(rcol)-cos(scol)*cos(*delta))/(sin(scol)*sin(*delta));
  az=atan2(sina,cosa);
  sina=sin(az); cosa=cos(az);
  if(cosa==0) {
      if(cos(scol)>0) eta=M_PI_2;
      else eta=-M_PI_2;
  }
  else
      eta=atan(cos(scol)/(cosa*sin(scol)));
  if(eta<0.0) eta+=M_PI;
  *gamma=M_PI_2+eta;
  mu=atan2(sina*sin(scol),cos(scol)/sin(eta));
  *beta=M_PI_2-mu;
  ep=atan2(sina*sin(eta),cos(eta)/sin(scol));
  *alpha=-M_PI_2+slon-ep;
	printf("M_PI = %f\n", M_PI);
	printf("M_PI_2 = %f\n", M_PI_2);
}

/*
  The subroutine rot_euler rotates coordinate frames based on the Euler
  angles alpha, beta and gamma (see Edmonds page 7 and page 53).
  The input theta and phi are the coordinates in the geographic
  coordinate system. The angle zeta is measured counterclockwise
  from due east. The returned values of theta, phi, and zeta are
  in the equatorial coordinate system.
  The returned value of theta is between 0 and pi, and the returned
  value of phi is between 0 and 2*pi.
  The inverse operation, i.e. from the rotated (equatorial)
  frame back to the original (geographic) frame is performed
  by replacing alpha by -gamma, beta by -beta, and gamma by -alpha
  in the call to the subroutine rot_euler.

  Copied from Jeroen Tromp 4/12/96
  Minor changes EWFL 4/12/96
*/

/* The following is called by the rot_euler subroutine. */
#define EVEN(i) ((i-2*((i)/2)==0) ? 1:0) /* test if integer i is even */
#define ODD(i) ((i-2*((i)/2)==0) ? 0:1) /* test if integer i is odd */

void reduce(float *pth, float *pph)
{
/* force theta between 0 and pi, and phi between 0 and 2*pi
   Copied from Jeroen Tromp */
  int i;
  float th,ph,PI2=M_PI*2;
  
  th=*pth; ph=*pph;
  i=abs((int)(ph/PI2));
  if(ph<0.0)
    ph+=(i+1)*PI2;
  else if(ph>PI2) ph-=i*PI2;
  *pph=ph;
  if(th<0.0 || th>M_PI) {
    i=(int)(th/M_PI);
    if(th>0.0) {
      if(ODD(i)) {
        th=(i+1)*M_PI-th;
        ph=(ph<M_PI)?ph+M_PI:ph-M_PI;
      }
      else th-=i*M_PI;
    }
    else {
      if(EVEN(i)) {
        th=-th+i*M_PI;
        ph=(ph<M_PI)?ph+M_PI:ph-M_PI;
      }
      else th-=i*M_PI;
    }
    *pth=th; *pph=ph;
  }
}


void rot_euler(float alpha, float beta, float gamma, float *theta,
          float *phi, float *zeta)
{
  int i,j;
  float r_hat[3],rp_hat[3],k_hat[3],kp_hat[3],rotation[3][3];
  double sint,cost,sinp,cosp,sina,cosa,sinb,cosb,sing,cosg,sinz,cosz;

  sint=sin(*theta); cost=cos(*theta);
  sinp=sin(*phi);   cosp=cos(*phi);
  sina=sin(alpha);  cosa=cos(alpha);
  sinb=sin(beta);   cosb=cos(beta);
  sing=sin(gamma);  cosg=cos(gamma);
  sinz=sin(*zeta);  cosz=cos(*zeta);
  r_hat[0]=sint*cosp; r_hat[1]=sint*sinp; r_hat[2]=cost;
  k_hat[0]=-(sinz*cost*cosp+cosz*sinp);
  k_hat[1]=cosz*cosp-sinz*cost*sinp;
  k_hat[2]=sinz*sint;
  rotation[0][0]=cosg*cosb*cosa-sing*sina;
  rotation[0][1]=cosg*cosb*sina+sing*cosa;
  rotation[0][2]=-cosg*sinb;
  rotation[1][0]=-sing*cosb*cosa-cosg*sina;
  rotation[1][1]=-sing*cosb*sina+cosg*cosa;
  rotation[1][2]=sing*sinb;
  rotation[2][0]=sinb*cosa;
  rotation[2][1]=sinb*sina;
  rotation[2][2]=cosb;
  for(i=0;i<3;i++) {
    rp_hat[i]=0.0;
    kp_hat[i]=0.0;
    for(j=0;j<3;j++) {
      rp_hat[i]+=rotation[i][j]*r_hat[j];
      kp_hat[i]+=rotation[i][j]*k_hat[j];
    }
  }
  *theta=atan2(sqrt(rp_hat[0]*rp_hat[0]+rp_hat[1]*rp_hat[1]),rp_hat[2]);
  *phi=atan2(rp_hat[1],rp_hat[0]);
  reduce(theta,phi);
  sint=sin(*theta); cost=cos(*theta);
  sinp=sin(*phi); cosp=cos(*phi);
  sinz=-(kp_hat[0]*cost*cosp+kp_hat[1]*cost*sinp-kp_hat[2]*sint);
  cosz=-kp_hat[0]*sinp+kp_hat[1]*cosp;
  *zeta=atan2(sinz,cosz);
}

void slicer_gcpath(int *numpt, float *ddelta, float *xlat, float *xlon)
{ /* 
   * allows user to input an eq-station pair and return great-circle path
   */
  int  i;
  float  PPI;
  float  RRAD;
  float  lat0, lon0, lat1, lon1, delta;
  float  depth, theta, phi, gamma, zeta, ddel;
  float  alpha, beta;
  float  scol, rcol;
  char   str[256];

  PPI=3.1415926535897932;
  RRAD=0.017453292519943;
  printf("enter beginning lat, lon:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%f %f", &lat1, &lon1);
  printf("enter ending lat, lon:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%f %f", &lat0, &lon0);
  lat0=lat0*RRAD;
  lon0=lon0*RRAD;
  lat1=lat1*RRAD;
  lon1=lon1*RRAD;
  scol=PPI/2.0-lat0;	/* colatitude */
  rcol=PPI/2.0-lat1;
  if(lon0<0.0) lon0+=PPI*2.0;
  if(lon1<0.0) lon1+=PPI*2.0;
  collon2euler(scol,lon0,rcol,lon1,&delta,&alpha,&beta,&gamma);
  printf("delta,alpha,beta,gamma: %f %f %f %f\n", delta, alpha,beta,gamma);
  printf("enter delta increment [in degrees]:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%f", &ddel);
  *ddelta=ddel;
  ddel *=RRAD;
  *numpt=(int)(delta/ddel+1);
  printf("total number of points: %d\n", *numpt);
  theta=scol;
  phi=lat0;
  for(i=0;i<*numpt;i++){
	rot_euler(alpha, beta,gamma , &theta, &phi, &zeta);
	theta=PPI/2.0;
	phi=(float)(i)*ddel;
     	rot_euler(-gamma, -beta, -alpha, &theta, &phi, &zeta);
     	xlat[i] = (PPI/2-theta)/RRAD;
     	phi = phi/RRAD;
     	if(phi >= 180.0) {
        	xlon[i] = phi-360.0;
     	}else {
       		xlon[i] = phi;
     	}
  } 
}


void slicer_by_pole(int *numpt, float *ddelta, float *xlat, float *xlon)
{ /* comuptes a global cross-section path from pole location */
  int  i;
  float  PPI;
  float  PPI_2;
  float  RRAD;
  float  pcolat, plon,ddel,delta;
  float  theta, phi, cphi, sphi;
  float  sf,cf,ctsf,ctcf,st;
  char   str[256];
  

  PPI=3.1415926535897932;
  PPI_2=PPI*0.5;
  RRAD=0.017453292519943;
  printf("pole of the great circle [lat, lon]:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%f %f", &pcolat, &plon);
  printf("delta increment [in degrees]:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%f", &ddel);
  *numpt=(int)(360.0/ddel);
  printf("ddel=%f, total path points=%d\n",ddel, *numpt);
  *ddelta=ddel;
  ddel *= RRAD;
  pcolat=90.-pcolat;
  pcolat *= RRAD;
  plon *= RRAD;
  sf=sin(plon);
  cf=cos(plon);
  ctsf=cos(pcolat)*sf;
  ctcf=cos(pcolat)*cf;
  st=sin(pcolat);
  for(i=0;i<*numpt;i++){
	delta=PPI_2+(float)i*ddel;
	theta=fabs(acos(-cos(delta)*st));
	sphi=(ctsf*cos(delta)+cf*sin(delta))/sin(theta);
	cphi=(ctcf*cos(delta)-sf*sin(delta))/sin(theta);
	phi=atan2(sphi,cphi);
	xlat[i]=90.0-theta/RRAD;
	if(xlat[i]<=(-90.0)) xlat[i]=-89.999999;
	if(xlat[i]>=(90.0)) xlat[i]=89.999999;
        xlon[i]=phi/RRAD;
	if(xlon[i]<=(-180.)) xlon[i]=-179.999999;
	if(xlon[i]>=(180.)) xlon[i]=179.999999;
  }
}

void print_title(FILE *fp)
{  /* simple header */
	fprintf(fp, "#!/bin/csh\n");
	fprintf(fp, "gmtset HEADER_FONT_SIZE 30 OBLIQUE_ANOTATION 0 DEGREE_FORMAT 0\n");
	fprintf(fp, "gmtset LABEL_FONT_SIZE 20\n");
	fprintf(fp, "gmtset ANOT_FONT_SIZE 10\n");
}

void write_wiggle_file(char *fname, double *xarray, double *yarray, double wtmin,
        double wtmax, int ndat, double norm, double ylab)
{ /* generate a gmt file for a number of traces */
   int i;
   FILE *fpwiggle;

   fpwiggle=fopen(fname, "w");
   for(i=0; i<ndat; i++){
	if(norm != 0.0) {
		fprintf(fpwiggle, "%lf %lf %lf\n", xarray[i], ylab, yarray[i]/norm);
	}else{
		fprintf(fpwiggle, "%lf %lf %lf\n", xarray[i], ylab, yarray[i]);
	}
   }
   fclose(fpwiggle);
}

void GenPspolar(fp,xlat,xlon,mag,bbsize,symsize,symchar,outfile,az,aih)
FILE *fp;
float mag;
float symsize,bbsize;
char  symchar;
char  outfile[];
float xlat,xlon,az,aih;
{

	bbsize *= mag/5.0;
	fprintf(fp, "pspolar  <<  END  -R  -JX -S%c%.2f -W4 -N -D%.3f/%.3f -M%.3f -O -K >> %s\n", 
    symchar,symsize,xlon,xlat,bbsize, outfile);
	fprintf(fp, "stat azim ih pol\n");
	fprintf(fp, "xxxx %.3f %.3f +\n", az, aih);
	fprintf(fp, "END\n");
}

void GenGmtEnd(FILE *fp, char *outfl){
/* write an empty text to end GMT plot, reason is that it seems to make GMT bounding box more
   accurate.
 */
		fprintf(fp, "pstext -R0/8.5/0/11 -JX8.5/11 -G255 -N -O << END >> %s\n", outfl);
		fprintf(fp, "7.0 11. 2 0.0 4 2 .\nEND\n");
}

void GenPsmeca(FILE *fp,float xlat,float xlon,int iftrans,float symsize,char *psname, int opt)
{
/* script for plotting stations on a focal-sphere
 * -Sc gives the size in inches for a magnitude 5 event
 */
  float depth;
  float mrr, mtt, mpp, mrt, mrp, mtp;
  float strike1, strike2, dip1, dip2, slip1, slip2;
  float mant;
  char  trans[2];
  int   mex;
  void  remove_edge();

	if(iftrans == 0) sprintf(trans, "%s", "");
	else	sprintf(trans, "%s", "-T");
	printf("enter depth of event:\n");
	scanf("%f", &depth);
	fprintf(fp, "psmeca << END -R  -W3 -JX -Sm%.2f -G160 %s -P -K -V -O >> %s\n",
		symsize,trans, psname);
/* make sure values != 0.000 since GMT causes an error */
	remove_edge(&xlon);
	remove_edge(&xlat);
	remove_edge(&depth);

	if(opt==1) { 
/* allows non-double couple components */
	   printf("moment tensor exponent:\n");
	   scanf("%d", &mex);
	   printf("enter moment tensor mrr, mtt mpp, mrt, mrp, rtp:\n");
	   scanf("%f %f %f %f %f %f", &mrr, &mtt, &mpp, &mrt, &mrp, &mtp);
	   remove_edge(&mrr);
	   remove_edge(&mtt);
	   remove_edge(&mpp);
	   remove_edge(&mrt);
	   remove_edge(&mrp);
	   remove_edge(&mtp);
	   fprintf(fp, "# lon lat depth mrr mtt mpp mrt mrp mtp iexp name \n"); 
	   fprintf(fp, "%.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %d\n",
	   	xlon, xlat, depth, mrr, mtt, mpp, mrt, mrp, mtp, mex);
	}else{ 
/* use strike dip rake double couple components */

	   printf("enter strike, dip, and slip for fault plane 1:\n");
	   scanf("%f %f %f", &strike1, &dip1, &slip1);
	   printf("enter strike, dip, and slip for fault plane 2:\n");
	   scanf("%f %f %f", &strike2, &dip2, &slip2);
	   remove_edge(&strike1);
	   remove_edge(&strike2);
	   remove_edge(&dip1);
	   remove_edge(&dip2);
	   remove_edge(&slip1);
	   remove_edge(&slip2);
	   printf("enter mantisa and exponent:\n");
	   scanf("%f %f", &mant, &mex);
	   fprintf(fp, "#lon lat depth str dip slip st dip slip mant exp plon plat\n");
	   fprintf(fp, "%.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %.3f %d 0\n",
		xlon, xlat,depth,strike1,dip1,slip1,strike2,dip2,slip2,mant,mex);	
	}
	fprintf(fp, "END\n");
}

void remove_edge(float *val)
{ /*gmt wont work when moment tensor element =0.000 */
	float val1;
	val1=*val;
	if(fabs(val1-0.0)<0.001) {
		if(val1>=0.0)  val1=0.001;
		else  val1=-0.001;
	}
	if(fabs(val1-90.0)<0.001) val1=89.999;
	if(fabs(val1+90.0)<0.001) val1=-89.999;
	if(fabs(val1-180.0)<0.001) val1=179.999;
	if(fabs(val1+180.0)<0.001) val1=-179.999;
	*val=val1;
}

void draw_regional_ocean(FILE *fp,float lonmin, float lonmax, float latmin, float latmax, char *outfl)
{
/*
 create bathymetry map
*/
	fprintf(fp, "img2grd bathy -Gtest.grd -R%.3f/%.3f/%.3f/%.3f -T1 -V\n", lonmin, lonmax, latmin, latmax);
   	fprintf(fp,"grdimage test.grd -Ccolorbar.cpt -JX -R -O -V -K >> %s\n", outfl);
}

void write_gmtscript(FILE *fp, char *filename,float dx,float dy, float displaymin,
		float displaymax,float grid_y,float grid_ystart)
{
  fprintf(fp, "#! /bin/csh\n");
  fprintf(fp, "gmtset HEADER_FONT_SIZE 25 OBLIQUE_ANOTATION 0 DEGREE_FORMAT 0 LABEL_FONT Times-Roman\n");
  fprintf(fp,"xyz2grd %s -G%s.bin -I%f/%f -R%f/%f/%f/%f -N0.0\n",
       filename, filename, dx, -dy, displaymin, displaymax, grid_y, grid_ystart);
}

void draw_colorbar(FILE *fp,char *colfile,float xmin,float xmax,
			float ymin,float ymax, char *legend, char *outfl)
{
   fprintf(fp,"psscale -C%s -D%f/%f/%f/%f -O -L -B:.%s: >> %s\n",
	colfile, xmin,xmax,ymin,ymax,legend,outfl);
}

void write_gmtheader(FILE *fp, char *outfl, int iffirst)
{
  int  headersz, labelsz;
  char str[256];
  if(iffirst==1){
  	fprintf(fp, "#! /bin/csh\n");
  	fprintf(fp, "rm %s\n", outfl);
  	headersz=24;
	labelsz=16;
  	fprintf(fp, "gmtset HEADER_FONT_SIZE %d OBLIQUE_ANOTATION 0 DEGREE_FORMAT 0\n", headersz);
  	fprintf(fp, "gmtset LABEL_FONT_SIZE %d\n", labelsz);
  } 
}

void write_pscoast_psbasemap_x(FILE *fp, float lonmin,float lonmax,float latmin, 
		float latmax, int latnet, int lonnet, char *outfl, int iffirst)
{  
/* 
 *	plot coast line and psbasemap 
 */
   if(iffirst==1) {
   	fprintf(fp,"pscoast -JX4.5/4.5 -R%.1f/%.1f/%.1f/%.1f -W2 -G200 -X1.3 -Y5.2 -A200 -V -P -K >! %s\n",
		lonmin,lonmax,latmin,latmax,outfl);
   }else{
   	fprintf(fp,"pscoast -JX4.5/4.5 -R%.1f/%.1f/%.1f/%.1f -W2 -G200 -X4.9 -Y2.5 -A200 -P -O -V -K >> %s\n",
		lonmin,lonmax,latmin,latmax,outfl);
   }
   fprintf(fp,"psbasemap -JX -R -B%dg%d/%dg%dneSW  -V -O -K >> %s\n",
		lonnet,lonnet/2,latnet,latnet/2,outfl);		
}


void write_pscoast_Joc(FILE *fp, float ctlat, float ctlon, float polat,
			float polon, float lblat,float lblon,float rtlat, 
		        float rtlon, int netwidth, char *outfl, int iffirst)
{  /* plot coast line for a oblique mercator plot for cross-sec */
   if(iffirst>0) {
   	fprintf(fp,"pscoast -Joc%.1f/%.1f/%.1f/%.1f/0.05 -R%.1f/%.1f/%.1f/%.1fr -B10g5 -W2 -G200 -A250 -X1.3 -P -Y5.2 -V -K >! %s\n",
		ctlon, ctlat, polon, polat, 
		lblon,lblat,rtlon,rtlat, outfl);
   }else{
   	fprintf(fp,"pscoast -Joc%.1f/%.1f/%.1f/%.1f/0.05 -R%.1f/%.1f/%.1f/%.1fr -B10g5 -W2 -G200 -A250 -X1.3 -P -Y2.5 -O  -V -K >> %s\n",
		ctlon, ctlat, polon, polat, 
		lblon,lblat,rtlon,rtlat, outfl);
   }
}

void write_grdimage_x(FILE *fp, char *binfl,float lonmin,float lonmax,float latmin,float latmax, char *outfl)
{
   	fprintf(fp,"grdimage %s -Ccolorbar.cpt -JX3.5/2.2 -R%.1f/%.1f/%.1f/%.1f  -B20:\"distance along arc (deg):\"/100:\"radius (km)\":WS -Y-2.5 -O -V -K >> %s\n",
		binfl,lonmin,lonmax,latmin,latmax,outfl);
}


void write_psxy_x(FILE *fp, char *locfile, float lonmin,float lonmax,float latmin,float latmax, char *outfl)
{
   	fprintf(fp,"psxy %s -JX -R%.1f/%.1f/%.1f/%.1f -W10 -G255/0/0 -O -L -V -K >> %s\n",
		locfile,lonmin,lonmax,latmin,latmax,outfl);
}

void write_psxy_x_double(FILE *fp, char *locfile, double lonmin,
     double lonmax, double latmin, double latmax, char *outfl)
{
   	fprintf(fp,"psxy %s -JX -R%.1f/%.1f/%.1f/%.1f -W3 -G255/0/0 -O -L -V -K >> %s\n",
		locfile,lonmin,lonmax,latmin,latmax,outfl);
}


void write_psxy_Joc(FILE *fp, char *locfile, float latmin,float lonmin,float latmax,float lonmax, char *outfl)
{
   	fprintf(fp,"psxy %s -JX -R%.1f/%.1f/%.1f/%.1fr -W10 -G255/0/0 -O -L -V -K >> %s\n",
		locfile,lonmin,latmin,lonmax,latmax,outfl);
}

void write_psxy_2symbol(FILE *fp, char *locfile, char *locfile1, char *outfl)
{ /* this makes a rectangular or mercator xy plot */
  	fprintf(fp, "#! /bin/csh\n");
  	fprintf(fp, "gmtset HEADER_FONT_SIZE 25 OBLIQUE_ANOTATION 0 DEGREE_FORMAT 0 LABEL_FONT Times-Roman\n");
   	fprintf(fp,"pscoast -JH225/4.0 -R0.0/360.0/-90.0/90.0 -W1 -G200 -A1000 -X1.2 -Y5 -P -V -K >! %s\n", outfl);
   	fprintf(fp,"psbasemap -JH -R -B90g45/90g45ENW -V -O -K >> %s\n",outfl);		
   	fprintf(fp,"psxy %s -JH -R0.0/360.0/-90.0/90.0 -W1/255/255/255 -G0/0/255 -St -P -K -O >> %s\n",
		locfile, outfl);
   	fprintf(fp,"psxy %s -JH -R0.0/360.0/-90.0/90.0 -W1/255/255/255 -G255/0/0 -St -P -O >> %s\n",
		locfile1, outfl);
	fprintf(fp, "\n\nghostview %s\n", outfl);
}

void write_psscale_x(FILE *fp, char *colfile, char *outfl, int iffirst)
{
	if(iffirst != 1) {
		fprintf(fp, "psscale -C%s -D5.0/2.3/3.2/0.2h -X-5.7 -Y-3.1 -B:.\"velocity perturbation (%s)\": -O -L -V -K >> %s\n", colfile,"%",outfl);
	} 
}

void write_pstext_x(FILE *fp, char  *outfl, int iffirst)
{
	if(iffirst==0){
		fprintf(fp, "# for pstext\n");
		fprintf(fp, "# (1,2) coord, 3=size, 4=rotation, 5=font\n");
		fprintf(fp, "pstext -R0/8.5/0/11 -X-0.5 -Y5.6 -JX -K -N -O << END >> %s\n",outfl);
		fprintf(fp, "2.0 10. 28 0.0 4 2 a)\nEND\n");

		fprintf(fp, "pstext -R0/8.5/0/11 -X4.8 -JX  -N -O << END >> %s\n", outfl);
		fprintf(fp, "2.0 10. 28 0.0 4 2 b)\nEND\n");
        }
}

void write_xyz2grd(char *gridfl, float dlon,float dlat,
  float lonmin,float lonmax,float latmin,float latmax, char *binfile)
{
  char xyzfile[256],systemstring[256];
  FILE *fp1, *fp2;
  char str[256];
  int  ierr;
 
  printf("xyz2grd file name:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%s", xyzfile);
  printf("bin file name:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%s", binfile);
  printf("dlat=%f, dlon=%f\n", dlat, dlon);
  printf("xyzfile=%s\n", xyzfile);
  fp1=fopen(xyzfile,"w");
  fprintf(fp1,"xyz2grd %s -G%s -I%f/%f -R%f/%f/%f/%f/ -N0.0\n",gridfl,binfile,dlon,dlat,
		lonmin,lonmax,latmin,latmax);
  fclose(fp1);
  sprintf(systemstring,"chmod u+x %s\n", xyzfile);
  ierr=system(systemstring);
  bzero(systemstring, sizeof(systemstring));
  sprintf(systemstring,"%s\n", xyzfile);
  ierr=system(systemstring);
  	
}
void write_xyz2grd_new(FILE *fp, char *gridfl, float dlon,float dlat,
  float lonmin,float lonmax,float latmin,float latmax, char *binfile)
{
  char xyzfile[256],systemstring[256], str[256];
  FILE *fp1, *fp2;
  int  ierr;
 
  printf("bin file name:\n");
  GetLine_gmt(str, 256);
  sscanf(str, "%s", binfile);
  printf("dlat=%f, dlon=%f\n", dlat, dlon);
  fprintf(fp,"xyz2grd %s -G%s -I%f/%f -R%f/%f/%f/%f/ -N0.0\n",gridfl,binfile,dlon,dlat,
		lonmin,lonmax,latmin,latmax);  	
}


void write_pswiggle_double(FILE *fppswig, char *locfile, double jxmin, double jxmax,
	double lonmin,double lonmax,double latmin,double latmax, double zvalue,
	int filcol, char *outfl, int iffirst)
{ /* Z is the data-unit/inch, so it is normalization */

     if(iffirst == 1) {
   	fprintf(fppswig,"pswiggle %s -JX%.1f/%.1f -R%.1f/%.1f/%.1f/%.1f -W3 -Z%.1f -G0/%d/0 -P -V -K >! %s\n",
		locfile,jxmin, jxmax, lonmin,lonmax,latmin,latmax,zvalue,filcol,outfl);
     }else{
   	fprintf(fppswig,"pswiggle %s -JX%.1f/%.1f -R%.1f/%.1f/%.1f/%.1f -W3 -Z%.1f -G0/%d/0  -P -O  -V -K >> %s\n",
		locfile,jxmin, jxmax, lonmin,lonmax,latmin,latmax,zvalue,filcol,outfl);
     }
}


void Color_Pt(float absmax,float ymin,float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
char str[256];
FILE *fp;

/* This program will generate a color plate of 18 colors.
 * There will be 9 redish to white and 9 blue to black colors. */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("choose one:\n");
   printf("==================================================\n");
   printf("1.uniformly divided coloring scheme\n");
   printf("2.emphasize specified secondary phase\n");
   printf("==================================================\n");
   printf("enter:\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%d", &op1);
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   if(op1==1) {
      dcolor = (scmax-scmin)/9.0;
   } else {
      dcolor = second_max/8.0;
   }
   red = 195;
   green = 0;
   blue = 64;
   col1= scmin;
   col2 = -second_max;
   if(op1 == 1) 
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);
   
   if(op1 == 2) {
      red = 195;
      green = 0;
      blue = 64;
      tempcol = col1 + 0.01;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col1,red, green,
      blue, tempcol, red, green, blue);

      red = 20;
      green = 0;
      blue = 130;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", tempcol,red, green,
      blue, col2, red, green, blue);
   }
   red = 0;
   green = 0;
   blue = 160;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 0;
   green = 10;
   blue = 200;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 0;
   green =40;
   blue = 224;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 13;
   green = 129;
   blue = 248;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 50;
   green = 190;
   blue = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 97;
   green = 225;
   blue = 240;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 255;
   green = 185;
   blue = 102;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 255;
   green = 160;
   blue = 69;
   col2 = col1 +dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 255;
   green = 155;
   blue = 124;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 255;
   green = 124;
   blue = 124;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 238;
   green = 80;
   blue = 78;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 217;
   green = 39;
   blue = 64;
   col2 = col1 + dcolor;
   if(op1 == 1)
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);

   if(op1 == 2) {
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);

      red = 195;
      green = 0;
      blue = 64;
      tempcol = scmax - 0.01;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
      blue, tempcol, red, green, blue);
      col2 = tempcol;
   }
   red = 195;
   green = 0;
   blue = 64;
   col1 = scmax;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d      U\n", col2,red, green,
      blue, col1, red, green, blue);
   fclose(fp);
}

void Read_table_double(char *tabname, double *arr1, double *arr2, double *arr3, 
				double *arr4, int ncol, int *nrow, int *ierr)
{  /* read travel time tables */
  FILE *fprdtab;
  int  i;

   fprdtab = fopen(tabname, "r");
   *ierr = 0;
   i = 0;
   switch(ncol) {
	case 1:
		while(fscanf(fprdtab, "%lf", &arr1[i]) != EOF) {
			i++;
		}
		break;
	case 2:
		while(fscanf(fprdtab, "%lf %lf", &arr1[i], &arr2[i]) != EOF) {
			i++;
		}
		break;
	case 3:
		while(fscanf(fprdtab, "%lf %lf %lf", &arr1[i], &arr2[i], &arr3[i]) != EOF) {
			i++;
		}
		break;
	case 4:
		while(fscanf(fprdtab, "%lf %lf %lf %lf", &arr1[i], &arr2[i], &arr3[i], &arr4[i]) != EOF) {
			i++;
		}
		break;
	default:
		*ierr = 1;
		break;
   }

   *nrow = i;
   printf("Number of table entries =%d\n", *nrow);
   fclose(fprdtab);
}

void Append_index_to_name(char *name, int sz, int num)
{ /* add a number to the end of a string */
  char tail[10];
  char *temp;
	
  temp = (char *) malloc((unsigned) sz * sizeof(char));
  strcpy(temp, name);
  sprintf(tail,  "%0d", num);
  strcat(temp, tail);
  strcpy(name, temp);
  free(temp);
}


void Color_Pt_Vel(float absmax,float ymin,float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
char str[256];
FILE *fp;

/* This program will generate a color plate of 18 colors.
 * There will be 9 redish to white and 9 blue to black colors. */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("choose one:\n");
   printf("==================================================\n");
   printf("1.uniformly divided coloring scheme\n");
   printf("2.emphasize specified secondary phase\n");
   printf("==================================================\n");
   printf("enter:\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%d", &op1);
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   if(op1==1) {
      dcolor = (scmax-scmin)/9.0;
   } else {
      dcolor = second_max/8.0;
   }
   red = 195;
   green = 0;
   blue = 64;
   col1= scmin;
   col2 = -second_max;
   if(op1 == 1) 
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);
   
   if(op1 == 2) {
      red = 195;
      green = 0;
      blue = 64;
      tempcol = col1 + 0.01;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col1,red, green,
      blue, tempcol, red, green, blue);

      red = 20;
      green = 0;
      blue = 130;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", tempcol,red, green,
      blue, col2, red, green, blue);
   }
   red = 0;
   green = 0;
   blue = 160;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 0;
   green = 10;
   blue = 200;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 0;
   green =40;
   blue = 224;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 13;
   green = 129;
   blue = 248;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 50;
   green = 190;
   blue = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 97;
   green = 225;
   blue = 240;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 200;
   green = 245;
   blue = 200;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 255;
   green = 185;
   blue = 102;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 255;
   green = 160;
   blue = 69;
   col2 = col1 +dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 255;
   green = 155;
   blue = 124;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 255;
   green = 124;
   blue = 124;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red, green, blue);

   red = 238;
   green = 80;
   blue = 78;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red, green, blue);

   red = 217;
   green = 39;
   blue = 64;
   col2 = col1 + dcolor;
   if(op1 == 1)
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);

   if(op1 == 2) {
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);

      red = 195;
      green = 0;
      blue = 64;
      tempcol = scmax - 0.01;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
      blue, tempcol, red, green, blue);
      col2 = tempcol;
   }
   red = 195;
   green = 0;
   blue = 64;
   col1 = scmax;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d      U\n", col2,red, green,
      blue, col1, red, green, blue);
   fclose(fp);
}



void Color_Pt_Smooth(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* This program will generate a color plate of 18 colors.
 * There will be 9 redish to white and 9 blue to black colors. */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("choose one:\n");
   printf("==================================================\n");
   printf("1.uniformly divided coloring scheme\n");
   printf("2.emphasize specified secondary phase\n");
   printf("==================================================\n");
   printf("enter:\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%d", &op1);
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   if(op1==1) {
      dcolor = (scmax-scmin)/9.0;
   } else {
      dcolor = second_max/8.0;
   }
   red = 20;
   green = 0;
   blue = 130;
   col1= scmin;
   col2 = -second_max;
   if(op1 == 1) 
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);
   
   if(op1 == 2) {
      red = 20;
      green = 0;
      blue = 130;
      tempcol = col1 + 0.01;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col1,red, green,
      blue, tempcol, red, green, blue);

      red = 20;
      green = 0;
      blue = 130;
      red2 = 0;
      green2 = 0;
      blue2 = 160;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", tempcol,red, green,
      blue, col2, red2, green2, blue2);
   }
   red = 0;
   green = 0;
   blue = 160;
   red2 = 0;
   green2 = 10;
   blue2 = 200;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 10;
   blue = 200;
   red2 = 0;
   green2=40;
   blue2 = 224;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green =40;
   blue = 224;
   red2 = 13;
   green2 = 129;
   blue2 = 248;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 13;
   green = 129;
   blue = 248;
   red2 = 50;
   green2 = 190;
   blue2 = 255;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 190;
   blue = 255;
   red2 = 97;
   green2 = 225;
   blue2 = 240;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 97;
   green = 225;
   blue = 240;
   red2 = 150;
   green2 = 235;
   blue2 = 240;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 150;
   green = 235;
   blue = 240;
   red2 = 200;
   green2 = 245;
   blue2 = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 200;
   green = 245;
   blue = 200;
   red2 = 200;
   green2 = 245;
   blue2 = 200;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 200;
   green = 245;
   blue = 200;
   red2 = 200;
   green2 = 245;
   blue2 = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 200;
   green = 245;
   blue = 200;
   red2 = 255;
   green2 = 185;
   blue2 = 102;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 185;
   blue = 102;
   red2 = 255;
   green2 = 160;
   blue2 = 69;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 160;
   blue = 69;
   red2 = 255;
   green2 = 155;
   blue2 = 124;
   col2 = col1 +dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 155;
   blue = 124;
   red2 = 255;
   green2 = 124;
   blue2 = 124;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 124;
   blue = 124;
   red2 = 238;
   green2 = 80;
   blue2 = 78;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 238;
   green = 80;
   blue = 78;
   red2 = 217;
   green2 = 39;
   blue2 = 64;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 217;
   green = 39;
   blue = 64;
   red2 = 195;
   green2 = 0;
   blue2 = 64;
   col2 = col1 + dcolor;
   if(op1 == 1)
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red2, green2, blue2);

   if(op1 == 2) {
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red2, green2, blue2);

      red = 195;
      green = 0;
      blue = 64;
      tempcol = scmax - 0.01;
      fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
      blue, tempcol, red, green, blue);
      col2 = tempcol;
   }
   red = 195;
   green = 0;
   blue = 64;
   col1 = scmax;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d      U\n", col2,red, green,
      blue, col1, red, green, blue);
   fclose(fp);
}

void Color_Pt_Smooth_Vel(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* This program will generate a color plate of 18 colors.
 * There will be 9 redish to white and 9 blue to black colors. */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/18.0;
   red = 195;
   green = 0;
   blue = 64;
   red2 = 217;
   green2 = 39;
   blue2 = 64;
   col1 =ymin-10.0;
   col2 = scmin;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);

   red = 195;
   green = 0;
   blue = 64;
   red2 = 217;
   green2 = 39;
   blue2 = 64;
   col1 = scmin;
   col2 = col1+dcolor; 
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d    L\n", col1,red, green,
      blue, col2, red2, green2, blue2);
   
   red = 217;
   green = 39;
   blue = 64;
   red2 = 238;
   green2 = 80;
   blue2 = 78;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 238;
   green = 80;
   blue = 78;
   red2 = 255;
   green2 = 124;
   blue2 = 124;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 124;
   blue = 124;
   red2 = 255;
   green2 = 155;
   blue2 = 124;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 155;
   blue = 124;
   red2 = 255;
   green2 = 160;
   blue2 = 69;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 160;
   blue = 69;
   red2 = 255;
   green2 = 185;
   blue2 = 102;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 185;
   blue = 102;
   red2 = 240;
   green2 = 220;
   blue2 = 102;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 240;
   green = 220;
   blue = 102;
   red2 = 200;
   green2 = 245;
   blue2 = 200;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 200;
   green = 245;
   blue = 200;
   red2 = 200;
   green2 = 245;
   blue2 = 200;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 200;
   green = 245;
   blue = 200;
   red2 = 200;
   green2 = 245;
   blue2 = 200;
   printf("col2 = %f\n", col2);
   col1 = col2+dcolor;
   fprintf(fp, "%10.0f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 200;
   green = 245;
   blue = 200;
   red2 = 150;
   green2 = 235;
   blue2 = 240;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 150;
   green = 235;
   blue = 240;
   red2 = 97;
   green2 = 225;
   blue2 = 240;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 97;
   green = 225;
   blue = 240;
   red2 = 50;
   green2 = 190;
   blue2 = 255;
   col2 = col1 +dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 190;
   blue = 255;
   red2 = 13;
   green2 = 129;
   blue2 = 248;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 13;
   green = 129;
   blue = 248;
   red2 = 0;
   green2 =40;
   blue2 = 224;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green=40;
   blue = 224;
   red2 = 0;
   green2 = 10;
   blue2 = 200;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 10;
   blue = 200;
   red2 = 0;
   green2 = 0;
   blue2 = 160;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red2, green2, blue2);

   red = 0;
   green = 0;
   blue = 160;
   red2 = 20;
   green2 = 0;
   blue2 = 130;
   col1 = scmax;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 20;
   green = 0;
   blue = 130;
   red2 = 20;
   green2 = 0;
   blue2 = 130;
   col2 = absmax+10.;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d    L\n", col1,red, green,
      blue, col2, red2, green2, blue2);
   fclose(fp);
}

void Color_Pt_Smooth_yellow(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* This program will generate a color plate of 18 colors.
 * There will be 9 redish to white and 9 blue to black colors. */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/18.0;
   red = 195;
   green = 0;
   blue = 64;
   red2 = 217;
   green2 = 39;
   blue2 = 64;
   col1 =-9999;
   col2 = scmin;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red, green, blue);

   red = 195;
   green = 0;
   blue = 64;
   red2 = 217;
   green2 = 39;
   blue2 = 64;
   col1 = scmin;
   col2 = col1+dcolor; 
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d    L\n", col1,red, green,
      blue, col2, red2, green2, blue2);
   
   red = 217;
   green = 39;
   blue = 64;
   red2 = 238;
   green2 = 80;
   blue2 = 78;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 238;
   green = 80;
   blue = 78;
   red2 = 255;
   green2 = 124;
   blue2 = 124;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 124;
   blue = 124;
   red2 = 255;
   green2 = 155;
   blue2 = 124;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 155;
   blue = 124;
   red2 = 255;
   green2 = 160;
   blue2 = 69;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 160;
   blue = 69;
   red2 = 255;
   green2 = 185;
   blue2 = 102;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 185;
   blue = 102;
   red2 = 240;
   green2 = 220;
   blue2 = 102;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 240;
   green = 220;
   blue = 102;
   red2 = 255;
   green2 = 255;
   blue2 = 100;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 255;
   blue = 100;
   red2 = 255;
   green2 = 255;
   blue2 = 100;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 255;
   blue = 100;
   red2 = 255;
   green2 = 255;
   blue2 = 100;
   printf("col2 = %f\n", col2);
   col1 = col2+dcolor;
   fprintf(fp, "%10.0f%7d%7d%7d%10.3f%7d%7d%7d     L\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 255;
   blue = 100;
   red2 = 150;
   green2 = 235;
   blue2 = 240;
   col2 = col1+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 150;
   green = 235;
   blue = 240;
   red2 = 97;
   green2 = 225;
   blue2 = 240;
   col1 = col2+dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 97;
   green = 225;
   blue = 240;
   red2 = 50;
   green2 = 190;
   blue2 = 255;
   col2 = col1 +dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 190;
   blue = 255;
   red2 = 13;
   green2 = 129;
   blue2 = 248;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 13;
   green = 129;
   blue = 248;
   red2 = 0;
   green2 =40;
   blue2 = 224;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green=40;
   blue = 224;
   red2 = 0;
   green2 = 10;
   blue2 = 200;
   col1 = col2 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 10;
   blue = 200;
   red2 = 0;
   green2 = 0;
   blue2 = 160;
   col2 = col1 + dcolor;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col1,red, green,
      blue, col2, red2, green2, blue2);

   red = 0;
   green = 0;
   blue = 160;
   red2 = 20;
   green2 = 0;
   blue2 = 130;
   col1 = scmax;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 20;
   green = 0;
   blue = 130;
   red2 = 20;
   green2 = 0;
   blue2 = 130;
   col2 = 9999.;
   fprintf(fp, "%10.3f%7d%7d%7d%10.3f%7d%7d%7d    L\n", col1,red, green,
      blue, col2, red2, green2, blue2);
   fclose(fp);
}



void Color_Pt_Vel_18col(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* draws 16 color colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("*** this version plots positive scale only ***\n");
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/16.0;
   red = 215;
   green = 0;
   blue = 0;
   col1=-10000;
   col2 = scmin;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 225;
   green = 71;
   blue =  0;
   red2 = 225;
   green2 = 71;
   blue2 = 0;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   L\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 227;
   green = 94;
   blue = 0;
   red2 = 227;
   green2 = 94;
   blue2 = 0;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 230;
   green = 125;
   blue = 0;
   red2 = 230;
   green2 = 125;
   blue2 = 0;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 235;
   green = 158;
   blue = 0;
   red2 = 235;
   green2 = 158;
   blue2 = 0;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 240;
   green = 181;
   blue = 0;
   red2 = 240;
   green2 = 181;
   blue2 = 0;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 248;
   green = 209;
   blue = 0;
   red2 = 248;
   green2 = 209;
   blue2 = 0;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 220;
   blue = 102;
   red2 = 255;
   green2 = 220;
   blue2 = 102;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 240;
   green = 235;
   blue = 102;
   red2 = 240;
   green2 = 235;
   blue2 = 102;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 220;
   green = 255;
   blue = 102;
   red2 = 220;
   green2 = 255;
   blue2 = 102;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
      blue, col1, red, green, blue);
   col2=col1+dcolor;
   red = 160;
   green = 255;
   blue = 160;
   red2 = 160;
   green2 = 255;
   blue2 = 160;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 100;
   green = 255;
   blue = 220;
   red2 = 100;
   green2 = 255;
   blue2 = 220;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 100;
   green = 230;
   blue = 255;
   red2 = 100;
   green2 = 230;
   blue2 = 255;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 170;
   blue = 255;
   red2 = 50;
   green2 = 170;
   blue2 = 255;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 13;
   green = 129;
   blue = 248;
   red2 = 13;
   green2 = 129;
   blue2 = 248;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green = 80;
   blue = 235;
   red2 = 0;
   green2 = 80;
   blue2 = 235;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 40;
   blue = 220;
   red2 = 0;
   green2 = 40;
   blue2 = 220;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 0;
   green = 0;
   blue = 130;
   red2 = 0;
   green2 = 0;
   blue2 = 130;
   col1 = 10000;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d	L\n", col2,red, green,
      	blue, col1, red, green, blue);
   fclose(fp);
}

void Color_Pt_Vel_18col_sharp0(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* draws 16 color colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("*** this version plots positive scale only ***\n");
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/16.0;
   red = 215;
   green = 0;
   blue = 0;
   col1=-10000;
   col2 = scmin;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 225;
   green = 71;
   blue =  0;
   red2 = 225;
   green2 = 71;
   blue2 = 0;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   L\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 227;
   green = 94;
   blue = 0;
   red2 = 227;
   green2 = 94;
   blue2 = 0;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 230;
   green = 125;
   blue = 0;
   red2 = 230;
   green2 = 125;
   blue2 = 0;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 235;
   green = 158;
   blue = 0;
   red2 = 235;
   green2 = 158;
   blue2 = 0;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 240;
   green = 181;
   blue = 0;
   red2 = 240;
   green2 = 181;
   blue2 = 0;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 248;
   green = 209;
   blue = 0;
   red2 = 248;
   green2 = 209;
   blue2 = 0;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 220;
   blue = 102;
   red2 = 255;
   green2 = 220;
   blue2 = 102;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 240;
   green = 235;
   blue = 102;
   red2 = 240;
   green2 = 235;
   blue2 = 102;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 180;
   green = 255;
   blue = 140;
   red2 = 180;
   green2 = 255;
   blue2 = 140;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
      blue, col1, red, green, blue);
   col2=col1+dcolor;
   red = 140;
   green = 255;
   blue = 180;
   red2 = 140;
   green2 = 255;
   blue2 = 180;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 100;
   green = 255;
   blue = 220;
   red2 = 100;
   green2 = 255;
   blue2 = 220;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 100;
   green = 230;
   blue = 255;
   red2 = 100;
   green2 = 230;
   blue2 = 255;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 170;
   blue = 255;
   red2 = 50;
   green2 = 170;
   blue2 = 255;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 13;
   green = 129;
   blue = 248;
   red2 = 13;
   green2 = 129;
   blue2 = 248;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green = 80;
   blue = 235;
   red2 = 0;
   green2 = 80;
   blue2 = 235;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 40;
   blue = 220;
   red2 = 0;
   green2 = 40;
   blue2 = 220;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 0;
   green = 0;
   blue = 130;
   red2 = 0;
   green2 = 0;
   blue2 = 130;
   col1 = 10000;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d	L\n", col2,red, green,
      	blue, col1, red, green, blue);
   fclose(fp);
}

void Color_Pt_Smooth_Pow_24col(float absmax,float ymin, float second_max)
{
float dcolor,dcolor2, dcolor3, col1, col2, tempcol;
float scmax0, scmin1, scmin, scmax, scmax1;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* This program will generate a color plate of 26 colors.
 * there are 8 blue, 8 green to yellow and 8 red colors
   2 more are added in to fill the gaps in the colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter 24 index color scale (min, max0, max1)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f %f", &scmin, &scmax, &scmax1);
   dcolor = (scmax-scmin)/23.0;
   red = 0;
   green = 0;
   blue = 130;
   red2 = 0;
   green2 = 0;
   blue2 = 220;
   col1= -9999.0;
   col2 = scmin+dcolor;
/****** BLUE COLOR *********/
/* index 1  */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d	L\n", col1,red, green,
   blue, col2, red2, green2, blue2);
   red = 0;
   green = 0;
   blue = 220;
   red2 = 0;
   green2 = 100;
   blue2 = 235;
   col1 = col2 + dcolor;
/* index 2 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d     \n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 100;
   blue = 235;
   red2 = 0;
   green2=170;
   blue2 = 220;
   col2 = col1+dcolor;
/* index 3 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green =170;
   blue = 220;
   red2 = 0;
   green2 = 200;
   blue2 = 230;
   col1 = col2+dcolor;
/* index 4 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 200;
   blue = 230;
   red2 = 100;
   green2 = 240;
   blue2 = 230;
   col2 = col1+dcolor;
/* index 5 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 100;
   green = 240;
   blue = 230;
   red2 = 150;
   green2 = 255;
   blue2 = 230;
   col1 = col2+dcolor;
/* index 6 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 150;
   green = 255;
   blue = 230;
   red2 = 180;
   green2 = 255;
   blue2 = 210;
   col2 = col1+dcolor;
/* index 7 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 180;
   green = 255;
   blue = 210;
   red2 = 180;
   green2 = 255;
   blue2 = 180;
   col1 = col2+dcolor;
/* index 8 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

/******* GREEN TO YELLOW *******/

   red = 180;
   green = 255;
   blue = 180;
   red2 = 110;
   green2 = 255;
   blue2 = 110;
   col2 = col1 + dcolor;
/* index 9 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 110;
   green = 255;
   blue = 110;
   red2 = 140;
   green2 = 255;
   blue2 = 140;
   col1 = col2+dcolor;
/* index 10 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d    \n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 140;
   green = 255;
   blue = 140;
   red2 = 180;
   green2 = 255;
   blue2 = 150;
   col2 = col1+dcolor;
/* index 11 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 180;
   green = 255;
   blue = 150;
   red2 = 200;
   green2 = 255;
   blue2 = 170;
   col1 = col2+dcolor;
/* index 12 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 200;
   green = 255;
   blue = 170;
   red2 = 210;
   green2 = 255;
   blue2 = 120;
   col2 = col1 +dcolor;
/* index 13 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 210;
   green = 255;
   blue = 120;
   red2 = 210;
   green2 = 255;
   blue2 = 150;
   col1 = col2 + dcolor;
/* index 14 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 210;
   green = 255;
   blue = 150;
   red2 = 235;
   green2 = 255;
   blue2 = 170;
   col2 = col1 + dcolor;
/* index 15 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 235;
   green = 255;
   blue = 170;
   red2 = 255;
   green2 = 255;
   blue2 = 140;
   col1 = col2 + dcolor;
/* index 16 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

/******* RED COLOR *******/

   red = 255;
   green = 255;
   blue = 140;
   red2 = 255;
   green2 =240;
   blue2 = 140;
   col2 = col1 + dcolor;
/* index 17 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      blue, col2, red2, green2, blue2);


   red = 255;
   green = 240;
   blue = 140;
   red2 = 255;
   green2 = 210;
   blue2 = 100;
   col1 = col2;
   col2 = col2+dcolor;
/* index 18 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 210;
   blue = 100;
   red2 = 255;
   green2 = 170;
   blue2 = 80;
   col1=col2;
   col2 = col2 +dcolor;
/* index 19 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 170;
   blue = 80;
   red2 = 255;
   green2 = 140;
   blue2 = 124;
   col1 = col2 + dcolor;
/* index 20 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 140;
   blue = 124;
   red2 = 255;
   green2 = 100;
   blue2 = 78;
   col2 = col1 + dcolor;
/* index 21 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 100;
   blue = 78;
   red2 = 255;
   green2 = 40;
   blue2 = 64;
   col1 = col2 + dcolor;
/* index 22 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 40;
   blue = 64;
   red2 = 220;
   green2 = 0;
   blue2 = 64;
   col2 = col1 + dcolor;
/* index 23 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      blue, col2, red2, green2, blue2);

   red = 220;
   green = 0;
   blue = 64;
   red2 = 180;
   green2 = 0;
   blue2 = 50;
   col1 = col2+dcolor;
/* index 24 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d     \n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 180;
   green = 0;
   blue = 50;
   red2 = 90;
   green2 = 0;
   blue2 = 60;
   col2 = col1;
   col1 = scmax1+scmax1/10;
/* index 25 */
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d     \n", col2,red, green,
      blue, col1, red2, green2, blue2);
   fclose(fp);
}


void Color_Pt_Vel_18col_vivid(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* draws 16 color colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("*** this version plots positive scale only ***\n");
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/16.0;
   red = 215;
   green = 0;
   blue = 0;
   col1=-10000;
   col2 = scmin;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 225;
   green = 60;
   blue =  0;
   red2 = 225;
   green2 = 60;
   blue2 = 0;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   L\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 227;
   green = 84;
   blue = 0;
   red2 = 227;
   green2 = 84;
   blue2 = 0;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 220;
   green = 125;
   blue = 0;
   red2 = 220;
   green2 = 125;
   blue2 = 0;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 235;
   green = 138;
   blue = 0;
   red2 = 235;
   green2 = 138;
   blue2 = 0;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 240;
   green = 161;
   blue = 0;
   red2 = 240;
   green2 = 161;
   blue2 = 0;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 200;
   blue = 0;
   red2 = 255;
   green2 = 200;
   blue2 = 0;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 220;
   blue = 82;
   red2 = 255;
   green2 = 220;
   blue2 = 82;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 240;
   green = 235;
   blue = 102;
   red2 = 240;
   green2 = 235;
   blue2 = 102;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 190;
   green = 245;
   blue = 150;
   red2 = 190;
   green2 = 245;
   blue2 = 150;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
      blue, col1, red, green, blue);
   col2=col1+dcolor;
   red = 160;
   green = 255;
   blue = 195;
   red2 = 160;
   green2 = 255;
   blue2 = 195;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 100;
   green = 230;
   blue = 230;
   red2 = 100;
   green2 = 230;
   blue2 = 230;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 100;
   green = 180;
   blue = 255;
   red2 = 100;
   green2 = 180;
   blue2 = 255;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 140;
   blue = 255;
   red2 = 50;
   green2 = 140;
   blue2 = 255;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 13;
   green = 110;
   blue = 240;
   red2 = 13;
   green2 = 110;
   blue2 = 240;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 0;
   green = 80;
   blue = 225;
   red2 = 0;
   green2 = 80;
   blue2 = 225;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 40;
   blue = 195;
   red2 = 0;
   green2 = 40;
   blue2 = 195;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 0;
   green = 0;
   blue = 130;
   red2 = 0;
   green2 = 0;
   blue2 = 130;
   col1 = 10000;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d	L\n", col2,red, green,
      	blue, col1, red, green, blue);
   fclose(fp);
}


void Color_Pt_Vel_gray(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* draws 8 color colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/10.0;
   red = 255;
   green = 255;
   blue = 255;
   col1=-1000;
   col2 = scmin;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   U\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 250;
   green = 250;
   blue =  250;
   red2 = 250;
   green2 = 250;
   blue2 = 250;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   L\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 240;
   green = 240;
   blue = 240;
   red2 = 240;
   green2 = 240;
   blue2 = 240;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 230;
   green = 230;
   blue = 230;
   red2 = 230;
   green2 = 230;
   blue2 = 230;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 218;
   green = 218;
   blue = 218;
   red2 = 218;
   green2 = 218;
   blue2 = 218;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 205;
   green = 205;
   blue = 205;
   red2 = 205;
   green2 = 205;
   blue2 = 205;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 158;
   green = 158;
   blue = 158;
   red2 = 158;
   green2 = 158;
   blue2 = 158;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 130;
   green = 130;
   blue = 130;
   red2 = 130;
   green2 = 130;
   blue2 = 130;
   col1 = col2;
   col2 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 95;
   green = 95;
   blue = 95;
   red2 = 95;
   green2 = 95;
   blue2 = 95;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
      blue, col1, red, green, blue);
   col2=col1+dcolor;
   red = 70;
   green = 70;
   blue = 70;
   red2 = 70;
   green2 = 70;
   blue2 = 70;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 50;
   blue = 50;
   red2 = 50;
   green2 = 50;
   blue2 = 50;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 40;
   green = 40;
   blue = 40;
   red2 = 40;
   green2 = 40;
   blue2 = 40;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d	L\n", col1,red, green,
      	blue, 1000.0, red, green, blue);
   fclose(fp);
}


void Color_Pt_Vel_gray_short(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* draws 16 color colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/8.0;
   red = 255;
   green = 255;
   blue = 255;
   col1=-1000;
   col2 = scmin;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   U\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 255;
   green = 255;
   blue =  255;
   red2 = 255;
   green2 = 255;
   blue2 = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   L\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 236;
   green = 236;
   blue = 236;
   red2 = 236;
   green2 = 236;
   blue2 = 236;
   col2 = col1;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 224;
   green = 224;
   blue = 224;
   red2 = 224;
   green2 = 224;
   blue2 = 224;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 212;
   green = 212;
   blue = 212;
   red2 = 212;
   green2 = 212;
   blue2 = 212;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 165;
   green = 165;
   blue = 165;
   red2 = 165;
   green2 = 165;
   blue2 = 165;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 110;
   green = 110;
   blue = 110;
   red2 = 110;
   green2 = 110;
   blue2 = 110;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
      blue, col1, red, green, blue);
   col2=col1+dcolor;
   red = 70;
   green = 70;
   blue = 70;
   red2 = 70;
   green2 = 70;
   blue2 = 70;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 50;
   green = 50;
   blue = 50;
   red2 = 50;
   green2 = 50;
   blue2 = 50;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 50;
   green = 50;
   blue = 50;
   red2 = 50;
   green2 = 50;
   blue2 = 50;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d	L\n", col1,red, green,
      	blue, 1000.0, red, green, blue);
   fclose(fp);
}

void Color_Pt_Vel_Smooth(float absmax,float ymin, float second_max)
{
float dcolor, col1, col2, tempcol;
float scmax, scmin;
int  second_min;
int  segment_pos, segment_neg, op1;
int  red, green, blue;
int  red2, green2, blue2;
char str[256];
FILE *fp;

/* draws 16 color colorbar */
   if((fp=fopen("colorbar.cpt", "w")) == NULL) {
      printf("error opening color file!\n");
      exit(1);
   }
   printf("ymin = %f,  ymax = %f\n", ymin, absmax);
   printf("*** this version plots positive scale only ***\n");
   printf("enter color scale (min, max)\n");
   GetLine_gmt(str, 256);
   sscanf(str, "%f %f", &scmin, &scmax);
   dcolor = (scmax-scmin)/16.0;
   red = 220;
   green = 0;
   blue = 0;
   col1=-1000;
   col2 = scmin;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   \n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 255;
   green = 0;
   blue =  0;
   red2 = 252;
   green2 = 32;
   blue2 = 32;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d   L\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   red = 255;
   green =32;
   blue = 32;
   red2 = 255;
   green2 = 64;
   blue2 = 64;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 64;
   blue = 64;
   red2 = 255;
   green2 = 96;
   blue2 = 96;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 96;
   blue = 96;
   red2 = 255;
   green2 = 128;
   blue2 = 128;
   col2 = col1 +dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 128;
   blue = 128;
   red2 = 255;
   green2 = 159;
   blue2 = 159;
   col1 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 255;
   green = 159;
   blue = 159;
   red2 = 255;
   green2 = 180;
   blue2 = 180;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 255;
   green = 180;
   blue = 180;
   red2 = 255;
   green2 = 195;
   blue2 = 195;
   col1 = col2;
   col2 = col2 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
      	blue, col2, red, green, blue);
   red = 255;
   green = 195;
   blue = 195;
   red2 = 255;
   green2 = 218;
   blue2 = 218;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
      blue, col1, red2, green2, blue2);
   col2=col1+dcolor;
   red = 218;
   green = 218;
   blue = 255;
   red2 = 195;
   green2 = 195;
   blue2 = 255;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 195;
   green = 195;
   blue = 255;
   red2 = 180;
   green2 = 180;
   blue2 = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   col2=col1+dcolor;
   red = 180;
   green = 180;
   blue = 255;
   red2 = 159;
   green2 = 159;
   blue2 = 255;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 159;
   green = 159;
   blue = 255;
   red2 = 127;
   green2 = 127;
   blue2 = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   col2=col1+dcolor;
   red = 127;
   green = 127;
   blue = 255;
   red2 = 96;
   green2 = 96;
   blue2 = 255;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 96;
   green = 96;
   blue = 255;
   red2 = 64;
   green2 = 64;
   blue2 = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   col2=col1+dcolor;
   red = 64;
   green = 64;
   blue = 255;
   red2 = 32;
   green2 = 32;
   blue2 = 255;
   col2 = col1 + dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col1,red, green,
   blue, col2, red2, green2, blue2);

   red = 32;
   green = 32;
   blue = 255;
   red2 = 0;
   green2 = 0;
   blue2 = 255;
   col1 = col2+dcolor;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d\n", col2,red, green,
   blue, col1, red2, green2, blue2);

   red = 0;
   green = 0;
   blue = 255;
   red2 = 0;
   green2 = 0;
   blue2 = 220;
   fprintf(fp, "%15.7f%7d%7d%7d%15.7f%7d%7d%7d  L\n", col1,red, green,
      	blue, 1000.0, red, green, blue);
   fclose(fp);
}


/*
 **********************input output functions***************************
 */
int GetLine_gmt(char *s, int lim)
{
int c, i;

    for(i=0; i<lim-1 && (c=getchar())!=EOF && c!='\n'; ++i)
        s[i]=c;
        
    if(c=='\n' && i==0){
       s[i] =c;
       ++i; 
    }
    s[i] = '\0';
    return i;
}

int FGetLine_gmt(FILE *fp, char *s, int lim)
{
char c;
int i;

    for(i=0; i<lim-1 && (fscanf(fp,"%c",&c))!=EOF && c!='\n'; ++i)
        s[i]=c;
    if(c=='\n'){
       s[i] =c;
       ++i;
    }
    s[i] = '\0';
    return i;
}



void xmgr_mktitle(FILE *fd,char *infile1,float xmin,float xmax,float ymin,float ymax,float wxl,float wxh,float wyh,float sc,
int ifl, char *title) {
/*
 *  plot a number of seismograms on a single plot
 */
  int xtkmajor, xtkminor;
  char str[30];
  
	fprintf(fd,"with g%d\n", ifl);
	fprintf(fd,"g%d on\n", ifl);
	fprintf(fd,"g%d  hidden false\n", ifl);
	fprintf(fd,"g%d  type xy\n", ifl);
	fprintf(fd, "yaxis off\n");
 	fprintf(fd, "xaxis tick op bottom\n");
	fprintf(fd, "xaxis tick minor size 0\n");
        fprintf(fd,"s0 linestyle 1\n");
       fprintf(fd,"s0 linewidth 1\n");
       fprintf(fd,"s0 color 1\n");
       fprintf(fd,"view xmin %5.2f\n", xmin);
       fprintf(fd,"view xmax %5.2f\n", xmax);
       fprintf(fd,"view ymin %5.2f\n", ymin);
       fprintf(fd,"view ymax %5.2f\n",  ymax);
       fprintf(fd,"world xmin %10.4e\n", wxl);
       fprintf(fd,"world xmax %10.4e\n", wxh);
       fprintf(fd,"world ymin %10.4e\n", -wyh);
       fprintf(fd,"world ymax %10.4e\n", wyh);
       xtkmajor=((wxh-wxl)/10.0);
       xtkminor=xtkmajor/2;
       fprintf(fd,"xaxis  tick on\n");
       fprintf(fd,"xaxis  tick major %d\n", xtkmajor);
       fprintf(fd,"xaxis  tick minor %d\n", xtkminor);
       fprintf(fd,"xaxis  tick major linewidth %5.2f\n",sc);
       fprintf(fd,"xaxis  tick minor linewidth %5.2f\n", sc);
       fprintf(fd,"xaxis ticklabel on\n");
       fprintf(fd,"xaxis ticklabel char size %5.2f\n",sc);
       fprintf(fd,"xaxis ticklabel font 4\n");
       fprintf(fd,"yaxis tick log off \n");
       fprintf(fd,"yaxis ticklabel on\n");
       fprintf(fd,"yaxis label layout perp\n");
       fprintf(fd,"yaxis label char size 1.000\n");
       fprintf(fd,"yaxis label \"TRANSVERSE\"\n");
       fprintf(fd,"yaxis ticklabel char size %5.2f\n",sc);
       fprintf(fd,"yaxis ticklabel font 4\n");
       fprintf(fd,"yaxis ticklabel linewidth %5.2f\n",sc);
       fprintf(fd,"yaxis tick spec 0\n");
       fprintf(fd,"subtitle \"%s\" \n", title);
       fprintf(fd,"subtitle font 0\n");
       fprintf(fd,"subtitle size 1.000\n");
       fprintf(fd,"subtitle color 1\n");
       fprintf(fd,"subtitle linewidth 1\n");
       fprintf(fd,"read block \"%s\"\n",infile1);
       fprintf(fd,"block xy \"1:2\" \n");
}

void find_min_max_mean(double *xarray, int ndat,float *xmin, float *xmax, float *absmax, float *xmean)
{
   int i, j;
   float xxmax, xxmin, xxmean,absm;
   xxmax=-1.0e19;
   absm=-1.0e19;
   xxmin=1.0e19;
   xxmean=0.0;
   for(i=0; i<ndat; i++){
	xxmean += xarray[i];
	if(xxmax<xarray[i]) xxmax=xarray[i];
	if(xxmin>xarray[i]) xxmin=xarray[i];
	if(absm<fabs(xarray[i])) absm=fabs(xarray[i]);
   }
   *xmin=xxmin;
   *xmax=xxmax;
   *absmax=absm;
   *xmean=xxmean/(float)ndat;
}

void compare3val(float a, float b, float c, float *xmax, float *xmin) 
{
	float xxmax, xxmin;
	if(a>=b){
	   if(b>=c) {
		xxmax=a;
		xxmin=c;
	   }else{
		xxmin=b;
		if(a>=c) xxmax=a;
		else  xxmax=c;
	   }
	}else{
		if(a>=c) {
			xxmax=b;
			xxmin=c;
		}else{
			xxmin=a;
			if(b>=c) xxmax=b;
			else   xxmax=c;
			
		}
	}
	*xmax=xxmax;
	*xmin=xxmin;
}

void write_data_db(FILE *fp, float xmin, float xmax, double *a, int ndata)
{
  int i, j;
  float x, dx;

  dx = (xmax-xmin)/(ndata-1);
  x=xmin; 
  for(i=0; i<ndata; i++){
	fprintf(fp, "%f %e\n", x, a[i]);
	x += dx;
  }
  fclose(fp);	
}


void plot_3comp_xmgr(char *title, float wxmin, float wxmax, float *traceV, float *traceR, float *traceT, int ndata)
{  /* plot a 3-component seismogram */
  int i, j, k, nfl;
  float dy, dspace, ymin, xmin, xmax, ymin0;
  float vmin, vmax, vabsmax, vmean;
  float rmin, rmax, rabsmax, rmean;
  float tmin, tmax, tabsmax, tmean;
  float wymax, ymax, tmp, scale;
  double *tracev, *tracer, *tracet;
  FILE *fp1, *fp2, *fp3, *fp4;
  char systemstring[128], tmpstr[128], title1[128];

  nfl=3;	/* plot 3-component seismogram */
  dy = 0.75/(float)nfl;
  dspace=0.33/(float)nfl;
  ymin=0.15;
  xmin=0.15;
  xmax=0.88;
  ymin0=ymin;
  scale=0.8;
	printf("xmin max %f %f ndata =%d\n", wxmin, wxmax, ndata);
  
  tracev = (double *) malloc((unsigned) ndata * sizeof(double));
  tracer = (double *) malloc((unsigned) ndata * sizeof(double));
  tracet = (double *) malloc((unsigned) ndata * sizeof(double));
	printf("ndata =%d\n", ndata);
  for(i=0; i<ndata; i++) { /* change to double */
	tracev[i]=traceV[i];
	tracer[i]=traceR[i];
	tracet[i]=traceT[i];
  }
  fp1=fopen("Tmpt.dat", "w");
  fp2=fopen("Tmpr.dat", "w");
  fp3=fopen("Tmpv.dat", "w");
  fp4=fopen("batch.xmgr", "w");
  
  find_min_max_mean(tracev, ndata, &vmin, &vmax, &vabsmax, &vmean);
  find_min_max_mean(tracer, ndata, &rmin, &rmax, &rabsmax, &rmean);
  find_min_max_mean(tracet, ndata, &tmin, &tmax, &tabsmax, &tmean);
  compare3val(vabsmax,rabsmax,tabsmax, &wymax, &tmp);
  write_data_db(fp1,wxmin, wxmax, tracet,ndata);
  write_data_db(fp2,wxmin, wxmax, tracer,ndata);
  write_data_db(fp3,wxmin, wxmax, tracev,ndata);
  for(i=0; i<nfl; i++){
	bzero(title1, 128);
	strcpy(title1, title);
	if(i==0) {
		sprintf(tmpstr, "Tmpt.dat");
		strcat(title1, " (Transverse)");
	}
	if(i==1) {
		sprintf(tmpstr, "Tmpr.dat");
		strcat(title1, " (Radial)");
	}
	if(i==2) {
		sprintf(tmpstr, "Tmpv.dat");
		strcat(title1, " (Vertical)");
	}
	ymax=ymin+dy;
  	xmgr_mktitle(fp4,tmpstr,xmin,xmax,ymin,ymax,wxmin,wxmax,wymax,scale,i,title1);
	ymin=ymax;
 }
  fprintf(fp4,"auto redraw on\n");
  fclose(fp4);
  sprintf(systemstring, "xmgr -batch batch.xmgr \n");
  system(systemstring);
  free(tracev);
  free(tracer);
  free(tracet);

}

void readGmtFigDef(char *name, float *figdef)
{
/*  Input: figure default file name and figure default parameter
	   array.
    Output:  figure default array
    Explanation of array figdef
line1:  orientation (0=horizontal; 1=vertical)   num_of_panels
        panel_size_horizontal     panel_size_vertical (in inches)
        horizontal_spacing_between_panels  vertical_spacing_BP
        line_thickness 
        norm_factor 1(in ratio with absolute_max)
        norm_factor 2(in ratio with absolute_max)
	...
        norm_factor npanel (in ratio with absolute_max)

line2-end:  each line contains the following segments.
float 1:  flag if a figure has been plotted, if value < 0, then it has 
	  not been previously plotted
float 2:  normalization for this existing figure
float 3:  total number of traces in this data window
*/
   FILE *fp;
   int  i;

   if(!(fp = fopen(name, "r"))) {
	printf("Error opening figure option file!\n");
	exit(-1);
   }
   printf("reading figure parameter file %s\n", name);
   fscanf(fp, "%f %f", &figdef[0], &figdef[1]);
   fscanf(fp, "%f %f", &figdef[2], &figdef[3]);
   fscanf(fp, "%f %f", &figdef[4], &figdef[5]);
   fscanf(fp, "%f", &figdef[6]);
   for(i=0; i<figdef[1]; i++){
	fscanf(fp, "%f",  &figdef[7+i]);
   }
   fclose(fp);
}

void setGmtFigDef(float *figdef, float norm, int ntrace)
{
/*  Input: a row of figure default parameter
	   array.
	   norm:  normalization factor
	   ntrace : number of traces. 
	   
    Output:  figure default array figdef[irow]
    Explanation of array figdef
line1:  orientation (0=horizontal; 1=vertical)   num_of_figures
        panel_size_horizontal     panel_size_vertical (in inches)
        horizontal_spacing_between_panels  vertical_spacing_BP
        line_thickness   norm_factor(in percent_of_absolute_max)
line2-end:  each line contains the following segments.
float 1:  flag if a figure has been plotted, if value < 0, then it has 
	  not been previously plotted
float 2:  normalization for this existing figure
float 3:  total number of traces in this data window
*/
	if(norm<=0.0) {
		printf("Unreasonable normalization factor!!!\n");
		exit(-1);
	}
	if(figdef[1]<norm) figdef[1]=norm;
	if(figdef[2l]<0.0) figdef[2]=(float)ntrace+0.45;
        if(figdef[1]>0.0 && figdef[2]>0.0 && figdef[0]<0.0) figdef[0]=1.0;
}
