#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#define MAXSTR 256

void Add_string(string1, string2, string3)
char string1[], string2[], string3[];
{
 char tempstring[MAXSTR];
   strcpy(tempstring, string1);
   strcat(tempstring, string2);
   sprintf(string3, tempstring);
}

void printArray(dataseries, inminx, inmaxx)
 float dataseries[];
  int  inminx, inmaxx;
{
  int i, j;

  for(i=inminx; i<=inmaxx; i++){
	printf("%d  %e\n", i, dataseries[i]);
  }
}

void Find_norm(imin, imax, xxarray, norm, opt)
int imin, imax;
int opt;	/* 1=find maximum;  2=find minimum */
float xxarray[], *norm;
{
 /* find maximum in a given part of an array */
 int i;
 float tempmax, tempmin;
 tempmax = -9999999999999.0;
 tempmin = 9999999999999.0;
 if(opt==1){
	for(i=imin; i<imax; i++){
		if(xxarray[i]>tempmax) tempmax = xxarray[i];
	}
	*norm = tempmax;
 }else if(opt==2){
	for(i=imin; i<imax; i++){
		if(xxarray[i]<tempmin) tempmin = xxarray[i];
	}
	*norm = tempmin;
 }else{
	printf("wrong choice!!!\n");
	exit(-1);
 }
 return;
}

#define MBIG 1000000000
#define MSEED 161803398
#define MZ 0
#define FAC (1.0/MBIG)

float ran3(idum)
int *idum;
{
	static int inext,inextp;
	static long ma[56];
	static int iff=0;
	long mj,mk;
	int i,ii,k;

	if (*idum < 0 || iff == 0) {
		iff=1;
		mj=MSEED-(*idum < 0 ? -*idum : *idum);
		mj %= MBIG;
		ma[55]=mj;
		mk=1;
		for (i=1;i<=54;i++) {
			ii=(21*i) % 55;
			ma[ii]=mk;
			mk=mj-mk;
			if (mk < MZ) mk += MBIG;
			mj=ma[ii];
		}
		for (k=1;k<=4;k++)
			for (i=1;i<=55;i++) {
				ma[i] -= ma[1+(i+30) % 55];
				if (ma[i] < MZ) ma[i] += MBIG;
			}
		inext=0;
		inextp=31;
		*idum=1;
	}
	if (++inext == 56) inext=1;
	if (++inextp == 56) inextp=1;
	mj=ma[inext]-ma[inextp];
	if (mj < MZ) mj += MBIG;
	ma[inext]=mj;
	return mj*FAC;
}


#undef MBIG
#undef MSEED
#undef MZ
#undef FAC

void Randomize(seed, ntotal, rand)
int *seed;
int ntotal;
int *rand;
{
   float ran3();
   float r1, dr, r;
   int count, tempseed;
   
   tempseed=*seed;
   r1 = ran3(&tempseed);
   r = 0.0;
   dr = (1.0)/(float)ntotal;
   count = 0;
   while(r<1.0) {
      if(r1 >= r && r1 < r+dr) {
	 *rand = count;
	 break;
      }
      r = r+dr;
      count++;
   }
   *seed=tempseed;
}

/* the following is a subroutine which finds the 
 * statistical parameters such as standard deviation
 * from the data.  It is used for bootstraping result
 */
void moment(data,n,ave,adev,sdev,svar,skew,curt)
int n;
float data[],*ave,*adev,*sdev,*svar,*skew,*curt;
{
	int j;
	float s,p;

	if (n <= 1) printf("n must be at least 2 in MOMENT");
	s=0.0;
	for (j=1;j<=n;j++) s += data[j];
	*ave=s/n;
	*adev=(*svar)=(*skew)=(*curt)=0.0;
	for (j=1;j<=n;j++) {
		*adev += fabs(s=data[j]-(*ave));
		*svar += (p=s*s);
		*skew += (p *= s);
		*curt += (p *= s);
	}
	*adev /= n;
	*svar /= (n-1);
	*sdev=sqrt(*svar);
	if (*svar) {
		*skew /= (n*(*svar)*(*sdev));
		*curt=(*curt)/(n*(*svar)*(*svar))-3.0;
	} else printf("No skew/kurtosis when variance = 0 (in MOMENT)");
}

int gettime_in_sec(){
 int i, j;
 struct timeval test0, test1;

	gettimeofday(&test0, NULL);
	j = 0;
	return test0.tv_sec;
}

void print_moment(n, ave, std, skew)
int n;
float ave, std, skew;
{
	printf("\nArray statistics:\n");
	printf("number of elements = %d\n", n);
	printf("average = %f\n", ave);
	printf("standard deviation = %f\n", std);
	printf("skew = %f\n", skew);
}

void correlate(x1,x2,n,rcoef)
/* finds correlation between array x1 and x2, save into output rcoef */
double *x1;
double *x2;
int  n;  /* number of data */
double  *rcoef;
{
	int   i;
	double x1mean, x2mean, x1res, x2res;
	double temp1, temp2, temp;
	double res1sq, res2sq;
/* computes averages of x1, x2 */
	printf("number of data = %d\n", n);
	temp1=temp2=0.0;
	for(i=0;i<n;i++){
		temp1 += x1[i];		
		temp2 += x2[i];		
	}
	x1mean=temp1/(double)n;
	x2mean=temp2/(double)n;
	temp=0.0;
	temp1=0.0;
	temp2=0.0;
	for(i=0;i<n;i++){
		x1res=x1[i]-x1mean;
		res1sq=x1res*x1res;
		x2res=x2[i]-x2mean;
		res2sq=x2res*x2res;
		temp += x1res*x2res;
		temp1 += res1sq;
		temp2 += res2sq;
	}
	*rcoef = temp/sqrt(temp1*temp2);
}
