#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <unistd.h>
#include <limits.h>
#include <assert.h>
#include "ttimelib.h"

// this is the bulk of the travel time program which has the following
// functions:
// 	SumArow()
// 	SumArow_topo()
//	SaveModelPara()
//	SaveArow
//	SaveArow_tttopo()
//	Contribution
//	Contribution_split
//	RadContrib
//	GetReg
//	EqSt::CalEqStDist
//	EqSt::ReadEq
//	EqSt::ReadStRes
//	EqSt::CalTrans4
//	TransMatrix::SetFromVec
//	Cartesian::Normalize
//	Cartesian::OutProdWith
//	Cartesian::ConvFromSph2
//	Spherical2::ForwardTrans
//	Spherical2::InverseTrans
//	EqSt::ForwardTrans4
//	EqSt::InverseTrans4
//	EqSt::InvTransRPhi
//	EqSt::Init
//	BSModel::ReadModel
//	BSModel::InitKnots
//	BSModel::CalAvgDist
//	BSModel::DistBt2
//	LinearInt
//	BsplineFun
//	SysOpen
//	SUMRES::OpenResFile
//	SUMRES::TotalNumRes
//	SUMRES::ReadSumRes
//	SUMRES::SeektoRec
//	CAPRES::Init
//	SUMRES::Init
//      GetCapinfo
//	CAPRES::ReadCapRes
//	FindSurfBspline
//	CalcDelta
//	CalcBspline
//	CalcDist
//	SurfTransKer
//	PHASE::TransBoundContri
//	PHASE::CapTransBoundContri
//	LocateCap
//	CAPRES::SurfKer
//	CapContribute
//	CapContribute_split
//	ConvEq
//	ConvSt
//	CAPRES::CalTrans4
//	CapSaveArow

real  SumArow(BSModel &bsmodel,float *arow,short *irow,int &ind)
{
int i,j;
real res=0.0;


      for(i=0;i<ind;i++){
          j=irow[i];
          res+=arow[i]*bsmodel.coeff[j];
      }
      return res;
}

real  SumArow_topo(BSModel &bsmodel,float *arow,short *irow,int &ind,
int &n3d)
{
int i,j;
real res=0.0;

      for(i=0;i<ind;i++){
          j=irow[i];
	  if(j<=n3d) res+=arow[i]*bsmodel.coeff[j];  //not considering topography portion
      }
      return res;
}

void SaveModelPara(int &fd, int &htps, int &maxk, int &opt)
{
    write(fd, &htps,sizeof(int));
    write(fd, &maxk,sizeof(int));
    write(fd, &opt, sizeof(int));
}

void SaveArow(int fd,class EqSt &eqst,float *arow,short *irow,int ind)
{
float buf[9];
int ibuf[3];

    buf[0]=(float)eqst.geq.lat;
    buf[1]=(float)eqst.geq.lon;
    buf[2]=(float)eqst.eqdep;
    buf[3]=(float)eqst.gst.lat;
    buf[4]=(float)eqst.gst.lon;
    buf[5]=(float)0.0;			// station elevation, set to 0.0 for sumres
    buf[6]=(float)eqst.res1;		// observed residual
    buf[7]=(float)eqst.res2;		// predicted residual
    buf[8]=(float)eqst.time;		// PREM predicted travel time
    ibuf[0]=0;				// earthquake index, not used, set to 0
    ibuf[1]=0;				// station index, not used, set to 0
    ibuf[2]=ind;				// number of nozero elements in the matrix
    write(fd,buf,sizeof(buf));
    write(fd,ibuf,sizeof(ibuf));
    if(ind>0){
       write(fd,irow,sizeof(short)*ind);
       write(fd,arow,sizeof(float)*ind);
    }
}

void SaveArow_tttopo(int fd,class EqSt &eqst,float *arow,short *irow,int ind, int nbsp_r)
{
float buf[9];
int ibuf[3];

    buf[0]=(float)eqst.geq.lat;
    buf[1]=(float)eqst.geq.lon;
    buf[2]=(float)eqst.eqdep;
    buf[3]=(float)eqst.gst.lat;
    buf[4]=(float)eqst.gst.lon;
    buf[5]=(float)0.0;			// station elevation, set to 0.0 for sumres
    buf[6]=(float)eqst.res1;		// observed residual
    buf[7]=(float)eqst.res2;		// predicted residual
    buf[8]=(float)eqst.time;		// PREM predicted travel time
    ibuf[0]=0;				// earthquake index, not used, set to 0
    ibuf[1]=0;				// station index, not used, set to 0
    ibuf[2]=ind;				// number of nozero elements in the matrix
    write(fd,buf,sizeof(buf));
    write(fd,ibuf,sizeof(ibuf));
    if(ind>0){
       write(fd,irow,sizeof(short)*ind);
       write(fd,arow,sizeof(float)*ind);
    }
}

void Contribution(class PHASE &ray, class BSModel &bsmodel, class EqSt &eqst, float *arow)
{
int gpt=bsmodel.gpt,mode,krad[2];	// horizontal grid points
int i,j,k,ii0,ii1,npt,ireg,numx,ind;
int numpt=ray.numpt;
real h,twoh,twohsq,aa,bb,xmin,xmax,del,*arowk;
real theta[2],rr[2], vv[2];		// for two end points
Spherical2 s2,s22;
TransMatrix trans;
real *rknt;
int  maxk;

   maxk=bsmodel.GetMaxK();
   rknt=bsmodel.rknt;
   numx=(maxk+1)*gpt;	// number of columns in maxtrix A
   for(k=0;k<numx;k++)arow[k]=0.0;
    
   arowk = new real[maxk+1];		  // Contribution to "A" maxtrix from a specific cell

   del=eqst.GetDist();
   trans=eqst.GetTrans4();
   h=bsmodel.GetAvgDist();
   twoh=2.0*h;
   twohsq=twoh*twoh;
   
   for(i=0;i<gpt;i++){			  // loop thru bells, break ray path
       s2.theta=bsmodel.theta[i];
       s2.phi = bsmodel.phi[i];
       s22=s2.ForwardTrans(trans.matrix); // rotate bell to path coordinates s22
       if(fabs(s22.theta-HPI)<=twoh){
          aa=HPI-s22.theta;		
          bb=sqrt(twohsq-aa*aa);	  // distance away from center of bell
          xmin=s22.phi-bb;		  // boundary of a bell
          xmax=s22.phi+bb;
          if(xmax>TWOPI){		  // make sure it is between 0 -> 2*PI
             xmax-=TWOPI;
             xmin-=TWOPI;
          }
          mode=IN;
          if(between(0.0,xmin,del) && between(0.0,xmax,del)){   // path passes  bell
            ii1=ii0=-1;
          }else if(between(0.0,xmin,del)&& xmax>del){		// path ends in  bell 
            xmax=del;
            ii0=-1;
            ii1=numpt-1;
          }else if(between(0.0,xmax,del)&& xmin<0.0){		// path begins in bell
            xmin=0.0;
            ii0=0;
            ii1=-1;
          }else if(between(xmin,0.0,xmax) &&  between(xmin,del,xmax)){ // path all in bell
            xmin=0.0;
            xmax=del;
            ii0=0;
            ii1=numpt-1;
          }else{						// path is out of bell
            mode=OUT;
          }
          if(mode==IN){
            krad[0]=krad[1]=0;					
            if(ii0==-1){					// means xmin is inside path
               for(j=0;j<numpt;j++){
                   if(ray.phi[j]>=xmin)break;			// find which point along raypath coincides xmin
               }
               ii0=j;						// save index of that point, lower limit
            }
            if(ii1==-1){
               for(j=numpt-1;j>=0;j--){
                   if(ray.phi[j]<=xmax)break;
               }
               ii1=j;						// save index of point coincides xmax, upper limit
            }
            if(ii0!=0){
               	rr[0]=LinearInt(xmin,ray.phi[ii0-1],ray.phi[ii0],ray.rad[ii0-1],ray.rad[ii0]);
              	vv[0]=LinearInt(rr[0],ray.rad[ii0-1],ray.rad[ii0],ray.vel[ii0-1],ray.vel[ii0]);
               	theta[0]=xmin;
              	krad[0]=GetReg(rr[0],rknt,maxk);
            }
            if(ii1!=numpt-1){
               	rr[1]=LinearInt(xmax,ray.phi[ii1],ray.phi[ii1+1],ray.rad[ii1],ray.rad[ii1+1]);
                vv[1]=LinearInt(rr[1],ray.rad[ii1],ray.rad[ii1+1],ray.vel[ii1],ray.vel[ii1+1]);
                theta[1]=xmax;
                ireg=krad[1]=GetReg(rr[1],rknt,maxk);
            }
	    RadContrib(ray,theta,rr,vv,krad,ii0,ii1,s22,h,rknt,maxk,arowk);
            for(ind=k=0;k<=maxk;k++,ind+=gpt)arow[ind+i]=arowk[k];
          }
       }
   }
   delete [] arowk;
}

void Contribution_split(class PHASE &ray, class BSModel &bsmodel, class EqSt &eqst, float *arow)
{
int gpt=bsmodel.gpt,mode;		// horizontal grid points
int i,j,k,ii0,ii1,npt,ireg,numx,ind;
int krad_lm[2], krad_um[2];
int numpt=ray.numpt;
real h,twoh,twohsq,aa,bb,xmin,xmax,del;
real *arowk_um, *arowk_lm;
real theta[2],rr[2], vv[2];		// for two end points
real *rknt_lm, *rknt_um;
Spherical2 s2,s22;
TransMatrix trans;
int  maxk, maxk_lm, maxk_um, umindex;
int  uplowop;

   maxk=bsmodel.GetMaxK();
   maxk_lm = bsmodel.GetMaxK_lm();
   maxk_um = bsmodel.GetMaxK_um();
   umindex = (maxk_lm+1)*gpt;
   arowk_lm = new real[maxk_lm+1];		  // Contribution to "A" maxtrix from a specific cell
   arowk_um = new real[maxk_um+1];		  // Contribution to "A" maxtrix from a specific cell
   rknt_lm=bsmodel.rknt_lm;
   rknt_um=bsmodel.rknt_um;
   numx=(maxk+1)*gpt;	// number of columns in maxtrix A
   for(k=0;k<numx;k++)arow[k]=0.0;

   del=eqst.GetDist();
   trans=eqst.GetTrans4();
   h=bsmodel.GetAvgDist();
   twoh=2.0*h;
   twohsq=twoh*twoh;
   
   for(i=0;i<gpt;i++){			  // loop thru bells, break ray path
       s2.theta=bsmodel.theta[i];
       s2.phi = bsmodel.phi[i];
       s22=s2.ForwardTrans(trans.matrix); // rotate bell to path coordinates s22
//       cout << i << " unroted theta=" << s2.theta << "   phi =" << s2.phi << endl;
//       cout << i << " rotated theta=" << s22.theta << "  phi =" << s22.phi << endl;
       if(fabs(s22.theta-HPI)<=twoh){
          aa=HPI-s22.theta;		
          bb=sqrt(twohsq-aa*aa);	  // distance away from center of bell
          xmin=s22.phi-bb;		  // boundary of a bell
          xmax=s22.phi+bb;
          if(xmax>TWOPI){		  // make sure it is between 0 -> 2*PI
             xmax-=TWOPI;
             xmin-=TWOPI;
          }
          mode=IN;
          if(between(0.0,xmin,del) && between(0.0,xmax,del)){   // path passes  bell
            ii1=ii0=-1;
          }else if(between(0.0,xmin,del)&& xmax>del){		// path ends in  bell 
            xmax=del;
            ii0=-1;
            ii1=numpt-1;
          }else if(between(0.0,xmax,del)&& xmin<0.0){		// path begins in bell
            xmin=0.0;
            ii0=0;
            ii1=-1;
          }else if(between(xmin,0.0,xmax) &&  between(xmin,del,xmax)){ // path all in bell
            xmin=0.0;
            xmax=del;
            ii0=0;
            ii1=numpt-1;
          }else{						// path is out of bell
            mode=OUT;
          }
          if(mode==IN){
            krad_lm[0]=krad_lm[1]=0;					
            krad_um[0]=krad_um[1]=0;					
            if(ii0==-1){					// means xmin is inside path
               for(j=0;j<numpt;j++){
                   if(ray.phi[j]>=xmin)break;			// find which point along raypath coincides xmin
               }
               ii0=j;						// save index of that point, lower limit
            }
            if(ii1==-1){
               for(j=numpt-1;j>=0;j--){
                   if(ray.phi[j]<=xmax)break;
               }
               ii1=j;						// save index of point coincides xmax, upper limit
            }
            if(ii0!=0){
               	rr[0]=LinearInt(xmin,ray.phi[ii0-1],ray.phi[ii0],ray.rad[ii0-1],ray.rad[ii0]);
              	vv[0]=LinearInt(rr[0],ray.rad[ii0-1],ray.rad[ii0],ray.vel[ii0-1],ray.vel[ii0]);
               	theta[0]=xmin;
                krad_lm[0]=GetReg(rr[0],rknt_lm,maxk_lm);
                krad_um[0]=GetReg(rr[0],rknt_um,maxk_um);
            }
            if(ii1!=numpt-1){
               	rr[1]=LinearInt(xmax,ray.phi[ii1],ray.phi[ii1+1],ray.rad[ii1],ray.rad[ii1+1]);
                vv[1]=LinearInt(rr[1],ray.rad[ii1],ray.rad[ii1+1],ray.vel[ii1],ray.vel[ii1+1]);
                theta[1]=xmax;
                krad_lm[1]=GetReg(rr[1],rknt_lm,maxk_lm);
                krad_um[1]=GetReg(rr[1],rknt_um,maxk_um);
            }
//		cout << "depth0=" << 6371.-rr[0] <<" depth1=" << 6371.-rr[1]<< " krad_lm[0]=" << krad_lm[0] << " krad_lm[1]=" << krad_lm[1]<<endl;
	    uplowop = 0;  					// lower mantle
            RadContrib_split(ray,theta,rr,vv,krad_lm,ii0,ii1,s22,h,rknt_lm,maxk_lm,arowk_lm, uplowop);
	    uplowop = 1;
	    RadContrib_split(ray,theta,rr,vv,krad_um,ii0,ii1,s22,h,rknt_um,maxk_um,arowk_um, uplowop);
            for(ind=k=0;k<=maxk_lm;k++,ind+=gpt)arow[ind+i]=arowk_lm[k];
            for(ind=umindex,k=0;k<=maxk_um;k++,ind+=gpt)arow[ind+i]=arowk_um[k];
          }
       }
   }
   delete [] arowk_lm;
   delete [] arowk_um;
}

int GetReg(real r,real *rknt,int maxk)
{
int i,ib=1,ie=maxk+1, k;

  if(r<rknt[0] || r>rknt[maxk])return 0;

  while(ie-ib>1){
	i=ib+((ie-ib)>>1);
//		cout << "r=" << r << "  ie =" << ie <<  "ib=" << ib << " i" << i <<endl;
	if(r<rknt[i-1])ie=i;
	else  ib=i;
  }
//	cout << "r=" << r << "  maxk=" <<maxk << " reg=" << ib << endl;
  return ib;
}


void RadContrib(class PHASE &ray,real *phi,real *rad, real *vv,int *krad,int ii0,int ii1,
Spherical2 &s22, real &h, real *rknt, int maxk, real *arowk)
{
int i,i0,i0m2,i1,j,k,n,reg,mode,ibeg=0,iend;
real rb,phib,vvb,dl,dr,dphi,ff0,ff1,dist,B0[4],B1[4],x,*func,rmin,rmax;
  
  for(k=0;k<=maxk;k++)arowk[k]=0.0;

  func = new real[ii1+1];
  
  for(i=ii0;i<=ii1;i++){				// horizontal functions
      dist=DIST(s22.theta,s22.phi,ray.phi[i]);
//      cout << s22.theta << "  " << s22.phi << endl;
      func[i]=-fdelta(dist,h);
  }
  if((i0=krad[0])!=0){					// there is an extra point at begining
     i0m2=i0-2;
     dist = DIST(s22.theta,s22.phi,phi[0]);		// distance to the center of bell
     ff0=-0.5*fdelta(dist,h)/vv[0];			// horizontal function value
     BsplineFun(krad[0],rad[0],rknt,maxk,B0);		// radial function values at rad[0]

     if(i0!=ray.krad[ii0]){				// this point and  next one is not in the same region
        if(rad[0]>=ray.rad[ii0]){			// find the radius at the knot
           rb=rknt[i0-1];
        }else{
           rb=rknt[i0];
        }
        phib=LinearInt(rb,ray.rad[ii0],rad[0],ray.phi[ii0],phi[0]);	// interpolate phi
        vvb =LinearInt(rb,ray.rad[ii0],rad[0],ray.vel[ii0],vv[0]);	// interpolate velocity
        dist = DIST(s22.theta,s22.phi,phib);
        ff1=-0.5*fdelta(dist,h)/vvb;
        BsplineFun(krad[0],rb,rknt,maxk,B1);
        
        dr=rad[0]-rb;
        dphi=(phi[0]-phib)*rad[0];
        dl=sqrt(dr*dr+dphi*dphi);
        for(j=0;j<4;j++){
            k=i0m2+j;
            if(k>=0 && k<=maxk)arowk[k]+=(ff0*B0[j]+ff1*B1[j])*dl;
        }
        for(j=0;j<4;j++)B0[j]=B1[j];
        ff0=ff1;
        i0=ray.krad[ii0];
        i0m2=i0-2;
     }else{
        rb=rad[0];
        phib=phi[0];
     }
     
     dr=ray.rad[ii0]-rb;				// integrate from rb to ray.rad[ii0]
     dphi=(ray.phi[ii0]-phib)*rb;
     dl=sqrt(dr*dr+dphi*dphi);
          
     ff1=func[ii0]*dl;
     ff0*=dl;
     for(j=0;j<4;j++){
         k=i0m2+j;
         if(k>=0 && k<=maxk)arowk[k]+=ff0*B0[j]+ff1*ray.rfac[j][ii0];
     }
  }
  for(i=ii0;i<ii1;i++){
      if((i0=ray.krad[i])==ray.krad[i+1]){	// two points in  same region
         i0m2=i0-2;
         if(ray.krad[i]!=0){			// in range, integrate from ray.krad[i] to ray.krad[i+1]
            ff0=ray.rlen[i]*func[i];
            ff1=ray.rlen[i]*func[i+1];
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
                   arowk[k] +=ff0*ray.rfac[j][i]+ff1*ray.rfac[j][i+1];
                }
            }
         }
      }else{					// two points are in different region
         rb=-1.0;
         if((i0=ray.krad[i+1])!=0){		// 2nd point in range, integrate from rb to ray.rad[i+1]
            i0m2=i0-2;
            if(ray.rad[i+1]>=ray.rad[i]){
               rb=rknt[ray.krad[i+1]-1];
            }else{
               rb=rknt[ray.krad[i+1]];
            }
            phib=LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.phi[i],ray.phi[i+1]);
            vvb =LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.vel[i],ray.vel[i+1]);
            dr=ray.rad[i+1]-rb;				// integrate from rb to ray.rad[i+1]
            dphi=(ray.phi[i+1]-phib)*rb;
            dl=sqrt(dr*dr+dphi*dphi);
            
            BsplineFun(ray.krad[i+1],rb,rknt,maxk,B0);
            dist = DIST(s22.theta,s22.phi,phib);	// factor for point at rb
            ff0=-0.5*dl*fdelta(dist,h)/vvb;
            ff1= dl*func[i+1];
            
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk)arowk[k]+=ff0*B0[j]+ff1*ray.rfac[j][i+1];
            }
         }
         if((i0=ray.krad[i])!=0){			// integrate from ray.rad[i] to rb
            i0m2=i0-2;
            if(ray.rad[i+1]>=ray.rad[i])rb=rknt[ray.krad[i]];
            else rb=rknt[ray.krad[i]-1];
            BsplineFun(ray.krad[i],rb,rknt,maxk,B0);
            phib=LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.phi[i],ray.phi[i+1]);
            vvb =LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.vel[i],ray.vel[i+1]);
               
            dist = DIST(s22.theta,s22.phi,phib);
            ff0=-0.5*ray.rlen[i]*fdelta(dist,h)/vvb;
            ff1= ray.rlen[i]*func[i];
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk)arowk[k]+=ff0*B0[j]+ff1*ray.rfac[j][i];
            }
         }
      }
  }
  if((i0=krad[1])!=0){				// integrate from rad[1] to ray.rad[ii1]
     i0m2=i0-2;
     dist = DIST(s22.theta,s22.phi,phi[1]);	// distance to the center of bell
     ff0=-0.5*fdelta(dist,h)/vv[1];		// horizontal function value
     BsplineFun(krad[1],rad[1],rknt,maxk,B0);	// radial function values

     if(i0!=ray.krad[ii1]){			// not in same region, integrate from rad[1] to rb
        if(rad[1]>=ray.rad[ii1]){		// find the radius at the knot
           rb=rknt[i0-1];
        }else{
           rb=rknt[i0];
        }
        phib=LinearInt(rb,ray.rad[ii1],rad[1],ray.phi[ii1],phi[1]);
        vvb =LinearInt(rb,ray.rad[ii1],rad[1],ray.vel[ii1],vv[1]);
        dr=rb-rad[1];
        dphi=(phi[1]-phib)*rad[1];
        dl=sqrt(dr*dr+dphi*dphi);
            
        BsplineFun(krad[1],rb,rknt,maxk,B1);
        dist = DIST(s22.theta,s22.phi,phib);	// factor for point at rb
        ff1=-0.5*fdelta(dist,h)/vvb;
            
        for(j=0;j<4;j++){
            k=i0m2+j;
            if(k>=0 && k<=maxk)arowk[k]+=(ff0*B0[j]+ff1*B1[j])*dl;
        }
        for(j=0;j<4;j++)B0[j]=B1[j];
        ff0=ff1;
        i0=ray.krad[ii1];
        i0m2=i0-2;
     }else{					// set rb to rad[1]
        rb=rad[1];
        phib=phi[1];
     }
     
     dr=ray.rad[ii1]-rb;			// integrate from rb to ray.rad[ii1]
     dphi=(ray.phi[ii1]-phib)*rb;
     dl=sqrt(dr*dr+dphi*dphi);
     ff0*=dl;
     ff1=func[ii1]*dl;
     
     for(j=0;j<4;j++){
         k=i0m2+j;
         if(k>=0 && k<=maxk)arowk[k]+=ff0*B0[j]+ff1*ray.rfac[j][ii1];
     }

  }
  delete  [] func;

}

void RadContrib_split(class PHASE &ray,real *phi,real *rad, real *vv,int *krad,int ii0,int ii1,
Spherical2 &s22, real &h, real *rknt, int maxk, real *arowk, int uplowopt)
{ // integration along ray path.  The output is saved in arowk which is the A matrix
  // this version is general for continuous, split and multiply split mantles.
  // 
int i,i0,i0m2,i1,j,k,n,reg,mode,ibeg=0,iend;
real rb,phib,vvb,dl,dr,dphi,ff0,ff1,dist,B0[4],B1[4],x,*func,rmin,rmax;
int *raykrad;
int numpt=ray.numpt;
real *rfac[4];
  
  for(k=0;k<=maxk;k++)arowk[k]=0.0;
  for(i=0;i<4;i++){
  	rfac[i] = new real [numpt];
  }
  if(uplowopt==1){			// upper mantle in 1 split at 670
     raykrad=ray.krad_um;
     for(i=0;i<numpt;i++){
         rfac[0][i]=ray.rfac_um[0][i];
         rfac[1][i]=ray.rfac_um[1][i];
         rfac[2][i]=ray.rfac_um[2][i];
         rfac[3][i]=ray.rfac_um[3][i];
     }
  }else if(uplowopt==0){		// lower mantle in 1 split at 670
     raykrad=ray.krad_lm;
     for(i=0;i<numpt;i++){
         rfac[0][i]=ray.rfac_lm[0][i];
         rfac[1][i]=ray.rfac_lm[1][i];
         rfac[2][i]=ray.rfac_lm[2][i];
         rfac[3][i]=ray.rfac_lm[3][i];
     }
  }
  func = new real[ii1+1];
  
  for(i=ii0;i<=ii1;i++){				// horizontal functions
      dist=DIST(s22.theta,s22.phi,ray.phi[i]);
//      cout << s22.theta << "  " << s22.phi << endl;
      func[i]=-fdelta(dist,h);
  }
  if((i0=krad[0])!=0){					// there is an extra point at begining
     i0m2=i0-2;
     dist = DIST(s22.theta,s22.phi,phi[0]);		// distance to the center of bell
     ff0=-0.5*fdelta(dist,h)/vv[0];			// horizontal function value
     BsplineFun(krad[0],rad[0],rknt,maxk,B0);		// radial function values at rad[0]

     if(i0!=raykrad[ii0]){				// this point and  next one is not in the same region
        if(rad[0]>=ray.rad[ii0]){			// find the radius at the knot
           rb=rknt[i0-1];
        }else{
           rb=rknt[i0];
        }
        phib=LinearInt(rb,ray.rad[ii0],rad[0],ray.phi[ii0],phi[0]);	// interpolate phi
        vvb =LinearInt(rb,ray.rad[ii0],rad[0],ray.vel[ii0],vv[0]);	// interpolate velocity
        dist = DIST(s22.theta,s22.phi,phib);
        ff1=-0.5*fdelta(dist,h)/vvb;
        BsplineFun(krad[0],rb,rknt,maxk,B1);
        
        dr=rad[0]-rb;
        dphi=(phi[0]-phib)*rad[0];
        dl=sqrt(dr*dr+dphi*dphi);
        for(j=0;j<4;j++){
            k=i0m2+j;
            if(k>=0 && k<=maxk)arowk[k]+=(ff0*B0[j]+ff1*B1[j])*dl;
        }
        for(j=0;j<4;j++)B0[j]=B1[j];
        ff0=ff1;
        i0=raykrad[ii0];
        i0m2=i0-2;
     }else{
        rb=rad[0];
        phib=phi[0];
     }
     
     dr=ray.rad[ii0]-rb;				// integrate from rb to ray.rad[ii0]
     dphi=(ray.phi[ii0]-phib)*rb;
     dl=sqrt(dr*dr+dphi*dphi);
          
     ff1=func[ii0]*dl;
     ff0*=dl;
     for(j=0;j<4;j++){
         k=i0m2+j;
         if(k>=0 && k<=maxk) {
		arowk[k]+=ff0*B0[j]+ff1*rfac[j][ii0]; 
	 }
     }
  }
  for(i=ii0;i<ii1;i++){
      if((i0=raykrad[i])==raykrad[i+1]){	// two points in  same region
         i0m2=i0-2;
         if(raykrad[i]!=0){			// in range, integrate from raykrad[i] to raykrad[i+1]
            ff0=ray.rlen[i]*func[i];
            ff1=ray.rlen[i]*func[i+1];
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
			arowk[k] +=ff0*rfac[j][i]+ff1*rfac[j][i+1];
                }
            }
         }
      }else{					// two points are in different region
         rb=-1.0;
         if((i0=raykrad[i+1])!=0){		// 2nd point in range, integrate from rb to ray.rad[i+1]
            i0m2=i0-2;
            if(ray.rad[i+1]>=ray.rad[i]){
               rb=rknt[raykrad[i+1]-1];
            }else{
               rb=rknt[raykrad[i+1]];
            }
            phib=LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.phi[i],ray.phi[i+1]);
            vvb =LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.vel[i],ray.vel[i+1]);
            dr=ray.rad[i+1]-rb;				// integrate from rb to ray.rad[i+1]
            dphi=(ray.phi[i+1]-phib)*rb;
            dl=sqrt(dr*dr+dphi*dphi);
            
            BsplineFun(raykrad[i+1],rb,rknt,maxk,B0);
            dist = DIST(s22.theta,s22.phi,phib);	// factor for point at rb
            ff0=-0.5*dl*fdelta(dist,h)/vvb;
            ff1= dl*func[i+1];
            
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
			arowk[k]+=ff0*B0[j]+ff1*rfac[j][i+1];
		}
            }
         }
         if((i0=raykrad[i])!=0){			// integrate from ray.rad[i] to rb
            i0m2=i0-2;
            if(ray.rad[i+1]>=ray.rad[i])rb=rknt[raykrad[i]];
            else rb=rknt[raykrad[i]-1];
            BsplineFun(raykrad[i],rb,rknt,maxk,B0);
            phib=LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.phi[i],ray.phi[i+1]);
            vvb =LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.vel[i],ray.vel[i+1]);
               
            dist = DIST(s22.theta,s22.phi,phib);
            ff0=-0.5*ray.rlen[i]*fdelta(dist,h)/vvb;
            ff1= ray.rlen[i]*func[i];
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
			arowk[k]+=ff0*B0[j]+ff1*rfac[j][i];
		}
            }
         }
      }
  }
  if((i0=krad[1])!=0){				// integrate from rad[1] to ray.rad[ii1]
     i0m2=i0-2;
     dist = DIST(s22.theta,s22.phi,phi[1]);	// distance to the center of bell
     ff0=-0.5*fdelta(dist,h)/vv[1];		// horizontal function value
     BsplineFun(krad[1],rad[1],rknt,maxk,B0);	// radial function values

     if(i0!=raykrad[ii1]){			// not in same region, integrate from rad[1] to rb
        if(rad[1]>=ray.rad[ii1]){		// find the radius at the knot
           rb=rknt[i0-1];
        }else{
           rb=rknt[i0];
        }
        phib=LinearInt(rb,ray.rad[ii1],rad[1],ray.phi[ii1],phi[1]);
        vvb =LinearInt(rb,ray.rad[ii1],rad[1],ray.vel[ii1],vv[1]);
        dr=rb-rad[1];
        dphi=(phi[1]-phib)*rad[1];
        dl=sqrt(dr*dr+dphi*dphi);
            
        BsplineFun(krad[1],rb,rknt,maxk,B1);
        dist = DIST(s22.theta,s22.phi,phib);	// factor for point at rb
        ff1=-0.5*fdelta(dist,h)/vvb;
            
        for(j=0;j<4;j++){
            k=i0m2+j;
            if(k>=0 && k<=maxk)arowk[k]+=(ff0*B0[j]+ff1*B1[j])*dl;
        }
        for(j=0;j<4;j++)B0[j]=B1[j];
        ff0=ff1;
        i0=raykrad[ii1];
        i0m2=i0-2;
     }else{					// set rb to rad[1]
        rb=rad[1];
        phib=phi[1];
     }
     
     dr=ray.rad[ii1]-rb;			// integrate from rb to ray.rad[ii1]
     dphi=(ray.phi[ii1]-phib)*rb;
     dl=sqrt(dr*dr+dphi*dphi);
     ff0*=dl;
     ff1=func[ii1]*dl;
     
     for(j=0;j<4;j++){
         k=i0m2+j;
         if(k>=0 && k<=maxk){
		arowk[k]+=ff0*B0[j]+ff1*rfac[j][ii1];
	 }
     }

  }
  delete  [] func;
  for(j=0; j<4; j++){
	delete[]  rfac[j];
  }

}

void EqSt::CalEqStDist()
{
  dist = cos(seq.theta)*cos(sst.theta)+sin(seq.theta)*sin(sst.theta)*cos(seq.phi-sst.phi);
  dist = acos(dist);
}

void Spherical2::ConvFromGeo(Geographic geo)
{
const static real fac=0.993305621334896;

   theta=(90.0-geo.lat)*RAD; 					// geographic colat  
   theta=HPI-atan(fac*cos(theta)/max(1.e-37,sin(theta)));	// geocentric colat
   
   phi = geo.lon;
   if(phi<0.0)phi+=360.0;
   phi *= RAD;
}


void EqSt::ReadEq(class SUMRES &sumres)
{

   geq.lat=sumres.elat;
   geq.lon=sumres.elon;
   eqdep=sumres.edep;
   seq.ConvFromGeo(geq);		// convert to spherical coordinate
   ceq.ConvFromSph2(seq);		// convert to cartesian coordinate
}

void EqSt::ReadStRes(class SUMRES &sumres)
{
   gst.lat=sumres.slat;
   gst.lon=sumres.slon;
   res1=sumres.res-sumres.elipc-sumres.crustcor-
        sumres.evcor-sumres.coric;		// observed residual
   res2=sumres.het;		// predicted residual
   
   sst.ConvFromGeo(gst);		// convert to spherical coordinate
   cst.ConvFromSph2(sst);		// convert to cartesian coordinate
}

void EqSt::CalTrans4()
{
Cartesian c1,c2,c3;

  c1.x =sin(seq.theta);
  c1.y =c1.x*sin(seq.phi);
  c1.x*=cos(seq.phi);
  c1.z =cos(seq.theta);

  c2.x =sin(sst.theta);
  c2.y =c2.x*sin(sst.phi);
  c2.x*=cos(sst.phi);
  c2.z =cos(sst.theta);
  c3=c1.OutProdWith(c2);
  c2=c3.OutProdWith(c1);
  c1.Normalize();  
  c2.Normalize();
  c3.Normalize();
  trans.SetFromVec(c1,c2,c3);
}


void TransMatrix::SetFromVec(class Cartesian &v1, class Cartesian &v2, class Cartesian &v3)
{
   matrix[0]=v1.x; matrix[1]=v1.y; matrix[2]=v1.z;
   matrix[3]=v2.x; matrix[4]=v2.y; matrix[5]=v2.z;
   matrix[6]=v3.x; matrix[7]=v3.y; matrix[8]=v3.z;
   

}

void Cartesian::Normalize()
{
real temp;

  temp=x*x+y*y+z*z;
  if(temp>1.0e-20){
     temp=sqrt(temp);
     x/=temp;
     y/=temp;
     z/=temp;
  }else{
     x=2.0;
     y=0.0;
     z=0.0;
  }
}

Cartesian Cartesian::OutProdWith(class Cartesian &c2)
{
Cartesian c3;

  c3.x=y*c2.z-z*c2.y;
  c3.y=z*c2.x-x*c2.z;
  c3.z=x*c2.y-y*c2.x;

  return c3;
}


void Cartesian::ConvFromSph2(class Spherical2 &sph)
{
  x =sin(sph.theta);		// assume normalize coordinate i.e sph.r=1.0;
  y =x*sin(sph.phi);
  x*=cos(sph.phi);
  z =cos(sph.theta);
}
/*
 * to do forward transformation on a normalized spherical vector
 */
Spherical2 Spherical2::ForwardTrans(real *matrix)
{
real x,y,z,x1,y1,z1;
Spherical2 temp;

  x =sin(theta);
  y =x*sin(phi);
  x*=cos(phi);
  z =cos(theta);
  
  x1=matrix[0]*x+matrix[1]*y+matrix[2]*z;
  y1=matrix[3]*x+matrix[4]*y+matrix[5]*z;
  z1=matrix[6]*x+matrix[7]*y+matrix[8]*z;

  temp.phi=0.0;
  temp.theta=acos(z1);
  if(fabs(temp.theta) > 1.0e-6){
     if(x1!=0.||y1!=0.0)temp.phi=atan2(y1,x1);
     if(temp.phi<0.0)temp.phi+=2.*PI;
  }
  return temp;

}

Spherical2 Spherical2::InverseTrans(real *matrix)
{
real x,y,z,x1,y1,z1;
Spherical2 temp;

  x =sin(theta);
  y =x*sin(phi);
  x*=cos(phi);
  z =cos(theta);
  
  x1=matrix[0]*x+matrix[3]*y+matrix[6]*z;
  y1=matrix[1]*x+matrix[4]*y+matrix[7]*z;
  z1=matrix[2]*x+matrix[5]*y+matrix[8]*z;

  temp.phi=0.0;
  temp.theta=acos(z1);
  if(fabs(temp.theta) > 1.0e-6){
     if(x1!=0.||y1!=0.0)temp.phi=atan2(y1,x1);
     if(temp.phi<0.0)temp.phi+=2.*PI;
  }
  return temp;

}

void EqSt::ForwardTrans4(const int &eqst)
{

   if(eqst==EQ || eqst==EQST)eqnew=seq.ForwardTrans(trans.matrix);
   if(eqst==ST || eqst==EQST)stnew=sst.ForwardTrans(trans.matrix);

}

void EqSt::InverseTrans4(const int &eqst)
{

   if(eqst==EQ || eqst==EQST)eqnew=eqnew.InverseTrans(trans.matrix);
   if(eqst==ST || eqst==EQST)stnew=stnew.InverseTrans(trans.matrix);

}
/*
 * transform a point on the equator to the old coordinate
 */
void EqSt::InvTransRPhi(const real *r1, const real *phi1, const int& np)
{
real rr0,rr1,pp0,pp1,theta0,phi0, x,y,x1,y1,z1,length,dr,dphi;

  numpt=np;
  pp0=length=0.0;
  rr0=r1[0];
  for(int i=0;i<np;i++){
	rr1=r1[i];
	pp1=phi1[i];
	dr=rr1-rr0;
	dphi=(pp1-pp0)*rr1;
	length+=sqrt(dr*dr+dphi*dphi);

	y =sin(pp1);
	x =cos(pp1);
  
	x1=trans.matrix[0]*x+trans.matrix[3]*y;
	y1=trans.matrix[1]*x+trans.matrix[4]*y;
	z1=trans.matrix[2]*x+trans.matrix[5]*y;
  
	theta0=acos(z1);
	if(fabs(theta0) > 1.0e-6){
	   if(x1!=0.||y1!=0.0)phi0=atan2(y1,x1);
	   if(phi0<0.0)phi0+=2.*PI;
	}else{
	   phi0=0.0;
	}
	r[i]=rr1;
	phi[i]=phi0;
	theta[i]=theta0;
      dl[i]=length;
      rr0=rr1;
      pp0=pp1;
  }
}

// below are added to deal with discontinuity topography

void EqSt::Init(int maxpt)
{

 r = new real[maxpt];
 theta = new real[maxpt];
 phi = new real[maxpt];
 dl  = new real[maxpt];
}

void BSModel::ReadModel(int choice, char *flname)
{
fstream inFile;
int i,j,k,m,nr;
real dum,lat,lon;

 cout << " model = " <<flname<<endl;
 inFile.open(flname,ios::in);		// open prem file for read
 
 if(!inFile) { 				// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 inFile >> gpt >> maxk;
 maxk--;				// to be consistent with other rad functions
 InitKnots(choice);		// allocate space
 
 for(m=i=0;i<gpt;i++){
   inFile >> j >> lat >> lon;
//   cout << "j=" << j << "lat=" << lat << "lon=" << lon << endl;
   theta[i]=(90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=360.0;
   phi[i] *= RAD;
   if(choice!=REGIONAL){	// also read adjacent knots if it is not a regional model
      for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
      }
   }
 }
 
 for(k=i=0;i<=maxk;i++){
     inFile >> m >>rknt[i];
     for(j=0;j<gpt;j++){
        inFile >> dum;
        if(choice==MODALL || choice==REGIONAL)coeff[k++]=dum;
     }
 }
 
 if(choice==MODALL || choice==REGIONAL){
    for(k=i=0;k<=maxk;k++){
        cout <<" k= " <<k<<" rknt = " <<rknt[k]<<endl;
 /*       for(j=0;j<gpt;j++){
            cout <<" "<< coeff[i++];
            if((j+1)%5==0)cout << endl;
        }
        cout << endl;
 */
    }
 }
 
 inFile.close();
 if(choice==REGIONAL){		// read in average distance between knots
    cout <<"input the average distance (in degrees) between knots\n";
    cin >> avgdist;
    avgdist*=RAD;
 }else{
    CalAvgDist();
 }
 
 if(rknt[0]<RADIUS-DEPCMB+1.0 && rknt[maxk]>RADIUS-DEPMOHO-1.0)
    region=WHOLEMANTLE;
 else {
    region=WHOLEMANTLE;		// pretent to be a whole mantle model
   // cerr << " Error 3, not implemented yet\n";
   // exit(-1);
 }
}

void BSModel::InitKnots(int choice)
{
	theta = new real[gpt];
	phi = new real[gpt];
	ajknts = new int[gpt*7];
	rknt = new real[maxk+1];
	if(choice==MODALL || choice==REGIONAL){
	   coeff = new real[(maxk+1)*gpt];
	}
}

void BSModel::CalAvgDist()
{
int i,j,m,n,nb;

  for(i=m=n=0, avgdist=0.0;i<gpt;i++){	/* cal avg distance between nodes */
      m++;						/* 1st element is # of ajacent points, skip it */
	for(j=0;j<6;j++){
	    nb=ajknts[m++];
	    if(nb>i){				/* calculate only once, skip if ==0 */
		n++;
		avgdist+=DistBt2(i,nb);
	    }
	}
  }
  
  avgdist/=(real)n;
}


real BSModel::DistBt2(const int &pt1, const int &pt2)
{
real theta1,theta2,phi1,phi2,dist;

  theta1=theta[pt1];
  phi1  =phi[pt1];
  theta2=theta[pt2];
  phi2  =phi[pt2];

  dist = cos(theta1)*cos(theta2)+sin(theta1)*sin(theta2)*cos(phi2-phi1);
  dist = acos(dist);
  
  return dist;
	
}


real LinearInt(real &x,real &x0,real &x1,real &y0,real &y1)
{
	if(x0==x1)return 0.5*(y0+y1);
	else return y0+(y1-y0)*(x-x0)/(x1-x0);
}


real fdelta(real &delta, real &h)
{  
// note, this version has been corrected by Yu Gu, 1999.
// original normalization of 2/3 is now 1 	
real deltil,value;

  deltil=delta/h;
  
  if(deltil>2.0){
     return 0.0;
  }else if(deltil<=1.0){
     value=(4.0+3.0*deltil*deltil*(deltil-2.0));
  }else{
     value=2.0-deltil;
     value*=value*value;
  }
//  return value/6.0;
  return value/4.0;
}


//
// radial functions
//
void BsplineFun(int ib, real x, real *xknt, int knt, real *B)
{
// calculates radial B-spline function.
//   input:   ib = the index of B-splines in which x falls
//            x  = the radius of the value to be calculated
float temp,temp1,X1,X2,X3,X4,x0,x1,x2,x3,x4,x5;
static float A1,A2,A3,A4;
static int i0=-1;


  if(ib==1 || ib==2){
    x0=x1=xknt[0];
    x4=xknt[ib+1];
    x5=xknt[ib+2];
  }else if(ib==knt-1 || ib==knt){
    x0=xknt[ib-3];
    x1=xknt[ib-2];
    x4=x5=xknt[knt];
  }else{
    x0=xknt[ib-3];
    x1=xknt[ib-2];
    x4=xknt[ib+1];
    x5=xknt[ib+2];
  }

  x2=xknt[ib-1];
  x3=xknt[ib];
  
//  if(ib!=i0){		// not the same region, need to cal. some coeff.
	temp=1.0/(x3-x2);
	A1=temp/(x3-x1);
	A3=temp/(x4-x2);
	A4=A3/(x5-x2);
	temp=x4-x1;
	A2=A1/temp;
	A3/=temp;
	A1/=x3-x0;
	i0=ib;
//  }
  
  X1=x-x1;
  X2=x-x2;
  X3=x-x3;
  X4=x-x4;

  temp=X3*X3*A1;
  temp1=X3*X1*A2+X2*X4*A3;
  B[0]=-temp*X3;
  B[1]=(x-x0)*temp+X4*temp1;
  temp=X2*X2*A4;
  B[3]=X2*temp;
  B[2]=-X1*temp1-(x-x5)*temp;
/*
 * for 2nd derivative = 0
 */
  if(ib==1){
     B[2]+=B[1]/3.0;
     B[1]=B[1]/1.5+B[0];
     B[0]=0.0;
  }
  if(ib==2){
     B[1]+=B[0]/3.0;
     B[0]/=1.5;
  }
  if(ib==knt){
     B[1]+=B[2]/3.0;
     B[2]=B[2]/1.5+B[3];
     B[3]=0.0;
  }
	
  if(ib==knt-1){
     B[2]+=B[3]/3.0;
     B[3]/=1.5;
  }
}

/*
 * mode = 0 --- for read, 1 for write and 2 for read and write
 */

int SysOpen(char *filenm,int mode)
{
int fd;

  if(mode==O_WRONLY){
     if(creat(filenm,PERMS)==-1){
        cerr<<"Error, cannot creat file "<<filenm<<endl;
        exit(-1);
     }
  }
  if((fd=open(filenm,mode))==-1){
      cerr<<"Erorr in  opening  file "<<filenm<<endl;
      exit(-1);
  }
  return fd;
}




void SUMRES::OpenResFile()
{
char filenm[132];

  cout <<"Open waveform derived residual file"<< endl;
  cout <<"File name:"<< endl;
  cin >> filenm;
  cout <<"File="<< filenm << endl;
  fd=SysOpen(filenm,O_RDONLY); 
  
}

int SUMRES::TotalNumRes()
{
int numres;

   numres=lseek(fd,0L,2)/resrcl;    
   lseek(fd,0,0);				// beginning of the file
   return numres;
}


int SUMRES::ReadSumRes()
{
int byterd;

typedef struct TEMP{	// structure defined for reading only
	char	eqname[8];	// earthquake name
	float elat;		// summary earthquake latitude (in degree)
	float elon;		// longitude
	float edep;		// earthquake depth
	char	stname[4];	// station name
	float slat;		// summary station latitude (in degree)
	float slon;		// summary station longitude (in degree)
	float stelv;	// station elevation (in km)
	float res;		// observed residual
	int	ig;		// grade
	float ttime;	// total travel time
	float elipc;	// ellipcity correction
	float crustcor;	// crustal correction
	float evcor;	// station elevation correction
	float delta;	// epicentral distance
	float coric;	// innor core correction
	float spare;
} TEMP;

TEMP temp;

  if((byterd=read(fd,&temp,resrcl))!=resrcl)return 0;
  strncpy(eqname,temp.eqname,8);
  eqname[8]='\0';
  elat=temp.elat;
  elon=temp.elon;
  edep=temp.edep;
  strncpy(stname,temp.stname,4);
  stname[4]='\0';
  slat=temp.slat;
  slon=temp.slon;
  stelv=temp.stelv;
  res=temp.res;
  ttime=temp.ttime;
  elipc=temp.elipc;
  crustcor=temp.crustcor;
  evcor=temp.evcor;
  delta=temp.delta;
  coric=temp.coric;
  ig=temp.ig;

//  cout <<eqname<<" "<<elat<<" "<<elon<<" "<<edep<<" "<<
//        stname<<" "<<slat<<" "<<slon<<" "<<stelv<<" "<< res<<" "<<
//   cout<<        ttime<<" "<<elipc<<" "<<crustcor<<" "<<evcor<<" "<<
//        delta<<" "<<coric<<" "<<ig<<endl;
  cout <<eqname<<" "<<elat<<" "<<elon<<" "<< slat <<" "<< slon << endl;
        
  return byterd;
}



void SUMRES::SeektoRec(long rec)
{
    lseek(fd,resrcl*rec,0);
}
//
// =================================================================
//  The following subroutines are topography inversion.
// =================================================================
//
void CAPRES::Init(int size)
{
       indsurf_r = new short[size];
       surfbs_r  = new real[size];
}

void SUMRES::Init(int size)
{
       indsurf_t = new short[size];
       surfbs_t  = new real[size];
}

void GetCapinfo(real readbuf[], CAPRES &capres)
{ // get SS precursor info from a buffer
           capres.elat = readbuf[0];
           capres.elon = readbuf[1];
           capres.dep  = readbuf[2];
           capres.slat = readbuf[3];
           capres.slon = readbuf[4];
           capres.dist = readbuf[5];
           capres.bplat = readbuf[6];
           capres.bplon = readbuf[7];
           capres.hettopo = readbuf[8];
// below does a test
//	   capres.dep  = 0.02;
//	   capres.elat = 0.01;
//	   capres.elon = 110.0;
//	   capres.slat = 0.01;
//	   capres.slon = -120.0;
//           capres.dist = 130.0;
//	   capres.bplat = 0.01;
//	   capres.bplon = 175.0;
	   
//	   cout << capres.elat << " " << capres.elon << " " << capres.dep
//		<< " " << capres.slat << " " << capres.slon << " " <<
//		capres.dist << " " << capres.bplat << " " << capres.bplon
//		<< " " << capres.hettopo << endl;
}

void CAPRES::ReadCapRes(FILE *fp) 
{
  int  i;

  i=fread(&ncap, sizeof(int), 1, fp);
  i=fread(&capsize, sizeof(real), 1, fp);
  cout << "number of caps=" << ncap << " cap radius =" << capsize << endl;
  i=fread(caplat, sizeof(real), ncap, fp);
  i=fread(caplon, sizeof(real),ncap , fp);
//  for(i=0; i< ncap;i++){
//     printf("caplat= %lf, caplon=%lf\n",caplat[i],caplon[i]);
//  }
   i=fread(&ntopo, sizeof(int), 1, fp);
   i=fread(indtopo, sizeof(int), ntopo, fp);
   i=fread(restopo, sizeof(real), ntopo, fp);
   i=fread(sdvtopo, sizeof(real), ntopo, fp);
   i=fread(npercap, sizeof(int), ntopo, fp);
   cout <<"last res=" << restopo[ntopo-1] << " sdv="<< sdvtopo[ntopo-1]
      << " ncap=" << ncap << " ntopo=" << ntopo << " num_of_dat =" << npercap[ntopo-1] <<endl;
}

void FindSurfBspline(BSModel &bsmodel,real kernel, real inlat,real inlon,
                    short indbs[], real bsval[], int &nbsp_r)
// calculate distance between a given cap location and horizontal knots
{
int i, j;
real bl, cpl, blon, cplon, temp;
real bslat;
real del, rim;
  rim=2*bsmodel.avgdist;
  j = 0;
  for(i=0; i<bsmodel.GetHPts(); i++) bsval[i]=0.0;
  for(i=0; i<bsmodel.GetHPts(); i++) {
      bslat = 90-bsmodel.theta[i]*REPRAD;
      temp = HPI-bsmodel.theta[i];
      if (abs(temp) != HPI) {
         bl=atan(GEOCO*tan(temp));
      } else {
         bl=temp;
      }
      bl = HPI - bl;
      blon = bsmodel.phi[i];
      if(bsmodel.phi[i]> PI) {
         blon = bsmodel.phi[i]-TWOPI;
      }    
      if (abs(inlat) != 90.0) {
         cpl=atan(GEOCO*tan(inlat*RAD));
      } else{
         cpl=inlat*RAD;
      }
      cpl = HPI - cpl;
      cplon = inlon*RAD;
      del = cos(cpl)*cos(bl)+sin(bl)*sin(cpl)*(cos(blon)*cos(cplon)+
              sin(blon)*sin(cplon));
      if(abs(del) > 1.0) {
            del = 1.0;
      }
      del = acos(del);
      if(del <= rim) {
         indbs[j] = i;
         temp = CalcBspline(bsmodel.avgdist,del);
         bsval[j]=kernel*temp;
//         printf("%f %f %f\n", bslat, blon*REPRAD, bsval[j]); 
         j++;
      }
  }
  nbsp_r=j;
}

void CalcDelta(real lat1, real lon1, real lat2, real lon2, real &del)
// calculates the distance between two points on the sphere
{
 int i;
 real temp, a1, a2, a3, a4;
 
   if(abs(lat1) != 90.0) {
      a1 = atan(GEOCO*tan(lat1*RAD));
   }else {
      a1 = lat1*RAD;
   }
   a1 = HPI-a1;
   if(abs(lat2) != 90.0) {
      a2 = atan(GEOCO*tan(lat2*RAD));
   } else{
      a2 = lat2*RAD;
   }
   a2 = HPI-a2;
   a3=lon1*RAD;
   a4 =lon2*RAD;
//   cout << a1 << " " << a3 << " " << a2 << " " << a4 << endl; 
   del=cos(a1)*cos(a2)+sin(a1)*sin(a2)*(cos(a3)*cos(a4)+sin(a3)*sin(a4));
   if(abs(del) > 1.0)  del = 1.0;
   del = acos(del); 
}

real CalcBspline(real h, real del)
// finds the value of bspline
{
 real r, dif, ff; 
   ff =0.0;
   r = del/h;
   if(r > 2.0) {
      printf("error in calculating surface B-spline function!!!\n");
      exit(1);
   }
   dif = r-1.0;
   if(dif<=0.0) {
      ff = (4.0+3.0*r*r*(dif-1.0));
   }else {
      ff=2.0-r;
      ff*=ff*ff;
   }
//   printf("h=%f del=%f  ff=%f\n", h, del, ff/6.0);
// normalization has been changed to 1/4 instead   
//   return ff/6.0;
   return ff/4.0;
}

real CalcDist(real theta1, real phi1, real theta2, real phi2)
{  // calculates distance given theta and phi of two points
  int i;
  real dist;
  
  dist=cos(theta1)*cos(theta2)+sin(theta1)*sin(theta2)
          *(cos(phi1)*cos(phi2)+sin(phi1)*sin(phi2));
   if(abs(dist) > 1.0)  dist = 1.0;
   dist = acos(dist); 
   return dist;
}

real SurfTransKer(int opt, real rparm)
//  dt due to dr Morrelli and Dziewonski
// note I took the negative of the above due to depth, nor radius
{
  real ker;
  
   if(opt==0) { // 400 (0.5 factor in front to offset the 2 in FACT400)
      ker =0.5*FACT400*(pow((ETA400TOP*ETA400TOP-rparm*rparm), 0.5)
                     -pow((ETA400BOT*ETA400BOT-rparm*rparm), 0.5));
   } else{	// 670
      ker =0.5*FACT670*(pow((ETA670TOP*ETA670TOP-rparm*rparm), 0.5)
                     -pow((ETA670BOT*ETA670BOT-rparm*rparm), 0.5));
   }
//   cout << "transmitted = " << ker << endl;
   return ker;
}

void PHASE::TransBoundContri(class BSModel& bsmodel, EqSt &eqst, float *transarow, real p0, int opt)
{ // this finds the contribution of the B-splines where the ray intercept the 
  // the major discontinuities.  This is a general version for any ray type.
  
Spherical2 s2,s22;
TransMatrix  trans;
int gpt=bsmodel.gpt;
int ncross670, ncross400, indskip, i,j;
real twoh, h, dist, bsweight, bstrans;
   
// calculate tranformation matrix
   trans=eqst.GetTrans4();
   h=bsmodel.GetAvgDist(); // bell size

   twoh=2.0*h;
   ncross670 = ncross400 = 0;
 if(opt==61 || opt == 63) {
   bstrans = SurfTransKer(0, p0);
   for(i=0;i<numpt;i++){
   //
   // the following print statement plots ray path
   //  cout << phi[i]*REPRAD << "  " << rad[i] << endl;
   //

    // below finds the 400 contribution due to crossing point
     if(fabs(rad[i]-(RADIUS-DEP400)) < 0.01){
//       cout << "400 crossing point " << ncross400 << endl;
       if(ncross400%2==0) {
         for(j=0;j<gpt;j++){ // loop thru bells
          // tranform horizontal b-spline to equator coordinate
             s2.theta=bsmodel.theta[j];
             s2.phi = bsmodel.phi[j];
             s22=s2.ForwardTrans(trans.matrix);
             dist = CalcDist(s22.theta, s22.phi, HPI, phi[i]);
             if(dist < twoh){
                 // B-spline value at this distance
                 // we have vel[i] -- prem velocity
                 // and ray parameter is known, we can
                 // calculate the contribution
                 bsweight=CalcBspline(h,dist);
//               cout << "dist=" << dist << " weight=" << bsweight << endl;
//               cout << "p0 = " << p0 << endl;
                 transarow[j]+=bstrans*bsweight;   // a row of a matrix for the transmitted wave

 //                cout << "j= " << j <<  " transarow=" << transarow[j] << endl;
             }
          }
        }
        ncross400++;
     }
   }
 }
 if(opt==63) {
    indskip = gpt;
 }else{
    indskip = 0; 
 }
// below for 670 case 
 if(opt==62 || opt == 63) {
    bstrans= SurfTransKer(1, p0);
    for(i=0;i<numpt;i++){
       if(fabs(rad[i]-(RADIUS-DEP670))<0.01){
//        cout << "660 crossing point " << ncross670 << endl;
          if(ncross670%2==0) {
          
             for(j=0;j<gpt;j++){ // loop thru bells
                 // tranform horizontal b-spline to equator coordinate
                 s2.theta=bsmodel.theta[j];
                 s2.phi = bsmodel.phi[j];
                 s22=s2.ForwardTrans(trans.matrix);
		 
                 dist = CalcDist(s22.theta, s22.phi, HPI, phi[i]);
                 if(dist < twoh){
                 // B-spline value at this distance
                 // we have vel[i] -- prem velocity
                 // and ray parameter is known, we can
                 // calculate the contribution
                    bsweight=CalcBspline(h,dist);
                    transarow[j+indskip]+=bstrans*bsweight;  // SS contribution
//                 cout << "j+indskip= " << j+indskip <<  " transarow[j+indskip]=" << transarow[j+indskip] << endl;
                 }
             }
          }
          ncross670 ++;
       }
   }
 }
}


void PHASE::CapTransBoundContri(class BSModel& bsmodel, CAPRES &capres, float *transarow, real p0, int iphase, int opt)
{ // this finds the contribution of the B-splines where the ray intercept the 
  // the major discontinuities
  
Spherical2 s2,s22;
TransMatrix  trans;
int gpt=bsmodel.gpt;
int ncross670, ncross400, indskip, i,j, nskip670, nskip400;
real twoh, h, dist, bsweight, bstrans, nodelat, nodelon;
   

// calculate tranformation matrix
   trans=capres.GetTrans4();

// bell size, for SdS, only two crossing point for a discontinuity (not including reflection)

   h=bsmodel.GetAvgDist();
   twoh=2.0*h;
   ncross670 = ncross400 = 0;
   if(iphase ==0) {       // if S410S skip the second crossing point, use reflection
      nskip400 = 2;
      nskip670 = 999;     // no skipping
   }else if(iphase == 1){ // if S670S skip the second crossing point, reflection
      nskip670 = 2;      
      nskip400 = 999;   
   }else {		  // if SS, use all  transmissions at 400 and 670 boundaries 
      nskip670 = 999;
      nskip400 = 999;
   }
 
 if(opt==61 || opt == 63) {
    bstrans = SurfTransKer(0, p0);
    for(i=0;i<numpt;i++){
// the following print statement plots ray path
//       cout << phi[i]*REPRAD << "  " << rad[i] << endl;
//
// below finds the 400 contribution due to crossing point
       if(fabs(rad[i]-(RADIUS-DEP400)) < 0.001){
          if(ncross400%2==0 && ncross400 != nskip400) { // use % since each boundary has a layer above and one below 
//	     cout << " crossing 400" << "  phi= " << phi[i]*REPRAD << "  rad =" << rad[i] << endl;
             for(j=0;j<gpt;j++){ // loop thru bells
                 // tranform horizontal b-spline to equator coordinate
                 s2.theta=bsmodel.theta[j];
                 s2.phi = bsmodel.phi[j];
                 s22=s2.ForwardTrans(trans.matrix);
                 dist = CalcDist(s22.theta, s22.phi, HPI, phi[i]);
                 if(dist < twoh){
                   // B-spline value at this distance
                   // we have vel[i] -- prem velocity
                   // and ray parameter is known, we can
                   // calculate the contribution
                    bsweight=CalcBspline(h,dist);

                    if(iphase < 2) {  
                       transarow[j]-=bstrans*bsweight;  // SdS contribution
                    }else{
                       transarow[j]+=bstrans*bsweight;  // SS contribution
                    }
//  the following block of code prints out kernels
//		    nodelat = (HPI-bsmodel.theta[j])*REPRAD;
//		    nodelon = bsmodel.phi[j]*REPRAD;
//		    if(nodelon>180.0) nodelon -= 360.0;
//		    cout << nodelat << "  " << nodelon << "  " <<  bstrans*bsweight << endl; 
//                    cout << "iphase= " << iphase << "  " << j << "transarow[j]=" << transarow[j] << endl;
                 }
             }
          }
          ncross400++;
       }
    }
 }
 if(opt==63) {
    indskip = gpt;
 }else{
    indskip = 0; 
 }
// below for 670 case 
 if(opt==62 || opt == 63) {
    bstrans= SurfTransKer(1, p0);
    for(i=0;i<numpt;i++){
//       cout << phi[i]*REPRAD << "  " << rad[i] << endl;
       if(fabs(rad[i]-(RADIUS-DEP670))<0.001){
          if(ncross670%2==0 && ncross670 != nskip670) {
//	     cout << " crossing 670" << "  phi= " << phi[i]*REPRAD  << "  rad =" << rad[i] << endl;
             for(j=0;j<gpt;j++){ // loop thru bells
                 // tranform horizontal b-spline to equator coordinate
                 s2.theta=bsmodel.theta[j];
                 s2.phi = bsmodel.phi[j];
                 s22=s2.ForwardTrans(trans.matrix);
                 dist = CalcDist(s22.theta, s22.phi, HPI, phi[i]);
 //               cout << s22.theta  << " " << s22.phi << " " << s22.phi << " " <<  phi[i] << endl;
                 if(dist < twoh){
                 // B-spline value at this distance
                 // we have vel[i] -- prem velocity
                 // and ray parameter is known, we can
                 // calculate the contribution
                    bsweight=CalcBspline(h,dist);
                    if(iphase < 2) {  
                       transarow[j+indskip]-=bstrans*bsweight;  // SdS contribution
                    }else{
                       transarow[j+indskip]+=bstrans*bsweight;  // SS contribution
                    }
//  the following block of code prints out kernels
//		    nodelat = (HPI-bsmodel.theta[j])*REPRAD;
//		    nodelon = bsmodel.phi[j]*REPRAD;
//		    if(nodelon>180.0) nodelon -= 360.0;
//		    cout << nodelat << "  " << nodelon << "  " <<  bstrans*bsweight << endl; 
//                    cout << "dist=" << dist << " weight=" << bsweight << endl;
                 }
             }
          }
          ncross670 ++;
       }
   }
 }
}


void LocateCap(real lat1, real lon1, CAPRES &capres, real &val, real &sdvval, real &dataval)
// find the caps where a reflection point falls, then assign the value
// to the reflection point based on the distance from each cap.
// The value is inversely proportional to the distance from the cap.
{
 int i, j;
 int ind;
 real latlow, lathigh;
 real del, csize, sumdel;
 real deltabuf[10], indexbuf[10], indexXbuf[10];
 real sdvbuf[10];
 real invnumdat[10];
 real delmin;

    latlow = lat1-(capres.capsize+capres.capsize);
    lathigh = lat1+(capres.capsize+capres.capsize);
    if(latlow<=-90.0) latlow=-90.0;
    if(lathigh>=90.0) lathigh=90.0;
    csize =capres.capsize*RAD;
    j = 0;
    sumdel = 0.0;
    for(i=0; i< capres.ntopo; i++){
          ind = capres.indtopo[i];
          if(capres.caplat[ind] >=latlow && capres.caplat[ind]<=lathigh){
             CalcDelta(lat1,lon1,capres.caplat[ind],capres.caplon[ind],
                       del);
             if(del<=csize) {
 		deltabuf[j]=CalcBspline(csize,del);
                sumdel += deltabuf[j];
                sdvbuf[j] = (int)(capres.sdvtopo[i]+1);
		invnumdat[j] = 10.0*pow(capres.npercap[i], -0.5);
                indexbuf[j] = ind;
                indexXbuf[j]=i;
                j++;
             }
          }  
    }
    if(j>=5) {
         printf("Strange bounce point %f %f\n", lat1, lon1);
         for(i=0; i<j; i++){
          ind = indexbuf[i];
          printf("ind=%d, delta= %f, %f %f\n", ind,deltabuf[i],
            capres.caplat[ind], capres.caplon[ind]); 
         } 
    }     
    val = 0.0;
    sdvval = 0.0;
    dataval = 0.0;
    for(i=0; i<j; i++){  // weighting function is spherical splines 
          ind      = indexXbuf[i];
          val     += deltabuf[i]*capres.restopo[ind];
          sdvval  += deltabuf[i]*sdvbuf[i];
	  dataval += deltabuf[i]*invnumdat[i];
//          printf("ind=%d, delta= %f, %f %f\n", ind, deltabuf[i],
//                 capres.restopo[ind], dataval  );
    }
    if(sumdel<0.000001 || sdvval < 0.00001 || dataval < 0.00001 || j==0) {
          printf("reflection point not touching any caps!!!\n");
          val = 9999.0;
	  sdvval = 9999.0;
	  dataval = 9999.0;
    }else{
          val /= sumdel;
	  sdvval /= sumdel;
	  dataval /= sumdel;  
    }

}

void CAPRES::SurfKer(real fact, real eta, real rparm, real &ker)
//       kernel based on Morrelli and Dziewonski
//        dr=dt*r/(2*(eta**2-(rparm*57.3)**2)**(0.5))
// note I took the negative of the above due to depth, not radius
{
   ker =-fact*(pow((eta*eta-rparm*rparm), 0.5));
}


void CapContribute(PHASE &ray, BSModel &bsmodel, CAPRES &capres, float *arow)
{
int gpt=bsmodel.gpt,mode,krad[2];	// horizontal grid points
int i,j,k,ii0,ii1,npt,ireg,numx,ind;
int numpt=ray.numpt;
real h,twoh,twohsq,aa,bb,xmin,xmax,del,*arowk;
real theta[2],rr[2], vv[2];		// for two end points
Spherical2 s2,s22;
TransMatrix trans;
real *rknt;
int  maxk;

   maxk=bsmodel.GetMaxK();
   rknt=bsmodel.rknt;
   numx=(maxk+1+1)*gpt;		    	// number of columns in maxtrix A
                                       	// includes the surface knots
   for(k=0;k<numx;k++)arow[k]=0.0;
    
   arowk = new real[maxk+1];           	// Contribution to "A" maxtrix from a specific cell

   del=capres.dist*RAD;
   
   trans=capres.GetTrans4();
   h=bsmodel.GetAvgDist();
   twoh=2.0*h;
   twohsq=twoh*twoh;
   
   for(i=0;i<gpt;i++){		    	// loop thru bells, break ray path
       s2.theta=bsmodel.theta[i];
       s2.phi = bsmodel.phi[i];
       s22=s2.ForwardTrans(trans.matrix);
       if(fabs(s22.theta-HPI)<=twoh){
          aa=HPI-s22.theta;
          bb=sqrt(twohsq-aa*aa);
          xmin=s22.phi-bb;
          xmax=s22.phi+bb;
          if(xmax>TWOPI){		// make sure it is between 0 -> 2*PI
             xmax-=TWOPI;
             xmin-=TWOPI;
          }
          mode=IN;
          if(between(0.0,xmin,del) && between(0.0,xmax,del)){ 	// path passes  bell
            ii1=ii0=-1;
          }else if(between(0.0,xmin,del)&& xmax>del){		// path ends in  bell 
            xmax=del;
            ii0=-1;
            ii1=numpt-1;
          }else if(between(0.0,xmax,del)&& xmin<0.0){		// path begins in bell
            xmin=0.0;
            ii0=0;
            ii1=-1;
          }else if(between(xmin,0.0,xmax) &&  between(xmin,del,xmax)){ // path all in bell
            xmin=0.0;
            xmax=del;
            ii0=0;
            ii1=numpt-1;
          }else{								/* path is out of bell */
            mode=OUT;
          }
          if(mode==IN){
            krad[0]=krad[1]=0;
            if(ii0==-1){
               for(j=0;j<numpt;j++){
                   if(ray.phi[j]>=xmin)break;
               }
               ii0=j;
            }
            if(ii1==-1){
               for(j=numpt-1;j>=0;j--){
                   if(ray.phi[j]<=xmax)break;
               }
               ii1=j;
            }
            if(ii0!=0){
               rr[0]=LinearInt(xmin,ray.phi[ii0-1],ray.phi[ii0],ray.rad[ii0-1],ray.rad[ii0]);
               vv[0]=LinearInt(rr[0],ray.rad[ii0-1],ray.rad[ii0],ray.vel[ii0-1],ray.vel[ii0]);
               theta[0]=xmin;
               krad[0]=GetReg(rr[0],rknt,maxk);
            }
            if(ii1!=numpt-1){
                rr[1]=LinearInt(xmax,ray.phi[ii1],ray.phi[ii1+1],ray.rad[ii1],ray.rad[ii1+1]);
                vv[1]=LinearInt(rr[1],ray.rad[ii1],ray.rad[ii1+1],ray.vel[ii1],ray.vel[ii1+1]);
                theta[1]=xmax;
                ireg=krad[1]=GetReg(rr[1],rknt,maxk);
            }
            RadContrib(ray,theta,rr,vv,krad,ii0,ii1,s22,h,rknt,maxk,arowk);
            for(ind=k=0;k<=maxk;k++,ind+=gpt)arow[ind+i]=arowk[k];
            
          }
       }
   }
   delete [] arowk;

   
}

void CapContribute_split(PHASE &ray, BSModel &bsmodel, CAPRES &capres, float *arow)
{// does the calculation of kernels from split B-spline model
int gpt=bsmodel.gpt,mode;		// horizontal grid points
int krad_lm[2], krad_um[2];
int i,j,k,ii0,ii1,npt,ireg,numx,ind;
int numpt=ray.numpt;
real h,twoh,twohsq,aa,bb,xmin,xmax,del;
real *arowk_um, *arowk_lm;
real theta[2],rr[2], vv[2];		// for two end points
Spherical2 s2,s22;
TransMatrix trans;
real *rknt_lm, *rknt_um;
int  maxk, maxk_lm, maxk_um, umindex;
int  uplowop;

   maxk=bsmodel.GetMaxK();
   maxk_lm = bsmodel.GetMaxK_lm();
   maxk_um = bsmodel.GetMaxK_um();
   umindex = (maxk_lm+1)*gpt;
   arowk_lm = new real[maxk_lm+1];	// Contribution to "A" maxtrix from a specific cell
   arowk_um = new real[maxk_um+1];	// Contribution to "A" maxtrix from a specific cell
   rknt_lm=bsmodel.rknt_lm;
   rknt_um=bsmodel.rknt_um;
   numx=(maxk+1+2)*gpt;		    	// number of columns in maxtrix A
                                       	// includes the surface knots
   for(k=0;k<numx;k++)arow[k]=0.0;
    
   del=capres.dist*RAD;
   
   trans=capres.GetTrans4();
   h=bsmodel.GetAvgDist();
   twoh=2.0*h;
   twohsq=twoh*twoh;
   
   for(i=0;i<gpt;i++){		    	// loop thru bells, break ray path
       s2.theta=bsmodel.theta[i];
       s2.phi = bsmodel.phi[i];
       s22=s2.ForwardTrans(trans.matrix);
       if(fabs(s22.theta-HPI)<=twoh){
          aa=HPI-s22.theta;
          bb=sqrt(twohsq-aa*aa);
          xmin=s22.phi-bb;
          xmax=s22.phi+bb;
          if(xmax>TWOPI){		// make sure it is between 0 -> 2*PI
             xmax-=TWOPI;
             xmin-=TWOPI;
          }
          mode=IN;
          if(between(0.0,xmin,del) && between(0.0,xmax,del)){ 	// path passes  bell
            ii1=ii0=-1;
          }else if(between(0.0,xmin,del)&& xmax>del){		// path ends in  bell 
            xmax=del;
            ii0=-1;
            ii1=numpt-1;
          }else if(between(0.0,xmax,del)&& xmin<0.0){		// path begins in bell
            xmin=0.0;
            ii0=0;
            ii1=-1;
          }else if(between(xmin,0.0,xmax) &&  between(xmin,del,xmax)){ // path all in bell
            xmin=0.0;
            xmax=del;
            ii0=0;
            ii1=numpt-1;
          }else{								/* path is out of bell */
            mode=OUT;
          }
          if(mode==IN){
            krad_lm[0]=krad_lm[1]=0;					
            krad_um[0]=krad_um[1]=0;					
            if(ii0==-1){
               for(j=0;j<numpt;j++){
                   if(ray.phi[j]>=xmin)break;
               }
               ii0=j;
            }
            if(ii1==-1){
               for(j=numpt-1;j>=0;j--){
                   if(ray.phi[j]<=xmax)break;
               }
               ii1=j;
            }
            if(ii0!=0){
               rr[0]=LinearInt(xmin,ray.phi[ii0-1],ray.phi[ii0],ray.rad[ii0-1],ray.rad[ii0]);
               vv[0]=LinearInt(rr[0],ray.rad[ii0-1],ray.rad[ii0],ray.vel[ii0-1],ray.vel[ii0]);
               theta[0]=xmin;
               krad_lm[0]=GetReg(rr[0],rknt_lm,maxk_lm);
               krad_um[0]=GetReg(rr[0],rknt_um,maxk_um);
            }
            if(ii1!=numpt-1){
               rr[1]=LinearInt(xmax,ray.phi[ii1],ray.phi[ii1+1],ray.rad[ii1],ray.rad[ii1+1]);
               vv[1]=LinearInt(rr[1],ray.rad[ii1],ray.rad[ii1+1],ray.vel[ii1],ray.vel[ii1+1]);
               theta[1]=xmax;
               krad_lm[1]=GetReg(rr[1],rknt_lm,maxk_lm);
               krad_um[1]=GetReg(rr[1],rknt_um,maxk_um);
            }
	    uplowop = 0;  					// lower mantle
            RadContrib_split(ray,theta,rr,vv,krad_lm,ii0,ii1,s22,h,rknt_lm,maxk_lm,arowk_lm, uplowop);
	    uplowop = 1;
	    RadContrib_split(ray,theta,rr,vv,krad_um,ii0,ii1,s22,h,rknt_um,maxk_um,arowk_um, uplowop);
            for(ind=k=0;k<=maxk_lm;k++,ind+=gpt)arow[ind+i]=arowk_lm[k];
            for(ind=umindex,k=0;k<=maxk_um;k++,ind+=gpt)arow[ind+i]=arowk_um[k];
            
          }
       }
   }
   delete [] arowk_lm;
   delete [] arowk_um;
}

void ConvEq(CAPRES &capres)
{

   capres.geq.lat=capres.elat;
   capres.geq.lon=capres.elon;
   
   capres.seq.ConvFromGeo(capres.geq);		// convert to spherical coordinate
   capres.ceq.ConvFromSph2(capres.seq);		// convert to cartesian coordinate
}

void ConvSt(CAPRES &capres)
{
   capres.gst.lat=capres.slat;
   capres.gst.lon=capres.slon;
   
   capres.sst.ConvFromGeo(capres.gst);		// convert to spherical coordinate
   capres.cst.ConvFromSph2(capres.sst);		// convert to cartesian coordinate
}

void CAPRES::CalTrans4()
{
Cartesian c1,c2,c3;
 
  c1.x =sin(seq.theta);
  c1.y =c1.x*sin(seq.phi);
  c1.x*=cos(seq.phi);
  c1.z =cos(seq.theta);

  c2.x =sin(sst.theta);
  c2.y =c2.x*sin(sst.phi);
  c2.x*=cos(sst.phi);
  c2.z =cos(sst.theta);
  c3=c1.OutProdWith(c2);
  c2=c3.OutProdWith(c1);
  c1.Normalize();  
  c2.Normalize();
  c3.Normalize();
  trans.SetFromVec(c1,c2,c3);
}

void CapSaveArow(int fd, CAPRES &capres,float *arow,short *irow,int ind, int iphase)
{
float buf[9];
int ibuf[3];

    buf[0]=(float)capres.geq.lat;
    buf[1]=(float)capres.geq.lon;
    buf[2]=(float)capres.dep;
    buf[3]=(float)capres.gst.lat;
    buf[4]=(float)capres.gst.lon;
    buf[5]=(float)0.0;	            // station elevation, set to 0.0 for sumres
    buf[6]=(float)capres.topores;   // observed residual
    buf[7]=(float)capres.hettopo;   // predicted residual
    buf[8]=(float)capres.PMtime;    // PREM predicted travel time
    ibuf[0]=0;		       // earthquake index, not used, set to 0
    ibuf[1]=0;		       // station index, not used, set to 0
    ibuf[2]=ind;	       // number of nozero elements in the matrix
 //   cout << " prem time = " << buf[8] << endl;
    
    write(fd,buf,sizeof(buf));
    write(fd,ibuf,sizeof(ibuf));
    if(ind>0 || capres.nbsp_r>0){
       write(fd,irow,sizeof(short)*ind);
       write(fd,arow,sizeof(float)*ind);
    }
}
