#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <unistd.h>
#include <limits.h>
#include <assert.h>
#include <string.h>
#include "ttimelib.h"

// The following subroutines are specifically designed for multiply split
// earth models, Jeffrey Gu, 1999.

void BSModel::ReadModel_multisplit(int choice, char *flname)
{
fstream inFile;
int i,j,k,m,nr, nc;
real dum,lat,lon;

 cout << " model = " <<flname<<endl;
 inFile.open(flname,ios::in);		// open prem file for read
 
 if(!inFile) { 				// open failed?
     cerr << "Cannot open "<<flname<<" for input" << endl;
     exit(-1);
 }
 Read1stLine(inFile,choice);
 cout << "maxk_lm=" << maxk_lm << " maxk_um=" << maxk_um << " total=" << maxk << endl;
 maxk_um--;				// to be consistent with other rad functions
 maxk_lm--;				// to be consistent with other rad functions
 maxk--;				// to be consistent with other rad functions
 InitKnots_split(choice);		// allocate space
 cout << "gpt = " << gpt << endl;
 for(m=i=0;i<gpt;i++){
   inFile >> j >> lat >> lon;
   theta[i]=(90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=360.0;
   phi[i] *= RAD;
   if(choice!=REGIONAL && choice !=SPLIT_REGIONAL){	// also read adjacent knots if it is not a regional model
      for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
      }
   }
 }
 cout << "maxk =" << maxk << endl;
 for(k=i=0;i<=maxk;i++){  // Model coefficients from CMB (lm) to Surface
     inFile >> m >>rknt[i];
     for(j=0;j<gpt;j++){
        inFile >> dum;
        coeff[k++]=dum;
     }
 }
 inFile.close();
 if(choice==REGIONAL || choice==SPLIT_REGIONAL){// read in average distance between knots
    cout <<"input the average distance (in degrees) between knots\n";
    cin >> avgdist;
    avgdist*=RAD;
 }else{
    CalAvgDist();
 }
 k=0;
 for(i=0;i<(nsplit_um+nsplit_lm);i++){
	for(j=0;j<nbsperlayer[i];j++){
		rknt_mult[i][j]=rknt[k];
     		cout <<"layer= " << i<<" rknt =" <<j << " "<<rknt[k]<<endl;
		k++;	
	}
	// add two more boundary points to make sure
	// spline calculation works.
	rknt_mult[i][j]=rknt_mult[i][j+1]=rknt_mult[i][j-1];
 }
 
 region=UPLOWMANTLE;
}

void BSModel::Read1stLine(fstream &inFile, int choice)
{
 int i,j;
 int nsplit; 
 char mod_desc[50];

 if(choice==SPLIT || choice==SPLIT_REGIONAL){
 	inFile >> mod_desc >> gpt >> maxk_lm >> maxk_um;
 	cout << mod_desc << " " << gpt << " " << maxk_lm
		 << " " << maxk_um << endl;
 }else if(choice==MULTISPLIT){
	inFile >> mod_desc  >> nsplit_lm >> nsplit_um
		>> gpt;
	nsplit=nsplit_um+nsplit_lm;
	cout << "nsplit = " << nsplit << endl;
	maxk_lm=0; maxk_um=0;
	if(nsplit>9) cout << "too many splits, not implemented!" <<endl;
	for(i=0;i<nsplit;i++){
		inFile >> nbsperlayer[i];
		cout << "i=" << i << "  layer =" <<  nbsperlayer[i] << endl;
		if(i<nsplit_lm) {
			maxk_lm += nbsperlayer[i];
		}else{
			maxk_um += nbsperlayer[i];
		}
	}
	maxk=maxk_lm+maxk_um;
 }
}


void Contribution_multisplit(class PHASE &ray, class BSModel &bsmodel, class EqSt &eqst, float *arow)
{ // calculates contribution of splines along the ray path.
int gpt=bsmodel.gpt,mode;		// horizontal grid points
int i,j,k,ii0,ii1,npt,ireg,numx,ind;
int ii, jj, ilayer[2];
int krad_mult[MAXLAYER][2];
int numpt=ray.numpt;
real h,twoh,twohsq,aa,bb,xmin,xmax,del;
real **arowk;
real theta[2],rr[2], vv[2];		// for two end points
real *rknt, **rknt_mult;
Spherical2 s2,s22;
TransMatrix trans;
int  maxk;
int  nsplit,uplowop;

   maxk=bsmodel.GetMaxK();
   nsplit=bsmodel.GetNumSplit();
   arowk=new real* [nsplit];
   for(i=0; i<nsplit; i++){
	arowk[i] = new real[maxk];
   }
   rknt=bsmodel.rknt;
   rknt_mult=bsmodel.rknt_mult;		// 2-D arrays to save multiple layers
   numx=(maxk+1)*gpt;	// number of columns in maxtrix A
   for(k=0;k<numx;k++)arow[k]=0.0;

   del=eqst.GetDist();
   trans=eqst.GetTrans4();
   h=bsmodel.GetAvgDist();
   twoh=2.0*h;
   twohsq=twoh*twoh;
   for(i=0;i<gpt;i++){			  // loop thru bells, break ray path
       s2.theta=bsmodel.theta[i];
       s2.phi = bsmodel.phi[i];
       s22=s2.ForwardTrans(trans.matrix); // rotate bell to path coordinates s22
       if(fabs(s22.theta-HPI)<=twoh){
          aa=HPI-s22.theta;		
          bb=sqrt(twohsq-aa*aa);	  // distance away from center of bell
          xmin=s22.phi-bb;		  // boundary of a bell
          xmax=s22.phi+bb;
          if(xmax>TWOPI){		  // make sure it is between 0 -> 2*PI
             xmax-=TWOPI;
             xmin-=TWOPI;
          }
          mode=IN;
          if(between(0.0,xmin,del) && between(0.0,xmax,del)){   // path passes  bell
            ii1=ii0=-1;
          }else if(between(0.0,xmin,del)&& xmax>del){		// path ends in  bell 
            xmax=del;
            ii0=-1;
            ii1=numpt-1;
          }else if(between(0.0,xmax,del)&& xmin<0.0){		// path begins in bell
            xmin=0.0;
            ii0=0;
            ii1=-1;
          }else if(between(xmin,0.0,xmax) &&  between(xmin,del,xmax)){ // path all in bell
            xmin=0.0;
            xmax=del;
            ii0=0;
            ii1=numpt-1;
          }else{						// path is out of bell
            mode=OUT;
          }
          if(mode==IN){
	    for(ii=0;ii<nsplit;ii++) krad_mult[ii][0]=krad_mult[ii][1]=0;
            if(ii0==-1){					// means xmin is inside path
               for(j=0;j<numpt;j++){
                   if(ray.phi[j]>=xmin)break;			// find which point along raypath coincides xmin
               }
               ii0=j;						// save index of that point, lower limit
            }
            if(ii1==-1){
               for(j=numpt-1;j>=0;j--){
                   if(ray.phi[j]<=xmax)break;
               }
               ii1=j;						// save index of point coincides xmax, upper limit
            }
            if(ii0!=0){
               	rr[0]=LinearInt(xmin,ray.phi[ii0-1],ray.phi[ii0],ray.rad[ii0-1],ray.rad[ii0]);
              	vv[0]=LinearInt(rr[0],ray.rad[ii0-1],ray.rad[ii0],ray.vel[ii0-1],ray.vel[ii0]);
               	theta[0]=xmin;
		for(ii=0;ii<nsplit;ii++){
//			cout << "bsmodel.nbsperlayer[ii]-1=" << bsmodel.nbsperlayer[ii]-1 << endl;
                	krad_mult[ii][0]=GetReg(rr[0],rknt_mult[ii],bsmodel.nbsperlayer[ii]-1);
			
		}
            }
            if(ii1!=numpt-1){
               	rr[1]=LinearInt(xmax,ray.phi[ii1],ray.phi[ii1+1],ray.rad[ii1],ray.rad[ii1+1]);
                vv[1]=LinearInt(rr[1],ray.rad[ii1],ray.rad[ii1+1],ray.vel[ii1],ray.vel[ii1+1]);
                theta[1]=xmax;
		for(ii=0;ii<nsplit;ii++){
                	krad_mult[ii][1]=GetReg(rr[1],rknt_mult[ii],bsmodel.nbsperlayer[ii]-1);
		}
            }
	    ind=0;
	    for(ii=0;ii<nsplit;ii++){
//		cout << "depth0=" << 6371.-rr[0] <<" depth1=" << 6371.-rr[1]<< " krad_mult[0]=" << krad_mult[ii][0] << " krad_mult[1]=" << krad_mult[ii][1]<<endl;
            	RadContrib_multisplit(ii,ray,theta,rr,vv,krad_mult[ii],ii0,ii1,s22,h,rknt_mult[ii],
			bsmodel.nbsperlayer[ii]-1,arowk[ii]);

	    	for(k=0;k<=bsmodel.nbsperlayer[ii]-1;k++){
//			cout << "ii=" << ii << " ind=" << ind << " k="<< k << " ind+i="<<ind+i << endl;
//			cout << "arowk[ii][k]=" << arowk[ii][k]<<endl;
			arow[ind+i]=arowk[ii][k];
			ind+=gpt;
		}
	    }
          }
       }
   }
   for(i=0; i<nsplit; i++){
	delete [] arowk[i];
   }
}


void RadContrib_multisplit(int ilayer,class PHASE &ray,real *phi,real *rad, real *vv,int *krad,int ii0,int ii1,
Spherical2 &s22, real &h, real *rknt, int maxk, real *arowk)
{ // integration along ray path.  The output is saved in arowk which is the A matrix
  // this version is general for continuous, split and multiply split mantles.
  // 
int i,i0,i0m2,i1,j,k,n,reg,mode,ibeg=0,iend;
real rb,phib,vvb,dl,dr,dphi,ff0,ff1,dist,B0[4],B1[4],x,*func,rmin,rmax;
int *raykrad;
int numpt=ray.numpt;
real *rfac[4];
  
  for(k=0;k<=maxk;k++)arowk[k]=0.0;
  for(i=0;i<4;i++){
  	rfac[i] = new real [numpt];
  }
  raykrad=ray.krad_mult[ilayer];
//  cout << "ilayer =" << ilayer << " maxk=" << maxk << endl;
//  for(i=0;i<=maxk+2;i++){
//	cout << i << " rknt=" << rknt[i] << endl;
//  }

//  for(i=0;i<790;i++){
//	cout << "i=" << i <<" r=" << ray.rad[i] <<  " raykrad=" << raykrad[i] << endl;
//  }
  for(i=0;i<numpt;i++){
	rfac[0][i]=ray.rfac_mult[ilayer][0][i];
	rfac[1][i]=ray.rfac_mult[ilayer][1][i];
	rfac[2][i]=ray.rfac_mult[ilayer][2][i];
	rfac[3][i]=ray.rfac_mult[ilayer][3][i];
  }

  func = new real[ii1+1];
  
  for(i=ii0;i<=ii1;i++){				// horizontal functions
      dist=DIST(s22.theta,s22.phi,ray.phi[i]);
      func[i]=-fdelta(dist,h);
  }
//  cout << "krad[0]=" << krad[0] << " raykrad[ii0]=" << raykrad[ii0] << " raykrad[ii1]=" << raykrad[ii1]<< endl;
  if((i0=krad[0])!=0){					// there is an extra point at begining
     i0m2=i0-2;
     dist = DIST(s22.theta,s22.phi,phi[0]);		// distance to the center of bell
     ff0=-0.5*fdelta(dist,h)/vv[0];			// horizontal function value
     BsplineFun(krad[0],rad[0],rknt,maxk,B0);		// radial function values at rad[0]

     if(i0!=raykrad[ii0]){				// this point and  next one is not in the same region
        if(rad[0]>=ray.rad[ii0]){			// find the radius at the knot
           rb=rknt[i0-1];
        }else{
           rb=rknt[i0];
        }
        phib=LinearInt(rb,ray.rad[ii0],rad[0],ray.phi[ii0],phi[0]);	// interpolate phi
        vvb =LinearInt(rb,ray.rad[ii0],rad[0],ray.vel[ii0],vv[0]);	// interpolate velocity
        dist = DIST(s22.theta,s22.phi,phib);
        ff1=-0.5*fdelta(dist,h)/vvb;
        BsplineFun(krad[0],rb,rknt,maxk,B1);
        
        dr=rad[0]-rb;
        dphi=(phi[0]-phib)*rad[0];
        dl=sqrt(dr*dr+dphi*dphi);
        for(j=0;j<4;j++){
            k=i0m2+j;
            if(k>=0 && k<=maxk)arowk[k]+=(ff0*B0[j]+ff1*B1[j])*dl;
        }
        for(j=0;j<4;j++)B0[j]=B1[j];
        ff0=ff1;
        i0=raykrad[ii0];
        i0m2=i0-2;
     }else{
        rb=rad[0];
        phib=phi[0];
     }
     
     dr=ray.rad[ii0]-rb;				// integrate from rb to ray.rad[ii0]
     dphi=(ray.phi[ii0]-phib)*rb;
     dl=sqrt(dr*dr+dphi*dphi);
          
     ff1=func[ii0]*dl;
     ff0*=dl;
     for(j=0;j<4;j++){
         k=i0m2+j;
         if(k>=0 && k<=maxk) {
		arowk[k]+=ff0*B0[j]+ff1*rfac[j][ii0]; 
	 }
     }
  }
  for(i=ii0;i<ii1;i++){
      if((i0=raykrad[i])==raykrad[i+1]){	// two points in  same region
         i0m2=i0-2;
         if(raykrad[i]!=0){			// in range, integrate from raykrad[i] to raykrad[i+1]
            ff0=ray.rlen[i]*func[i];
            ff1=ray.rlen[i]*func[i+1];
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
			arowk[k] +=ff0*rfac[j][i]+ff1*rfac[j][i+1];
                }
            }
         }
      }else{					// two points are in different region
         rb=-1.0;
         if((i0=raykrad[i+1])!=0){		// 2nd point in range, integrate from rb to ray.rad[i+1]
            i0m2=i0-2;
            if(ray.rad[i+1]>=ray.rad[i]){
               rb=rknt[raykrad[i+1]-1];
            }else{
               rb=rknt[raykrad[i+1]];
            }
            phib=LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.phi[i],ray.phi[i+1]);
            vvb =LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.vel[i],ray.vel[i+1]);
            dr=ray.rad[i+1]-rb;				// integrate from rb to ray.rad[i+1]
            dphi=(ray.phi[i+1]-phib)*rb;
            dl=sqrt(dr*dr+dphi*dphi);
            
            BsplineFun(raykrad[i+1],rb,rknt,maxk,B0);
            dist = DIST(s22.theta,s22.phi,phib);	// factor for point at rb
            ff0=-0.5*dl*fdelta(dist,h)/vvb;
            ff1= dl*func[i+1];
            
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
			arowk[k]+=ff0*B0[j]+ff1*rfac[j][i+1];
		}
            }
         }
         if((i0=raykrad[i])!=0){			// integrate from ray.rad[i] to rb
            i0m2=i0-2;
            if(ray.rad[i+1]>=ray.rad[i])rb=rknt[raykrad[i]];
            else rb=rknt[raykrad[i]-1];
            BsplineFun(raykrad[i],rb,rknt,maxk,B0);
            phib=LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.phi[i],ray.phi[i+1]);
            vvb =LinearInt(rb,ray.rad[i],ray.rad[i+1],ray.vel[i],ray.vel[i+1]);
               
            dist = DIST(s22.theta,s22.phi,phib);
            ff0=-0.5*ray.rlen[i]*fdelta(dist,h)/vvb;
            ff1= ray.rlen[i]*func[i];
            for(j=0;j<4;j++){
                k=i0m2+j;
                if(k>=0 && k<=maxk){
			arowk[k]+=ff0*B0[j]+ff1*rfac[j][i];
		}
            }
         }
      }
  }
  if((i0=krad[1])!=0){				// integrate from rad[1] to ray.rad[ii1]
     i0m2=i0-2;
     dist = DIST(s22.theta,s22.phi,phi[1]);	// distance to the center of bell
     ff0=-0.5*fdelta(dist,h)/vv[1];		// horizontal function value
     BsplineFun(krad[1],rad[1],rknt,maxk,B0);	// radial function values

     if(i0!=raykrad[ii1]){			// not in same region, integrate from rad[1] to rb
        if(rad[1]>=ray.rad[ii1]){		// find the radius at the knot
           rb=rknt[i0-1];
        }else{
           rb=rknt[i0];
        }
        phib=LinearInt(rb,ray.rad[ii1],rad[1],ray.phi[ii1],phi[1]);
        vvb =LinearInt(rb,ray.rad[ii1],rad[1],ray.vel[ii1],vv[1]);
        dr=rb-rad[1];
        dphi=(phi[1]-phib)*rad[1];
        dl=sqrt(dr*dr+dphi*dphi);
            
        BsplineFun(krad[1],rb,rknt,maxk,B1);
        dist = DIST(s22.theta,s22.phi,phib);	// factor for point at rb
        ff1=-0.5*fdelta(dist,h)/vvb;
            
        for(j=0;j<4;j++){
            k=i0m2+j;
            if(k>=0 && k<=maxk)arowk[k]+=(ff0*B0[j]+ff1*B1[j])*dl;
        }
        for(j=0;j<4;j++)B0[j]=B1[j];
        ff0=ff1;
        i0=raykrad[ii1];
        i0m2=i0-2;
     }else{					// set rb to rad[1]
        rb=rad[1];
        phib=phi[1];
     }
     
     dr=ray.rad[ii1]-rb;			// integrate from rb to ray.rad[ii1]
     dphi=(ray.phi[ii1]-phib)*rb;
     dl=sqrt(dr*dr+dphi*dphi);
     ff0*=dl;
     ff1=func[ii1]*dl;
     
     for(j=0;j<4;j++){
         k=i0m2+j;
         if(k>=0 && k<=maxk){
		arowk[k]+=ff0*B0[j]+ff1*rfac[j][ii1];
	 }
     }

  }
  delete  [] func;
  for(j=0; j<4; j++){
	delete[]  rfac[j];
  }

}


void PHASE::CalDlength_multisplit(class BSModel& bsmodel)
{  // calculte multiplication factor rfac
real vv,rr0,rr1,pp0,pp1,dr,dphi,*rknt,B[4];
int i,j,k,ii,ireg,ireg1,maxk, nsplit, ilayer;

   rknt=bsmodel.rknt;
   maxk=bsmodel.maxk;
   rr0=rad[0];		// raypath which can be plotted using rr[], pp[]
   pp0=phi[0];		// numpt is the number of points for raypath
 
   for(i=0;i<numpt;i++){
      if(i!=numpt-1){
	   rr1=rad[i+1];
	   pp1=phi[i+1];
      }else{
	   rr1=rr0;
	   pp1=pp0;
      }
      vv=0.5/vel[i];
      nsplit=bsmodel.GetNumSplit();
      for(ii=0;ii<nsplit;ii++){		// save radial factor for a layer at each path point
	ireg=krad_mult[ii][i]=GetReg(rad[i],bsmodel.rknt_mult[ii],bsmodel.nbsperlayer[ii]-1);
//	cout << rad[i] << " ii=" << ii << " ireg = " << ireg << endl;
      	rfac_mult[ii][0][i]=rfac_mult[ii][1][i]=rfac_mult[ii][2][i]=rfac_mult[ii][3][i]=0.0;
      	if(ireg>0 && ireg<=bsmodel.nbsperlayer[ii]-1){
         	BsplineFun(ireg, rad[i], bsmodel.rknt_mult[ii],
				bsmodel.nbsperlayer[ii]-1, B); // note kmax here = kmax+1
/*
	 	cout << "depth = " << 6371.-rad[i] << endl;
	 	cout << " B[0]=" << B[0] << endl;
	 	cout << " B[1]=" << B[1] << endl;
	 	cout << " B[2]=" << B[2] << endl;
	 	cout << " B[3]=" << B[3] << endl;
*/
         	rfac_mult[ii][0][i]=vv*B[0];
         	rfac_mult[ii][1][i]=vv*B[1];
         	rfac_mult[ii][2][i]=vv*B[2];
         	rfac_mult[ii][3][i]=vv*B[3];
      	}
      }	
      dr=rr1-rr0;
      dphi=(pp1-pp0)*rr1;
      rlen[i]=sqrt(dr*dr+dphi*dphi);
      rr0=rr1;
      pp0=pp1;
   }
}


void Get_MultiReg(int nsplit, int *bsknot,  int *krad, int *ilayer)
{ // get layer number in the corresponding layer
  // in a multiply split mantle
  // Input:
  //		*krad   ----- old segment number in a 1 layer mantle
  //	 	*bsknot  ---  bspline knots throughout the mantle
  // Output:
  //		*krad   ----- new segment number in the corresponding
  //			      layer within a multiply split mantle
  //		ilayer  ----- layer number where krad0 falls
  int i, j;

  j=0;
  *ilayer=-1;
  if(*krad==0) return;
	
  for(i=0;i<nsplit;i++){
	j += bsknot[i];
	if(*krad<=j){
		*krad = *krad-j+bsknot[i];
		*ilayer=i;
		break;
	}	
  }
  return;
}

