#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <unistd.h>
#include <limits.h>
#include <assert.h>
#include <string.h>
#include "ttimelib.h"

// this are the routines for the split model

void CheckMenu(int argc, char *argv[], char &model, int &begrec, int &endrec, char &modeltype)
{
// This checks the menu options

      void PrintMenu();
      int i, j=0;
      char ch;

      model = 'W';
      modeltype = 'C';
      begrec = 1;
      endrec = 1000000;
      if(argc > 1) { // if to use options
	 for(i=1; i< argc;i++) {
		if(argv[i][0] == 'W' || argv[i][0] == 'w') {
			model = 'W';
			cout << "  Global Model";
		} else if(argv[i][0] == 'R' || argv[i][0] == 'r'){
			model = 'R';
			cout << "  Regional Model";
		} else if(argv[i][0] == 'C' || argv[i][0] == 'c'){
			modeltype = 'C';
			cout << "  Continuous";
		} else if(argv[i][0] == 'S' || argv[i][0] == 's'){
			modeltype = 'S';
			cout << "  Split";
		} else if(argv[i][0] == 'D' || argv[i][0] == 'd') {
			assert(argc >=i+2);
			i++;
			sscanf(argv[i], "%d", &begrec);
			i++;
			if(strlen(argv[i])<2) {
				cout << "WARNING, SINGLE DIGIT endrec value!!" << endl;
				cout << "Continue(y/n)?" << endl;
				cin >> ch;
				if(ch != 'Y' && ch != 'y') {
					exit(-1);
				}
			}
			sscanf(argv[i], "%d", &endrec);
			assert(begrec <= endrec);
			i += 2;
			cout << "  begrec=" << begrec << " endrec=" << endrec;
		} else {
			cout << "option(s) not found !!" << endl << endl;
	 		PrintMenu();
		}
	 }
	 cout << endl;
      } else {
	 cout << "Global Model " << "  Continuous" << "  begrec=" << begrec << " endrec=" << endrec <<endl;
      }
      
}

void PrintMenu() 
{
      cout << endl<<endl;
      cout << "Options: " << endl;
      cout << "-------------------------------------------------------------------------" << endl;
      cout << "W             --Global model (default)" << endl;
      cout << "R             --Regional model" << endl;
      cout << "C             --Continuous model across 670 (default)" << endl;
      cout << "S             --Split model across 670" << endl;
      cout << "D rec1 rec2   --Read record from rec1 to rec2 (default 1--1000000)" << endl;
      cout << "-------------------------------------------------------------------------" << endl;
      cout << endl << endl;
      exit(-1);
}

void ChooseFormat(int &amatopt, int hpts, int maxk, int &num3d, int &num2d,int &numx)
{
   cout << "calculate : " << endl;
   cout << "---------------------------------------------- " << endl;
   cout << "1. OLD FORMAT A Mat for 3-D inversion " << endl;
   cout << "2. NEW FORMAT A Mat for 3-D inversion " << endl;
   cout << "3. Joint A Mat for 1 layer of topography" << endl;
   cout << "4. Joint A Mat for 2 layers of topography" << endl;
   cout << "---------------------------------------------- " << endl;
   cout << "Enter: " << endl;
   cin >> amatopt;
   num2d = 0;
   num3d = hpts*(maxk+1);
   if(amatopt < 3) {
      numx = num3d;
   } else if(amatopt == 3) {
      num2d = hpts;
      numx=num3d+num2d;      // 3D + 1 layer of 2D B-splines
   } else {
      num2d = hpts+hpts;
      numx=num3d+num2d; // 3D + 2 layers of 2D B-splines
   }
   if(numx>SHRT_MAX){
      cerr<<"Number of unknowns exceed limit\n";
      cerr<<"Can not use irow as short integer to save matrix A element index\n";
      cerr<<"Change irow to int instead\n";
      exit(-1);
   }
}

void ChooseDelRange(int &Delopt, real &delow, real &delup)
{
   real temp;
   
   cout << endl;
   cout << "Set limits on epicentral distance? (0=no; 1=yes)" << endl;
   cin >> Delopt;
   if(Delopt != 0 && Delopt != 1) {
      cerr << "Delta range option not valid!!!" << endl;
      exit(-1);
   }
   if(Delopt == 1) {
      cout << "Enter desired delta range (min, max): " << endl;
      cin >> delow >> delup;
      if(delup < delow) {
         temp = delow;
         delow = delup;
         delup = temp;
      }
   }
}


void BSModel::ReadModel_split(int choice, char *flname)
{

fstream inFile;
int i,j,k,m,nr;
char mod_desc[50];
real dum,lat,lon;

 cout << " model = " <<flname<<endl;
 inFile.open(flname,ios::in);		// open prem file for read
 
 if(!inFile) { 				// open failed?
     cerr << "Cannot open "<<flname<<" for input" << endl;
     exit(-1);
 }
 inFile >> mod_desc >> gpt >> maxk_lm >> maxk_um;
 cout << mod_desc << " " << gpt << " " << maxk_lm << " " << maxk_um << endl;
 maxk = maxk_um+maxk_lm-1;		// to be consistent with other rad functions
 maxk_um--;				// to be consistent with other rad functions
 maxk_lm--;				// to be consistent with other rad functions
 InitKnots_split(choice);		// allocate space
 
 for(m=i=0;i<gpt;i++){
   inFile >> j >> lat >> lon;
   theta[i]=(90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=360.0;
   phi[i] *= RAD;
   if(choice!=REGIONAL && choice !=SPLIT_REGIONAL){	// also read adjacent knots if it is not a regional model
      for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
      }
   }
 }
 
 for(k=i=0;i<=maxk;i++){  // Model coefficients from CMB (lm) to Surface
     inFile >> m >>rknt[i];
     if(i<=maxk_lm) {
	rknt_lm[i] = rknt[i];
     }else{
	rknt_um[i-maxk_lm-1] = rknt[i];
     }	
     for(j=0;j<gpt;j++){
        inFile >> dum;
        coeff[k++]=dum;
     }
 }
 
 for(k=i=0;k<=maxk;k++){
     cout <<" k= " <<k<<" rknt = " <<rknt[k]<<endl;
 }
 
 inFile.close();
 if(choice==REGIONAL || choice==SPLIT_REGIONAL){// read in average distance between knots
    cout <<"input the average distance (in degrees) between knots\n";
    cin >> avgdist;
    avgdist*=RAD;
 }else{
    CalAvgDist();
 }
 
 region=UPLOWMANTLE;
}

void BSModel::InitKnots_split(int choice)
{
	theta = new real[gpt];
	phi = new real[gpt];
	ajknts = new int[gpt*7];
	rknt = new real[maxk+1];
	rknt_um = new real[maxk_um+1];
	rknt_lm = new real[maxk_lm+1];
	coeff = new real[(maxk+1)*gpt];
}

void PHASE::CalDlength_split(class BSModel& bsmodel)
{  
// calculte multiplication factor rfac_um and rfac_lm
real vv,rr0,rr1,pp0,pp1,dr,dphi,*rknt_lm, *rknt_um,B[4];
int i,ireg,maxk_lm, maxk_um;

   rknt_um=bsmodel.rknt_um;
   rknt_lm=bsmodel.rknt_lm;
   maxk_um=bsmodel.maxk_um;
   maxk_lm=bsmodel.maxk_lm;
   
   rr0=rad[0];		// raypath which can be plotted using rr[], pp[]
   pp0=phi[0];		// numpt is the number of points for raypath
 
   for(i=0;i<numpt;i++){
      if(i!=numpt-1){
	   rr1=rad[i+1];
	   pp1=phi[i+1];
      }else{
	   rr1=rr0;
	   pp1=pp0;
      }
      rfac_um[0][i]=rfac_um[1][i]=rfac_um[2][i]=rfac_um[3][i]=0.0;
      rfac_lm[0][i]=rfac_lm[1][i]=rfac_lm[2][i]=rfac_lm[3][i]=0.0;
      vv=0.5/vel[i];

      ireg=krad_um[i] = GetReg(rad[i],rknt_um,maxk_um);
      if(ireg>0 && ireg<=maxk_um){
         BsplineFun(ireg, rad[i], rknt_um, maxk_um, B);
         rfac_um[0][i]=vv*B[0];
         rfac_um[1][i]=vv*B[1];
         rfac_um[2][i]=vv*B[2];
         rfac_um[3][i]=vv*B[3];
      } 
      ireg=krad_lm[i] = GetReg(rad[i],rknt_lm,maxk_lm);
      if(ireg>0 && ireg<=maxk_lm){
         BsplineFun(ireg, rad[i], rknt_lm, maxk_lm, B);
         rfac_lm[0][i]=vv*B[0];
         rfac_lm[1][i]=vv*B[1];
         rfac_lm[2][i]=vv*B[2];
         rfac_lm[3][i]=vv*B[3];
      }
      dr=rr1-rr0;
      dphi=(pp1-pp0)*rr1;
      rlen[i]=sqrt(dr*dr+dphi*dphi);
      rr0=rr1;
      pp0=pp1;
   }
}
