#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "ttimelib.h"

//
//  phase dependent functions
// 

void RayInfo(int &amatphase, char *filenm)
{
// the following checks the input data type
   cout << "Input data type : " << endl;
   cout << "---------------------------------------------- " << endl;
   cout << "1. travel time of ISC or Xianfeng's T-time" << endl;
   cout << "2. travel time of SS and SdS phases" << endl;
   cout << "---------------------------------------------- " << endl;
   cout << "Enter: " << endl;
   cin >> amatphase;
   
   assert(amatphase == 1 || amatphase == 2);

   if(amatphase ==2 ) { 
      cout <<"SS precursor info file name (binary):"<< endl;
      cin >> filenm;
      cout <<"File="<< filenm << endl;
   }

}

Phase ChooseRay()
{
Phase rc;
int itemp;

  printf("___________________________________________\n");
  printf("Choose ray:\n");
  printf("P=%1d, S=%1d, PP=%1d, SS=%1d, PcP=%1d, ScS=%1d\n",P,S,PP,SS,PcP,ScS);
  printf("PKPab=%1d, PKPbc=%1d, PKPdf=%1d, PPP=%2d SSS=%2d\n",PKPab,PKPbc,PKPdf,PPP,SSS);
  printf("PS=%2d, SP=%2d, pP=%d, sP=%2d, sS=%2d, pS=%2d\n",PS,SP,pP,sP,sS,pS);
  printf("P670S=%2d (P converted to S at 670 km discontinuity)\n",P670S);
  printf("S670P=%2d (S converted/scattered to P at 670 km discontinuity)\n",S670P);
  printf("P400P=%2d (P reflected at 400 km discontinuity)\n", P400P);
  printf("P670P=%2d (P reflected at 670 km discontinuity)\n", P670P);
  printf("S400S=%2d (S reflected at 400 km discontinuity)\n", S400S);
  printf("S670S=%2d (S reflected at 670 km discontinuity)\n", S670S);
  printf("SKS=%2d, SKKS=%2d, SKKKS=%d\n", SKS, SKKS, SKKKS);
  
  printf("___________________________________________\n");
  cout << "Ray:\n";
  for(int i=0;i<10;i++){
     cin >> itemp;
     rc=(Phase) itemp;
     if(rc>=P && rc<LAST){
        return rc;
     }else{
        cerr << "Wrong choice\n";
        cout << "Choose number beteen " << P << " and " << LAST-1 << endl;
     }
  }
  exit(-1);
}

Phase SetRay(int iphase)
{
Phase rc;
   rc = (Phase) iphase;
   if(rc>=P && rc<LAST){
      return rc;
   }else{
      cerr << "Wrong choice\n";
      cout << "Choose number beteen " << P << " and " << LAST-1 << endl; 
   }
   exit(-1);
}

P_wave::P_wave()
{
  numseg=2;
  dir=new int[numseg];
  dir[0]=DOWN;
  dir[1]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=P;
  
  rayname=new char[2];
  strcpy(rayname,"P");
}

void P_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i670-1];			// assume teleseismic P
  if(eqlay<DPrem.i670)	pmax=min(pmax,p1);	// need to adjust pmax

  rend[0]=rbeg[1]=DPrem.lmbot;
  tlay[P]=DPrem.icmb;					// deepest layer a P ray can go
  tr[P]  =DPrem.rr[tlay[P]];
}


S_wave::S_wave()
{
  numseg=2;
  dir=new int[numseg];
  dir[0]=DOWN;
  dir[1]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=S;
  
  rayname=new char[2];
  strcpy(rayname,"S");
}

void S_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];
int size=2*DPrem.numlay;

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  rend[0]=rbeg[1]=DPrem.lmbot;
  tlay[S]=DPrem.icmb;			// deepest layer an S ray can go
  tr[S]  =DPrem.rr[tlay[S]];
  
}

PP_wave::PP_wave()
{
  numseg=4;
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
  
  rayname=new char[3];
  strcpy(rayname,"PP");
}

void PP_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  rend[1]=rbeg[2]=DPrem.umtop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;  

  tlay[P]=DPrem.icmb;			// deepest layer a PP ray can go
  tr[P]  =DPrem.rr[tlay[P]];
}

SS_wave::SS_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=S;
  
  rayname=new char[3];
  strcpy(rayname,"SS");
}

void SS_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  rend[1]=rbeg[2]=DPrem.umtop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;
  tlay[S]=DPrem.icmb;				// deepest layer an SS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
}

PcP_wave::PcP_wave()
{
  numseg=2;  
  dir=new int[numseg];
  dir[0]=DOWN;
  dir[1]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=P;
  
  rayname=new char[4];
  strcpy(rayname,"PcP");
}


void PcP_wave::init(class DeckPrem &DPrem)
{
  pmin = 0.0;
  pmax = DPrem.u[P][DPrem.icmb];
  rend[0]=rbeg[1]=DPrem.lmbot;
  tlay[P]=DPrem.icmb;			// deepest layer a PcP ray can go
  tr[P]  =DPrem.rr[tlay[P]];
}

ScS_wave::ScS_wave()
{
  numseg=2;  
  dir=new int[numseg];
  dir[0]=DOWN;
  dir[1]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=S;
  
  rayname=new char[4];
  strcpy(rayname,"ScS");
}

void ScS_wave::init(class DeckPrem &DPrem)
{
  pmin = 0.0;
  pmax = DPrem.u[S][DPrem.icmb];
  rend[0]=rbeg[1]=DPrem.lmbot;
  tlay[S]=DPrem.icmb;			// deepest layer a ScS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
}


PKPab_wave::PKPab_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[1]=DOWN;
  dir[2]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
    
  rayname=new char[6];
  strcpy(rayname,"PKPab");
}

void PKPab_wave::init(class DeckPrem &DPrem)
{

  pmin = DPrem.u[P][DPrem.iiob];		// C point
  pmax = DPrem.u[P][DPrem.icmb];		// A point
  rend[0]=rbeg[3]=DPrem.lmbot;
  rbeg[1]=rend[2]=DPrem.octop;
  rend[1]=rbeg[2]=DPrem.ocbot;
  tlay[P]=DPrem.iiob;
  tr[P]  =DPrem.rr[tlay[P]];
}

PKPbc_wave::PKPbc_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[1]=DOWN;
  dir[2]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
    
  rayname=new char[6];
  strcpy(rayname,"PKPab");
}

void PKPbc_wave::init(class DeckPrem &DPrem)
{
  pmin = DPrem.u[P][DPrem.iiob];		// C point
  pmax = DPrem.u[P][DPrem.icmb];		// A point
  rend[0]=rbeg[3]=DPrem.lmbot;
  rbeg[1]=rend[2]=DPrem.octop;
  rend[1]=rbeg[2]=DPrem.ocbot;  
  tlay[P]=DPrem.iiob;
  tr[P]  =DPrem.rr[tlay[P]];
}

PKPdf_wave::PKPdf_wave()
{
  numseg=6;  
  dir=new int[numseg];
  dir[0]=dir[1]=dir[2]=DOWN;
  dir[3]=dir[4]=dir[5]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
    
  rayname=new char[6];
  strcpy(rayname,"PKPdf");
}

void PKPdf_wave::init(class DeckPrem &DPrem)
{
  pmin = 0.0;
  pmax = DPrem.u[P][DPrem.iiob-1];
  rend[0]=rbeg[5]=DPrem.lmbot;
  rbeg[1]=rend[4]=DPrem.octop;
  rend[1]=rbeg[4]=DPrem.ocbot;
  rbeg[2]=rend[3]=DPrem.ictop;
  rend[2]=rbeg[3]=DPrem.icbot;
  tlay[P]=0;			// deepest layer a PKPdf ray can go
  tr[P]  =0.0;
  // do not need to adjust pmax for depth unless the source is in IC
}

PPP_wave::PPP_wave()
{
  numseg=6;
  dir=new int[numseg];
  dir[0]=dir[2]=dir[4]=DOWN;
  dir[1]=dir[3]=dir[5]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
  
  rayname=new char[4];
  strcpy(rayname,"PPP");
}

void PPP_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  for(int i=0;i<5;i+=2){
      if(i!=0)rbeg[i]=DPrem.umtop;		// do not overwrite rbeg[0] in PHASE::initarray
      if(i!=4)rend[i+1]=DPrem.umtop;	// do not overwrite rend[5]
      rend[i]=rbeg[i+1]=DPrem.lmbot;
  }
  tlay[P]=DPrem.icmb;			// deepest layer a PPP ray can go
  tr[P]  =DPrem.rr[tlay[P]];
}

SSS_wave::SSS_wave()
{
  numseg=6;  
  dir=new int[numseg];
  dir[0]=dir[2]=dir[4]=DOWN;
  dir[1]=dir[3]=dir[5]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=S;
  
  rayname=new char[4];
  strcpy(rayname,"SSS");
}



void SSS_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];
  
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);		// need to adjust pmax
  
  for(int i=0;i<5;i+=2){
      if(i!=0)rbeg[i]=DPrem.umtop;
      if(i!=4)rend[i+1]=DPrem.umtop;
      rend[i]=rbeg[i+1]=DPrem.lmbot;
  }
  tlay[S]=DPrem.icmb;			// deepest layer a SSS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
}


PS_wave::PS_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=P;
  pors[2]=pors[3]=S;
    
  rayname=new char[3];
  strcpy(rayname,"PS");
}

void PS_wave::init(class DeckPrem &DPrem)
{
real pmins = DPrem.u[S][DPrem.icmb];
real pmaxs = DPrem.u[S][DPrem.i670-1];
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i670-1]; 			// assume teleseismic P
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);		// need to adjust pmax
 
  pmin=max(pmins,pmin);
  pmax=min(pmaxs,pmax);

  rend[1]=rbeg[2]=DPrem.umtop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;  
  tlay[P]=tlay[S]=DPrem.icmb;				// deepest layer a PS ray can go
  tr[P]=tr[S]=DPrem.rr[tlay[P]];
}

SP_wave::SP_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=S;
  pors[2]=pors[3]=P;
    
  rayname=new char[3];
  strcpy(rayname,"SP");
}

void SP_wave::init(class DeckPrem &DPrem)
{
real pminp=DPrem.u[P][DPrem.icmb];
real pmaxp0=DPrem.u[P][DPrem.i670-1];
real p1=eqr/eqv[S];

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];		// assume teleseismic S
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax
 
  pmin=max(pminp,pmin);
  pmax=min(pmaxp0,pmax);

  rend[1]=rbeg[2]=DPrem.umtop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;
  tlay[P]=tlay[S]=DPrem.icmb;			// deepest layer a SP ray can go
  tr[P]=tr[S]=DPrem.rr[tlay[P]];
}

sP_wave::sP_wave()
{
  numseg=3;  
  dir=new int[numseg];
  dir[0]=dir[2]=UP;
  dir[1]=DOWN;
  
  pors= new int[numseg];
  pors[0]=S;
  pors[1]=pors[2]=P;
  
  rayname=new char[3];
  strcpy(rayname,"sP");
}

void sP_wave::init(class DeckPrem &DPrem)
{
real pmaxs=eqr/eqv[S];		// pmax for S leg, ungoing only

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i670-1];	// assume teleseismic P
  pmax = min(pmaxs,pmax);

  rend[0]=DPrem.umtop;
  rbeg[1]=DPrem.umtop;
  rend[1]=rbeg[2]=DPrem.lmbot;  
  tlay[P]=DPrem.icmb;			// deepest layer an sP ray (P leg) can go
  tr[P]  =DPrem.rr[tlay[P]];
}

pS_wave::pS_wave()
{
  numseg=3;  
  dir=new int[numseg];
  dir[0]=dir[2]=UP;
  dir[1]=DOWN;
  
  pors= new int[numseg];
  pors[0]=P;
  pors[1]=pors[2]=S;
  
  rayname=new char[3];
  strcpy(rayname,"pS");
}

void pS_wave::init(class DeckPrem &DPrem)
{
real pmaxp0=eqr/eqv[P];				// pmax for P leg, upgoing only

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];	// assume teleseismic P
  pmax = min(pmaxp0,pmax);

  rend[0]=DPrem.umtop;
  rbeg[1]=DPrem.umtop;
  rend[1]=rbeg[2]=DPrem.lmbot;
  tlay[S]=DPrem.icmb;			// deepest layer a pS ray (S leg) can go
  tr[S]  =DPrem.rr[tlay[S]];
}

pP_wave::pP_wave()
{
  numseg=3;  
  dir=new int[numseg];
  dir[0]=dir[2]=UP;
  dir[1]=DOWN;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
  
  rayname=new char[3];
  strcpy(rayname,"pP");
}

void pP_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[P];

  pmin=DPrem.u[P][DPrem.icmb];
  pmax=DPrem.u[P][DPrem.i670-1];	// assume teleseismic P
  pmax=min(pmax,p1);
  
  rend[0]=DPrem.umtop;
  rbeg[1]=DPrem.umtop;
  rend[1]=rbeg[2]=DPrem.lmbot;  
  tlay[P]=DPrem.icmb;			// deepest layer a pP ray can go
  tr[P]  =DPrem.rr[tlay[P]];
}

sS_wave::sS_wave()
{
  numseg=3;  
  dir=new int[numseg];
  dir[0]=dir[2]=UP;
  dir[1]=DOWN;
  
  pors= new int[numseg];
  pors[0]=pors[1]=pors[2]=S;
  
  rayname=new char[3];
  strcpy(rayname,"sS");
}

void sS_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];

  pmin=DPrem.u[S][DPrem.icmb];
  pmax=DPrem.u[S][DPrem.i670-1];	// assume teleseismic S
  pmax=min(p1,pmax);

  rend[0]=DPrem.umtop;
  rbeg[1]=DPrem.umtop;
  rend[1]=rbeg[2]=DPrem.lmbot;  
  tlay[S]=DPrem.icmb;			// deepest layer a sS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
}


P670S_wave::P670S_wave()
{
  numseg=3;  
  dir=new int[numseg];
  dir[0]=DOWN;
  dir[1]=dir[2]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=P;
  pors[2]=S;
  
  rayname=new char[6];
  strcpy(rayname,"P670S");
}

void P670S_wave::init(class DeckPrem &DPrem)
{
real pmaxs= DPrem.u[S][DPrem.i670];	// pmax for S
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];	// for P part
  pmax = DPrem.u[P][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  pmax  = min(pmaxs,pmax);
  
  rend[0]=rbeg[1]=DPrem.lmbot;
  rend[1]=DPrem.lmtop;  
  rbeg[2]=DPrem.umbot;
  tlay[P]=DPrem.icmb;			// deepest layer a P670S ray (P leg) can go
  tlay[S]=DPrem.i670;			// deepest layer a P670S ray (S leg) can go
  tr[P]  =DPrem.rr[tlay[P]];
  tr[S]  =DPrem.rr[tlay[S]];
}

S670P_wave::S670P_wave()
{
  numseg=3;  
  dir=new int[numseg];
  dir[0]=DOWN;
  dir[1]=dir[2]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[1]=S;
  pors[2]=P;
  
  rayname=new char[6];
  strcpy(rayname,"S670P");
}

void S670P_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];

  pmaxp= DPrem.u[P][DPrem.i670];	// pmax for P
  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];
  
   
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);		// need to adjust pmax

  rend[0]=rbeg[1]=DPrem.lmbot;
  rend[1]=DPrem.lmtop;  
  rbeg[2]=DPrem.umbot;
  
  tlay[P]=DPrem.i670;			// deepest layer a S670P ray (P leg) can go
  tlay[S]=DPrem.icmb;			// deepest layer a S670P ray (S leg) can go  
  tr[P]  =DPrem.rr[tlay[P]];
  tr[S]  =DPrem.rr[tlay[S]];
}

P400P_wave::P400P_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
  
  rayname=new char[6];
  strcpy(rayname,"P400P");
}

void P400P_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i400-1];
  if(eqlay<DPrem.i400)pmax=min(pmax,p1);	// need to adjust pmax

  rend[1]=rbeg[2]=DPrem.tztop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;
  tlay[P]=DPrem.icmb;
  tr[P]  =DPrem.rr[tlay[P]];
}


P670P_wave::P670P_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=P;
  
  rayname=new char[6];
  strcpy(rayname,"P670P");
}

void P670P_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[P];

  pmin = DPrem.u[P][DPrem.icmb];
  pmax = DPrem.u[P][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  rend[1]=rbeg[2]=DPrem.lmtop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;
  tlay[P]=DPrem.icmb;
  tr[P]  =DPrem.rr[tlay[P]];
}

S400S_wave::S400S_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=S;
  
  rayname=new char[6];
  strcpy(rayname,"S400S");
}

void S400S_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i400-1];
  if(eqlay<DPrem.i400)pmax=min(pmax,p1);	// need to adjust pmax
  rend[1]=rbeg[2]=DPrem.tztop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;
  tlay[S]=DPrem.icmb;				// deepest layer an SS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
}

S670S_wave::S670S_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[2]=DOWN;
  dir[1]=dir[3]=UP;
  
  pors= new int[numseg];
  for(int i=0;i<numseg;i++)pors[i]=S;
  
  rayname=new char[6];
  strcpy(rayname,"S670S");
}

void S670S_wave::init(class DeckPrem &DPrem)
{
real p1=eqr/eqv[S];

  pmin = DPrem.u[S][DPrem.icmb];
  pmax = DPrem.u[S][DPrem.i670-1];
  if(eqlay<DPrem.i670)pmax=min(pmax,p1);	// need to adjust pmax

  rend[1]=rbeg[2]=DPrem.lmtop;
  rend[0]=rbeg[1]=rend[2]=rbeg[3]=DPrem.lmbot;
  tlay[S]=DPrem.icmb;				// deepest layer an SS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
}

SKS_wave::SKS_wave()
{
  numseg=4;  
  dir=new int[numseg];
  dir[0]=dir[1]=DOWN;
  dir[2]=dir[3]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[3]=S;
  pors[1]=pors[2]=P;
  
  rayname=new char[4];
  strcpy(rayname,"SKS");
}



void SKS_wave::init(class DeckPrem &DPrem)
{

  pmin = DPrem.u[P][DPrem.iiob];
  pmax = DPrem.u[P][DPrem.icmb-1];
  
// begin and end of each segment  
  rend[0]=rbeg[3]=DPrem.lmbot;
  rbeg[1]=rend[2]=DPrem.octop;
  rend[1]=rbeg[2]=DPrem.ocbot;

  tlay[S]=DPrem.icmb;			// deepest layer an SKS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
  
  tlay[P]=DPrem.iiob;			// deepest layer of P part can go
  tr[P]  =DPrem.rr[tlay[P]];
}


SKKS_wave::SKKS_wave()
{
  numseg=6;  
  dir=new int[numseg];
  dir[0]=dir[1]=dir[3]=DOWN;
  dir[2]=dir[4]=dir[5]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[5]=S;
  pors[1]=pors[2]=pors[3]=pors[4]=P;
  
  rayname=new char[5];
  strcpy(rayname,"SKKS");
}

void SKKS_wave::init(class DeckPrem &DPrem)
{

  pmin = DPrem.u[P][DPrem.iiob];
  pmax = DPrem.u[P][DPrem.icmb-1];
  
// begin and end of each segment  
  rend[0]=rbeg[5]=DPrem.lmbot;
  rbeg[1]=rend[2]=rbeg[3]=rend[4]=DPrem.octop;
  rend[1]=rbeg[2]=rend[3]=rbeg[4]=DPrem.ocbot;

  tlay[S]=DPrem.icmb;			// deepest layer an SKKS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
  
  tlay[P]=DPrem.iiob;			// deepest layer of P part can go
  tr[P]  =DPrem.rr[tlay[P]];
}

SKKKS_wave::SKKKS_wave()
{
  numseg=8;  
  dir=new int[numseg];
  dir[0]=dir[1]=dir[3]=dir[5]=DOWN;
  dir[2]=dir[4]=dir[6]=dir[7]=UP;
  
  pors= new int[numseg];
  pors[0]=pors[7]=S;
  for(int i=2;i<=6;i++)pors[i]=P;
  
  rayname=new char[6];
  strcpy(rayname,"SKKKS");
}

void SKKKS_wave::init(class DeckPrem &DPrem)
{

  pmin = DPrem.u[P][DPrem.iiob];
  pmax = DPrem.u[P][DPrem.icmb-1];
  
// begin and end of each segment  
  rend[0]=rbeg[7]=DPrem.lmbot;
  rbeg[1]=rend[2]=rbeg[3]=rend[4]=rbeg[5]=rend[6]=DPrem.octop;
  rend[1]=rbeg[2]=rend[3]=rbeg[4]=rend[5]=rbeg[6]=DPrem.ocbot;

  tlay[S]=DPrem.icmb;			// deepest layer an SKKS ray can go
  tr[S]  =DPrem.rr[tlay[S]];
  
  tlay[P]=DPrem.iiob;			// deepest layer of P part can go
  tr[P]  =DPrem.rr[tlay[P]];
}



