#ifndef WSMATHEADER

#define WSMATHEADER
#define UNISTD			// there is unix stand lib

#include <stdio.h>
#include <stdlib.h>
#include <wsconst.h>
#include <wsmacro.h>

const int   GEN_MAT=0;		// matrices in general, not related to any model
const int   BSP_R_MAT=1;	// 3D tvl time B-spline model, rknots described as radius
const int   BSP_R_MAT_NEW=11;	// 3D tvl time B-spline model, NEW FORMAT
const int   BSP_D_MAT=2;	// as above, but rknots described as depth
const int   SPH_B_MAT=3;	// spherical harmonic model, radial func described by Bspline
const int   SPH_C_MAT=4;	// as above, but radial func described by Chebyshev polynomial
const int   BSP_R_COM_MAT=5;   	// 2d or 3d B-spline, joint models (high order regional + lower order
const int   BSP_R_TOPO_MAT=6;  	// 2d topography + 3D B-spline, knots described as radius
const int   BSP_R_TOPO_410=61; 	// 2d 410 topo + 3D b-spline, knots described as radius
const int   BSP_R_TOPO_660=62; 	// 2d 670 topo + 3d b-spline, knots described as radius
const int   BSP_R_TOPO_ALL=63; 	// 3D b-spline + 410 + 670, radius as knots
				

class AMAT_ROW{
public:
	~AMAT_ROW();
	void Init(int na=0);
	void OpenMatFile(char *);
	void CloseMatFile(){/* close(fd); */}
	void ReduceARow(int *);
	void SubARow(AMAT_ROW&);
	float SumArow(float *);
	float SumArow_dcshift(float, float, int, int);
	float* GetArow()const { return arow;}
	short* GetIrow()const { return irow;}
protected:
	float *arow;	// a row of A matrix
	short *irow;	// index of element in A
	int numnon0;	// number of nonzero elements	
	int	numa;	// number of columns
	int	fd;	// input A matrix row file
	int	fd_out;	// input A matrix row file
	int	file_opened;
};

class BS_AMAT_ROW: public AMAT_ROW{ 		// B-spline A matrix
public:
	BS_AMAT_ROW(int na=0);
	void RdAmatModPara(int&, int&);
        void RdAmatModPara(int&, int &, int&);
	int ReadARow();
	void SaveARow(int, float);
	void SaveARow_Scripts(int, float, float);
	float GetObsRes() const { return res1;}
	float GetPreRes() const { return res2;}
	float GetPremRes() const { return time;}
	float GetEqlat() const{ return eqlat;}
	float GetEqlon() const{ return eqlon;}
	float GetStlat() const{ return stlat;}
	float GetStlon() const{ return stlon;}
	
	int CompARow(BS_AMAT_ROW&);
	int NumOfNonzeroElm() const { return numnon0;}
protected:
	int numgpt;		// number of horizontal grid points of corresponding model
	int numknt;		// # of radial knots
	int neq;
        int ist;	
        float eqlat;
        float eqlon;
        float eqdep;
        float stlat;
	float stlon;
        float stelev;
        float res1;		// observed residual
        float res2;		// predicted residual
        float time;		// PREM predicted travel time
        int amatopt;             // by default -99
};


class IPM{			// inner production matrix
public:
   float atamin,atamax,atavg,atavar;	  // matrix statistics
   float atdmin,atdmax,atdvg,atdvar;
   float atamin_2d, atamax_2d, atavg_2d;  // 2d topography stats
   float atdmin_2d, atdmax_2d, atdvg_2d;
   float atavar_2d, atdvar_2d;
   IPM() {ata = 0; atd = 0; atb = 0; btb = 0;}
   int  GetNumatd() const { return numatd;}
   int  GetNumatd_save() const { return numatd_save;}
   int  GetNsurf()  const { return nsurf;}
   int  GetNumata() const {return numata;}
   int  GetFormat() const {return format;}
   void ReadAtaEtc(int &);		// read ATA matrix etc, never call this func
   					// directory, call ReadIpm through derived class
   void AddAtaEtc(IPM&, float&);
   void AddAnisoAta(IPM&, int, int, int, int);
   void ModIpm();			// not complete yet
   void WriteAtaEtc(int&);		// not complete yet
   void PrintAtaEtc();
   void SubInitModel(float *, int, int, int);
   void SubInitModel_resolution(float *, int, int, int);
   void MultAX(float *, float *, int, float *);
   void FlipAta(int,int);
   void DelAtaEtc();			// free allocated memory. Do not call this directly, 
   					// call through destructor of the derived class
   void  Stats(int);
   void  PrintStats(int);
   void  PrintStats_topo(int);
   int  Choles(float*);			// cholesky inversion
   void DampDiag(float&, float *,int *, int);// damping diagonal element of ata
   void DampDiag_Block(float&,float *, int *, int, int, float *,int *,int);// damping diagonal for split model
   void DampRadialAta(float*,float, int);// add radial damping matrix
   void DampHorizAta(float *,float&,float*,int*, int,int); // horizontal damping matrix
   void DampHBDAta(float *, int,int,float *, float *); 	// horizontal damping matrix
   void CKBlockDiag_topo(int,int,float*); // check block diagonal of ATA matrix
   void CKDiag_topo(int,int,float*); 	// check regular diagonal of ATA matrix
   void RedAtaAtd(int*, float&);	// reduce matrices
//
// the following member func is defined in /scratch/troika23/gu/cpp/src_damping
// by convert_atatype.cc.  This is ugly, but ...  Jeff Gu, 1999
//
   
   void ChangeFmt();			// change the format option  *** NOTE:  TEMPORARY FIX!!!
//
//
protected:
   int  format;				// format of ipm 
   int  numatd,numata,numatd_save;
   int  numev;				// number of event
   int  numsor;				// number of source parameters
   int  nsurf;                          // number of 2D b-spline
   float *ata,*atd,*atb,*btb;
};

class GEN_IPM: public IPM{ 			// inner production matrix
public:
   int  ReadIpm();		// allocate space and read related information
   void  PrintIpm();
   void WriteIpm();
   ~GEN_IPM();
};


class BS_IPM: public IPM{ 	// inner production matrix
public:
   int  ReadIpm();		// allocate space and read related information
   int  ReadIpm_Topo(int);     	// allocate space and read related information
   void ReadAtaEtc_Topo(int &,int);  // reading for topography
   void CheckAtaDiag();
   void WeightAta();
   void ScaleAta();
   void PrintIpm();
   void AddIpm(BS_IPM&, float&);
   void AddAnisoIpm(BS_IPM&, int, int, int, int);
   void WriteIpm_topo(int,int);
//
// the following member func is defined in /scratch/troika23/gu/cpp/src_damping
// by convert_atatype.cc.  This writes out an ATA matrix in the format
// readable by calc_3ddamp.cc, Jeff Gu, 1999.
//
   void ReadFull3DDamp();
   void WriteIpm_Weijia(int,int);
   void DampFull3D(BS_IPM&, float);
//
// 
   void Stats_topo();
   void Replace_Info(int);
   BS_IPM();
   ~BS_IPM();
private:
   int  numgpt;		// number of points for spherical B-splines
   int  numrknt;	// number of radial B-splines
   float *rknt;		// radial knots
   float *theta,*phi;	// grid points for spherical B-splines
};

class BS_COM_IPM: public IPM{ 	// combined regional-whole earth ipm
public:
   int  ReadIpm();		// allocate space and read related information
   void PrintIpm();
   void AddIpm(BS_COM_IPM&, float&);
   void WriteIpm();
   int*  GetOverlapInd() const { return index;}
   int   GetNumOverlapPts() const { return numovl;}
   BS_COM_IPM();
   ~BS_COM_IPM();
   void  StatsPart();
   void DampRadialAta(float*, float, float, int);
   void DampRadialAta(float*, float*, float, float, int, int);
   void DampDiag(float&, float&, int);
private:
   int  numgpt_we;	// number of points for spherical B-splines (whole earth)
   int  numrknt_we;	// number of radial B-splines (whole earth)
   float *rknt_we;	// radial knots (whole earth)
   float *theta_we,*phi_we;	// grid points for spherical B-splines (whole earth)
   int  numgpt_re;	// number of points for spherical B-splines (regional)
   int  numrknt_re;	// number of radial B-splines (regional)
   float *rknt_re;	// radial knots (whole earth)
   float *theta_re,*phi_re;	// grid points for spherical B-splines (whole earth)
   int  *index;		// index of these overlapped grids
   int  numovl;		// number of overlapped points
};



int SysOpen(char *,int);


class Geographic{
public:
	float lat,lon;
};

class Geocentric{
public:
	void FromSph2(class Spherical2&);
	void FromSph(class Spherical&);
	float lat,lon;
};

class Cartesian{
public:
	void Normalize();
	void FromSph2(class Spherical2&);
	void FromSph(class Spherical&);
	Cartesian OutProdWith(Cartesian&);
	float x,y,z;
};

class Cartesian2{
public:
	float x,y;
};
class Spherical{
public:
	Spherical ForwardTrans(float *);
	Spherical InverseTrans(float *);
	void FromGeographic(class Geographic&);
	void FromGeocentric(class Geocentric&);
	float r,theta,phi;
};


class Spherical2{
public:
	Spherical2 ForwardTrans(float *);
	Spherical2 InverseTrans(float *);
	void FromGeographic(class Geographic&);
	void FromGeocentric(class Geocentric&);
	float theta,phi;
};

class TransMatrix{
public:
	void SetFromVec(class Cartesian&, class Cartesian&, class Cartesian&);
	float matrix[9];
};

class TwoPtsSph{		// two points on a sphere   
public:
	float CalDist();
	void SetCoordinate(const int&);
	void SetCoordinate(Spherical2&, Spherical2&);
	float GetDist() const { return dist;}
	Spherical2 GetSph(const int &pt) const { return (pt==1)?sph1:sph2;}
	void CalTrans4();
	TransMatrix GetTrans() const { return trans;}
	void FwdTrans_sph2s(const int &pt);	// performat forward trans on point 1 or 2
	void FwdTrans_sph(const int &pt);	// trans sph1, sph2 and save in place
	void InvTrans_s2sph(const int &pt);	// performat inverse trans on point 1 or 2
	void InvTrans_s2s(const int &pt);	// performat inverse trans on point 1 or 2
	Spherical2 s1,s2;
	TransMatrix trans;		// transformation matrix
private:
	Geographic geo1,geo2;	// geographic location of points 1 and 2
	Spherical2 sph1,sph2;	// spherical coordinates
	Cartesian cart1,cart2;	// in Cartesian coordinates
	float dist;		// arc distance (in radian) between two points

};

class POLARMAPELM {			// polar map element
public:
	void SetPoleTransMat(class Spherical2&, class Spherical2&);
	void SetPoleTransMat(class Spherical2&, float &);
	void PrintLine();
	Cartesian2* GetLinePt(int&);
	void Frame(float &);
	void MapCloseLine(Spherical2*, int&, float&);
	void MapOpenLine(Spherical2*, int&);
	void CalMeridianLine(float&, float&);
	void CalParallelLine(float&, float&);
	void ReleaseMem();
	void PathBt2Pts(TwoPtsSph*, int, float);
	void SetMapRange(float &range_in_deg) { Range=range_in_deg*RAD;}
	void GetPoleCoor(float *x,float *y, float *z)
	       {*x=PoleCart.x; *y=PoleCart.y; *z=PoleCart.z;}
	Cartesian2 *Geo2UserCoor();
	int  GetNumLine() const { return numline;}
	int  GetNumPt(int &i) const { return numpt[i];}
	POLARMAPELM();
	~POLARMAPELM();
private:
	int		numline;	// total number of lines
	int		tnumpt;		// total number of points
	int		*numpt;		// number of points for each line
	float		Azim;
	float		Range;		// map range ( in radian );
	Spherical2	Pole;		// pole location
	Cartesian	PoleCart;	// pole in Cartesian coordinate
	TwoPtsSph	P12;		// two points define the pole
	Spherical2	*line;
	Cartesian2	*xy;
	float asc,bsc,csc,dsc;	// screen mapping factors
	float aps,bps,cps,dps;	// postscript mapping factors
};

class LatLatLine{		// a line described by (lat, lon)
friend class POLARMAPELM;
public:
	int ReadData(char *ch);
	Spherical2 *GetLinePt(int &linenum);
	int GetNumPt(int &i) const { return numpt[i];}
	int GetNumLine() const { return numline;}
	~LatLatLine();
private:
	int	numline;	// total number of lines
	int	tnumpt;		// total number of points
	int	*numpt;		// number of points for each line
	Spherical2 *line;	// in spherical coordinate
};

const int SPHERICAL=1;
const int GEOGRAPHICAL=2;
const int LANDSCAPE=1;
const int PORTRAIT=2;

int equation2(float*, float, float, float);
int SysOpen(char*, int);
float *ReadDamMat(int);
void Read3DDamMat(int &, int *inddamp, float *dmat);
float *Read_H_DamMat(int);
float *Read_H_DamMat_New(int);

#endif
