#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>

#ifndef MODHEADER
#include "wsmodel.h"
#endif

void GetMenu(int argc, char *argv[], int &ifregional, char &modeltype, char *flname,int &invopt)
{
// This checks the menu options

      void PrintMenu();
      int i, j=0;
      char ch;
      char model;

      model = 'W';
      modeltype = 'C';
      ifregional = WHOLE_EARTH;

      cout << "-----------------Chosen Options-----------------"<< endl;
      if(argc > 1) { //  options
         for(i=1; i< argc;i++) {
                if(argv[i][0] == 'W' || argv[i][0] == 'w') {
                        model = 'W';
                        cout << "  Global Model";
                } else if(argv[i][0] == 'R' || argv[i][0] == 'r'){            
                        model = 'R';
                        cout << "  Regional Model";
			ifregional = REGIONAL;
                } else if(argv[i][0] == 'C' || argv[i][0] == 'c'){
                        modeltype = 'C';
                        cout << "  Continuous";
                } else if(argv[i][0] == 'S' || argv[i][0] == 's'){
                        modeltype = 'S';
                        cout << "  Split";
                } else if(argv[i][0] == 'M' || argv[i][0] == 'm'){
                        modeltype = 'M';
                        cout << "  Multiple Split";
                } else {
                        cout << "option(s) not found !!" << endl << endl;
                        PrintMenu();
                }
         }
         cout << endl;
      } else {
         cout << "Global Model " << "  Continuous"  <<endl;
      }
      cout << "------------------------------------------------"<< endl;
      cout<<"file name for the earth model:\n";
      cin >>flname;
      cout << "invert the following: " << endl;
      cout << "1. 3D earth structure using OLD ISC FORMAT" << endl;
      cout << "2. 3D earth structure using NEW ISC FORMAT" << endl;
      cout << "3. 3D structure with ONE 2D layer" << endl;
      cout << "4. 3D structure with TWO 2D layers" << endl;
      cout << "enter: " << endl;
      cin >> invopt;
      assert (invopt < 5);
} 

void PrintMenu()
{
      cout << endl<<endl;
      cout << "Options: " << endl;
      cout << "-------------------------------------------------------------------------" << endl;
      cout << "W             --Global model (default)" << endl;
      cout << "R             --Regional model" << endl;
      cout << "C             --Continuous model across 670 (default)" << endl;
      cout << "S             --Split model across 670" << endl;
      cout << "M             --General Split model at any depth(s)" << endl;
      cout << "-------------------------------------------------------------------------" << endl;
      cout << endl << endl;
      exit(-1);
}                                                                              

SP_MODEL::~SP_MODEL()
{
	cout<<".......delete SP_MODEL......\n";
	delete [] coeff;
	if(lcrust!=-1)delete [] coeff_crust;
	if(l670!=-1)delete [] coeff_670;
	delete [] degmask;
	delete [] laymask;
	delete [] coeff_r;
	delete [] dknt;
	delete [] rknt;
}

void MODEL::PrintModInfor()
{
   if(format==HVDHARM)
	  cout<<"Harvard format spherical harmonic model\n";
   else if(format==HVDBS)
	  cout<<"Harvard format B-spline model\n";
   else if(format==CHARM)
        cout<<"Complex spherical harmonics model\n";
   
   cout<<"model dimension = "<<dim<<endl;

   if(rfun!=NORADFUN){
      cout<<"radial function=";
      if(rfun==LEGENDRE)cout<<"LEGENDRE\n";
      else if(rfun==CHEBYSHEV)cout<<"CHEBYSHEV\n";
      else if(rfun==SHELL)cout<<"SHELL\n";
      else if(rfun==BSPLINE)cout<<"B-SPLINE\n";

      cout<<"rtop="<<rtop<<endl;
      if(region==UPLOWMANTLE)cout<<"rmid="<<rmid<<endl;
      cout<<"rbot="<<rbot<<endl;
   }
}

void SP_MODEL::PrintModel()
{
int k,ind;
   PrintModInfor(); 
   cout<<"degree mask="<<degmask<<endl;
   cout<<"layer mask="<<laymask<<endl;
   if(region==WHOLEMANTLE){
	  cout<<"whole mantle model\n";
	  cout<<"maxk="<<maxwk<<endl;
	  cout<<"maxl="<<maxwl<<endl;
	  for(ind=k=0;k<=maxwk;k++){
		  cout<<"k="<<k<<" ";
		  PrintModCoeff(&coeff[ind],maxwl);
		  ind+=NumCoeff(maxwl);
	  }
   }else if(region==UPLOWMANTLE){
	  cout<<"split upper/lower mantle model\n";
	  cout<<"maxk (upper mantle)="<<maxuk<<endl;
	  cout<<"maxl (upper mantle)="<<maxul<<endl;
	  for(ind=k=0;k<=maxuk;k++){
		  cout<<"k="<<k<<" ";
		  PrintModCoeff(&coeff[ind],maxul);
		  ind+=NumCoeff(maxul);
	  }
	  cout<<"maxk (lower mantle)="<<maxlk<<endl;
	  cout<<"maxl (lower mantle)="<<maxll<<endl;
	  for(ind=k=0;k<=maxlk;k++){
		  cout<<"k="<<k<<" ";
		  PrintModCoeff(&coeff[ind],maxll);
		  ind+=NumCoeff(maxll);
	  }
   }else if(region==UPPERMANTLE){
	  cout<<"upper mantle model\n";
	  cout<<"maxk="<<maxuk<<endl;
	  cout<<"maxl="<<maxul<<endl;
	  for(ind=k=0;k<=maxuk;k++){
		  cout<<"k="<<k<<" ";
		  PrintModCoeff(&coeff[ind],maxul);
		  ind+=NumCoeff(maxul);
	  }
   }else if(region==LOWERMANTLE){
	  cout<<"lower mantle model\n";
	  cout<<"maxk="<<maxlk<<endl;
	  cout<<"maxl="<<maxll<<endl;
	  for(ind=k=0;k<=maxlk;k++){
		  cout<<"k="<<k<<" ";
		  PrintModCoeff(&coeff[ind],maxll);
		  ind+=NumCoeff(maxll);
	  }
   }

   if(lcrust!=-1){
	  cout<<"model contains a crust layer\n";
	  cout<<"max degree for crust layer="<<lcrust<<endl;
	  PrintModCoeff(coeff_crust, lcrust);
   }
   if(l670!=-1){
	  cout<<"model contains a 670 layer\n";
	  cout<<"max degree for 670 ="<<l670<<endl;
      PrintModCoeff(coeff_670, l670);
   }

}

void SP_MODEL::PrintModCoeff(float *c, int &deg)
{
  cout<<"c[0]="<<c[0]<<"  c["<<NumCoeff(deg)-1<<"]="<<
         c[NumCoeff(deg)-1]<<endl;
}


void BS_MODEL::ReadModel(int choice, char *flname)
{

fstream inFile;
int i,j,k,m;
float dum,lat,lon;

 format=HVDBS;			// Harvard B-spline model
 hfun=rfun=BSPLINE;		// bspline for radial and horizontal
 HRegion=choice;
 cout<<"file name="<<flname<<endl;
 inFile.open(flname,ios::in);	// open bspline model for read
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 inFile >> numgpt >> numknt;
 if(numknt>1){			// 3-D B-spline model
	dim=3;
 }else{				// 2-D B-spline model
	dim=2;
	cout<<"Error in BS_MODEL::ReadModel, not complete yet\n";
	exit(-1);
 }
 InitKnots(choice);		// allocate space
 
 for(m=i=0;i<numgpt;i++){
   inFile >> j >> lat >> lon;
   assert((i+1)==j);
   theta[i]=((float)90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=(float)360.0;
   phi[i] *= RAD;
   if(choice==WHOLE_EARTH){
     for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
     }
   }
 }
 
 for(k=i=0;i<numknt;i++){
     inFile >> m >>rknt[i];
     assert((i+1)==m);
     for(j=0;j<numgpt;j++){
        inFile >> dum;
        coeff[k++]=dum;
       
     }
 } 
 
 inFile.close();
 if(choice==WHOLE_EARTH)CalAvgDist();
 
 if(rknt[0]<RADIUS-DEPCMB+1.0 && rknt[numknt-1]>RADIUS-DEPMOHO-1.0){
    region=WHOLEMANTLE;
    rtop=rknt[numknt-1];
    rbot=rknt[0];
 }else {
    region=WHOLEMANTLE;		// pretend to be a whole mantle model
    rtop=rknt[numknt-1];
    rbot=rknt[0];
//    cerr << " Error 3, not implemented yet\n";
//    exit(-1);
 }
}

void BS_MODEL::ReadModel_split(int choice, char *flname)
{

fstream inFile;
int i,j,k,m;
float dum,lat,lon;
char mod_desc[50];

 format=HVDBS;			// Harvard B-spline model
 hfun=rfun=BSPLINE;		// bspline for radial and horizontal
 HRegion=choice;
 cout<<"file name="<<flname<<endl;
 inFile.open(flname,ios::in);	// open bspline model for read
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 inFile >> mod_desc >> numgpt >> numknt_lm >> numknt_um;
 numknt = numknt_lm+numknt_um;
 
 if(numknt>1){			// 3-D B-spline model
	dim=3;
 }else{				// 2-D B-spline model
	dim=2;
	cout<<"Error in BS_MODEL::ReadModel, not complete yet\n";
	exit(-1);
 }
 InitKnots(choice);		// allocate space
 
 for(m=i=0;i<numgpt;i++){
   inFile >> j >> lat >> lon;
   assert((i+1)==j);
   theta[i]=((float)90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=(float)360.0;
   phi[i] *= RAD;
   if(choice==WHOLE_EARTH){
     for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
     }
   }
 }
 
 for(k=i=0;i<numknt;i++){
     inFile >> m >>rknt[i];
     assert((i+1)==m);
     for(j=0;j<numgpt;j++){
        inFile >> dum;
        coeff[k++]=dum;
       
     }
 } 
 
 inFile.close();
 if(choice==WHOLE_EARTH)CalAvgDist();
 
 if(rknt[0]<RADIUS-DEPCMB+1.0 && rknt[numknt-1]>RADIUS-DEPMOHO-1.0){
    region=WHOLEMANTLE;
    rtop=rknt[numknt-1];
    rbot=rknt[0];
 }else {
    region=WHOLEMANTLE;		// pretend to be a whole mantle model
    rtop=rknt[numknt-1];
    rbot=rknt[0];
//    cerr << " Error 3, not implemented yet\n";
//    exit(-1);
 }
}

void BS_MODEL::ReadModel_multisplit(int choice, char *flname)
{  // read models with multiple spliting layers
fstream inFile;
int i,j,k,m, nsplit;
float dum,lat,lon;

 format=HVDBS;			// Harvard B-spline model
 hfun=rfun=BSPLINE;		// bspline for radial and horizontal
 HRegion=choice;
 cout<<"file name="<<flname<<endl;
 nbsperlayer = new int[10];  	// number of b-spline knots in each layer
 inFile.open(flname,ios::in);	// open bspline model for read
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 Read1stLine(inFile, choice);
 nsplit=GetNumSplit();
 numknt=0;
 for(i=0;i<nsplit;i++){
 	numknt = numknt+nbsperlayer[i];
 }
 cout << "numknt =" << numknt << endl;
 
 if(numknt>1){			// 3-D B-spline model
	dim=3;
 }else{				// 2-D B-spline model
	dim=2;
	cout<<"Error in BS_MODEL::ReadModel, not complete yet\n";
	exit(-1);
 }
 InitKnots(choice);		// allocate space
 
 for(m=i=0;i<numgpt;i++){
   inFile >> j >> lat >> lon;
   assert((i+1)==j);
   theta[i]=((float)90.0-lat)*RAD;
   phi[i] = lon;
   if(phi[i]<0.0)phi[i]+=(float)360.0;
   phi[i] *= RAD;
   if(choice==WHOLE_EARTH){
     for(j=0;j<7;j++){
       inFile >> k;
       if(j!=0)ajknts[m++]=k-1;
       else ajknts[m++]=k;
     }
   }
 }
 
 for(k=i=0;i<numknt;i++){
     inFile >> m >>rknt[i];
     assert((i+1)==m);
     for(j=0;j<numgpt;j++){
        inFile >> dum;
        coeff[k++]=dum;
       
     }
 } 
 
 inFile.close();
 if(choice==WHOLE_EARTH)CalAvgDist();
 
 if(rknt[0]<RADIUS-DEPCMB+1.0 && rknt[numknt-1]>RADIUS-DEPMOHO-1.0){
    region=WHOLEMANTLE;
    rtop=rknt[numknt-1];
    rbot=rknt[0];
 }else {
    region=WHOLEMANTLE;		// pretend to be a whole mantle model
    rtop=rknt[numknt-1];
    rbot=rknt[0];
//    cerr << " Error 3, not implemented yet\n";
//    exit(-1);
 }
}

void BS_MODEL::Read1stLine(fstream &inFile, int choice)
{
 int i;
 int nsplit; 
 char mod_desc[50];

	inFile >> mod_desc  >> nsplit_lm >> nsplit_um
		>> numgpt;
	nsplit=nsplit_um+nsplit_lm;
	if(nsplit>9) cout << "too many splits, not implemented!" <<endl;
	for(i=0;i<nsplit;i++){
		inFile >> nbsperlayer[i];
		cout << "i=" << i << "  layer =" <<  nbsperlayer[i] << endl;
	}
}



void SP_MODEL::ReadModel(int choice, char *flname)
{
fstream inFile;
int i,k,numstr,maxk,maxdeg,numcoeff,numc;

cout<<"filename = "<<flname<<endl;

 inFile.open(flname,ios::in);	// open spherical harmonic model for read
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 inFile >>maxdeg;		// max spherical harmonic degree
 degmask=new char[maxdeg+2];
 inFile >>degmask;
 inFile >> numstr;		// max # of radial structures
 laymask=new char[numstr+1];
 inFile >>laymask>>rfun;
 if(rfun==SHELL || rfun==BSPLINE){
	dknt=new float[numstr+1];
	rknt=new float[numstr+1];
 }else{
	dknt=new float[2];	// for later use
	rknt=new float[2];
 }

 maxk=numstr-3;		// subtract crust, 670 and 1 (k=0,maxk)
 if(maxk<0 || maxdeg<0){
	cerr<<"Error in SP_MODEL::ReadModel\n";
	cerr<<"Invalid harmonic degrees or radial order\n";
	exit(-1);
 }
 if(maxk>1)dim=3;
 else dim=2;

 numc=NumCoeff(maxdeg);
 numcoeff=numc*(maxk+1);
 coeff=new float[numcoeff];
 coeff_r=new float[numc];	// for later use

 if(rfun<=NORADFUN || rfun>BSPLINE){
	cerr<<"Error in SP_MODEL::ReadModel\n";
	cerr<<"Unknown radial function\n";
	exit(-1);
 }

 lcrust=l670=maxul=maxll=maxwl=maxuk=maxlk=maxwk=-1;

 if(laymask[0]!='0'){
	lcrust=maxdeg;		// there is a crust layer
	numc=NumCoeff(lcrust);
	coeff_crust= new float[numc];
	for(i=0;i<numc;i++)inFile >> coeff_crust[i];
 }
 if(laymask[1]!='0'){
	l670=maxdeg;		// there is a 670 layer
	numc=NumCoeff(l670);
	coeff_670= new float[numc];
	for(i=0;i<numc;i++)inFile >> coeff_670[i];
 }
 for(k=0;k<=maxk;k++)laymask[k]=laymask[k+2];	// exclude crust & 670
 laymask[maxk+1]='\0';
 
 format=HVDHARM;	// Harvard format model
 hfun=HARMONICS;	// spherical harmonics as horizontal func

 // whole mantle model should have mask  of all 1's
 // upper+lowe mantle modle should have mask of 1's for um part
 //			and 2's for lm part such as 111112222222
 // upper mantle only model should have mask of all 3's
 // lower mantle only model should have mask of all 4's
 
 for(k=0;k<=maxk;k++){
	 if(laymask[k]=='1')maxwk++;
	 else if(laymask[k]=='2' || laymask[k]=='4')maxlk++;
	 else if(laymask[k]=='3')maxuk++;
 }

 rmid=(float)0.0;
 if(maxwk!=-1){			// whole mantle or upper+lower mantle model
	rtop=RMOHO;
	rbot=RCMB;
	if(maxlk!=-1){		// upper+lower mantle model
	   maxuk=maxwk;
	   maxwk=-1;
	   maxul=maxll=maxdeg;
	   rmid=R670;
	   region=UPLOWMANTLE;
	}else{
	   maxwl=maxdeg;
	   region=WHOLEMANTLE;
	}
 }else{					// upper mantle only or lower mantle only
	if(maxuk!=-1 && maxlk==-1){			// upper mantle
	   maxul=maxdeg;
	   rtop=RMOHO;
	   rbot=R670;
	   region=UPPERMANTLE;
	}else if(maxlk!=-1 && maxuk==-1){	// lower mantle
	   maxll=maxdeg;
	   rtop=R670;
	   rbot=RCMB;
	   region=LOWERMANTLE;
	}else{								// wrong
	   cerr<<"Error in SP_MODEL::ReadModel\n";
	   cerr<<"Invalid radial strcture mask.\n";
	   exit(-1);
	}
 }

 for(i=0;i<numcoeff;i++)inFile >> coeff[i];
 
 inFile.close();

}

void BS_MODEL::ReadTopoModel(int choice, char *flname, float *topocoef)
{  // read topography model

fstream inFile;
int i,j,k,m, npt, nlayer;
float dum,lat,lon;

 format=HVDBS;		// Harvard B-spline model
 hfun=rfun=BSPLINE;		// bspline for radial and horizontal
 HRegion=choice;
 cout<<"file name="<<flname<<endl;
 inFile.open(flname,ios::in);	// open bspline model for read
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 inFile >> npt >> nlayer;
 for(m=i=0;i<numgpt;i++){
   inFile >> j >> lat >> lon;
   if(choice==WHOLE_EARTH){
     for(j=0;j<7;j++){
       inFile >> k;
     }
   }
 }
 
 for(i=0; i< numgpt;i++) { // initialize for 2D b-spline
     topocoef[i] = 0.0;
 }

 for(j=0;j<numgpt;j++){
        inFile >> dum;
        topocoef[j]=dum;
 //       cout << "j=" << j << "   topocoef=" << topocoef[j] << endl;
 }
 inFile.close();
}

int BS_MODEL::ExcludeRegMod(BS_MODEL &regmod, int *ind)
{
int i,j,m,n,numoverlap,*index;
float tt,pp;

  index=new int[numgpt];

  for(j=0;j<numgpt;j++)index[j]=TRUE;


  if(HRegion!=WHOLE_EARTH || regmod.HRegion !=REGIONAL){
    cout <<"Error in BS_MODEL::ExcludeRegMod\n";
    cout <<"model 1 is not WHOLE_EARTH or\n";
    cout <<"model 2 is not REGIONAL\n";
    exit(1);
  }
  cout <<"whole earth model: numgpt="<<numgpt<<endl;
  cout <<"regional    model: numgpt="<<regmod.numgpt<<endl;
  cout <<"exclude the common grid points from whole Earth model\n";
  
   for(m=i=0;i<numgpt;i++){
     tt=theta[i];
     pp=phi[i];
     for(j=0;j<regmod.numgpt;j++){
         if(fabs(regmod.theta[j]-tt)<0.001 &&
            fabs(regmod.phi[j]-pp)<0.001){
            m++;
            index[i]=FALSE;
            cout<<m<<" found i="<<i<<" j ="<<j<<endl;
            break;
         } 
     }
   }
   
   for(numoverlap=m=i=0;i<numknt;i++){
     for(j=0;j<numgpt;j++,m++){
        if(i<regmod.numknt){
           ind[m]=index[j];
           if(!index[j])numoverlap++;
        }else{
           ind[m]=TRUE;
        }
     }
   }

   for(n=m=i=0;i<numknt;i++){
     for(j=0;j<numgpt;j++,m++){
        if(!ind[m])
           ind[m]=-1;
        else
           ind[m]=n++;
     }
   }
/*
   for(n=m=i=0;i<numknt;i++){
     for(j=0;j<numgpt;j++,m++){
         cout << "m = "<<m<<" ind[m]="<< ind[m] << endl;
         if(ind[m]==-1)n++;
     }
   }
   
   cout <<" n = "<< n<<endl;
*/
   delete [] index;
   return numoverlap;
}


void BS_MODEL::InitKnots(int choice)
{
  int i;
	theta = new float[numgpt];
	phi = new float[numgpt];
	if(choice==WHOLE_EARTH)
	   ajknts = new int[numgpt*7];
	rknt = new float[numknt];
// The following NEEDS TO BE CHANGED IF MORE THAN 2-2D LAYERS!!!
	coeff = new float[numknt*numgpt+numgpt+numgpt]; // include surface b-spl
	for(i=0; i< numknt*numgpt+numgpt+numgpt;i++) { // initialize
           coeff[i] = 0.0;
        }
}

void BS_MODEL::CalAvgDist()
{
int i,j,m,n,nb;

  for(i=m=n=0, avgdist=(float)0;i<numgpt;i++){	// cal avg distance between nodes
      m++;						// 1st element is # of ajacent points, skip it
	for(j=0;j<6;j++){
	    nb=ajknts[m++];
	    if(nb>i){				// calculate only once, skip if ==0
		n++;
		avgdist+=DistBt2(i,nb);
	    }
	}
  }
  
  avgdist/=(float)n;
}


float BS_MODEL::DistBt2(const int &pt1, const int &pt2)
{
float theta1,theta2,phi1,phi2;
double dist;

  theta1=theta[pt1];
  phi1  =phi[pt1];
  theta2=theta[pt2];
  phi2  =phi[pt2];

  dist = cos(theta1)*cos(theta2)+sin(theta1)*sin(theta2)*cos(phi2-phi1);
  dist = acos(dist);
  
  return (float)dist;
	
}


BS_MODEL::BS_MODEL()
{
  InitModel();
  numgpt=0;			// number of horizontal grid points
  theta=NULL;
  phi=NULL;
  rknt=NULL;		// knots expressed as radius
  avgdist=(float)0;	// average distance between grid points
  ajknts=NULL;		// to save adjacent points 
}

void MODEL::InitModel()
{
  format=UNDEFINED;	// model format, HVDHARM or HVDBS etc.
  dim=0;			// model dimension
  region=UNDEFINED;	// the region for which model is defined
  rfun=NORADFUN;		// radial function type
  hfun=BSPLINE;		// horizontal function type
  numknt=0;			// # of radial knots
  rtop=rmid=rbot=(float)0;
  coeff=NULL;		// model coefficients pointer
  desc[0]='\0';		// model description (NULL string for now)
}

BS_MODEL::~BS_MODEL()
{
  cout<<"......delete BS_MODEL......\n";
  if(theta)delete [] theta;
  if(phi)delete [] phi;
  if(ajknts)delete [] ajknts;
  if(rknt)delete [] rknt;
  if(coeff)delete [] coeff;

}


void BS_MODEL::PrintModel()
{
int k,ind;
int i,j;

   PrintModInfor();
   cout<<"number of horizontal points = "<<numgpt<<endl;
   cout<<"number of radial knots = "<<numknt<<endl;
   cout<<"average distance between grid (in degree)="<<avgdist/RAD<<endl;
   ind = 0;
   cout << endl;
   for(k=i=0;k<numknt;k++){
       cout << " k= " <<k<<" rknt = " <<rknt[k]<<endl;
//       for(j=0;j<numgpt;j++){
//         cout <<" "<< coeff[i++];
//         if((j+1)%5==0)cout << endl;
//       }
//     cout<<" coeff["<<ind-1<<"]="<<coeff[ind-1];
       ind+=numgpt;
   }
/*************coordinates of two adjacent grid points  
 for(m=i=0;i<numgpt;i++){
//   cout << theta[i] << phi[i];
//   cout << i <<" ";
//   theta[i]=((float)90.0-lat)*RAD;
//   phi[i] = lon;

   lat0=90.0-theta[i]/RAD;
   lon0=phi[i]/RAD;
   if(lon0>180.0)lon0-=360.0;

   for(j=0;j<7;j++,m++){
       k=ajknts[m];
       if(j!=0 && k>=0){
          // cout<< ajknts[m]<<" ";
          cout<<lat0<<" "<<lon0<<" ";

          lat=90.0-theta[k]/RAD;
          lon=phi[k]/RAD;
          if(lon>180.0)lon-=360.0;
          cout<<lat<<" "<<lon<<endl;
       }
   }
 }
*******************************/
}

int BS_MODEL::TotNumCoeff()
{
  return numgpt*numknt;
}

void BS_MODEL::ScaleModel(int opt)
{  // change model normalization
int i, j, k;
float fac;
	cout << "numknt=" << numknt << "  numgpt = " << numgpt << endl;
	cout << "***Before normalizing:" << endl;
	cout << "coeff[0]=" << coeff[0] << "  coeff[100]=" << coeff[100]
		<< "  coeff[1000]=" << coeff[1000] << endl;
	if(opt == 3) {
		cout << "scaling factor:" << endl;
		cin >> fac;
	}
 	for(k=i=0;i<numknt;i++){
     		for(j=0;j<numgpt;j++){
			if(opt==1){
				coeff[k] *=2.0/3.0;
			}else if(opt==2){
				coeff[k] *=1.5;
			}else{
				coeff[k] *=fac;
			}
			k++;
		}
	}
	cout << "***After normalizing:" << endl;
	cout << "coeff[0]=" << coeff[0] << "  coeff[100]=" << coeff[100]
		<< "  coeff[1000]=" << coeff[1000] << endl; 
}

void BS_MODEL::WriteModel(int ntopo, char ifsplit)
{
int i,j,k,m, ifl;
int  layer2d;
float lat,lon;
char flname[132], flname1[10][132];
FILE *fpout1, *fpout[10];


 if(ntopo==0) {
    cout<<"Output model file:\n";
    cin>>flname;
    cout<<"output file = "<<flname<<endl;
 } else{
    layer2d = 1;
    while(numgpt*layer2d < ntopo) layer2d++;
    cout << "There are " << layer2d << " layers" << endl;
    cout<<"Output 3D model file:\n";
    cin>>flname;
    cout<<"output 3D file = "<<flname<<endl;
    for(i=0; i<layer2d;i++){
      cout<<"Output 2D model file" << i+1 << ":"<< endl;
      cin>>flname1[i];
      j =i+1;
      cout<<"output 2D file "<< j <<" = "<< flname1[i]<<endl;
    }
 }
 fpout1=fopen(flname, "w");
 
 if(HRegion==WHOLE_EARTH || HRegion==WERECOM ){
    if(HRegion==WHOLE_EARTH) {
	if(ifsplit=='C') {
        	fprintf(fpout1, "%d   %d\n", numgpt,numknt);
  	}else if(ifsplit=='S'){
		printf("what is going on??\n");
        	fprintf(fpout1, "UPPERLOWERM %d %d %d\n",numgpt,numknt_lm,numknt_um);
		printf("UPPERLOWERM %d %d %d\n",numgpt,numknt_lm,numknt_um);
  	}else if(ifsplit=='M'){	// multiple split model
        	fprintf(fpout1, "UPPERLOWERM %d %d %d ", nsplit_um, nsplit_lm, numgpt);
		for(i=0;i<nsplit_um+nsplit_lm;i++){
			fprintf(fpout1, "%d ", nbsperlayer[i]);
		}
		fprintf(fpout1, "\n");
	}
    }
    if(HRegion==WERECOM)
        fprintf(fpout1, "2 %d   %d\n", numgpt,numknt);

    cout << "Output node information..." << endl << endl;
    for(m=i=0;i<numgpt;i++){
      lat=(float)90-theta[i]/RAD;
      lon=phi[i]/RAD;
      if(lon>(float)180)lon-=(float)360;
      fprintf(fpout1, "%4d %9.4f %9.4f", i+1,lat,lon);
      for(j=0;j<7;j++){
          if(j==0)fprintf(fpout1,"%4d ",ajknts[m++]);
          else fprintf(fpout1, "%4d ", ajknts[m++]+1);
      }
      fprintf(fpout1, "\n");
    }
 }else if(HRegion==REGIONAL){
    fprintf(fpout1, "%d   %d  %f\n", numgpt, numknt,avgdist/RAD);
    for(m=i=0;i<numgpt;i++){
      lat=(float)90-theta[i]/RAD;
      lon=phi[i]/RAD;
      if(lon>(float)180)lon-=(float)360;
      fprintf(fpout1, "%4d %9.4f %9.4f", i+1,lat,lon);
    }
 }
 for(k=i=0;i<numknt;i++){
     fprintf(fpout1,"%d %f\n", i+1,rknt[i]);
     for(j=0;j<numgpt;j++){
        fprintf(fpout1, "%12.7f ", coeff[k++]);
        if((j+1)%5==0)fprintf(fpout1, "\n");
     }
     fprintf(fpout1, "\n");
 } 
 fclose(fpout1);

 if(ntopo!=0) {
    for(ifl=0; ifl<layer2d; ifl++){
       fpout[ifl]=fopen(flname1[ifl],"w");	// open 2D bspline model
       fprintf(fpout[ifl], "%d   1\n", numgpt);
       for(m=i=0;i<numgpt;i++){
         lat=(float)90-theta[i]/RAD;
         lon=phi[i]/RAD;
         if(lon>(float)180)lon-=(float)360;
         fprintf(fpout[ifl], "%4d %9.4f %9.4f", i+1,lat,lon);
         for(j=0;j<7;j++){
            if(j==0) fprintf(fpout[ifl], "%4d ", ajknts[m++]);
            else fprintf(fpout[ifl], "%4d ", ajknts[m++]+1);
         }
         fprintf(fpout[ifl], "\n");
       }
       m=0;
       for(i=k; i<k+numgpt; i++){   // write out topography 2D b-spline
         fprintf(fpout[ifl], "%10.7f ",coeff[i]);
         if((m+1)%5==0)fprintf(fpout[ifl], "\n");
         m++;
       }
       fclose(fpout[ifl]);
       k +=numgpt;
    }
 }
}

void MODEL::Add(float &fac, MODEL &mod2)
{
int i,itotal=TotNumCoeff();

   for(i=0;i<itotal;i++)coeff[i]+=fac*mod2.coeff[i];
}


void BS_MODEL::Add2Model(float *pert, int numcoeff, int nadd, int opt)
{ 
// opt < 3 : CMB to surface  =1 include initial model  =2 not include
// opt >=3 : surface to CMB  =3 include initial model  =4 not include
int i,j,k,m,itotal;
   itotal=numknt*numgpt;
   if(nadd > 0) for(i=0; i<nadd; i++) coeff[itotal+i] = 0.0; // init 2D coef
   itotal += nadd;  // includes surface topo bspline or others
   cout << "numcoeff=" << numcoeff << endl;
   assert(itotal==numcoeff);
  
   if(opt<3) { // model from CMB to surface
      for(i=0;i<itotal;i++){
         cout << pert[i];
         if((i+1)%5==0)cout << endl;       
	 if(opt==1){	
         	coeff[i]=pert[i];
	 }else{
         	coeff[i]+=pert[i];
	 }
      }
   }else {     // model from surface to CMB
      m=0;
      for(i=0;i<numknt; i++){
         k= (numknt-i-1)*numgpt;
         cout << i+1 <<" "<<rknt[i]<<endl;
         for(j=0; j<numgpt; j++){
            cout << pert[k] << " ";
            if((k+1)%5==0)cout << endl;       
	    if(opt==4){	
         	coeff[m]+=pert[k];
	    }else{
         	coeff[m]=pert[k];
	    }
            k++;
            m++;
         }
         cout << endl;
      }
      printf("number of knot=%d, numgpt=%d\n", numknt, numgpt);
   }
}

void BS_MODEL::ScaleCoeff(float fac)
{
int i,itotal=numknt*numgpt;

   for(i=0;i<itotal;i++)
       coeff[i]*=fac;
}

void BS_MODEL::Add2Model(float *pert, int *index)
{
int i,ind, itotal;

   itotal=numknt*numgpt;
   for(i=0;i<itotal;i++){
       ind=index[i];
       if(ind>=0){
          coeff[i]+=pert[ind];
       }
   }
   cout << "Perturbation added to the starting model..." <<endl;
}

void MODEL::Check3D()
{
  if(dim<3){
	cerr<<"Error in MODEL::SumModelAtR, dim="<<dim<<endl;
	exit(-1);
  }
}

int SP_MODEL::TotNumCoeff()
{
  cout<<"not implemented yet\n";
  return 0;
}

void SP_MODEL::SumModelAtR(float &r)
{
int maxk,ireg,cur_region;
float *func;

  Check3D();
  if(rfun==CHEBYSHEV){
	 rknt[0]=rbot;
	 rknt[1]=rtop; 
	 if(region==WHOLEMANTLE)maxk=maxwk;
	 else if(region==LOWERMANTLE)maxk=maxuk;
	 else if(region==UPPERMANTLE)maxk=maxlk;
     else if(region==UPLOWMANTLE){
       if(r>=rmid && r<=rtop){
	      rknt[0]=rmid;
          maxk=maxuk;
          cur_region=UPPERMANTLE;
       }else if(r>=rbot && r<=rmid){
	      rknt[1]=rmid; 
          maxk=maxlk;
          cur_region=LOWERMANTLE;
       }
     }
	 func=new float[maxk+1];
     ireg=ChebyshevFun(r,rknt,maxk,func);
   }else if(rfun==BSPLINE){		// rknt[] is already defined
	 maxk=numknt-1;
     if(region==UPLOWMANTLE){
        cerr<<"Error in MODEL::SumModelAtR, doesn't work for split model yet\n";
        exit(-1);
     }
	 func=new float[maxk+1];
     ireg=BsplineFun(r,rknt,maxk,func);
   }else{
      cerr<<"Error in SP_MODEL::SumModelAtR, radial function not implemented yet\n";
      exit(-1);
   }


   SumRadCoeff(func,ireg,cur_region);
   delete [] func;
}
//
// ireg is not used here
//
void SP_MODEL::SumRadCoeff(float *func, int &ireg, int &cur_region)
{
float *model_coeff,funck;
int  i,k,numc,maxl,maxk,ind;

  cout<<" get in SP_MODEL::SumRadCoeff\n";
  model_coeff=coeff;		// begining of the model coefficient array
  if(laymask[0]!='0'){
     numc=NumCoeff(lcrust);
     model_coeff+=numc;		// skip crust
  }
  if(laymask[1]!='0'){
     numc=NumCoeff(l670)*(maxuk+1);
     model_coeff+=numc;		// skip 670
  }

  if(region==LOWERMANTLE || 
    (region==UPLOWMANTLE && cur_region==LOWERMANTLE)){
     numc=NumCoeff(maxul)*(maxuk+1);
     model_coeff+=numc;		// skip upper mantle
     maxl=maxll;
     maxk=maxlk;
  }else if(region==UPPERMANTLE || 
          (region==UPLOWMANTLE && cur_region==UPPERMANTLE)){
     maxl=maxul;
     maxk=maxuk;
  }else if(region==WHOLEMANTLE){
     maxl=maxwl;
     maxk=maxwk;
  }

  ind=numc=NumCoeff(maxl);

  funck=func[0];
  for(i=0;i<numc;i++)coeff_r[i]=model_coeff[i]*funck;	// k=0 part
  for(k=1;k<=maxk;k++,ind+=numc){
      funck=func[k];
      for(i=0;i<numc;i++)coeff_r[ind+i]+=model_coeff[ind+i]*funck;
  }
}

void BS_MODEL::SumRadCoeff(float *func, int &ireg, int &cur_region)
{
int k;

    for(k=ireg-2;k<=ireg+1;k++){
        if(k>=0 && k<numknt){
        }
    }
	cout<<" get in BS_MODEL::SumRadCoeff\n";
}

void CS_MODEL::ReadModel(int choice, char *flname)
{

fstream inFile;
int lm,l,m,l0,m0,numc;
float coeff_r,coeff_i;

 format=CHARM;		// complext spherical harmonics model
 hfun=HARMONICS;
 rfun=NORADFUN;
 dim=2;
 cout<<"file name="<<flname<<endl;
 inFile.open(flname,ios::in);	// open model for read
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     exit(-1);
 }
 
 cout<<"Max degree?\n";
 cin>>maxl;

 degmask=new char[maxl+2];
 
 for(l=0;l<=maxl;l++)degmask[l]='1';
 degmask[maxl+1]='\0';

 numc=NumCoeff(maxl);
 coeff=new float[numc];
 
 for(lm=l=0;l<=maxl;l++){
     for(m=0;m<=l;m++){
         inFile>>l0>>m0>>coeff_r>>coeff_i;
         
         assert(l0==l && m0==m);
         coeff[lm++]=coeff_r;
         if(m!=0)coeff[lm++]=coeff_i;
     }
 }
 
 cout <<" lm= "<<lm<<endl;
  
 inFile.close();
}

void BS_MODEL::SumModelAtR(float &r)
{
}

void CS_MODEL::SumRadCoeff(float *func, int &ireg, int &cur_region)
{
}

void CS_MODEL::SumModelAtR(float &r)
{
}

void CS_MODEL::PrintModel()
{
int l,m,lm;

 PrintModInfor();

 for(lm=l=0;l<=maxl;l++){
     for(m=0;m<=l;m++){
         if(m==0){
            cout<<l<<"  "<<m<<" "<<coeff[lm++]<<"   0"<< endl;
         }else{
            cout<<l<<"  "<<m<<" "<<coeff[lm++]<<"  "<<coeff[lm++]<< endl;
         }
     }
 }

}

void CS_MODEL::WriteModel()
{
fstream outFile;
char flname[132];
int l,m,lm;

 cout<<"\n Output model file:\n";
 cin>>flname;
 cout<<"output file = "<<flname<<endl;

 outFile.open(flname,ios::out);	// open  for write
 
 if(!outFile) { 				// open failed?
     cerr << "Cannot open "<<flname<<" for output\n";
     exit(-1);
 }
 for(lm=l=0;l<=maxl;l++){
     for(m=0;m<=l;m++){
         if(m==0)
            outFile<<l<<" "<<m<<" "<<coeff[lm++]<<" 0"<< endl;
         else
            outFile<<l<<" "<<m<<" "<<coeff[lm++]<<" "<<coeff[lm++]<< endl;
     }
 }
 outFile.close();

}

CS_MODEL::~CS_MODEL()
{
	cout<<".......delete CS_MODEL......\n";
	delete [] coeff;
	delete [] degmask;
}

//*****************************************************************************
//					Written by Weijia Su	1/4/97
// B-spline function
// Input:	maxk -- order (or number of knots-1)
//		xknt -- knots in ascending order. xknt[0]<=xknt[1]...<=xknt[maxk]
//		x    -- where function is to be evaluated
// Output:	y    -- function value y[0] --- y[maxk], only four of them are 
//			non-zero
//
//*****************************************************************************
int BsplineFun(float x, float *xknt, int maxk, float *y)
{
float y0,yknt1,temp,temp1,X1,X2,X3,X4,x0,x1,x2,x3,x4,x5;
int i,ib,ie;
static float A1,A2,A3,A4;
static int i0=-1;

  for(i=0;i<=maxk;i++)y[i]=(float)0;
  
  if(x<xknt[0] || x>xknt[maxk])return -1;
  ib=1;
  ie=maxk+1;

  while(ie-ib>1){
	i=ib+((ie-ib)>>1);
	if(x<xknt[i-1])	ie=i;
	else		ib=i;
  }
  
  if(ib==1 || ib==2){
    x0=x1=xknt[0];
    x4=xknt[ib+1];
    x5=xknt[ib+2];
  }else if(ib==maxk-1 || ib==maxk){
    x0=xknt[ib-3];
    x1=xknt[ib-2];
    x4=x5=xknt[maxk];
  }else{
    x0=xknt[ib-3];
    x1=xknt[ib-2];
    x4=xknt[ib+1];
    x5=xknt[ib+2];
  }

  x2=xknt[ib-1];
  x3=xknt[ib];
  
  if(ib!=i0){
	temp=(float)1.0/(x3-x2);
	A1=temp/(x3-x1);
	A3=temp/(x4-x2);
	A4=A3/(x5-x2);
	temp=x4-x1;
	A2=A1/temp;
	A3/=temp;
	A1/=x3-x0;
	i0=ib;
  }
  
  X1=x-x1;
  X2=x-x2;
  X3=x-x3;
  X4=x-x4;
  
  temp=X3*X3*A1;
  temp1=X3*X1*A2+X2*X4*A3;

  if(ib!=1)y[ib-2]=-temp*X3;
  else    y0    =-temp*X3;
  y[ib-1]=(x-x0)*temp+X4*temp1;
  temp=X2*X2*A4;
  y[ib]=-X1*temp1-(x-x5)*temp;
  if(ib!=maxk)y[ib+1]=X2*temp;
  else       yknt1=X2*temp;

  if(ib==1){			/* for 2nd derivative = 0 */
     y[1]+=y[0]/(float)3;
     y[0]=y[0]/(float)1.5+y0;
  }else if(ib==2){
     y[1]+=y[0]/(float)3;
     y[0]/=(float)1.5;
  }else if(ib==maxk){
     y[maxk-1]+=y[maxk]/(float)3;
     y[maxk]=y[maxk]/(float)1.5+yknt1;
  }else if(ib==maxk-1){
     y[maxk-1]+=y[maxk]/(float)3;
     y[maxk]/=(float)1.5;
  }
  return ib;
}
//*****************************************************************************
//					Written by Weijia Su	1/4/97
// Chebyshev polynomials
// Input:	maxk -- order of the function
//		xnode -- xnode[0]=xmin, xnode[1]=xmax, others are not used
//		x    -- where function is to be evaluated
// Output:	f    -- function value f[0] --- f[maxk]
//
//*****************************************************************************
int ChebyshevFun(float x,float *xnode, int maxk, float *f)
{
float twou, u;
static float cheby_coef[14]={
(float)0.7071067811865, (float)1.224744871392, (float)1.0350983390135, 
(float)1.0145993123918, (float)1.008032257548, (float)1.0050890913907, 
(float)1.0035149493262, (float)1.002574006832, (float)1.0019665702378,
(float)1.0015515913133, (float)1.001255493275, (float)1.0010368069141,
(float)1.0008707010792, (float)1.000741564803};
int i;

   if(maxk>13){
      cout << " error in function ChebyshevFun, kmax>13\n";
      return -1;
   }
   if(x<xnode[0] || x > xnode[1]){
     for(i=0;i<=maxk;i++)f[i]=(float)0;
     return -1;
   }
     
   u = (x+x-xnode[1]-xnode[0])/(xnode[1]-xnode[0]);
   
   f[0] = (float)1;
   f[1] = u;
   twou = u+u;
   for(i=2;i<=maxk;i++)f[i] = twou*f[i-1]-f[i-2];
   for(i=0;i<=maxk;i++)f[i]*= cheby_coef[i];

   return 1;
}

void PolyPrem::InitPolyPrem(int nr)
{
  numreg =  nr;
  rtop = new float[numreg];
  rbot = new float[numreg];

  rhopol = new float[numreg][5];
  qspol  = new float[numreg][5];
  qkpol  = new float[numreg][5];
  etapol = new float[numreg][5];
  vpvpol = new float[numreg][5];
  vsvpol = new float[numreg][5]; 
  vphpol = new float[numreg][5];
  vshpol = new float[numreg][5];

}

void PolyPrem::PrintPolyPrem()
{
int i,j;
   cout << "Number of regions in prem = " << numreg << endl;
   cout << "Radius of Earth = " << radius << endl;
   cout.precision(6);
   
   for(i=0;i<numreg;i++){
      cout <<i<<" dep0 = "<<radius-rbot[i]<<" dep1 = "
		   <<radius-rtop[i]<<endl;
   }
   cout << endl << endl;
   for(i=0;i<numreg;i++){
      cout << "i = " << i << " rbot = " << rbot[i] << " rtop = " << rtop[i] << endl;
      for(j=0;j<5;j++){cout << rhopol[i][j]<<" ";} cout << endl;
      for(j=0;j<5;j++){cout << vpvpol[i][j]<<" ";} cout << endl;
      for(j=0;j<5;j++){cout << vsvpol[i][j]<<" ";} cout << endl;
      for(j=0;j<5;j++){cout << qspol[i][j]<<" ";}  cout << endl;
      for(j=0;j<5;j++){cout << qkpol[i][j]<<" ";}  cout << endl;
      for(j=0;j<5;j++){cout << vphpol[i][j]<<" ";} cout << endl;
      for(j=0;j<5;j++){cout << vshpol[i][j]<<" ";} cout << endl;
      for(j=0;j<5;j++){cout << etapol[i][j]<<" ";} cout << endl;
      cout << endl;
   }
}

void PolyPrem::ReadPolyPrem()
{
fstream inFile;
int j,nr;

 inFile.open("premod_520.dat",ios::in);		// open prem file for read
 
 if(!inFile) { 					// open failed?
     cerr << "Cannot open ``premod_520.dat'' for input\n";
     exit(-1);
 }
 inFile >> nr >> radius;
 
 InitPolyPrem(nr);
 
 for(int i=0;i<numreg;i++){
   inFile >> rbot[i] >> rtop[i];
   for(j=0;j<5;j++)inFile >> rhopol[i][j];
   for(j=0;j<5;j++)inFile >> vpvpol[i][j];
   for(j=0;j<5;j++)inFile >> vsvpol[i][j];
   for(j=0;j<5;j++)inFile >> qspol[i][j];
   for(j=0;j<5;j++)inFile >> qkpol[i][j];
   for(j=0;j<5;j++)inFile >> vphpol[i][j];
   for(j=0;j<5;j++)inFile >> vshpol[i][j];
   for(j=0;j<5;j++)inFile >> etapol[i][j];
 }
 inFile.close();
}

PolyPrem::~PolyPrem()
{
  delete [] rtop;
  delete [] rbot;
  delete [] rhopol;
  delete [] qspol;
  delete [] qkpol;
  delete [] etapol;
  delete [] vpvpol;
  delete [] vsvpol;
  delete [] vphpol;
  delete [] vshpol;  
}


