/*
 **********************input output functions***************************
 */
int       GetLine(char*, int);			/* read a line from the console          */
int       FGetLine(FILE*,char*,int);		/* read a line from a file               */
int       GetInt(FILE*,int*);			/* read an integer from a file           */
int       CGetInt();          			/* read an integer from console          */
char      CGetChar();         			/* read a char from console              */
int       GetStr(FILE*,char*);			/* read a string from a file             */
float     GetFloat(FILE*,float*);		/* read a real number from a file        */
FILE      *OpenFl(char*);			/* open a file for read                  */
FILE      *OpenFile(char*);			/* read in file name and open the  file  */
FILE      *OpenFileWt(char*);			/* open file for write purpose only      */
FILE      *OpenFlWt(char*);			/* open file for write purpose only      */
int       SysOpen(char*,int);			/* open a file for system level rd/wt    */
/*
 ********************allocate memory functions*************************
 */
float     **Matrix(int,int,int,int);	/* allocate memory for 2-D array        */
void      FreeMatrix(float**,int,int,int,int);	/* free the allocated memory      */
int       **IntMatrix(int,int,int,int);	/* allocate memory for 2-D int array    */
void      FreeIntMatrix(int**,int,int,int,int);/* free the allocated memory     */
short     **ShortMatrix(int,int,int,int);
void      FreeShortMatrix(short**,int,int,int,int);
char      **CMatrix(int,int,int,int);
void      FreeCMatrix(char**,int,int,int,int);
float     *Vector(int,int);			/* allocate memory for 1-D array         */
char	    *CharVector(int,int);
CARTESIAN_2 *Cart2Vec(int,int);
CMT       *CMTVec(int,int);			/* allocate memory for 1-D array         */
ISCEQ     *ISCEqVec(int,int);
ISCST     *ISCStVec(int,int);
void      FreeVector(float *,int,int);
void      FreeCMTVec(CMT *,int,int); 
void      FreeISCEqVec(ISCEQ *,int,int);
void      FreeISCStVec(ISCST *,int,int);
int       *IntVec(int,int);			/* allocate space for integer array      */
void      FreeIntVec(int*,int,int);		/* free the allocated memory             */
short     *ShortVec(int);			/* allocate space for short int vector   */
short     *ShortVec1(int,int);		/* allocate space for short int array    */
void      FreeShortVec1(short*,int,int);	/* free the allocated memory             */
SPHERE    *SphereVec(int,int);		/* allocate space for spherical array    */
void      FreeSphereVec(SPHERE*,int,int);	/* free the allocated memory             */
PREM      *PremVec(int,int);			/* allocate space for PREM model         */
void      FreePremVec(PREM*,int,int);	/* free the allocated memory             */
float     ***Array3d(int,int,int,int,int,int);
void      FreeArray3d(float***,int,int,int,int,int,int);
void	    FreeCharVec(char*,int,int);
/*
 ************************math functions*********************************
 */
CARTESIAN   OutProduct(CARTESIAN,CARTESIAN);	/* outproduct of two vectors           */
float       DotVec(CARTESIAN,CARTESIAN);		/* dot product of two vectors          */
CARTESIAN   NormCarVec(CARTESIAN);			/* to normalize a vector               */
CARTESIAN   DiffVec(CARTESIAN,CARTESIAN);		/* sub one vec from another            */
SPHERE      GetSphere(CARTESIAN);			/* convert cart coor to sphere coor    */
CARTESIAN   GetCartesian(SPHERE);			/* convert spherical coor to cart coor */
CARTESIAN_2 GetCartesian_2(CYLINDRIC);		/* cylindric to cartesian_2            */
CYLINDRIC   GetCylindric(CARTESIAN_2);		/* cartesian_2 to cylindric            */
float       ArcDist(SPHERE,SPHERE);			/* minor arc dist of two points        */
float       DistP1P2(CARTESIAN,CARTESIAN);	/* dist between two points             */
float       SqDist(CARTESIAN,CARTESIAN);		/* squared dist between two points     */
float       LenVec(CARTESIAN);			/* the length of a vector              */
void        Ylm(int,float,float*);
void		CalYlm(int, float, float, int,float **);
void		CSmphi(int, float, float, int, float **, float **);
void        indexx(int, float*, int *);		/* sort routine                        */
void        BSpline(float, float*, float *, int, float, float);
int		DBBSplKer(float*, float*, float, float*, int);
int		BSplKer(float *, float, float*, int);
int		BSplKer0(float *, float, float*, int);
void        tridag(float*, float*, float*, float*, float*, int); /* to solve a tridiagnal matrix        */
void        choles(float*, float*, float*, int, int*);
void        FSwap(float*, float*);
void        DSwap(double*, double*);
void        ISwap(int*, int*);
/*
 ***********************map functions***********************************
 */
 SPHERE      Geo2Sph(float, float, float);	/* geo coor to spherical coordinate   */
 void        Sph2Geo(SPHERE, float*, float*);	/* geo coor to spherical coordinate   */
 TRANSMATRIX GetTrans1(CARTESIAN, float);		/* get transformation matrix          */
 TRANSMATRIX GetTrans2(float, float, float);	/* get transformation matrix          */
 TRANSMATRIX GetTrans3(SPHERE,SPHERE);		/* get transformation matrix          */
 TRANSMATRIX GetTrans4(SPHERE2,SPHERE2);        /* get transformation matrix          */
 TRANSMATRIX GetTrans5(float, float, float);	/* get transformation matrix          */
 CARTESIAN   Transform1(TRANSMATRIX, CARTESIAN);	/* new coor under a transformation    */
 CARTESIAN   InvTransform1(TRANSMATRIX, CARTESIAN);
 SPHERE      Transform2(TRANSMATRIX, SPHERE);	/* new coor under a transformation    */
 SPHERE      InvTransform2(TRANSMATRIX, SPHERE);
 SPHERE2      Transform3(TRANSMATRIX, SPHERE2);	/* new coor under a transformation    */
 SPHERE2	 Geo2Sph2(float, float);
 
 CARTESIAN_2 DoProject(CARTESIAN, float);
 short       *ReadCoast(int, int);
void SaveGrid2D(float**, float, float, float, float, float, int, int);
 void		ReadGrid(MODEL*);
/*
 ***********************seismic functions*******************************
 */
void        ReadPrem(MODEL*);			/* read PREM model cooefficients       */
void        ReadBspline3(MODEL*);		/* read 3-D B-spline model             */
void        RdBSCartMd(MODEL*);		/* read 3-D B-spline model             */
void        ReadGrand(MODEL*);		/* read 3-D B-spline model             */
void        ReadSPHBSPL2(MODEL*);		/* read sherical B-spline model             */
void        ReadBSPSPLT(MODEL *model);  /* read split B-spline model  */
void        ReadBSMULTISPLIT(MODEL *model);  /* read split B-spline model  */
void        PremVelDen(float, float*, PREM*, char);	/* calculate PREM velocity or density  */
void        GetModel(MODEL*, int);		/* alloc space and read model          */
int         SelModel(MODEL*);			/* select model                        */
void        DetPara(MODEL *);
void        AllocCoeffSp(MODEL *);	/* alloc the space for model coeff     */
void        Read1stLine(MODEL *);	/* read the first line of HRV model    */
void        ReadBS1stLine(FILE *,int *,int *,int *, int *, int *);/* read the first line of B-spline model with multiple layers   */
void        ReadModel(MODEL*);		/* read models                         */
void        ReadMDLSH(MODEL*);		/* read MDLSH                          */
void        ReadMB92(MODEL*);		/* read Master and Bolton's model      */
void        ReadMRD(MODEL*);		/* read Mark Richard's density model   */
void        ReadSAF(MODEL*);		/* read X.-D Li's model                */
void        ReadHdModel(MODEL*);	/* read harvard model                  */
void        PrintModel(MODEL, int);	/* print model                         */
void        PrintCartBS(MODEL);		/* print   Cartesian BS model		*/
void        WtBSCartMd(MODEL);		/* print   Cartesian BS model		*/
void        PrintBSModel(MODEL);	/* print spherical BS model		*/
void	    PrintBSSplit(MODEL);	/* print split B-spline model 		*/
void        ReadLayer(FILE*, float*, float*, int, char*);	/* read one layer of the model         */
void        PrintLayer(int, float*, float*, int);		/* print one layer of the model        */
void        PrintModelHD(MODEL);	/* print model header                  */
void        PrintHarvard();    		/* print harvard model                 */
void        PrintShell(MODEL);		/* print shell                         */
void        ReleaseModel(MODEL*);	/* release the memory alloc for model  */
void        SumRad(float r, MODEL *model, int cur_mantle);           /* sum radial function                 */
void        SumMKRad(float r, MODEL model, int cur_mantle, float xi, float vsp2, int choice);         /* sum mu kappa radial function        */
void        Harvard2Complex();  	/* convert harvard format to complex   */
void        Complex2Harvard(MODEL*);	/* convert complex format to harvard   */
void        CST2Harvard();      	/* convert CST format to harvard format*/
void        Harvard2HarvardPub(); 	/* hrv format to hrv published fmt   */
void        HarvardPub2Harvard(); 	/* hrv pub format to hrv internal fmt*/
void        PrintSphereModel(); 	/* print spherical model               */
void        RadFunc(float, float*, int, int, int);          /* radial function                     */
void        ChebyshevFun(float, int, float*);
void        DChebyshevDx(double, int, double*, double*);
void        ComplexPower(FILE *fp, MODEL *model);  /* cal power of complex spherical harm */
float       ComplexPower_RMS(MODEL *model);     /* cal RMS power of complex spherical harm */
void        HarvardPower(MODEL*);		/* cal power for harvard model         */
float        HarvardPower_RMS(MODEL*,float,int);	/* cal rms heterogeneity for harvard model         */
int         ChooseFormat();     	/* select output format                */
void        Harvard2Zhang(MODEL*);    	/* convert hrv model to zhang's fmt    */
void        MDLSH2Harvard(MODEL *);    	/* MDLSH to harvard format             */
void        MDLSH2Complex(MODEL *);    	/* MDLSH to complex format             */
void        ZT2Complex(MODEL*);		/* ZT to complex format                */
void        ZT2Harvard(MODEL*);       	/* ZT to Harvard format                */
void		PrintZhang(MODEL);
void        MB92_2Complex(MODEL*);    	/* MB model to complex format          */
void        MRD2ComOrHarvard(MODEL*, int); /* Mark's to com or harvard format     */
void        SAF2Complex(MODEL*);		/* SAF (Li's model) to complex         */
void        ReadZT36(MODEL*);         	/* read zhang's model                  */
void        ReadL0256(MODEL*);       	/* read L02.56                         */
void        ReadSurCoeff(MODEL *);     	/* read spherical harmonics coeff      */
float       SumSurCoeff(MODEL, float*, float*,int, float*, float*, float*);
void        CSmPhi(float, float*, float*, int);
void        ShellRad(float, MODEL);
void        ReadICAniDt(MODEL*, int);
void        VdvPREM_P(float, float*, float*, int);
void        VdvPREM_S(float, float*, float*, int);
int         GetZone(float);
void        Setab2Zero(float*, float*, int);
float		SumSphBspline(MODEL, float, float);
float		fdelta(float, float);
void UnPackEq(ISCEQ,int *, int *, int *, int *,
     int *, float *, float *, float *, float *, int *,
     float *, float *, int *, int *, int *);
void		PackRes(int *, int, int, int, int, float, float, float);
void		UnPackRes(int *, int *, int *, int *, int *,
			float *, float *, float *);
void		WriteIpm(float*, float*, int);
void		ReadIpm(IPM*);
void		PrintBSModel();
/*
 **********************CMT functions************************************
 */
void        OpenCMT(int*, int*);		/* open CMT allorder.bin file		*/
int         ReadAllCMT(int, int, int, CMT*);	/* read in all the CMT solutions	*/
ISCEQ*      ReadISCEq(int*);	/* open and read all eqs from ISC	*/
ISCST*	ReadISCSt(int*); 	/* open and read all isc stations	*/
int		OpenResFile();

/*
 **********************num_r functions************************************
 */
 
void rk4(float*, float *, int, float, float, float *, void (*)(float,float *,float *));
float *vector(int, int);
float **matrix(int, int, int, int);
float **convert_matrix(float*, int, int, int, int);
double *dvector(int, int);
double **dmatrix(int, int, int, int);
int *ivector(int, int);
int **imatrix(int, int, int, int);
float **submatrix(float**, int, int, int, int, int, int);
void free_vector(float *, int, int);
void free_dvector(double*, int, int);
void free_ivector(int*, int, int);
void free_matrix(float**, int, int, int, int);
void free_dmatrix(double**, int, int, int, int);
void free_imatrix(int**, int, int, int, int);
void free_submatrix(float**, int, int, int, int);
void free_convert_matrix(float**, int, int, int, int);
void nrerror(char *);
void gauleg(double, double, double*, double*, int);

int SetOptions(int, char**, char**);
int GetOptions(char*, int, char**, char**, char**);
int HelpMessage(char *, char**, char*);
void FormBSMatrix(float *rker,int irr, float *hker, int *iker,
		int nrad, float dd, float *ata, float *atd, float *arow, int *iar, int nnode);
void ReadDepth(float *rr, int *n);
float CalBSVal_2D(float *coef, float *hker, int *iker,int nnode);
float CalBSVal(MODEL, float *rker,int irr, float *hker, int *iker,
		int nrad, int nnode);
float CalBSVal_UM(MODEL, float *rker,int irr, float *hker, int *iker,
		int nrad_um, int nnode);
float CalBSVal_LM(MODEL, float *rker,int irr, float *hker, int *iker,
		int nrad_lm, int nnode);
float CalBSVal_MULTISPLIT(MODEL model, float *rker,int irr, float *hker, int *iker,int *bslayer);
int find_region(float r, int nrad, MODEL model);
int find_region_split(float r, int nrad, MODEL model, int uplow);
int find_region_multisplit(float r,MODEL model, int *);
