/*
 ******************************wslib.c**********************************
 * Originally written by Wei-Jia Su
 * Modified by Yu Gu in Mar, 1998 to run on solaris
 */
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include  "wslib.h"

/*
 **********************input output functions***************************
 */
int GetLine(char *s, int lim)
{
int c, i;

    for(i=0; i<lim-1 && (c=getchar())!=EOF && c!='\n'; ++i)
        s[i]=c;
        
    if(c=='\n' && i==0){
       s[i] =c;
       ++i; 
    }
    s[i] = '\0';
    return i;
}

int GetLine_trim(char *s, int lim)
{
int c, i;

    for(i=0; i<lim-1 && (c=getchar())!=EOF && c!='\n'; ++i)
        s[i]=c;
        
    if(c=='\n' && i==0){
       s[i] =c;
    }
    s[i] = '\0';
    return i;
}

int FGetLine(FILE *fp, char *s, int lim)
{
char c;
int i;

    for(i=0; i<lim-1 && (fscanf(fp,"%c",&c))!=EOF && c!='\n'; ++i)
        s[i]=c;
    if(c=='\n'){
       s[i] =c;
       ++i;
    }
    s[i] = '\0';
    return i;
}

int FGetLine_trim(FILE *fp, char *s, int lim)
{
char c;
int i;

    for(i=0; i<lim-1 && (fscanf(fp,"%c",&c))!=EOF && c!='\n'; ++i)
        s[i]=c;
    if(c=='\n'){
       s[i] =c;
    }
    s[i] = '\0';
    return i;
}

int GetInt(FILE *fp, int *i)
{
char c,s[MAXSTR];
   FGetLine(fp,s,MAXSTR);
   sscanf(s,"%d",i);
   return *i;
}

int CGetInt()
{
char s[MAXSTR];
int i=-1;

   GetLine(s,MAXSTR);
   sscanf(s,"%d",&i);
   printf("here i=%d\n", i);
   return i;
}

char CGetChar(void)
{
char s[MAXSTR];
char c;

   GetLine(s,MAXSTR);
   sscanf(s,"%c",&c);
   return c;
}

int GetStr(FILE *fp, char *st)
{
char s[MAXSTR];

   int i = FGetLine(fp,s,MAXSTR);
   sscanf(s,"%s",st);
   return i;
}

float GetFloat(FILE *fp, float *x)
{
char s[MAXSTR];
   FGetLine(fp,s,MAXSTR);
   sscanf(s,"%f",x);
   return *x;
}

FILE *OpenFl(char *name)
{
FILE *fp;

  if((fp=fopen(name,"r"))==NULL){
      printf("Cannot open file %s\n",name);
      exit(1);              
  }
  return fp;
}

FILE *OpenFile(char *message)
{
FILE *fp;
char name[MAXSTR],s[MAXSTR];
int i;

  printf(" Type the name of the %s file\n",message);
    
  GetLine(s,MAXSTR);
  sscanf(s,"%s",name); 
  i = 0;   
  while ((fp=fopen(name,"r"))==NULL){
      printf(" Cannot open file %s, input another file name\n",name);
      if(++i>20)exit(1);
      GetLine(s,MAXSTR);
      sscanf(s,"%s",name);
  }
  return fp;
}

FILE *OpenFileWt(char *message)
{
FILE *fp;
char name[MAXSTR],s[MAXSTR];
int i;

  printf("Type the name of the %s file\n",message);
    
  GetLine(s,MAXSTR);
  sscanf(s,"%s",name);
  i = 0;  
  while ((fp=fopen(name,"w"))==NULL){
      printf(" Cannot open file %s, give another file name\n",name);
      if(++i>20)exit(1);
      GetLine(s,MAXSTR);
      sscanf(s,"%s",name);
  }
  return fp;
}

FILE *OpenFlWt(char *flname)
{
FILE *fp;

  if((fp=fopen(flname,"w"))==NULL){
      printf(" Cannot open file %s\n",flname);
      exit(1);
  }
  return fp;
}

/*
 * mode = 0 --- for read, 1 for write and 2 for read and write
 */

int SysOpen(char *filenm, int mode)
{
int fd;

  if(mode==1){
     if(creat(filenm,PERMS)==-1){
        printf("error, cannot creat file\n");
        return -1;
     }
  }
  if((fd=open(filenm,mode))==-1){
      printf(" erorr in  opening  file %s\n",filenm);
      return -1;
  }
  return fd;
}


/*
 ********************allocate memory functions*************************
 */

/*
 * these functions are modified from <<Numerical Recipes in C>>
 */
float **Matrix(int nrl, int nrh, int ncl, int nch)
{
int i;
float **m;

  m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
  if (!m) printf("allocation failure 1 in Matrix()");
  m -= nrl;

  for(i=nrl;i<=nrh;i++) {
      m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float)); 
      if (!m[i]) printf("allocation failure 2 in Matrix()");
           m[i] -= ncl;
  }
  return m;
}

void FreeMatrix(float **m, int nrl, int nrh, int ncl, int nch)
{
  int i;

  for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
      free((char*) (m+nrl));
}

int **IntMatrix(int nrl, int nrh, int ncl, int nch)
{
int i, **m;

  m=(int **) malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
  if (!m) printf("allocation failure 1 in IntMatrix()");
  m -= nrl;

  for(i=nrl;i<=nrh;i++) {
      m[i]=(int *) malloc((unsigned) (nch-ncl+1)*sizeof(int)); 
      if (!m[i]) printf("allocation failure 2 in IntMatrix()");
           m[i] -= ncl;
  }
  return m;
}

void FreeIntMatrix(int **m, int nrl, int nrh, int ncl, int nch)
{
  int i;

  for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
      free((char*) (m+nrl));
}

short **ShortMatrix(int nrl, int nrh, int ncl, int nch)
{
int i;
short **m;

  m=(short **) malloc((unsigned) (nrh-nrl+1)*sizeof(short*));
  if (!m) printf("allocation failure 1 in ShortMatrix()");
  m -= nrl;

  for(i=nrl;i<=nrh;i++) {
      m[i]=(short *) malloc((unsigned) (nch-ncl+1)*sizeof(short)); 
      if (!m[i]) printf("allocation failure 2 in ShortMatrix()");
           m[i] -= ncl;
  }
  return m;
}

void FreeShortMatrix(short int **m, int nrl, int nrh, int ncl, int nch)
{
  int i;

  for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
      free((char*) (m+nrl));
}

char **CMatrix(int nrl, int nrh, int ncl, int nch)
{
int     i;
char  **m;

  m=(char **)malloc((unsigned) (nrh-nrl+1)*sizeof(char*));
  if (!m) printf("allocation failure 1 in CMatrix()");
  m -= nrl;

  for(i=nrl;i<=nrh;i++) {
      m[i]=(char *)malloc((unsigned) (nch-ncl+1)*sizeof(char));
      if (!m[i]) printf("allocation failure 2 in CMatrix()");
      m[i] -= ncl;
  }
   return m;
}

void FreeCMatrix(char **m, int nrl, int nrh, int ncl, int nch)
{
int i;

  for(i=nrh;i>=nrl;i--) free(m[i]+ncl);
      free(m+nrl);
}


float *Vector(int nl, int nh)
{
float *v;

  v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
  if (!v) printf("allocation failure in Vector()");
  return v-nl;
}

char *CharVector(int nl, int nh)
{
char *v;

  v=(char *)malloc((unsigned) (nh-nl+1)*sizeof(char));
  if (!v) printf("allocation failure in CharVector()");
  return v-nl;
}

CARTESIAN_2 *Cart2Vec(int nl, int nh)
{
CARTESIAN_2 *v;

  v=(CARTESIAN_2 *)malloc((unsigned) (nh-nl+1)*sizeof(CARTESIAN_2));
  if (!v) printf("allocation failure in CARTESIAN_2()");
  return v-nl;
}

CMT *CMTVec(int nl, int nh)
{
CMT *v;

  v=(CMT *)malloc((unsigned) (nh-nl+1)*sizeof(CMT));
  if (!v) printf("allocation failure in CMTVec()");
  return v-nl;
}

ISCEQ *ISCEqVec(int nl, int nh)
{
ISCEQ *v;

  v=(ISCEQ *)malloc((unsigned) (nh-nl+1)*sizeof(ISCEQ));
  if (!v) printf("allocation failure in ISCEqVec()");
  return v-nl;
}

ISCST *ISCStVec(int nl, int nh)
{
ISCST *v;

  v=(ISCST *)malloc((unsigned) (nh-nl+1)*sizeof(ISCST));
  if (!v) printf("allocation failure in ISCStVec()");
  return v-nl;
}


void FreeISCEqVec(ISCEQ *v, int nl, int nh)
{
  free((char*) (v+nl));
}

void FreeISCStVec(ISCST *v, int nl, int nh)
{
  free((char*) (v+nl));
}

void FreeCart2Vec(CARTESIAN_2 *v, int nl, int nh)
{
  free((char*) (v+nl));
}

void FreeVector(float *v, int nl, int nh)
{
  free((char*) (v+nl));
}

void FreeCharVec(char *v, int nl, int nh)
{
  free((char*) (v+nl));
}

void FreeCMTVec(CMT *v, int nl, int nh)
{
  free((char*) (v+nl));
}

int *IntVec(int nl, int nh)
{
  int *v;

  v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
  if (!v) printf("allocation failure in IntVec()\n");
  return v-nl;
}

void FreeIntVec(int *v, int nl, int nh)
{
  free((char*) (v+nl));
}

short *ShortVec1(int nl, int nh)
{
short *v;

  v=(short *)malloc((unsigned) (nh-nl+1)*sizeof(short));
  if (!v) printf("allocation failure in ShortVec()\n");
  return v-nl;
}

void FreeShortVec1(short int *v, int nl, int nh)
{
  free((char*) (v+nl));
}

SPHERE *SphereVec(int nl, int nh)
{
SPHERE *v;

  v=(SPHERE *)malloc((unsigned) (nh-nl+1)*sizeof(SPHERE));
  if (!v) printf("allocation failure in SphereVec()");
  return v-nl;
}

void FreeSphereVec(SPHERE *v, int nl, int nh)
{
  free((char*) (v+nl));
}

short *ShortVec(int size)
{
short *pt;

  if((pt=(short *)malloc(sizeof(short)*size))==NULL)
      printf("allocation failure in ShortVec()\n"); 
  return pt;
}  


PREM *PremVec(int nl, int nh)
{
PREM *v;

  v=(PREM *)malloc((unsigned) (nh-nl+1)*sizeof(PREM));
  if (!v) printf("allocation failure in PremVec()");
  return v-nl;
}

void FreePremVec(PREM *v, int nl, int nh)
{
  free((char*) (v+nl));
}

float ***Array3d(int nrl, int nrh, int ncl, int nch, int nll, int nlh)
{
int   i,j;
float ***m;

   m=(float ***) malloc((unsigned) (nrh-nrl+1)*sizeof(float**));
   if (!m) printf("allocation failure 1 in array3d()\n");
	
   for(i=nrl;i<=nrh;i++) {
         m[i]=(float **) malloc((unsigned) (nch-ncl+1)*sizeof(float* ));
         if (!m[i]) printf("allocation failure 2 in Array3d()\n");
         m[i] -= ncl;
         for(j=ncl;j<=nch;j++) {
             m[i][j]=(float *) malloc((unsigned) (nlh-nll+1)*sizeof(float));
             if (!m[i][j]) printf("allocation failure 3 in matrix()\n");
             m[i][j] -= nll;
         }
    }
    return m;
}

void  FreeArray3d(float ***m, int nrl, int nrh, int ncl, int nch, int nll, int nlh)
{
int i,j;

   for(i=nrh;i>=nrl;i--){
      for(j=nlh;j>=nll;j--){
          free((char*) (m[i][j]+ncl));
      }
      free((char*) (m[i]+ncl));
   }
   free((char*) (m+nrl));
}

/*
 ************************math functions*********************************
 */
 
/*
 * to compute the outproduct of two Vectors (in cartesian coordinate)
 */
CARTESIAN OutProduct(CARTESIAN c1, CARTESIAN c2)
{
CARTESIAN c3;

  c3.x=c1.y*c2.z-c1.z*c2.y;
  c3.y=c1.z*c2.x-c1.x*c2.z;
  c3.z=c1.x*c2.y-c1.y*c2.x;
  return c3;
}

/*
 * to normalize a Vector
 */
CARTESIAN NormCarVec(CARTESIAN c)
{
float temp;

  temp=c.x*c.x+c.y*c.y+c.z*c.z;
  if(temp>1.0e-20){
     temp=sqrt((double)temp);
     c.x/=temp;
     c.y/=temp;
     c.z/=temp;
  }else{
     c.x=2.0;
     c.y=0.0;
     c.z=0.0;
  }
  return c;
}
/*
 * to sub one vec from another
 */
CARTESIAN DiffVec(CARTESIAN c1, CARTESIAN c2)
{
CARTESIAN c;

  c.x=c2.x-c1.x;
  c.y=c2.y-c1.y;
  c.z=c2.z-c1.z;
  return c;
}
/*
 * to calculate dot product of two Vectors
 */
float DotVec(CARTESIAN a1, CARTESIAN a2)
{
float dot;

  return dot=a1.x*a2.x+a1.y*a2.y+a1.z*a2.z;
}

/*
 * to transform cartesian coordinate to spherical coordinate
 */
SPHERE GetSphere(CARTESIAN c)
{
SPHERE s;

 s.r    =sqrt(c.x*c.x+c.y*c.y+c.z*c.z);
 s.theta=s.phi=0.0;
    if(s.r>1.0e-10){
       s.theta=acos((double)(c.z/s.r));
       if(fabs((double)s.theta) > 1.0e-6){
          if(c.x!=0.||c.y!=0.0)s.phi=atan2((double)c.y,(double)c.x);
          if(s.phi<0.0)s.phi+=2.*PI;
       }
    }
 return s;
}
/*
 * to transform spherical coordinate to cartesian coordinate
 */
CARTESIAN GetCartesian(SPHERE s)
{
CARTESIAN c;

  c.x =s.r*sin(s.theta);
  c.y =c.x*sin(s.phi);
  c.x*=cos(s.phi);
  c.z =s.r*cos(s.theta);
  return c;
}

/*
 * to transform cylindrical coordinate to cartesian coordinate
 */
CARTESIAN_2 GetCartesian_2(CYLINDRIC c)
{
CARTESIAN_2 c_2;

  c_2.x =c.r*cos((double)c.phi);
  c_2.y =c.r*sin((double)c.phi);
  return c_2;
}

/*
 * to transform cartesian coordinate to cylindric coordinate
 */
CYLINDRIC GetCylindric(CARTESIAN_2 cart)
{
CYLINDRIC c;

   c.r  =sqrt(cart.x*cart.x+cart.y*cart.y);
   c.phi=0.0;
   if(c.r>1.0e-10){
      c.phi=(float)atan2((double)cart.y,(double)cart.x);
      if(c.phi<0.0)c.phi+=2.*PI;
   }
   return c;
}

/* arctwo.c 
 * To calculate the minor arc distance between two points on a sphere
 * input       : s1,s2 -- the coordinates of the two points 
 * return value: the arc length between two poin (in radian)
 */
float  ArcDist(SPHERE s1, SPHERE s2)
{
float arclen;

  arclen = cos((double)s1.theta)*cos((double)s2.theta)+
           sin((double)s1.theta)*sin((double)s2.theta)*
           cos((double)(s1.phi-s2.phi));
  return   acos((double)arclen);
}

/* DistP1P2 
 * To calculate the distance between two points
 */
float  DistP1P2(CARTESIAN p1, CARTESIAN p2)
{
float dist;

  dist=square(p1.x-p2.x)+square(p1.y-p2.y)+square(p1.z-p2.z);
  dist=sqrt(dist);

  return   dist;
}

/* DistP1P2 
 * To calculate the squared distance between two  points
 */
float  SqDist(CARTESIAN p1, CARTESIAN p2)
{

  return square(p1.x-p2.x)+square(p1.y-p2.y)+square(p1.z-p2.z);

}

float  LenVec(CARTESIAN p)
{
float len;

  len=square(p.x)+square(p.y)+square(p.z);
  len=sqrt(len);

  return   len;
}

void Ylm(int lmax, float theta, float *y)
{
float x,y0,fac,tlm1,tlm3,tlp1,lm,sqrx2,tl,tl1,lpm;
int i,j,dj,l,l0,l1,l2,m;

  x=cos((double)theta);
  y0=y[0]=y00; 		/*  sqrt(1.0/fpi)            */
  y[1]=y10*x;      	/*  sqrt(3.0/fpi)*x          */
  fac=2.0;
  sqrx2=sqrt(1.0-x*x);
     
  for(i=1,j=dj=2;i<=lmax;j+=++dj){
      y0 = y[j] =-sqrt(1.0+1.0/fac)*sqrx2*y0;
      fac += 2.0;
      if(i++!=lmax)y[j+i]= sqrt(fac+1.0)*x*y0;
  }
  for(l=2,tl=2.0,j=dj=0;l<=lmax;l++,j+=++dj){
      tl  +=2.0;
      tl1  =tl-1.0;
      tlm1 =sqrt(tl1);
      tlp1 =sqrt(tl+1.0);
      tlm3 =sqrt(tl-3.0);
      for(m=0;m<l-1;m++){
          lpm =(float)((l-m)*(l+m));
          fac =tlp1/sqrt(lpm);
          lm  =sqrt(lpm-tl1)/tlm3;
          l2=j+m;
          l1=l2+l-1;
          l0=l1+l;
          y[l0]=(x*tlm1*y[l1]-lm*y[l2])*fac;
/*	  printf("l0=%d, y[l0]=%f\n", l0, y[l0]); */
      }
  }
}

void  CalYlm(int lmax, float theta0, float dtheta, int row, float **y)
{
int   i;
float theta;

   for(theta=theta0,i=1;i<=row;i++,theta+=dtheta){
       Ylm(lmax,theta,y[i]); 
   }
}

void  CSmphi(int lmax, float phi0, float dphi, int col, float **cmphi, float **smphi)
{
float phi,cmph,smph,tmph,sp,cp;
int j,m;

    for(phi=phi0,j=1;j<=col;j++,phi+=dphi){
        cp=cos(phi);
        sp=sin(phi);
        for(m=0,cmph=1.0,smph=0.0;m<=lmax;m++){
            cmphi[j][m]=cmph;
            smphi[j][m]=smph;
            tmph=cmph*cp-smph*sp;
            smph=smph*cp+cmph*sp;
            cmph=tmph;
        }
    }

}

void indexx(int n, float *arrin, int *indx)
{
	int l,j,ir,indxt,i;
	float q;

	for (j=1;j<=n;j++) indx[j]=j;
	l=(n >> 1) + 1;
	ir=n;
	for (;;) {
		if (l > 1)
			q=arrin[(indxt=indx[--l])];
		else {
			q=arrin[(indxt=indx[ir])];
			indx[ir]=indx[1];
			if (--ir == 1) {
				indx[1]=indxt;
				return;
			}
		}
		i=l;
		j=l << 1;
		while (j <= ir) {
			if (j < ir && arrin[indx[j]] < arrin[indx[j+1]]) j++;
			if (q < arrin[indx[j]]) {
				indx[i]=indx[j];
				j += (i=j);
			}
			else j=ir+1;
		}
		indx[i]=indxt;
	}
}

void BSpline(float dx, float *y, float *f, int n, float ddy1, float ddyn)
{
float *r,*a,*b,*c;
float oneosix=1.0/6.0,twoothree=2.0/3.0;
int  i;

   r = Vector(1,n+2);
   a = Vector(1,n+2);
   b = Vector(1,n+2);
   c = Vector(1,n+2);
   
   r[1] = -12.0*y[1]/(dx*dx)+ddy1;
/*   a[1] = 0.0;       /* is not referenced by tridag */
   b[1] = -1.0/(dx*dx);
   c[1] = -10.0/(dx*dx);
   for(i=2;i<=n+1;i++){
       r[i]=y[i-1];
       a[i]=oneosix;
       b[i]=twoothree;
       c[i]=oneosix;
   }
   r[n+2] = -6.0*y[n]/(dx*dx)+ddyn;
   a[n+2] = -5.0/(dx*dx);
   b[n+2] =  0.0;
/*   c[n+2] =  0.0;   /* is not referenced by tridag */
   
   tridag(a,b,c,r,f,n+2);
   
   FreeVector(a,1,n+2);
   FreeVector(b,1,n+2);
   FreeVector(c,1,n+2);
   FreeVector(r,1,n+2);

}

/*
 * to calculate cubic B-spline Kernel and its derivative
 */
int DBBSplKer(float *B, float *DB, float x, float *xnode, int N)
{
static float A1,A2,A3,A4;
float  *xn,temp,X0,X1,X2,X3,X4,X5;
float XX1,XX2,XX3,XX4;
static int i0=0;
int i,ib,ie;

  if(N<=0){		/* initialize, A1, A2 etc will be recalculated */
     i0=0;
     return 0;
  }
  if(x<xnode[1] || x>xnode[N])return 0;
  ib=1;
  ie=N;

  while(ie-ib>1){
	i=ib+((ie-ib)>>1);
	if(x<xnode[i])ie=i;
	else		  ib=i;
  }
  
  i=ib;
  xn=&xnode[i-2];
  
  if(i!=i0){
	temp=1.0/(xn[3]-xn[2]);
	A1=temp/(xn[3]-xn[1]);
	A3=temp/(xn[4]-xn[2]);
	A4=A3/(xn[5]-xn[2]);
	temp=xn[4]-xn[1];
	A2=A1/temp;
	A3/=temp;
	A1/=xn[3]-xn[0];
	i0=i;
  }
  
  X0=x-xn[0];
  X1=x-xn[1];
  X2=x-xn[2];
  X3=x-xn[3];
  X4=x-xn[4];
  X5=x-xn[5];
  
  XX1=X1*X1;
  XX2=X2*X2;
  XX3=X3*X3;
  XX4=X4*X4;
  
  temp=XX3*A1;
  B[0]=-temp*X3;
  B[1]=X0*temp;
  DB[0]=-3.0*temp;
  DB[1]=temp;
  temp=XX2*A4;
  B[3]=X2*temp;
  DB[3]=3.0*temp;
  B[2]=-X5*temp;
  DB[2]=-temp;
  
  temp=XX1*A2;
  B[2]-=X3*temp;
  DB[2]-=temp;
  
  temp=XX4*A3;
  B[1]+=X2*temp;
  DB[1]+=temp;
  
  temp=X1*X3*A2;
  
  B[1]+=X4*temp;
  
  DB[1]+=temp;
  DB[2]-=2.0*temp;
  
  temp=X1*X4;
  
  DB[1]+=temp*A2;

  temp*=A3;
  B[2]-=X2*temp;
  DB[2]-=temp;
  
  temp=X2*X4*A3;
  DB[1]+=2.0*(temp+X0*X3*A1)+X3*X4*A2;
  DB[2]-=temp+X2*(X1*A3+2.0*X5*A4);
/*
 * for 2nd derivative = 0
 */
  if(i==1){
     B[2]+=B[1]/3.0;
     B[1]=B[1]/1.5+B[0];
     DB[2]+=DB[1]/3.0;
     DB[1]=DB[1]/1.5+DB[0];
     B[0]=DB[0]=0.0;
  }
  if(i==2){
     B[1]+=B[0]/3.0;
     B[0]/=1.5;
     DB[1]+=DB[0]/3.0;
     DB[0]/=1.5;
  }
  if(i==N-1){
     B[1]+=B[2]/3.0;
     B[2]=B[2]/1.5+B[3];
     DB[1]+=DB[2]/3.0;
     DB[2]=DB[2]/1.5+DB[3];
     B[3]=DB[3]=0.0;
  }
	
  if(i==N-2){
     B[2]+=B[3]/3.0;
     B[3]/=1.5;
     DB[2]+=DB[3]/3.0;
     DB[3]/=1.5;
  }
 
  return i;
}
/* unwarrantted fortran version for calculating second derivative of BS
	subroutine ddbspl()
	
	dimension ddb(0:3),xnode(-1:nnode+2)
	integer ib0
	data ib0/0/
	save ib0,a1,a2,a3,a4
	
	if(ib.ne.ib0)then
	   temp=1.0/(xnode(ib+1)-xnode(ib))
	   a1=temp/(xnode(ib+1)-xnode(ib-1))
	   a2=a1
	   a3=temp/(xnode(ib+2)-xnode(ib))
	   a4=a3
	   a1=a1/(xnode(ib+1)-xnode(ib-2))
	   temp=xnode(ib+2)-xnode(ib-1)
	   a2=a2/temp
	   a3=a3/temp
         a4=a4/(xnode(ib+3)-xnode(ib))
	   ib0=ib
	endif
	
	x0=x-xnode(ib-2)		-2
	x1=x-xnode(ib-1)		-1
	x2=x-xnode(ib)		0
	x3=x-xnode(ib+1)		1
	x4=x-xnode(ib+2)		2
	x5=x-xnode(ib+3)
	
	temp=x1+x4
	
	ddb(0)=6.0*x2*a4	
	ddb(1)=(2.0*x1+x3)*a2+(temp+x2)*a3+(2.0*x2+x5)*a4
	ddb(2)=(2.0*x3+x0)*a1+(temp+x3)*a2+(x2+2.0*x4)*a3
	ddb(3)=-6.0*x3*a1

	ddb(1)=-2.0*ddb(1)
	ddb(2)=2.0*ddb(2)
	
	return
	end
*/

/*
 * to calculate cubic B-spline
 */
int BSplKer(float *B, float x, float *xnode, int N)
{
float  *xn,temp,temp1,X1,X2,X3,X4;
int i,ib,ie;
static float A1,A2,A3,A4;
static int i0=0;

  if(N<=0){		/* initialize, A1, A2 etc will be recalculated */
     i0=0;
     return 0;
  }
  if(x<xnode[1] || x>xnode[N])return 0;
  ib=1;
  ie=N;

  while(ie-ib>1){
	i=ib+((ie-ib)>>1);
	if(x<xnode[i])ie=i;
	else ib=i;
  }
  
  i=ib;
  xn=&xnode[i-2];
/*
  printf("xn[0] = %f\n", xn[0]);
  printf("xn[1] = %f\n", xn[1]);
  printf("xn[2] = %f\n", xn[2]);
  printf("xn[3] = %f\n", xn[3]);
  printf("xn[4] = %f\n", xn[4]);
  printf("xn[5] = %f\n", xn[5]);
*/  
  if(i!=i0){
	temp=1.0/(xn[3]-xn[2]);
	A1=temp/(xn[3]-xn[1]);
	A3=temp/(xn[4]-xn[2]);
	A4=A3/(xn[5]-xn[2]);
	temp=xn[4]-xn[1];
	A2=A1/temp;
	A3/=temp;
	A1/=xn[3]-xn[0];
	i0=i;
  }
  
  X1=x-xn[1];
  X2=x-xn[2];
  X3=x-xn[3];
  X4=x-xn[4];
  
  temp=X3*X3*A1;
  temp1=X3*X1*A2+X2*X4*A3;
  B[0]=-temp*X3;
  B[1]=(x-xn[0])*temp+X4*temp1;
  temp=X2*X2*A4;
  B[3]=X2*temp;
  B[2]=-X1*temp1-(x-xn[5])*temp;
/*
 * for 2nd derivative = 0
 */

  if(i==1){
     B[2]+=B[1]/3.0;
     B[1]=B[1]/1.5+B[0];
     B[0]=0.0;
  }
  if(i==2){
     B[1]+=B[0]/3.0;
     B[0]/=1.5;
  }
  if(i==N-1){
     B[1]+=B[2]/3.0;
     B[2]=B[2]/1.5+B[3];
     B[3]=0.0;
  }
	
  if(i==N-2){
     B[2]+=B[3]/3.0;
     B[3]/=1.5;
  }
 
  return i;
}

int BSplKer0(float *B, float x, float *xnode, int N)
{
float  *xn,temp,temp1,X1,X2,X3,X4;
int i,ib,ie;
float A1,A2,A3,A4;

  if(x<xnode[1] || x>xnode[N])return 0;
  ib=1;
  ie=N;

  while(ie-ib>1){
	i=ib+((ie-ib)>>1);
	if(x<xnode[i])ie=i;
	else		  ib=i;
  }
  
  i=ib;
  xn=&xnode[i-2];
  
  temp=1.0/(xn[3]-xn[2]);
  A1=temp/(xn[3]-xn[1]);
  A3=temp/(xn[4]-xn[2]);
  A4=A3/(xn[5]-xn[2]);
  temp=xn[4]-xn[1];
  A2=A1/temp;
  A3/=temp;
  A1/=xn[3]-xn[0];
  
  X1=x-xn[1];
  X2=x-xn[2];
  X3=x-xn[3];
  X4=x-xn[4];
  
  temp=X3*X3*A1;
  temp1=X3*X1*A2+X2*X4*A3;
  B[0]=-temp*X3;
  B[1]=(x-xn[0])*temp+X4*temp1;
  temp=X2*X2*A4;
  B[3]=X2*temp;
  B[2]=-X1*temp1-(x-xn[5])*temp;
/*
 * for 2nd derivative = 0
 */

  if(i==1){
     B[2]+=B[1]/3.0;
     B[1]=B[1]/1.5+B[0];
     B[0]=0.0;
  }
  if(i==2){
     B[1]+=B[0]/3.0;
     B[0]/=1.5;
  }
  if(i==N-1){
     B[1]+=B[2]/3.0;
     B[2]=B[2]/1.5+B[3];
     B[3]=0.0;
  }
	
  if(i==N-2){
     B[2]+=B[3]/3.0;
     B[3]/=1.5;
  }
 
  return i;
}

void tridag(float *a, float *b, float *c, float *r, float *u, int n)
{
int   j;
float bet,*gam;

   gam=Vector(1,n);
   if (b[1] == 0.0) printf("Error 1 in TRIDAG");
   u[1]=r[1]/(bet=b[1]);
   for (j=2;j<=n;j++) {
      gam[j]=c[j-1]/bet;
      bet=b[j]-a[j]*gam[j];
      if (bet == 0.0)   printf("Error 2 in TRIDAG");
      u[j]=(r[j]-a[j]*u[j-1])/bet;
   }
   for (j=(n-1);j>=1;j--)
      u[j] -= gam[j+1]*u[j+1];
   FreeVector(gam,1,n);
}

/*
 * to solve a positive definite sysmetric system ax=b
 *   a[1->n*(n+1)/2]        row-wise, lower triangle
 *   g[1->n*(n+1)/2]        cholesky storage
 *   b(1->n]                
 *   x[1->n]                answer vector
 *   n                      system dimension
 *   nono > 0 is the level at which p.d. failed
 *   (a,g) and (b,y,x) may be equivalenced.
 */
void choles(float *a, float *b, float *x, int n, int *nono)
{
float *y,*g;
int i,j,k,kj,kz,jmin,jmax,kmax;
float sg,gkz;
     
     y = Vector(1,n);
   /*  g = Vector(1,(n*(n+1))/2); */
     g=a;
     *nono = 0;
     if(a[1]<0.0){
        *nono = 1;
        return;
     }
     
     g[1]=sqrt(a[1]);
     y[1]=b[1]/g[1];
     
     for(i=2;i<=n;i++){
         kz=(i*(i-1))/2;
         g[kz+1]=a[kz+1]/g[1];
         sg=g[kz+1]*g[kz+1];
         y[i]=b[i]-g[kz+1]*y[1];
         if(i>2){
            jmax=i-1;
            for(j=2;j<=jmax;j++){
                gkz=a[kz+j];
                kj=(j*(j-1))/2;
                kmax=j-1;
      
                for(k=1;k<=kmax;k++)
                    gkz-=g[kz+k]*g[kj+k];
          
                g[kz+j]=gkz/g[kj+j];
                y[i]-=g[kz+j]*y[j];
                sg+=g[kz+j]*g[kz+j];
            }
         }
      
         gkz=a[kz+i]-sg;
      
         if(gkz<=0.0){
            *nono=i;
            return;
         }
      
         g[kz+i]=sqrt(gkz);
         y[i]/=g[kz+i];
      }
      
      kz=(n*(n-1))/2;
      x[n]=y[n]/g[kz+n];
      if(n<=1) return;
      
      for(k=2;k<=n;k++){
          i=n+1-k;
          x[i]=y[i];
          jmin=i+1;
      
          for(j=jmin;j<=n;j++){
              kj=(j*(j-1))/2;
              x[i]-=g[kj+i]*x[j];
          }

          kz=(i*(i+1))/2;
          x[i]/=g[kz];
      }
}

void FSwap(float *x, float *y)
{
float temp;

   temp=*x;
   *x=*y;
   *y=temp;
}

void DSwap(double *x, double *y)
{
double temp;

   temp=*x;
   *x=*y;
   *y=temp;
}

void ISwap(int *x, int *y)
{
int temp;

   temp=*x;
   *x=*y;
   *y=temp;
}


/*
 ******************************map functions****************************
 */
/*
 * to convert geographic latitude and longitude (in degree)
 * to spherical coordinate
 */
SPHERE Geo2Sph(float lat, float lon, float r)
{
SPHERE s;

   s.theta=(90.0-lat)*RAD;
   if(lon<0.0)lon+=360.0;
   s.phi  = lon*RAD;
   s.r    = r;
   
   return s;
}

/*
 * to convert geographic latitude and longitude (in degree)
 * to spherical coordinate
 */
SPHERE2 Geo2Sph2(float lat, float lon)
{
SPHERE2 s;

   s.theta=(90.0-lat)*RAD;
   if(lon<0.0)lon+=360.0;
   s.phi  = lon*RAD;
   
   return s;
}

/*
 * to convert spherical coordinate to
   geographic latitude and longitude (degree)
 */
void Sph2Geo(SPHERE s, float *lat, float *lon)
{
   *lat=90.0-s.theta/RAD;
   *lon=s.phi/RAD;
   if(*lon>180.0)*lon-=360.0;
}

/*
 * viewpt (theta, phi) to be the new z axis, the direction between new
 * ? axis to the old z axis to be  angle.
 */

TRANSMATRIX GetTrans1(CARTESIAN viewpt, float angle)
{
CARTESIAN x,y,z,x1,y1;
TRANSMATRIX t;
float ca,sa;

   z   = NormCarVec(viewpt);
   x.x =-viewpt.y;
   x.y = viewpt.x; 
   x.z = 0.0;
   x   = NormCarVec(x);
   y.x =-viewpt.x*viewpt.z;
   y.y =-viewpt.z*viewpt.y;
   y.z = viewpt.x*viewpt.x+viewpt.y*viewpt.y;
   y   = NormCarVec(y);
   
   ca  = cos(angle);
   sa  = sin(angle);
   
   x1.x =  ca*x.x+sa*y.x;
   x1.y =  ca*x.y+sa*y.y;
   x1.z =  ca*x.z+sa*y.z;
   
   y1.x = -sa*x.x+ca*y.x;
   y1.y = -sa*x.y+ca*y.y;
   y1.z = -sa*x.z+ca*y.z;
   
   t.x  = x1;
   t.y  = y1;
   t.z  = z;
   
   return t;
}
/*
 * point (theta, phi) to be the new x axis, the direction between new
 * y axis to the old z axis to be an angle azim.
 */
TRANSMATRIX GetTrans2(float azim, float theta, float phi)
{
CARTESIAN   x,y,z;
TRANSMATRIX t;
float ca,sa,ct,st,cp,sp;

  ca=cos(azim);
  sa=sin(azim);
  ct=cos(theta);
  st=sin(theta);
  cp=cos(phi);
  sp=sin(phi);
  
  x.x= st*cp;
  x.y= st*sp;
  x.z= ct;
  
  y.x=-ca*cp*ct-sa*sp;
  y.y= sa*cp-ca*ct*sp;
  y.z= ca*st;
  
  z.x= ca*sp-sa*ct*cp;
  z.y=-ca*cp-sa*ct*sp;
  z.z= sa*st;
  
  t.x=x;                         /* transformation matrix   */
  t.y=y;
  t.z=z;
  
  return t;
}


/*
 * to compute the transformation matrix t, which converts the first 
 * point to be on  the x axis and the second point to be on the x-y
 * plane on the  new coordinate.
 */
TRANSMATRIX GetTrans3(SPHERE s1, SPHERE s2)
{
CARTESIAN c1,c2,c3;
TRANSMATRIX t;

  c1=GetCartesian(s1);            /* direction of new x axis */
  c2=GetCartesian(s2);
  c3=OutProduct(c1,c2);           /* direction of new z axis */
  c2=OutProduct(c3,c1);           /* direction of new y axis */
  c1=NormCarVec(c1);              /* normalize vectors       */
  c2=NormCarVec(c2);
  c3=NormCarVec(c3);
  t.x=c1;                         /* transformation matrix   */
  t.y=c2;
  t.z=c3;
  
  return t;
}

/*
 * to compute the transformation matrix t, which converts the first 
 * point to be on  the x axis and the second point to be on the x-y
 * plane on the  new coordinate.
 */
TRANSMATRIX GetTrans4(SPHERE2 s1, SPHERE2 s2)
{
CARTESIAN c1,c2,c3;
TRANSMATRIX t;

  c1.x =sin(s1.theta);
  c1.y =c1.x*sin(s1.phi);
  c1.x*=cos(s1.phi);
  c1.z =cos(s1.theta);
  
  c2.x =sin(s2.theta);
  c2.y =c2.x*sin(s2.phi);
  c2.x*=cos(s2.phi);
  c2.z =cos(s2.theta);

  c3=OutProduct(c1,c2);           /* direction of new z axis */
  c2=OutProduct(c3,c1);           /* direction of new y axis */
  c1=NormCarVec(c1);              /* normalize vectors       */
  c2=NormCarVec(c2);
  c3=NormCarVec(c3);
  t.x=c1;                         /* transformation matrix   */
  t.y=c2;
  t.z=c3;
  
  return t;
}

/*
 * point (theta, phi) to be the new z axis, the direction between new
 * y axis to the old z axis to be an angle azim.
 */
TRANSMATRIX GetTrans5(float azim, float theta, float phi)
{
CARTESIAN   x,y,z;
TRANSMATRIX t;
float ca,sa,ct,st,cp,sp;

  ca=cos(azim);
  sa=sin(azim);
  ct=cos(theta);
  st=sin(theta);
  cp=cos(phi);
  sp=sin(phi);
  
  z.x= st*cp;
  z.y= st*sp;
  z.z= ct;
  
  y.x=-ca*cp*ct-sa*sp;
  y.y= sa*cp-ca*ct*sp;
  y.z= ca*st;
  
  x.x=-ca*sp+sa*ct*cp;
  x.y= ca*cp+sa*ct*sp;
  x.z=-sa*st;
  
  t.x=x;                         /* transformation matrix   */
  t.y=y;
  t.z=z;
  
  return t;
}

/*
 * to compute the new coordinate under a transformation t 
 */
CARTESIAN   Transform1(TRANSMATRIX t, CARTESIAN c)
{
CARTESIAN   c1;

  c1.x=t.x.x*c.x+t.x.y*c.y+t.x.z*c.z;
  c1.y=t.y.x*c.x+t.y.y*c.y+t.y.z*c.z;
  c1.z=t.z.x*c.x+t.z.y*c.y+t.z.z*c.z;
  
  return c1;
}

/*
 * to compute the new coordinate under an inverse transformation t 
 */
CARTESIAN   InvTransform1(TRANSMATRIX t, CARTESIAN c)
{
CARTESIAN   c1;

  c1.x=t.x.x*c.x+t.y.x*c.y+t.z.x*c.z;
  c1.y=t.x.y*c.x+t.y.y*c.y+t.z.y*c.z;
  c1.z=t.x.z*c.x+t.y.z*c.y+t.z.z*c.z;
  
  return c1;
}

/*
 * to compute the new coordinate under a inverse transformation t ( get
 * from function get_trans() ).
 */
SPHERE InvTransform2(TRANSMATRIX t, SPHERE s)
{
CARTESIAN c,c1;

  c=GetCartesian(s);
  c1.x=t.x.x*c.x+t.y.x*c.y+t.z.x*c.z;
  c1.y=t.x.y*c.x+t.y.y*c.y+t.z.y*c.z;
  c1.z=t.x.z*c.x+t.y.z*c.y+t.z.z*c.z;
  s=GetSphere(c1);
  return s;
}

CARTESIAN_2 DoProject(CARTESIAN x, float viewptz)
{
CARTESIAN_2 x1;
float fac;

   fac=viewptz/(viewptz-x.z);

   x1.x=x.x*fac;
   x1.y=x.y*fac;
   return x1;
}

/*
 * This is a function to  allocate memory and read the coastline data.
 */ 
short *ReadCoast(int flag, int size)
{
int  i,fd,n_read;
short *pt;
/*
 * Allocate enough memory to keep the coastline data
 */
     pt=ShortVec(size);
/*
 * Open data file and read data into the allocated memory  
 */      
  if     (flag==COAST   ){
     fd=SysOpen("coast.dat",0);
     printf("Reading coastlines\n");
  }else if(flag==BOUNDARY){
     fd=SysOpen("boundary.dat",0);
     printf("Reading plate boundaries\n");
  }
     
  lseek(fd,0L,0);              /* begining of the file */
  n_read=read(fd,pt,size*sizeof(short));
  if(n_read!=size*sizeof(short)){
     if(flag==COAST   ){
        printf("Error in read file coast.dat\n"); 
        exit(1);
     }
     if(flag==BOUNDARY){
        printf("Error in read file boundary.dat\n");
        exit(1);
     }
  }
   
  return pt;    
}


/*
 * to compute the new coordinate under a transformation t ( get
 * from function get_trans() ).
 */
SPHERE Transform2(TRANSMATRIX t, SPHERE s)
{
CARTESIAN c,c1;

  c=GetCartesian(s);
  c1.x=t.x.x*c.x+t.x.y*c.y+t.x.z*c.z;
  c1.y=t.y.x*c.x+t.y.y*c.y+t.y.z*c.z;
  c1.z=t.z.x*c.x+t.z.y*c.y+t.z.z*c.z;
  s=GetSphere(c1);
  return s;
}

/*
 *****************************seismic functions*************************
 */
/*
 * read PREM model coefficients
 */
void  ReadPrem(MODEL *model)
{
PREM  *par;      
int  size,fd,n_read;
char *premname;

   premname=model->name2;
   par     =model->par;
   
   fd=SysOpen(premname,0);
   size=sizeof(PREM)*(LAYER+1);
   n_read=read(fd,&par[1],size);
   if(n_read!=size)printf(" Error in read PREM data file\n");
   close(fd);
}

/*
 * calculate velocity or density for PREM
 */
void  PremVelDen(float r, float *vd, PREM *par, char choice)
{
float vn,rn,xa,xrho,t,avgd;
int n=LAYER;

  vn  =0.001*par[LAYER+1].rho1;
  avgd=0.001*par[LAYER+1].rho3;
     
  rn=r/RADIUS;
  while(rn>par[n--].r || rn<par[n].r);
  t=rn-par[n].r;
  xrho=par[n].rho0+t*(par[n].rho1+t*(par[n].rho2+t*par[n].rho3));
  
  if       (choice=='p' || choice=='P'){                       /* Vph */ 
      xa=par[n].a0+t*(par[n].a1+t*(par[n].a2+t*par[n].a3));
      *vd =vn*sqrt(xa/xrho);
  } else if(choice=='s' || choice=='S'){                       /* Vsv */
      xa=par[n].n0+t*(par[n].n1+t*(par[n].n2+t*par[n].n3));
      *vd =vn*sqrt(xa/xrho);
  } else if(choice=='d' || choice=='D'){
      *vd =avgd*xrho;
  } else {
    printf("Error in function PremVelDen\n");
    exit(1);
  }
}



/*
 * allocate space for model, and read model
 */
void  GetModel(MODEL *model, int flag)
{
    while(!SelModel(model));     /* select model                  */
    DetPara(model);              /* determine parameterization    */
    AllocCoeffSp(model);         /* space for model coefficients  */
    if(flag==TRUE){              /* read data, otherwise allocate */
       ReadModel(model);         /* space for the model only      */
    /*   if(model->mod!=ICANI)
       PrintModel(*model,FALSE); */
    }
    
}

void DetPara(MODEL *model)
{
int lmaxum,lmaxlm,kmaxum,kmaxlm,lmax,llmax;

    lmaxum=lmaxlm=0;
    
    printf(" selected model=%d\n",model->mod);
    
    if(model->mod==SH425   || model->mod==SH8U4L8  || model->mod==ULMODEL ||
       model->mod==SH8WM13 || model->mod==SH12WM13 || model->mod==WMMODEL ||
       model->mod==M84C    || model->mod==M84A     || model->mod==WMMK){
       Read1stLine(model);
       model->dim    = 3;
       if(model->mod==M84C || model->mod==M84A){
          model->velfm  = DV2;
          model->rtop   = RMOHO;
          model->rmid   = R670;
          model->rbot   = R670;
          model->mantle = UPPERMANTLE;
       }else if(model->mod==WMMK){   /* whole mantle mu kappa model */ 
          model->velfm  = DEM;
          model->rtop   = RMOHO;
          model->rmid   = R670;
          model->rbot   = RCMB;
          model->mantle = WHOLEMANTLE;
       }else{
          model->velfm  = DVOVFORMAT;
          model->rtop   = RMOHO;
          model->rmid   = R670;
          model->rbot   = RCMB;
       }
    }else if(model->mod==ICHET){   /* inner core heterogeneity model */
       Read1stLine(model);
       model->dim    = 3;
       model->velfm  = DVOVFORMAT;
       model->rtop   = RICB;
       model->rmid   = 0.0;
       model->rbot   = 0.0;
       model->mantle = INNERCORE;
    }else if(model->mod==ICANI){
       ReadICAniDt(model,1);       /* read in some information */
       model->dim    = 3;
       model->velfm  = DT;
       model->rtop   = 120.0;      /* epicentral distance      */
       model->rmid   = 0.0;        /* not used                 */
       model->rbot   = 180.0;
       model->mantle = INNERCORE;  /* Inner core               */
    }else if(model->mod==MDLSH){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = SMODEL;       /* S-velocity model               */
       model->velfm  = DVOVFORMAT;   /* perturbation is dv/v           */
       model->rtop   = RMOHO;        /* top is moho                    */
       model->rmid   = R670;         /* mid is not used                */
       model->rbot   = RCMB;         /* bottom is cmb                  */
       model->mantle = WHOLEMANTLE;  /* whole mantle model             */
       model->lmaxum =  6;           /* whole mantle max angular order */
       model->kmaxum = 10;           /* whole mantle max radial  order */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun = SHELL;        /* model is defined as SHELLs     */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod==ZT36){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = SMODEL;       /* S-velocity model               */
       model->velfm  = DVOVFORMAT;   /* perturbation is dv/v           */
       model->rtop   = RADIUS;       /* top is Earth RADIUS            */
       model->rmid   = 0.0;          /* mid is not used                */
       model->rbot   = RADIUS-500.0; /* bottom is 500 km               */
       model->mantle = UPPERMANTLE;  /* upper mantle model             */
       model->lmaxum = 36;           /* upper mantle max angular order */
       model->kmaxum = 24;           /* upper mantle max radial  order */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun = SHELL;        /* model is defined as SHELLs     */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod==MB92){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = SMODEL;       /* S-velocity model               */
       model->velfm  = DVOVFORMAT;   /* perturbation is dv/v           */
       model->rtop   = RMOHO;        /* top is moho                    */
       model->rmid   = R670;         /* mid is not used                */
       model->rbot   = RCMB;         /* bottom is cmb                  */
       model->mantle = WHOLEMANTLE;  /* whole mantle model             */
       model->lmaxum = 8;            /* whole mantle max angular order */
       model->kmaxum = 10;           /* whole mantle max radial  order */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun = SHELL;        /* model is defined as SHELLs     */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod==INOUE){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = PMODEL;       /* p-velocity model               */
       model->velfm  = DVOVFORMAT;   /* perturbation is dv/v           */
       model->rtop   = RADIUS;       /* top is Earth RADIUS            */
       model->rmid   = 0.0;          /* mid is not used                */
       model->rbot   = RADIUS-2900.0;/* bottom depth is 2900 km        */
       model->mantle = WHOLEMANTLE;  /* whole mantle model             */
       model->lmaxum = 36;           /* whole mantle max angular order */
       model->kmaxum = 15;           /* whole mantle max radial  order */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun = SHELL;        /* model is defined as SHELLs     */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod==SAF){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = SMODEL;       /* p-velocity model               */
       model->velfm  = DVOVFORMAT;   /* perturbation is dv/v           */
       model->rtop   = RADIUS-3.0;   /* top is sea floor               */
       model->rmid   = 0.0;          /* mid is not used                */
       model->rbot   = RCMB;         /* bottom depth is 2900 km        */
       model->mantle = WHOLEMANTLE;  /* whole mantle model             */
       model->lmaxum =  4;           /* whole mantle max angular order */
       model->kmaxum =  6;           /* whole mantle max radial  order */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun = LEGENDRE;     /* model is defined as SHELLs     */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod== L0256){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = PMODEL;       /* p-velocity model               */
       model->velfm  = DVFORMAT;     /* perturbation is dv/v           */
       model->rtop   = R670;         /* top is 670 discontinuity       */
       model->rmid   = 0.0;          /* mid is not used                */
       model->rbot   = RCMB;         /* bottom depth is 2900 km        */
       model->mantle = LOWERMANTLE;  /* lower mantle model             */
       model->lmaxlm =  6;           /* lower mantle max angular order */
       model->kmaxlm =  4;           /* lower mantle max radial  order */
       model->lmaxum =  0;           /* not used                       */
       model->kmaxum =  0;           /* not used                       */
       model->radfun = LEGENDRE;     /* radial fun is   LEGENDRE       */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod== SURCOM){
       model->dim    = 2;            /* 2-dimensional model            */
       model->pors   = UNKNOWN;      /* p-velocity model               */
       model->velfm  = DVFORMAT;     /* perturbation is dv/v           */
       model->rtop   = 0.0;          /* top                            */
       model->rmid   = 0.0;          /* mid                            */
       model->rbot   = 0.0;          /* bottom                         */
       model->mantle = SURFACE;      /* not used                       */
       model->lmaxum = 128;          /*  max angular order             */
       model->kmaxum =  0;           /* not used                       */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun =  0;           /*                                */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod== CRUSTMD){
       model->mantle = SURFACE;      /* no radial function is used     */
       Read1stLine(model);
       model->dim    = 2;            /* 2-dimensional model            */
       model->pors   = UNKNOWN;      /* not defined                    */
       model->velfm  = DH;           /* boundary perturbation          */
       model->rtop   = 0.0;          /* top is not defined             */
       model->rmid   = 0.0;          /* mid is not defined             */
       model->rbot   = 0.0;          /* bottom is not defined          */
       model->radfun =  0;           /*                                */
    }else if(model->mod==MRD){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = DMODEL;       /* S-velocity model               */
       model->velfm  = DD;           /* perturbation is delta d        */
       model->rtop   = RADIUS;       /* top is Earth RADIUS            */
       model->rmid   = 0.0;          /* mid is not used                */
       model->rbot   = RADIUS-2900.0;/* bottom is cmb                  */
       model->mantle = WHOLEMANTLE;  /* whole mantle model             */
       model->lmaxum = 20;           /* whole mantle max angular order */
       model->kmaxum = 19;           /* whole mantle max radial  order */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->radfun = SHELL;        /* model is defined as shell      */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */
    }else if(model->mod== BSPL3){
       model->dim    = 3;            /* 3-dimensional model            */
       model->pors   = UNKNOWN;      /* S-velocity model               */
       model->velfm  = OTHERFT;      /* perturbation is other format   */
       model->mantle = REGIONAL;     /* regional model                 */
    }else if(model->mod== GRDM){
       model->dim    = 2;            /* 3-dimensional model            */
       model->pors   = UNKNOWN;      /* S-velocity model               */
       model->velfm  = OTHERFT;      /* perturbation is other format   */
       model->mantle = WHOLEMANTLE;  /* whole mantle model             */
    }else if(model->mod== PREMDL){
       model->dim    = 1;            /* 1-dimensional model            */
       model->pors   = PREMODEL;     /* prem model                     */
       model->velfm  = PREMFORMAT;   /* expressed as PREM format       */
       model->rtop   = RADIUS;       /* top is Earth's Radius          */
       model->rmid   = 0.0;          /* mid is not used                */
       model->rbot   = 0.0;          /* bottom is the center of Earth  */
       model->mantle = WHOLEEARTH;   /* whole Earth  model             */
       model->lmaxlm =  0;           /* not used                       */
       model->kmaxlm =  0;           /* not used                       */
       model->lmaxum =  0;           /* not used                       */
       model->kmaxum =  0;           /* not used                       */
       model->radfun = SHELL;        /* layered model                  */
       model->crust  = FALSE;        /* doesnot contains crust pert    */
       model->b670   = FALSE;        /* doesnot contains 670   pert    */    
    }else if(model->mod== GRAND){
       return;   /* everything is done in ReadGrand */
    }else if(model->mod== SPHBSPL2 || model->mod == SPHBSPL3){
       return;   /* everything is done in ReadSPHBSPL2 */
    }else if(model->mod== BSPSPLT){
       return;   /* everything is done in ReadBSPSPLT */
    }else if(model->mod== BSMULTISPLIT){
       printf("Read model ...\n");
       return;   /* everything is done in ReadBSPSPLT */
    }else {
       printf(" not complete yet\n");
       exit(1);
    }
}

void AllocCoeffSp(MODEL *model)
{
int lmaxum,lmaxlm,kmaxum,kmaxlm,lmax,llmax;

       if(model->mod==BSPL3){
          model->xmima=Vector(0,5);  /* to save xmin, xmax, ymin, ymax, zmin, zmax */
          model->ixyz =IntVec(0,3);
          return;
       }else if(model->mod==GRAND 
	|| model->mod==SPHBSPL2 
	|| model->mod == SPHBSPL3 
	|| model->mod==GRDM 
	|| model->mod==BSPSPLT 
	|| model->mod==BSMULTISPLIT){
          model->xmima=Vector(0,5);
          model->ixyz =IntVec(0,3);
          return;         /* everything is done in read functions */
       }
       if(model->mod==ICANI){
          lmaxum     = model->lmaxum;
          kmaxum     = model->kmaxum;
          llmax      = (lmaxum+2)*(lmaxum+1)/2-1;       
          model->au  = Matrix(0,kmaxum,0,llmax);   
          model->bu  = Matrix(0,kmaxum,0,llmax);
          model->ar  = Vector(0,llmax);          /* storage */
          model->br  = Vector(0,llmax);
          model->ylm = Vector(0,llmax);
          return;
       }
       lmaxum     = 0;
       lmaxlm     = 0;
/* 
 * allocate memory for upper mantle, whole mantle or inner core model coeffcients
 */   
    if(model->mantle==UPPERMANTLE ||  model->mantle==UPPERLOWERM ||
       model->mantle==WHOLEMANTLE ||  model->mantle==INNERCORE){
       lmaxum     = model->lmaxum;
       kmaxum     = model->kmaxum;
       llmax      = (lmaxum+2)*(lmaxum+1)/2-1;       
       model->au  = Matrix(0,kmaxum,0,llmax);   
       model->bu  = Matrix(0,kmaxum,0,llmax);
       model->ac  = Vector(0,llmax);   
       model->bc  = Vector(0,llmax);
       model->a670= Vector(0,llmax);
       model->b670= Vector(0,llmax);
       model->ylm = Vector(0,llmax);
       if(model->radfun==BSPLINE){
          model->xmima=Vector(0,kmaxum);
       }
    }else if(model->mantle==SURFACE){
       lmaxum     = model->lmaxum;
       llmax      = (lmaxum+2)*(lmaxum+1)/2-1;       
       model->ac  = Vector(0,llmax);   
       model->bc  = Vector(0,llmax);
       model->a670= Vector(0,llmax);
       model->b670= Vector(0,llmax);
       model->ylm = Vector(0,llmax);
    }
/* 
 * allocate memory for lower mantle
 */ 
    if(model->mantle==LOWERMANTLE || model->mantle==UPPERLOWERM){
       lmaxlm    = model->lmaxlm;
       kmaxlm    = model->kmaxlm;
       llmax     =(lmaxlm+2)*(lmaxlm+1)/2-1;       
       model->al = Matrix(0,kmaxlm,0,llmax);   
       model->bl = Matrix(0,kmaxlm,0,llmax);
    }
/*
 * allocate memory for RADIUS at knots and avarage velocity
 */
    if(model->mod == ZT36 || model->mod == INOUE){
       kmaxum        = model->kmaxum;
       model->rknot  = Matrix(0,1,0,kmaxum);
       model->vavg   = Vector(0,kmaxum);
    }else if(model->mod == MB92 || model->mod == MDLSH ||
             model->mod == MRD ){
       kmaxum        = model->kmaxum;
       model->rknot  = Matrix(0,1,0,kmaxum);
    }
/*
 * allocate memory for spherical earth model PREM
 */
    model->par=PremVec(1,LAYER+1);
/*
 * allocate memory for temp storage
 */   
    lmax  = max(lmaxum,lmaxlm);
    llmax =(lmax+2)*(lmax+1)/2-1;  
    model->ar=Vector(0,llmax);
    model->br=Vector(0,llmax);
    
}

int SelModel(MODEL *model)
{
static char *defname[29]={ 
       "prem.dat",
       "m84a.dat",
       "m84c.dat",
       "l02_56.dat",
       "v3.dat",
       "sh425.2.dat",
       "sh8wm13.dat",
       "sh8u4l8.dat",
       "sh12wm13.dat",
       "ulmodel.dat",
       "wmmodel.dat",
       "surcom.dat",
       "mdlsh.dat",
       "zhangmodel.dat",
       "mbmodel.dat",
       "inoue_sph.dat",
       "SAF.dat",
       "gridm.dat",
       "richards_d.dat",
       "dtaniso.dat",
       "crust.dat",
       "grand.dat",
       "kappamu.dat",
       "ichetero.dat",
       "sphbspline.dat",
       "sphbsp2d.dat",
       "bspline.dat",
       "bspline_split.dat",
       "bspline_multisplit.dat"
        };
static char *mdldsc[29]={
       "PREM spherical Earth's model         ",
       "M84A,    upper mantle model          ",
       "M84C,    upper mantle model          ",
       "L02.56,  lower mantle model          ",
       "V3,      lower mantle model          ",
       "SH425.2, upper and lower mantle model",
       "SH8WM13, whole mantle model          ",
       "SH8U4L8, upper and lower mantle model",
       "S12_WM13, whole mantle model         ",
       "Harvard  upper-lower mantle model    ",
       "Harvard format whole mantle model    ",
       "Complex spherical  harmonics model   ",
       "Tanimoto's 11 layer model (MDLSH)    ",
       "Zhang & Tanimoto degree 36 model     ",
       "Masters & Bolton's model             ",
       "Inoue's model in spherical harmonics ",
       "SAF or SAT model   (Li, et al.)      ",
       "Grid value model                     ",
       "Mard Richard's density model         ",
       "Travel-time res due to IC anisotropy ",
       "Crust Model                          ",
       "Steve Grand's Model                  ",
       "Mu & Kappa model                     ",
       "Inner core heterogeneity model       ",
       "B-spline model (3D, spherical)       ",
       "B-spline model (2D, spherical)       ",
       "B-spline model (3D, cartesian)       ",
       "Split B-spline model (3D, spherical) ",
       "Multiply-Split B-spline model (3D)   "
       };

static char name1[MAXSTR];
char s[MAXSTR];
int i,sel=-1;

    printf("---------------------------------------------\n");
    for(i=PREMDL;i<=BSMULTISPLIT;i++)
        printf( "|%2d----%s|\n",i,mdldsc[i]);
    printf("---------------------------------------------\n");
    
    printf("select  model:");
    sel = CGetInt();
          
    if(sel>=PREMDL  && sel<= BSMULTISPLIT){
       model->dsc = mdldsc[sel];
       printf("model description = %s\n",model->dsc);
       printf("the name of the model file (%s)\n",defname[sel]);
       printf("hit return to use default name:");
       GetLine(s,MAXSTR);
       if     (sel==L0256 || sel==ICHET) model->pors=PMODEL;
       else if(sel==MRD  ) model->pors=DMODEL;
       else if(sel==PREMDL)model->pors=PREMODEL;
       else                model->pors=SMODEL;
       if(s[0]=='\n'){
          model->name1=defname[sel];
          if(sel==PREMDL)model->name2=defname[sel];
       }else{
          sscanf(s,"%s",name1);
          model->name1=name1;
       }
    } else {
          return FALSE;
    }
    model->mod=sel;
/*
 * get file name for the PREM parameter
 */
    if(sel==SH12WM13 || sel==SH8WM13 || sel==WMMODEL || sel==L0256 ||
       sel==SH425    || sel==SH8U4L8 || sel==ULMODEL || sel==MRD   ||
       sel==M84C     || sel==M84A    || sel==WMMK){
/*       printf("the name of PREM parameter file (%s)\n",defname[PREMDL]);
       printf("hit return to use default name:");
       GetLine(s,MAXSTR); */
       s[0]='\n';
       if(s[0]=='\n'){
          model->name2=defname[PREMDL];
       }else {
          sscanf(s,"%s",name1);
          model->name2=name1;
       }
    }
    return TRUE;
}

/*
 * Read model of travel time residual due to the inner core anisotropy
 */
void  ReadICAniDt(MODEL *model, int flag)
{
char  c,s[132],*degmask,*layermask;
int   i,k,kmax,lmax,maxlayer,radfun;
float **au,**bu;
FILE *fp;


   if(flag==1){
      layermask = model->layermask;
      degmask   = model->degmask;
   
      if((fp=fopen(model->name1,"r"))==NULL){
         printf("cannot open file %s.\n",model->name1);
         exit(1);
      }
    
      fscanf(fp,"%d %s %d %s %d",&lmax,degmask,&maxlayer,layermask,&radfun);
      fclose(fp);
   
      if(radfun!=3){
        printf("error, unknown radial function %d\n",radfun);
        exit(1);
      }
   
      model->lmaxum = lmax;       /* max degree */
   
      kmax = i=0;
      while((c=layermask[i++])!='\0')kmax++;
      model->kmaxum = kmax-1;
      printf("ani max degree =%d\n",model->lmaxum);
      printf("ani max k =%d\n",model->kmaxum);
   }else if(flag==2){
      fp=OpenFl(model->name1);
      degmask   = model->degmask;
      FGetLine(fp,s,MAXSTR);                       /* skip the first line */
      au = model->au;
      bu = model->bu;
      lmax=model->lmaxum;
      kmax=model->kmaxum;
      for(k=0;k<=kmax;k++)                         /* read data   */
          ReadLayer(fp,au[k],bu[k],lmax,degmask);

   }
}

/*
 * to calculate PREM P-wave velocity
 */
void VdvPREM_P(float r, float *v, float *dvdr, int zone)
{
float x;

  x=r/RADIUS;
      
  switch(zone){
    case 1:                          /* inner core */   
       *v    = 11.2622-6.3640*x*x;
       *dvdr =-12.7280*x;
       break;
    case 2:
       *v    = 11.0487+(-4.0362+(4.8023-13.5732*x)*x)*x;
       *dvdr = -4.0362+(9.6046-40.7196*x)*x;
       break;
    case 3:
       *v    = 15.3891+(-5.3181+(5.5242-2.5514*x)*x)*x;
       *dvdr = -5.3181+(11.0484-7.6542*x)*x;
       break;
    case 4:
       *v    = 24.9520+(-40.4673+(51.4832-26.6419*x)*x)*x;
       *dvdr =-40.4673+(102.9664-79.9257*x)*x;
       break;
    case 5:        
       *v    = 29.2766+(-23.6027+(5.5242-2.5514*x)*x)*x;
       *dvdr =-23.6027+(11.0484-7.6542*x)*x;
       break;
    case 6:
       *v    = 19.0957-9.8672*x;
       *dvdr = -9.8672;
       break;
    case 7:
       *v    = 39.7027-32.6166*x;
       *dvdr =-32.6166;
       break;
    case 8:
       *v    = 20.3926-12.2569*x;
       *dvdr =-12.2569;
       break;
    case 9:
       *v    =  4.1875+3.9382*x;
       *dvdr =  3.9382;
       break;
    case 10:
       *v    =  4.1875+3.9382*x;
       *dvdr =  3.9382;
       break;
    case 11:
       *v    =  6.8;
       *dvdr =  0.0;
       break;
    case 12:
       *v    =  5.8;
       *dvdr =  0.0;
       break;
    case 13:
       *v    =  1.45;
       *dvdr =  0.0;
  }
  *dvdr/=RADIUS;
  
}       

/*
 * to calculate PREM S-wave velocity
 */
void VdvPREM_S(float r, float *v, float *dvdr, int zone)
{
float x;

  x=r/RADIUS;
      
  switch(zone){
    case 1:                          /* inner core */   
       *v    =  3.6678-4.4475*x*x;
       *dvdr = -8.8895*x;
       break;
    case 2:
       *v     =  0.0;
       *dvdr  =  0.0;
       break;
    case 3:
       *v    =  6.9254+(1.4672+(-2.0834+0.9783*x)*x)*x;
       *dvdr =  1.4672+(-4.1668+2.9349*x)*x;
       break;
    case 4:
       *v    = 11.1671+(-13.7818+(17.4575-9.2777*x)*x)*x;
       *dvdr =-13.7818+(34.915-27.8331*x)*x;
       break;
    case 5:        
       *v    = 22.3459+(-17.2473+(-2.0834+0.9783*x)*x)*x;
       *dvdr =-17.2473+(-4.1668+2.9349*x)*x;
       break;
    case 6:
       *v    =  9.9839-4.9324*x;
       *dvdr = -4.9324;
       break;
    case 7:
       *v    = 22.3512-18.5856*x;
       *dvdr =-18.5856;
       break;
    case 8:
       *v    =  8.9496-4.4597*x;
       *dvdr = -4.4597;
       break;
    case 9:
       *v    =  2.1519+2.3481*x;
       *dvdr =  2.3481;
       break;
    case 10:
       *v    =  2.1519+2.3481*x;
       *dvdr =  2.3481;
       break;
    case 11:
       *v    =  3.9;
       *dvdr =  0.0;
       break;
    case 12:
       *v    =  3.2;
       *dvdr =  0.0;
       break;
    case 13:
       *v    =  0.0; 
       *dvdr =  0.0;
  }
  *dvdr/=RADIUS;
  
} 

int GetZone(float r)
{
static float rz[14]= {    0.0, 1221.49, 3479.96, 3630.0, 5600.0,
                       5701.0, 5771.00, 5971.00, 6151.0, 6219.0,
                       6346.63, 6356.00, 6368.00, 6371.0};
int i;

   if(r<0.0)return 0;
    
   for(i=1;i<=13;i++){
       if(r>rz[i-1] && r<=rz[i])break;
   }
   return i;
    
} 
void Read1stLine(MODEL *model)
{
char  c,*degmask,*layermask;
int    i,lmax,maxlayer,radfun,kmaxum,kmaxlm;
FILE  *fp;
    
   layermask = model->layermask;
   degmask   = model->degmask;
   printf("model->name1=%s\n", model->name1);
   if((fp=fopen(model->name1,"r"))==NULL){
      printf("cannot open file %s.\n",model->name1);
      exit(1);
   }
    
   fscanf(fp,"%d %s %d %s %d",&lmax,degmask,&maxlayer,layermask,&radfun);
   printf("%d %s %d %s %d\n",lmax,degmask,maxlayer,layermask,radfun);
   fclose(fp);
   
   if(layermask[0]=='1')model->crust = TRUE;
   else                 model->crust = FALSE;
   
   if(layermask[1]=='1')model->p670  = TRUE;
   else                 model->p670  = FALSE;

   if     (radfun==1)radfun=LEGENDRE;
   else if(radfun==2)radfun=CHEBYSHEV;
   else if(radfun==4)radfun=BSPLINE;
   else {
      printf("error, unknown radial function %d\n",radfun);
      exit(1);
   }
   
   if(model->mod==CRUSTMD){
      model->lmaxum = lmax;         /* crust     lmax                       */
      model->lmaxlm = 0;            /* lower mantle lmax not used           */
      model->kmaxum = 0;            /* upper mantle kmax not used           */
      model->kmaxlm = 0;            /* lower mantle kmax not used           */
      return;
   }
   
   
   i = 2;
   kmaxum=0;
   kmaxlm=0;
   while((c=layermask[i++])!='\0'){
        if(c=='1')kmaxum++;
        if(c=='2')kmaxlm++;
   }
   printf("kmaxum = %d\n", kmaxum);
   printf("kmaxlm = %d\n", kmaxlm);
/*
 * some error checking
 */
   if((model->mod==SH425||model->mod==SH8U4L8||model->mod==ULMODEL) && 
       (kmaxlm==0||kmaxum==0)){
       printf("error in reading upper-lower mantle model\n");
       printf("upper mantle kmax=%d\n",kmaxum);
       printf("lower mantle kmax=%d\n",kmaxlm);
       exit(1);
   }else if((model->mod==SH8WM13||model->mod==SH12WM13||model->mod==WMMODEL)&&
       kmaxlm!=0){
       printf("error in reading whole mantle model\n");
       printf("upper mantle kmax=%d\n",kmaxum);
       printf("lower mantle kmax=%d\n",kmaxlm);
       exit(1);
   }
       
       
   if(kmaxlm==0 && kmaxum!=0){
      model->mantle = WHOLEMANTLE;
      model->kmaxum = kmaxum-1;            /* upper mantle or whole mantle kmax    */
      model->lmaxum = lmax;                /* whole mantle lmax                    */
      model->lmaxlm = 0;                   /* lower mantle lmax not used           */
   }else if(kmaxlm!=0 && kmaxum!=0){
      model->mantle = UPPERLOWERM;
      model->lmaxum = lmax;                /* upper mantle lmax                    */
      model->lmaxlm = lmax;                /* lower mantle lmax                    */
      model->kmaxum = kmaxum-1;            /* upper mantle kmax                    */
      model->kmaxlm = kmaxlm-1;            /* lower mantle kmax                    */
   }
   model->radfun = radfun;                 /* radial function                      */
   fclose(fp);
}

void ReadModel(MODEL *model)
{
     if(model->mod==SH425   || model->mod==SH8U4L8  || model->mod==ULMODEL||
        model->mod==SH8WM13 || model->mod==SH12WM13 || model->mod==WMMODEL||
        model->mod==M84C    || model->mod==M84A     || model->mod==WMMK ||
        model->mod==ICHET){
        ReadHdModel(model);
        if(model->mod!=ICHET)ReadPrem(model); 
     } else if(model->mod==MDLSH){
        ReadMDLSH(model);
     } else if(model->mod==ZT36){
        ReadZT36(model); 
     } else if(model->mod==INOUE){    /* Inoue's spherical harm model */

     } else if(model->mod==MB92){
        ReadMB92(model);
     } else if(model->mod==SAF){
        ReadSAF(model);
     } else if(model->mod==L0256){
        ReadL0256(model);
        ReadPrem(model);
     } else if(model->mod==SURCOM){
        ReadSurCoeff(model);
     } else if(model->mod==MRD){
        ReadMRD(model);
        ReadPrem(model);
     } else if(model->mod==PREMDL){
        ReadPrem(model);
     } else if(model->mod==GRDM){
        ReadGrid(model);
     } else if(model->mod==BSPL3){
      /*  ReadBspline3(model); */
        RdBSCartMd(model);
     } else if(model->mod==ICANI){
        ReadICAniDt(model,2);
     } else if(model->mod==CRUSTMD){
        ReadHdModel(model);
     } else if(model->mod==GRAND){
        ReadGrand(model);
     } else if(model->mod==SPHBSPL2 || model->mod==SPHBSPL3){
	/* 3D or 2D continuous splines */
        ReadSPHBSPL2(model);
     } else if(model->mod==BSPSPLT){
        ReadBSPSPLT(model);
     } else if(model->mod==BSMULTISPLIT) { 
	/* B-spline model with multiple splits */
	ReadBSMULTISPLIT(model);
     }else {
       printf("ReadModel has not completed for this option yet!\n");
       exit(1);
     }
}

void ReadSurCoeff(MODEL *model)
{
int l,m,l1,m1,lm,lmax;
float aa, bb, *ac, *bc;
FILE *fp;
char line[132];

  printf("Model name %s\n",model->name1);
  fp=OpenFl(model->name1);
  printf("angular order lmax =");
  lmax = CGetInt();
  if(lmax>model->lmaxum)exit(1);
/*  lmax= model->lmaxlm; */
  model->lmaxum=model->lmaxlm=lmax;
  ac  = model->ac;
  bc  = model->bc;

  for(l=0;l<=lmax;l++)           /* set degree mask   */
      (*model).degmask[l]='1';
  
  (*model).degmask[lmax+1]='\0';

  for(lm=0,l=0;l<=lmax;l++){
      for(m=0;m<=l;m++,lm++){
          FGetLine(fp,line,132);
          if(m==0){
             sscanf(line,"%d %d %f",&l1,&m1,&aa);
             bb=0.0;
          }else{
             sscanf(line,"%d %d %f %f",&l1,&m1,&aa,&bb);
      /*     printf("%6d%6d%10.2f%10.2f\n",l1,m1,aa,bb); */
          }
          if(l!=l1 || m!=m1){
             printf(" Erorr in reading model %s\n",model->name1);
             exit(1);
          }
          ac[lm]=aa;
          bc[lm]=bb;
      }
  }
  fclose(fp);
}
 
float SumSurCoeff(MODEL model, float *az, float *bz, int lmax, float *cmphi, float *smphi, float *y)
{
float sum=0.0;
int l,m,lm;

  for(l=lm=0;l<=lmax;l++){
      if(model.degmask[l]!='0')sum+=az[lm]*y[lm];
      lm++;
      for(m=1;m<=l;m++,lm++){
          if(model.degmask[l]!='0'){
             sum+=(az[lm]*cmphi[m]+bz[lm]*smphi[m])*y[lm];
          }
      }
  }
  return sum;

}

float SumSphBspline(MODEL model, float theta, float phi)
{
float h,twoh,**au,val,delta;
int i,nnode;
SPHERE s1,s2;

  h=model.rmid;		/* average bell size */
  twoh=2.0*h;
  nnode=model.lmaxum;
  au=model.au;
  s1.theta=theta;
  s1.phi=phi;
  
 /* printf("%f %d %f %f\n",h,nnode,theta,phi); */
  
  for(val=0.0,i=1;i<=nnode;i++){
	s2.theta=au[1][i];
	if(fabs(s1.theta-s2.theta)<twoh){
	   s2.phi=au[2][i];
	   delta=ArcDist(s1,s2);
	   if(delta<twoh)val+=fdelta(delta,h)*au[3][i];
	}
  }
  return val;
}

float fdelta(float delta, float h)
{
float deltil,value;

  deltil=delta/h;
  
  if(deltil>2.0){
     return 0.0;
  }else if(deltil<=1.0){
     value=(4.0+3.0*deltil*deltil*(deltil-2.0));
  }else{
     value=2.0-deltil;
     value*=value*value;
  }
/*  return value/6.0;  */
   return value/4.0;
}


void CSmPhi(float phi, float *cmph, float *smph, int lmax)
{
float cmphi,smphi,tmphi,cp,sp;
int m;

    
    cp=cos((double)phi);
    sp=sin((double)phi);
    for(m=0,cmphi=1.0,smphi=0.0;m<=lmax;m++){
        cmph[m]=cmphi;
        smph[m]=smphi;
        tmphi=cmphi*cp-smphi*sp;
        smphi=smphi*cp+cmphi*sp;
        cmphi=tmphi;
    }
}


void ReadL0256(MODEL *model)
{
int k,l,m,l1,m1,lm,lmax,kmax;
float aa,bb,fac,**al,**bl;
FILE *fp;

  printf(" model name in readl0256 %s\n",model->name1);

  fp=OpenFl(model->name1);
  
  fac=sqrt(4.0*PI)*0.001;
  al  = model->al;
  bl  = model->bl;
  lmax= model->lmaxlm;
  kmax= model->kmaxlm;
  
  for(l=0;l<=lmax;l++)           /* set degree mask   */
      (*model).degmask[l]='1';

  (*model).degmask[lmax+1]='\0';
   
  for(l=0;l<=kmax;l++)           /* set layer mask   */
      (*model).layermask[l]='1';

  (*model).layermask[lmax+1]='\0';
   
      
  for(lm=0,l=0;l<=lmax;l++){
      for(m=0;m<=l;m++,lm++){
          fscanf(fp,"%d %d",&l1,&m1);
          if(l!=l1 || m!=m1){
             printf(" Erorr in reading model L02.56\n");
              exit(1);
          }
          for(k=0;k<=kmax;k++){
              fscanf(fp," %f %f",&aa,&bb);
              al[k][lm]=fac*aa;
              bl[k][lm]=fac*bb;
          }
          fscanf(fp,"\n");
      }
  }
  fclose(fp);
}

void ReadMDLSH(MODEL *model)
{
float **au,**bu,a,b,**rknot;
int  l,m,k,l1,m1,lm,lmax,kmax;
static float rtanw[12]={6371.0,6151.0,5971.0,5701.0,5349.0,5087.0,
                        4816.0,4555.0,4283.0,4012.0,3741.0,3480.0};
FILE  *fp;

  lmax  = model->lmaxum;
  kmax  = model->kmaxum;
  au    = model->au;
  bu    = model->bu;
  rknot = model->rknot;

  fp=OpenFl(model->name1);

  for(l=lm=0;l<=lmax;l++){
      for(m=0;m<=l;m++,lm++){
          fscanf(fp,"%d %d",&l1,&m1);
          if(l1!=l || m1!=m){
             printf(" error in reading model MDLSH\n");
             exit(1);
          }
          for(k=0;k<=kmax;k++){
              fscanf(fp,"%f",&a);
              au[k][lm]=a;
          }
          if(m!=0){
             fscanf(fp,"%d %d",&l1,&m1);
             if(l1!=l || m1!=m){
                printf(" error in reading model MDLSH\n");
                exit(1);
             }
             for(k=0;k<=kmax;k++){
                 fscanf(fp,"%f",&b);
                 bu[k][lm]=b;
             }
          }
      }
  }
/*
 * knots
 */ 
  for(k=0;k<=kmax;k++){
      rknot[0][k]=rtanw[k];
      rknot[1][k]=rtanw[k+1];
  } 
  
  fclose(fp);
}

void ReadMB92(MODEL *model)
{
float **au,**bu,**rknot;
int  k,l1,k1,lm,lmax,kmax;
static float rtanw[12]={6371.0,6151.0,5971.0,5701.0,5349.0,5087.0,
                        4816.0,4555.0,4283.0,4012.0,3741.0,3480.0};
char  c,*degmask;
FILE  *fp;

  lmax  = model->lmaxum;
  kmax  = model->kmaxum;
  au    = model->au;
  bu    = model->bu;
  rknot = model->rknot;
  degmask=model->degmask;

  fp=OpenFl(model->name1);
  fscanf(fp,"%d %d",&k1,&l1);
  
  if(k1!=kmax || l1 != lmax){
     printf("error in reading model MB92, kmax=%d lmax=%d\n",k1,l1);
     exit(1);
   }
   for(k=0;k<=kmax;k++)                      /* read data   */
       ReadLayer(fp,au[k],bu[k],lmax,degmask);
       
   for(k=0;k<=kmax;k++){
       rknot[0][k]=rtanw[k];
       rknot[1][k]=rtanw[k+1];
   }
         
  fclose(fp);
}

void ReadMRD(MODEL *model)
{
float **au,**bu,**rknot,a,b,r;
int  k,l,m,l1,m1,lm,lmax,kmax;
FILE  *fp;

  lmax  = model->lmaxum;
  kmax  = model->kmaxum;
  au    = model->au;
  bu    = model->bu;
  rknot = model->rknot;

  fp=OpenFl(model->name1);
  
  for(k=0;k<=kmax;k++){
     fscanf(fp,"%f",&r); 
     rknot[0][k]=RADIUS-(r-72.5);
     rknot[1][k]=RADIUS-(r+72.5);
     
     for(lm=0,l=0;l<=lmax;l++){
        for(m=0;m<=l;m++,lm++){
            fscanf(fp,"%d %d %f %f",&l1,&m1,&a,&b);
            if(l1!=l || m1!=m){
               printf(" error in ReadMRD\n");
               exit(1);
            }
            au[k][lm]=a;
            bu[k][lm]=b;
        }
     }
  }
   
  fclose(fp);
}


void ReadBspline3(MODEL *model)
{
float *ar;
int  fd,num,n_read;


  fd=SysOpen(model->name1,2);
  n_read=read(fd,model->xmima,6*sizeof(float));
  printf("bytes read =%d\n",n_read);
  printf("xmin = %g xmax=%g\n",model->xmima[0],model->xmima[1]);
  printf("ymin = %g ymax=%g\n",model->xmima[2],model->xmima[3]);
  printf("zmin = %g zmax=%g\n",model->xmima[4],model->xmima[5]);
  
  n_read=read(fd,model->ixyz,3*sizeof(float));
  printf("bytes read =%d\n",n_read);
  printf(" ix0 =%d\n",model->ixyz[0]);
  printf(" iy0 =%d\n",model->ixyz[1]);
  printf(" iz0 =%d\n",model->ixyz[2]);
  
  num=model->ixyz[0]*model->ixyz[1]*model->ixyz[2];
  
  ar=model->ar=Vector(1,num);
  
  n_read=read(fd,ar,num*sizeof(float));
  printf("bytes read =%d\n",n_read);

}

void RdBSCartMd(MODEL *model)
{
int i,num,n,fd,dim,nlat,nlon,ndep;
float **rknot;

   fd=SysOpen(model->name1,0);
   
   model->mod=BSPL3;
   n=read(fd,&(model->dim),sizeof(int));
   dim=model->dim;
   model->ixyz=ivector(1,dim);
   n=read(fd,&(model->ixyz[1]),dim*sizeof(int));	/* number of nodes */
   
   nlat=model->ixyz[1];
   nlon=model->ixyz[2];
   ndep=model->ixyz[3];
  
   n=max(nlat,nlon);
   num=nlat*nlon;
   if(dim==3){
      n=max(ndep,n);
      num*=ndep;
   }
   
   rknot=model->rknot=matrix(1,dim,-1,n+2);	/* two extral points are needed on both ends */
   model->ar=vector(1,num);
   
   for(i=1;i<=dim;i++){					/* coordinate of nodes */
      n=read(fd,&(model->rknot[i][1]),model->ixyz[i]*sizeof(float)); 
   }
   
   n=read(fd,model->ar,num*sizeof(float));
   
   for(i=1;i<=3;i++)rknot[i][-1]=rknot[i][0]=rknot[i][1];
   
   rknot[1][nlat+2]=rknot[1][nlat+1]=rknot[1][nlat];
   rknot[2][nlon+2]=rknot[2][nlon+1]=rknot[2][nlon];
   rknot[3][ndep+2]=rknot[3][ndep+1]=rknot[3][ndep];
   
   close(fd);

}

void ReadGrid(MODEL *model)
{
float bllat,bllon,trlat,trlon,r;
float **grid;
int  fd,num,n_read,row,col;
int  i,j,fsize;


  fd=SysOpen(model->name1,0);
  read(fd,&bllat,sizeof(float));
  read(fd,&bllon,sizeof(float));
  read(fd,&trlat,sizeof(float));
  read(fd,&trlon,sizeof(float));
  read(fd,&r,sizeof(float));
  read(fd,&row,sizeof(int));
  read(fd,&col,sizeof(int));
  
  model->xmima[0]=bllat;
  model->xmima[1]=bllon;
  model->xmima[2]=trlat;
  model->xmima[3]=trlon;
  model->xmima[4]=r;
  model->ixyz[0]=row;
  model->ixyz[1]=col;
  
  grid=model->au=matrix(0,row-1,0,col-1);

  fsize=sizeof(float);
  for(i=0;i<row;i++){
      n_read=read(fd,grid[i],col*fsize);
      if(n_read!=(fsize*col)){
         printf(" read error\n");
         exit(1);
      }
  }
    
  close(fd);

}

void ReadGrand(MODEL *model)
{
float **au;
int  fd,i,j,num,n_read,nlayer,maxpt,bytes;
float dum[2];

  fd=SysOpen(model->name1,2);                /* open model file                */
  model->xmima=Vector(0,6);                  /* to save latmin, latmax, lonmin,  
                                                lonmax, and depmin, depmax     */
  n_read=read(fd,model->xmima,6*sizeof(float));
  model->rtop   = RADIUS-model->xmima[4];
  model->rbot   = RADIUS-model->xmima[5];
  model->rmid   = 0.0;                       /* mid is not used                */

  n_read=read(fd,&nlayer,sizeof(int));
  model->kmaxum = nlayer;                    /* number of layers               */
  
  n_read=read(fd,&maxpt,sizeof(int));
  model->kmaxlm = maxpt;

  model->rknot  = Matrix(0,nlayer-1,0,1);     /* to save layer radius           */
  model->ixyz   = IntVec(0,nlayer-1);         /* # of data points in each layer */
  model->vavg   = Vector(0,nlayer-1);         /* to save average velocity       */
  model->au     = Matrix(0,nlayer,0,3*maxpt-1); /* vel pert &  cell location      */

  for(i=0;i<nlayer;i++){
      printf("reading layer %d\n",i+1);
      n_read=read(fd,dum,8);                  /* read layer depth range         */
      model->rknot[i][0]=RADIUS-dum[0];
      model->rknot[i][1]=RADIUS-dum[1];
      n_read=read(fd,&model->ixyz[i],sizeof(int)); /* # of points in this layer*/
      n_read=read(fd,&model->vavg[i],4);           /* average velocity         */
      bytes=model->ixyz[i]*3*4;
      n_read=read(fd,model->au[i],bytes);         /* lat,lon and velocity     */
  }
  
  model->dim    = 3;                        /* 3-dimensional model            */
  model->pors   = SMODEL;                   /* S-velocity model               */
  model->radfun = SHELL;                    /* layered model                  */
  model->velfm  = DVFORMAT;                 /* perturbation is dv             */
  
  for(i=0;i<nlayer;i++)model->layermask[i]='1';
  model->layermask[nlayer]='\0';

  printf("latmin = %10.2f latmax=%10.2f\n",model->xmima[0],model->xmima[1]);
  printf("lonmin = %10.2f lonmax=%10.2f\n",model->xmima[2],model->xmima[3]);
  printf("depmin = %10.2f depmax=%10.2f\n",model->xmima[4],model->xmima[5]);
  printf("total number of layers =%4d\n",model->kmaxum);
  printf("max number of points in any single layer = %5d\n",model->kmaxlm);
  
  for(i=0;i<nlayer;i++){
      printf("\nlayer %d\n",i+1);
      printf("number of points =%5d\n",model->ixyz[i]);
      printf("average velocity is set to %10.2f\n",model->vavg[i]);
      printf("depth  =%8.2f ->%8.2f\n", 
              RADIUS-model->rknot[i][0],
              RADIUS-model->rknot[i][1]);
/*      au=model->au;
      printf("%f %f %f\n",au[i][0],au[i][1],au[i][2]);
      j=model->ixyz[i];
      printf("%f %f %f\n",au[i][(j-1)*3],au[i][(j-1)*3+1],au[i][(j-1)*3+2]); */
  }
      
  

}

void ReadSPHBSPL2(MODEL *model)
{
float **au,**bu,*ar,*br,lat,lon,coeff, **bsloc;
int  i,j,n,nb,np,nnode,nrad,**ajnode,dum[7];
char txt[132];
SPHERE s1,s2;
FILE *fp;

  fp=OpenFl(model->name1); 	/* open model file    */

  FGetLine(fp,txt,132);		/* get the first line */
  sscanf(txt,"%d %d",&nnode,&nrad); /* # of nodes and # of radial nodes */
  model->mod=SPHBSPL2;
  model->lmaxum=nnode;			/* save number of nodes */
  model->kmaxum=nrad;			/* number of nodes in radial direction */
  au=model->au=matrix(1,2,1,nnode);	/* au[1][*] --- lat, au[2][*] --- lon */
  model->ajnode=ajnode=imatrix(0,7,1,nnode);	/* ajacent nodes */
  bsloc=model->bsloc=matrix(1,2,1,nnode); /* bsloc[1][*] --- lat, bsloc[2][*] --- lon */

  if(nrad<=1){				/* model defined on surface */
	printf("number of nodes =%d\n",nnode);
	model->mantle=SURFACE;		/* defined on spherical surface		*/
	model->radfun=0;
	br=model->br=vector(1,nnode);	/* model coefficients */
	model->dim=2;
  }else if(nrad>1){			/* defined as 3D model */
	printf("number of nodes in radial direction=%d\n",nrad);
	printf("number of horizontal nodes for each radial node =%d\n",nnode);
	model->radfun=0;
	model->dim=3;
	ar=model->ar=vector(-1,nrad+2);			/* for cooridinate of radial nodes */
      bu=model->bu=matrix(1,nrad,1,nnode);	/* for model coefficients */
  }else{
      printf("number of radial nodes has to be greater than four\n");
      exit(1);
  }
  
  for(i=1;i<=nnode;i++){	/* read coor of horizontal nodes */
      FGetLine(fp,txt,132);
	sscanf(txt,"%*d %f %f %d %d %d %d %d %d %d",
	  &lat,&lon,&n,&dum[1],&dum[2],&dum[3],&dum[4],&dum[5],&dum[6]);
	bsloc[1][i]=lat;
	bsloc[2][i]=lon;
	ajnode[0][i]=n;
	for(j=1;j<=n;j++)ajnode[j][i]=dum[j];
	s1=Geo2Sph(lat,lon,1.0);
	au[1][i]=s1.theta;
	au[2][i]=s1.phi;
  }
  
  if(nrad<=1){
     for(i=1;i<=nnode;i++)fscanf(fp,"%f",&br[i]);	/* model coefficients */
/*
     for(i=1;i<=nnode;i++){
         printf("%12.4e",br[i]);
         if(!(i%5))printf("\n");
     }
*/
  }else{
     for(j=1;j<=nrad;j++){
         fscanf(fp,"%*d %f",&ar[j]);				/* radial knots	*/
         for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[j][i]);	/* model coefficients */
     }
     model->rbot=RADIUS-ar[nrad];
     model->rtop=RADIUS-ar[1];
     ar[-1]=ar[0]=ar[1];
     ar[nrad+2]=ar[nrad+1]=ar[nrad];
  }
/*  
  for(i=1;i<=nnode;i++){
	printf("%d %f %f %d %d %d %d %d %d %d\n",
	i,au[1][i],au[2][i],ajnode[0][i],ajnode[1][i],ajnode[2][i],
	ajnode[3][i],ajnode[4][i],ajnode[5][i],ajnode[6][i]);
  }
*/
    
  for(n=0,coeff=0.0,i=1;i<=nnode;i++){	/* cal avg distance between nodes */
	np=ajnode[0][i];				/* total number of neighboring points */
	s1.theta=au[1][i];
	s1.phi=au[2][i];
	for(j=1;j<=np;j++){
	    nb=ajnode[j][i];
	    if(nb>i){				/* calculate only once */
		s2.theta=au[1][nb];
		s2.phi=au[2][nb];
		coeff+=ArcDist(s1,s2);
		n++;
	    }
	}
  }
  
  coeff/=(float)n;
  
  printf("average distance between nodes = %12.4f\n",coeff/RAD);
  
  model->rmid=coeff;				/* save average distance */
/*  free_imatrix(ajnode,0,7,1,nnode);		/* dot need ajacent nodes */
  
  fclose(fp);
}

void ReadBSPSPLT(MODEL *model)
{
/* this is for the split (discontinuous) B-spline model,
     implemented by Yu Gu, 1998.  */
float **au,**bu, **bl, *ar,*br,lat,lon,coeff;
float **bsloc;
int  i,j,n,nb,np,nnode,nrad_lm, nrad_um,**ajnode,dum[7];
char typestr[50],txt[132];
SPHERE s1,s2;
FILE *fp;

  fp=OpenFl(model->name1); 	/* open model file    */

  FGetLine(fp,txt,132);		/* get the first line */
  sscanf(txt,"%s %d %d %d",typestr,&nnode,&nrad_lm, &nrad_um); /* # of nodes and # of radial nodes */
  printf("%d %d %d\n", nnode, nrad_um, nrad_lm);
  model->mod=BSPSPLT;
  model->lmaxum=nnode;			/* save number of nodes */
  model->lmaxlm=nnode;
  model->kmaxum=nrad_um;		/* number of nodes in upper man */
  model->kmaxlm=nrad_lm;		/* number of nodes in lower man */
  au=model->au=matrix(1,2,1,nnode);	/* au[1][*] --- lat, au[2][*] --- lon */
  bsloc=model->bsloc=matrix(1,2,1,nnode); /* bsloc[1][*] --- lat, bsloc[2][*] --- lon */
  model->ajnode=ajnode=imatrix(0,7,1,nnode);	/* ajacent nodes */

  if(nrad_um==1){			/* model defined on surface */
	printf("Check Split Model input!!\n");
	printf("number of nodes =%d\n",nnode);
	model->mantle=SURFACE;		/* defined on spherical surface		*/
	model->radfun=0;
	br=model->br=vector(1,nnode);	/* model coefficients */
	model->dim=2;
  }else if(nrad_um>1){			/* defined as 3D model */
	printf("number of radial knots in lower mantle=%d\n",nrad_lm);
	printf("number of radial knots in upper mantle=%d\n",nrad_um);
	printf("number of horizontal nodes for each radial node =%d\n",nnode);
	model->radfun=0;
	model->dim=3;
	ar=model->ar=vector(-1,nrad_um+2);      /* for location of UM radial nodes */
	br=model->br=vector(-1,nrad_lm+2);      /* for location of LM radial nodes */
        bu=model->bu=matrix(1,nrad_um,1,nnode);	/* Upper Mantle coefficients */
        bl=model->bl=matrix(1,nrad_lm,1,nnode);	/* Lower Mantle coefficients */
  }else{
      printf("number of radial nodes has to be greater than 4\n");
      exit(1);
  }
  
  for(i=1;i<=nnode;i++){	/* read coor of horizontal nodes */
      FGetLine(fp,txt,132);
	sscanf(txt,"%*d %f %f %d %d %d %d %d %d %d",
	  &lat,&lon,&n,&dum[1],&dum[2],&dum[3],&dum[4],&dum[5],&dum[6]);
	bsloc[1][i]=lat;
	bsloc[2][i]=lon;
	ajnode[0][i]=n;
	for(j=1;j<=n;j++)ajnode[j][i]=dum[j];
	s1=Geo2Sph(lat,lon,1.0);
	au[1][i]=s1.theta;
	au[2][i]=s1.phi;
  }
  
  if(nrad_lm<=1){
     for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[1][i]);	        /* model coefficients */
  }else{
     for(j=1;j<=nrad_lm;j++){
         fscanf(fp,"%*d %f",&br[j]);				/* radial knots	*/
	 printf( "lower mantle knot %d  %f\n", j, br[j]);
         for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bl[j][i]);	/* model coefficients */
     }
     for(j=1;j<=nrad_um;j++){
         fscanf(fp,"%*d %f",&ar[j]);				/* radial knots	*/
	 printf( "Upper mantle knot %d  %f\n", j, ar[j]);
         for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[j][i]);	/* model coefficients */
     }
     model->rbot=RADIUS-ar[nrad_um];    /* surface of the earth */
     model->rtop=RADIUS-br[1]; 		/* CMB */
     ar[-1]=ar[0]=ar[1];		/* to satisify boundary condition of B-splines*/
     br[-1]=br[0]=br[1];
     ar[nrad_um+2]=ar[nrad_um+1]=ar[nrad_um];
     br[nrad_lm+2]=br[nrad_lm+1]=br[nrad_lm];
  }
/*  
  for(i=1;i<=nnode;i++){
	printf("%d %f %f %d %d %d %d %d %d %d\n",
	i,au[1][i],au[2][i],ajnode[0][i],ajnode[1][i],ajnode[2][i],
	ajnode[3][i],ajnode[4][i],ajnode[5][i],ajnode[6][i]);
  }
*/
    
  for(n=0,coeff=0.0,i=1;i<=nnode;i++){	/* cal avg distance between nodes */
	np=ajnode[0][i];		/* total number of neighboring points */
	s1.theta=au[1][i];
	s1.phi=au[2][i];
	for(j=1;j<=np;j++){
	    nb=ajnode[j][i];
	    if(nb>i){			/* calculate only once */
		s2.theta=au[1][nb];
		s2.phi=au[2][nb];
		coeff+=ArcDist(s1,s2);
		n++;
	    }
	}
  }
  
  coeff/=(float)n;
  
  printf("average distance between nodes = %12.4f\n",coeff/RAD);
  
  model->rmid=coeff;				/* save average distance */
  
  fclose(fp);
}

void ReadBSMULTISPLIT(MODEL *model)
{
/*  This is for the split (discontinuous) B-spline model.
 *  This version allows multiple splits in the model, it
 *  should also work for single split.
 *  implemented by Yu Gu, 1999.  
 */
float **au,**bu, **bl, *ar,*br,lat,lon,coeff;
float **rknot;
float **bsloc;
int  i,j,k,m,n,nb,np,nnode;
int  nrad_lm, nrad_um,**ajnode,dum[7], nlayer;
char txt[MAXSTR];
SPHERE s1,s2;
FILE *fp;

  fp=OpenFl(model->name1); 	/* open model file    */
  
  ReadBS1stLine(fp, model->nsplit, &nnode, model->nbsperlay, &nrad_um, &nrad_lm);
  model->mod=BSMULTISPLIT;
  model->lmaxum=nnode;			/* save number of nodes */
  model->lmaxlm=nnode;
  model->kmaxum=nrad_um;		/* number of nodes in upper man */
  model->kmaxlm=nrad_lm;		/* number of nodes in lower man */
  au=model->au=matrix(1,2,1,nnode);	/* au[1][*] --- lat, au[2][*] --- lon */
  bsloc=model->bsloc=matrix(1,2,1,nnode); /* bsloc[1][*] --- lat, bsloc[2][*] --- lon */
  model->ajnode=ajnode=imatrix(0,7,1,nnode);	/* ajacent nodes */

  if(nrad_um==1){			/* model defined on surface */
	printf("Check Split Model input!!\n");
	printf("number of nodes =%d\n",nnode);
	model->mantle=SURFACE;		/* defined on spherical surface		*/
	model->radfun=0;
	br=model->br=vector(1,nnode);	/* model coefficients */
	model->dim=2;
  }else if(nrad_um>1){			/* defined as 3D model */
	printf("number of radial knots in lower mantle=%d\n",nrad_lm);
	printf("number of radial knots in upper mantle=%d\n",nrad_um);
	printf("number of horizontal nodes for each radial node =%d\n",nnode);
	model->radfun=0;
	model->dim=3;
	nlayer = model->nsplit[0]+model->nsplit[1];
	printf("nlayer = %d\n", nlayer);
	rknot=model->rknot=matrix(1,nlayer,-1,nrad_um+nrad_lm+2);  /* for radial node loc */
        bu=model->bu=matrix(1,nrad_um,1,nnode);	/* Upper Mantle coefficients */
        bl=model->bl=matrix(1,nrad_lm,1,nnode);	/* Lower Mantle coefficients */
  }else{
      printf("number of radial nodes has to be greater than 4!\n");
      exit(1);
  }
  
  for(i=1;i<=nnode;i++){	/* read coor of horizontal nodes */
      FGetLine(fp,txt,132);
	sscanf(txt,"%*d %f %f %d %d %d %d %d %d %d",
	  &lat,&lon,&n,&dum[1],&dum[2],&dum[3],&dum[4],&dum[5],&dum[6]);
/*	printf("%f %f %d %d %d %d %d %d %d\n",
	  lat,lon,n,dum[1],dum[2],dum[3],dum[4],dum[5],dum[6]);
*/
	bsloc[1][i]=lat;
	bsloc[2][i]=lon;
	ajnode[0][i]=n;
	for(j=1;j<=n;j++)ajnode[j][i]=dum[j];
	s1=Geo2Sph(lat,lon,1.0);
	au[1][i]=s1.theta;
	au[2][i]=s1.phi;
  }
  n=1;
  if(nrad_lm<=1){
     for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[1][i]);	   /* model coefficients */
  }else{
     for(k=1;k<=nlayer;k++){
     	for(j=1;j<=model->nbsperlay[k-1];j++){
         	fscanf(fp,"%*d %f",&rknot[k][j]);	   /* radial knots	*/
		if(n<=nrad_lm){
         		for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bl[n][i]);   /* model coefficients */
	 		printf( "lower mantle knot k=%d, j=%d, %f\n", k,j,rknot[k][j]);
		}else{
			m=n-nrad_lm;
         		for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[m][i]);   /* model coefficients */
	 		printf( "upper mantle knot k=%d, j=%d, %f\n", k,j,rknot[k][j]);
		}
		n++;
     	}
     }
     model->rbot=RADIUS-rknot[nlayer][model->nbsperlay[nlayer-1]];    /* surface of the earth */
     model->rtop=RADIUS-rknot[1][1]; 		/* CMB */
     printf("rtop = %f, rbot = %f\n", model->rtop, model->rbot);
     for(i=1;i<=nlayer;i++){
     	rknot[i][-1]=rknot[i][0]=rknot[i][1];		/* to satisify boundary condition of B-splines*/
     	rknot[i][model->nbsperlay[i-1]+2]=rknot[i][model->nbsperlay[i-1]];
	rknot[i][model->nbsperlay[i-1]+1]=rknot[i][model->nbsperlay[i-1]];
     }
  }
/*  
  for(i=1;i<=nnode;i++){
	printf("%d %f %f %d %d %d %d %d %d %d\n",
	i,au[1][i],au[2][i],ajnode[0][i],ajnode[1][i],ajnode[2][i],
	ajnode[3][i],ajnode[4][i],ajnode[5][i],ajnode[6][i]);
  }
*/
    
  for(n=0,coeff=0.0,i=1;i<=nnode;i++){	/* cal avg distance between nodes */
	np=ajnode[0][i];		/* total number of neighboring points */
	s1.theta=au[1][i];
	s1.phi=au[2][i];
	for(j=1;j<=np;j++){
	    nb=ajnode[j][i];
	    if(nb>i){			/* calculate only once */
		s2.theta=au[1][nb];
		s2.phi=au[2][nb];
		coeff+=ArcDist(s1,s2);
		n++;
	    }
	}
  }
  
  coeff/=(float)n;
  
  printf("average distance between nodes = %12.4f\n",coeff/RAD);
  
  model->rmid=coeff;				/* save average distance */
  
  fclose(fp);
}

void ReadBS1stLine(FILE *fp, int *nsplit, int *nnode,int *nlayer, int *nrad_um, int *nrad_lm)
{  /* this is for multiple splits in the B-spline model 
    * Correct format (separated by space(s)):
    *  	1.   MULTISPLITM  
    *   2.   number of lower mantle split layers
    *   3.   number of upper mantle split layers
    *   4.   number of horizontal splines
    *   5.   number of radial splines in lower mantle layer(s)
    *   6.   number of radial splines in lower mantle layer(s)
    *   sample input:   (split at 670, 1 layer in LM, 2 in UM)
			MULTISPLITM 1 2 362 7 3 4
    */
char txt[MAXSTR], typestr[50];
int ntotal, i,j;

  FGetLine(fp,txt,MAXSTR);		/* get the first line */
  sscanf(txt, "%s %d %d", typestr, &nsplit[0], &nsplit[1]);
  printf("# lower mantle split  = %d, upper mantle = %d\n", nsplit[0], nsplit[1]);
  ntotal = nsplit[0]+nsplit[1];
  switch(ntotal) {
	case 1:
		sscanf(txt, "%s %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0]);
		printf("%s %d %d %d %d\n", typestr, &nsplit[0], &nsplit[1], *nnode, nlayer[0]);
		break;
	case 2:
		sscanf(txt, "%s %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], &nlayer[1]);
		printf("%s %d %d %d %d %d\n", typestr, &nsplit[0], &nsplit[1], *nnode, nlayer[0], nlayer[1]);
		break;
	case 3:
		sscanf(txt, "%s %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], &nlayer[1],
			&nlayer[2]);
		printf("%s %d %d %d %d %d %d\n", typestr, nsplit[0], nsplit[1], *nnode, nlayer[0], nlayer[1], nlayer[2]);
		break;
	case 4:
		sscanf(txt, "%s %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], &nlayer[1],
			&nlayer[2], &nlayer[3]);
		printf("%s %d %d %d %d %d %d %d\n", typestr, nsplit[0], nsplit[1], *nnode, nlayer[0], nlayer[1], nlayer[2], nlayer[3]);
		break;
	case 5:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], 
			&nlayer[1],&nlayer[2], &nlayer[3], &nlayer[4]);
		printf("%s %d %d %d %d %d %d %d %d\n", typestr, nsplit[0], nsplit[1], *nnode, nlayer[0], nlayer[1], nlayer[2], nlayer[3], nlayer[3]);
		break;
	case 6:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], 
			&nlayer[1],&nlayer[2], &nlayer[3], &nlayer[4], &nlayer[5]);
		break;
	case 7:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], 
			&nlayer[1],&nlayer[2], &nlayer[3], &nlayer[4], &nlayer[5], &nlayer[6]);
		break;
	case 8:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode,&nlayer[0], 
			&nlayer[1],&nlayer[2],&nlayer[3],&nlayer[4],&nlayer[5],&nlayer[6],
			&nlayer[7]);
		break;
	case 9:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode,
			&nlayer[0],&nlayer[1],&nlayer[2],&nlayer[3],&nlayer[4],&nlayer[5],
			&nlayer[6],&nlayer[7],&nlayer[8]);
		break;
	case 10:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode,
			&nlayer[0],&nlayer[1],&nlayer[2],&nlayer[3],&nlayer[4],&nlayer[5],
			&nlayer[6],&nlayer[7],&nlayer[8],&nlayer[9]);
		break;
	default:
		printf("First line of model not reasonable!\n");
		break;
  }
  *nrad_lm=0;
  *nrad_um=0;
  for(i=0;i<nsplit[0];i++){
	*nrad_lm=*nrad_lm+nlayer[i];
	printf("lower mantle, splines=%d\n", nlayer[i]);
  }
  for(i=0;i<nsplit[1];i++){
	*nrad_um=*nrad_um+nlayer[i+nsplit[0]];
	printf("upper mantle, splines=%d\n", nlayer[i+nsplit[0]]);
  }
}

void ReadSAF(MODEL *model)
{
float **au,**bu;
int  a,b,k,l,m,lm,lmax,kmax;
FILE  *fp;

  lmax  = model->lmaxum;
  kmax  = model->kmaxum;
  au    = model->au;
  bu    = model->bu;

  fp=OpenFl(model->name1);
/*
 * read model
 */
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          if(l==2 || l==4 ){
             fscanf(fp,"%d",&a);
          }else{
             a=0;
          }
          au[k][lm]  = (float)a*1.0e-6; 
          bu[k][lm++]= 0.0;
          for(m=1;m<=l;m++,lm++){
             if(l==2 || l==4 ){
                fscanf(fp,"%d %d",&a,&b);
             }else{
                a=0;
             }
             au[k][lm]=(float)a*1.0e-6;
             bu[k][lm]=(float)b*1.0e-6;
          }
      }
  }
         
  fclose(fp);
}

void ReadHdModel(MODEL *model)
{
float *ac,*bc,*a670,*b670,**au,**bu,**al,**bl,ftemp;
int   k,lmax,kmax;
char  s[MAXSTR],*degmask,*layermask;
FILE  *fp;

  fp=OpenFl(model->name1);
  FGetLine(fp,s,MAXSTR);                       /* skip the first line */
  
  degmask=model->degmask;
  layermask=model->layermask;

  if(model->crust==TRUE){
     lmax=model->lmaxum;
     ac  =model->ac;
     bc  =model->bc;
     ReadLayer(fp,ac,bc,lmax,degmask);
  }
  if(model->p670 ==TRUE){
     lmax=model->lmaxum;
     a670=model->a670;
     b670=model->b670;
     ReadLayer(fp,a670,b670,lmax,degmask);
  }
  if(model->mantle==WHOLEMANTLE || model->mantle==UPPERMANTLE ||
     model->mantle==INNERCORE){
     au = model->au;
     bu = model->bu;
     lmax=model->lmaxum;
     kmax=model->kmaxum;
     for(k=0;k<=kmax;k++){                      /* read data   */
         ReadLayer(fp,au[k],bu[k],lmax,degmask);
     }
  }else if (model->mantle==UPPERLOWERM){
     au = model->au;
     bu = model->bu;
     lmax=model->lmaxum;
     kmax=model->kmaxum;
     for(k=0;k<=kmax;k++){                      /* read data   */
         ReadLayer(fp,au[k],bu[k],lmax,degmask);
     }
     al = model->al;
     bl = model->bl;
     lmax=model->lmaxlm;
     kmax=model->kmaxlm;
     for(k=0;k<=kmax;k++){                      /* read data   */
         ReadLayer(fp,al[k],bl[k],lmax,degmask);
     }
  }
  if(model->radfun==BSPLINE){
     printf(" get here radfun\n");
     for(k=0;k<=kmax;k++){
         fscanf(fp,"%f",&ftemp);
         model->xmima[kmax-k]=RADIUS-ftemp;
     }
  }
  fclose(fp);
}

void PrintModel(MODEL model, int print)
{
float *ac,*bc,*a670,*b670,**au,**bu,**al,**bl,aa,bb;
int   l,k,m,lm,lmax,kmax;
char  s[MAXSTR];
FILE  *fp;
  
  if(model.mod==BSPSPLT) {
     PrintBSSplit(model);
     return;
  }
  if(model.mod==BSPL3){
     PrintCartBS(model);
     return;
  }
  if(model.mod==SPHBSPL2){
     PrintBSModel(model);
     return;
  }
  PrintModelHD(model);
  if(model.crust==TRUE){
     lmax=model.lmaxum;
     ac=model.ac;
     bc=model.bc;
     if(print==TRUE){
        printf("coefficients for crust perturbation\n");
        PrintLayer(-1,ac,bc,lmax);
     }
  }
  if(model.p670 ==TRUE){
     lmax=model.lmaxum;
     a670=model.a670;
     b670=model.b670;
     if(print==TRUE){
        printf("coefficients for 670 perturbation\n");
        PrintLayer(-1,a670,b670,lmax);
     }
  }
  if(model.mantle==WHOLEMANTLE || model.mantle==INNERCORE){
     au = model.au;
     bu = model.bu;
     lmax=model.lmaxum;
     kmax=model.kmaxum;
     if(model.mantle==WHOLEMANTLE)printf("whole mantle model coefficients\n");
     if(model.mantle==INNERCORE)printf("Inner core anisotropy model\n");
     if(print==TRUE){
        for(k=0;k<=kmax;k++)
            PrintLayer(k,au[k],bu[k],lmax);
     }
  }else if (model.mantle==UPPERMANTLE){
     au = model.au;
     bu = model.bu;
     lmax=model.lmaxum;
     kmax=model.kmaxum;
     if(print==TRUE){
        printf(" coefficients for upper mantle\n");
        for(k=0;k<=kmax;k++)     
            PrintLayer(k,au[k],bu[k],lmax);
     }
         
     al = model.al;
     bl = model.bl;
     lmax=model.lmaxlm;
     kmax=model.kmaxlm;
     if(print==TRUE){
        printf("coefficients for lower mantle\n");
        for(k=0;k<=kmax;k++)     
            PrintLayer(k,al[k],bl[k],lmax);
     }
  }
  
}

void PrintBSModel(MODEL model)
{
int   i,j,nnode,nrad,**ajn;
float **a,**b,*ar,*br,**au,**bu,lat,lon;
SPHERE s;
FILE  *fp;

  nnode=model.lmaxum;				/* save number of nodes */
  nrad =model.kmaxum;				/* number of nodes in radial direction */
  au=model.au;					/* au[1][*] --- lat, au[2][*] --- lon */
  ajn=model.ajnode;				/* ajacent nodes */
  
  if(model.dim==2){				/* model defined on surface */
	printf("number of nodes =%d\n",nnode);
	br=model.br;				/* model coefficients */
  }else if(model.dim==3){			/* defined as 3D model */
	printf("# of nodes in radial direction=%d\n",nrad);
	printf("# of horizontal nodes for each radial node =%d\n",nnode);
	ar=model.ar;				/* for cooridinate of radial nodes */
      bu=model.bu;				/* for model coefficients */
  }
/*
 * open file for output 
 */
   fp=OpenFileWt("output model");
   fprintf(fp,"%5d %4d\n",nnode,nrad);	/* # of nodes and # of radial nodes */

  for(i=1;i<=nnode;i++){			/* print coor of horizontal nodes */
	s.theta=au[1][i];
	s.phi=au[2][i];
	Sph2Geo(s,&lat,&lon);

	fprintf(fp,"%4d %7.3f %8.3f %2d %4d %4d %4d %4d %4d %4d\n",
	  i,lat,lon,ajn[0][i],ajn[1][i],ajn[2][i],ajn[3][i],ajn[4][i],ajn[5][i],ajn[6][i]);
  }
  if(model.dim<=2){
     for(i=1;i<=nnode;i++){			/* print model coefficients for 2D model */
         fprintf(fp,"%12.4f",br[i]);
         if(!(i%5))printf("\n");
     }

  }else{
     for(j=1;j<=nrad;j++){
         fprintf(fp,"%4d %10.2f\n",j,ar[j]);				/* radial knots	*/
         for(i=1;i<=nnode;i++){			/* print model coefficients for 2D model */
             fprintf(fp,"%12.4e",bu[j][i]);
             if(!(i%5) || i==nnode)fprintf(fp,"\n");
         }
     }
  }
  
  fclose(fp);
    
}

void PrintBSSplit(MODEL model)
{  /* write out the split B-spline model */
int i, j;
int nnode, nrad_lm, nrad_um, nrad;
FILE *fp;
char outfl[256];

 printf("output file name :\n");
 scanf("%s", outfl);
 fp = fopen(outfl, "w");
 nnode=model.lmaxum;
 nrad_um=model.kmaxum;
 nrad_lm=model.kmaxlm;
 nrad = nrad_um+nrad_lm;
 fprintf(fp, "UPPERLOWERM %d %d %d\n",nnode, nrad_lm, nrad_um);
 for(i=1;i<=nnode;i++){
	fprintf(fp, "%5d %7.3f %8.3f %2d %4d %4d %4d %4d %4d %4d\n",
	i,model.bsloc[1][i],model.bsloc[2][i],model.ajnode[0][i],model.ajnode[1][i],
	model.ajnode[2][i], model.ajnode[3][i], model.ajnode[4][i], model.ajnode[5][i],
	model.ajnode[6][i]);
 }
 for(i=1;i<=nrad_lm; i++) {   /* lower mantle knots */
	printf("%4d %10.2f\n", i, model.br[i]);
        fprintf(fp,"%4d %10.2f\n",i,model.br[i]);
     	for(j=1;j<=nnode;j++) {
		fprintf(fp,"%12.4e",model.bl[i][j]);
        	if((j%5)==0)fprintf(fp,"\n");
	}
	fprintf(fp, "\n");
 }

 for(i=1;i<=nrad_um;i++){		/* upper mantle knots */
	 printf("%4d %10.2f\n", i+nrad_lm, model.ar[i]);
         fprintf(fp,"%4d %10.2f\n",i+nrad_lm,model.ar[i]);
     	 for(j=1;j<=nnode;j++) {
		fprintf(fp,"%12.4e",model.bu[i][j]);
         	if((j%5)==0) fprintf(fp,"\n");
	 }
     	 fprintf(fp,"\n");
 }
 fclose(fp);
}

void PrintCartBS(MODEL model)
{

int i,j,num,n,dim;
float **rknot;

   dim=model.dim;

   printf("printing Cartesian B-spline model\n");
   printf("Dimension =%d\n",dim);
   
   num=1;
   for(i=1;i<=dim;i++){
      num*=model.ixyz[i];
      printf(" number of knots ixyz[%1d]=%d\n",i,model.ixyz[i]);
   }

   printf(" total number of unknowns =%d\n",num);
   rknot=model.rknot;
   
   for(i=1;i<=dim;i++){					/* coordinate of nodes */
      printf(" knots in direction %d\n",i);
      for(j=-1;j<=model.ixyz[i]+2;j++)
         printf(" %3d %.4g\n",j,rknot[i][j]);
   }
   
   for(i=1;i<=num;i++){
      printf("%.2g ",model.ar[i]);
      if(!(i%10))printf("\n");
   }  

}

void WtBSCartMd(MODEL model)
{
int i,j,num,n,fd,dim;
char *filenm;
float **rknot;

   filenm=model.name1;
   dim=model.dim;

   fd=SysOpen(filenm,1);
   
   n=write(fd,&(model.dim),sizeof(int));
   printf("dim n=%d\n",n);
   
   printf(" model ixyz =%d\n",model.ixyz[1]);
   printf(" model ixyz =%d\n",model.ixyz[2]);
   printf(" model ixyz =%d\n",model.ixyz[3]);

   n=write(fd,&(model.ixyz[1]),dim*sizeof(int));	/* number of nodes */
   
   printf(" n=%d\n",n);
   
   for(i=1;i<=dim;i++){				/* coordinate of nodes */
      n=write(fd,&(model.rknot[i][1]),model.ixyz[i]*sizeof(float));
      printf("i=%d n=%d\n",i,n/sizeof(int));
   }
   
   if(dim==3){
      num=model.ixyz[1]*model.ixyz[2]*model.ixyz[3];
   }else if(dim==2){
      num=model.ixyz[1]*model.ixyz[2];
   }
   n=write(fd,model.ar,num*sizeof(float));
   printf(" number of model coeff=%d\n",n/sizeof(float));
   
   close(fd);

}


void ReadLayer(FILE *fp, float *ak, float *bk, int lmax, char *degmask)
{
int l,m,lm;
float aa,bb;

  for(l=lm=0;l<=lmax;l++){
      if(degmask[l]!='0'){
         fscanf(fp,"%f",&aa);
         ak[lm] = aa;
      }else{
         ak[lm] = 0.0;
      } 
      bk[lm++]=0.0;
      for(m=1;m<=l;m++,lm++){
          if(degmask[l]!='0'){
             fscanf(fp,"%f %f",&aa,&bb);
             ak[lm]=aa;
             bk[lm]=bb;
          }else{
             ak[lm]=0.0;
             bk[lm]=0.0;
          }
      }
  }
}

void WriteLayer(FILE *fp, float *ak, float *bk, int lmax, char *degmask)
{
int l,m,lm;
float aa,bb;

  for(l=lm=0;l<=lmax;l++){
      fprintf(fp, "%12.4e", ak[lm]);
      lm ++;
      for(m=1;m<=l;m++,lm++){
          fprintf(fp,"%12.4E",ak[lm]);
          fprintf(fp,"%12.4E",bk[lm]);
      }
      fprintf(fp, "\n");
  }

}

void Setab2Zero(float *ak, float *bk, int lmax)
{
int l,m,lm;

  for(l=lm=0;l<=lmax;l++){
      ak[lm] = 0.0;
      bk[lm++]=0.0;
      for(m=1;m<=l;m++,lm++){
          ak[lm]=0.0;
          bk[lm]=0.0;
      }
  }

}

void PrintLayer(int k, float *ak, float *bk, int lmax)
{
int l,m,lm;
float aa,bb;

  for(l=lm=0;l<=lmax;l++){
      if(k>=0)printf("k=%3d ",k);
      printf("l=%3d, m=%3d a=%12.4e b=%12.4e\n",l,0,ak[lm],0.0);
      lm++;
      for(m=1;m<=l;m++,lm++){
          if(k>=0)printf("k=%3d ",k);
          printf("l=%3d, m=%3d a=%12.4e b=%12.4e\n",l,m,ak[lm],bk[lm]);
      }
  }
}


void  PrintModelHD(MODEL model)
{
    printf("%s\n",model.dsc);      
    if     (model.pors==PMODEL)
       printf(" %3d-dimensional P velocity",model.dim);
    else if(model.pors==SMODEL)
       printf(" %3d-dimensional S velocity",model.dim);
    else if(model.pors==DD)
       printf(" %3d-dimensional density",model.dim);
    else if(model.pors==PREMFORMAT)
       printf(" %3d-dimensional PREM",model.dim);
       
    if     (model.mantle==UPPERMANTLE){
       printf(" upper mantle model..\n");
       printf(" lmax = %3d kmax=%3d\n",model.lmaxum,model.kmaxum);
    }else if(model.mantle==LOWERMANTLE){
       printf(" lower mantle model\n");
       printf(" lmax = %3d kmax=%3d\n",model.lmaxlm,model.kmaxlm);
    }else if(model.mantle==WHOLEMANTLE){
       printf(" whole mantle model.\n");
       printf(" lmax = %3d kmax=%3d\n",model.lmaxum,model.kmaxum);
    }else if(model.mantle==UPPERLOWERM){
       printf(" separate upper- and lower-mantle model\n");
       printf(" upper mantle: lmax = %3d kmax=%3d\n",model.lmaxum,model.kmaxum);
       printf(" lower mantle: lmax = %3d kmax=%3d\n",model.lmaxlm,model.kmaxlm);
    }else if(model.mantle==WHOLEEARTH){
       printf(" whole Earth model\n");
    }
    
    if     (model.radfun==CHEBYSHEV)
       printf(" radial function used is Chebyshev polynomial\n");
    else if(model.radfun==LEGENDRE)
       printf(" radial function used is LEGENDRE  polynomial\n");
    else if(model.radfun==SHELL)
       printf(" radial function used is LAYER\n");
       
    if     (model.velfm==DVFORMAT)
       printf(" velocity is expressed as dv\n");
    else if(model.velfm==DVOVFORMAT)
       printf(" velocity is expressed as dv/v\n");
    else if(model.velfm==PREMFORMAT)
       printf("model is expressed as PREM format\n");
    
    if(model.crust==TRUE)
       printf(" model contains a layer for crust\n");
    if(model.p670 ==TRUE)
       printf(" model contains a layer for 670 km discontinuity\n");
       
    if(model.mod!=GRAND)printf(" degree mask = %s\n",model.degmask);
    if(model.dim==3)
       printf(" layer  mask = %s\n",model.layermask);
}

void ReleaseModel(MODEL *model)
{
int num,lmaxum,lmaxlm,kmaxum,kmaxlm,lmax,llmax,nnode,nrad;
   
    if(model->mod==BSPL3){   /* for B-spline model */
       num=model->ixyz[0]*model->ixyz[1]*model->ixyz[2];
       FreeVector(model->ar,1,num);
       return;
    }else if(model->mod==SPHBSPL2){	 /* for spherical spline model */
       nnode=model->lmaxum;			/* number of nodes */
       nrad=model->kmaxum;			/* number of nodes in radial direction */
       FreeMatrix(model->au,1,2,1,nnode);	/* node coordinates */

       if(nrad==0){
          FreeVector(model->br,1,nnode);	/* model coefficients */
       }else{
	    FreeVector(model->ar,-1,nrad+2);		/* for cooridinate of radial nodes */
          FreeMatrix(model->bu,1,nrad,1,nnode);		/* for model coefficients */
       }  
       return;
    }
/* 
 * free the allocated memory for upper- and whole mantle
 */   
    if(model->mantle==UPPERMANTLE ||  model->mantle==UPPERLOWERM ||
       model->mantle==WHOLEMANTLE ){
       lmaxum=model->lmaxum;
       kmaxum=model->kmaxum;
       llmax =(lmaxum+2)*(lmaxum+1)/2-1;       
       FreeMatrix(model->au,0,kmaxum,0,llmax);   
       FreeMatrix(model->bu,0,kmaxum,0,llmax);
       FreeVector(model->ac,0,llmax);   
       FreeVector(model->bc,0,llmax);
       FreeVector(model->a670,0,llmax);
       FreeVector(model->b670,0,llmax);
    }
/* 
 * free allocated memory for lower mantle
 */ 
    if(model->mantle==LOWERMANTLE || model->mantle==UPPERLOWERM){
       lmaxlm=model->lmaxlm;
       kmaxlm=model->kmaxlm;
       llmax =(lmaxlm+2)*(lmaxlm+1)/2-1;       
       FreeMatrix(model->al,0,kmaxlm,0,llmax);   
       FreeMatrix(model->bl,0,kmaxlm,0,llmax);
    }
    
    lmax  = max(lmaxum,lmaxlm);
    llmax = (lmax+2)*(lmax+1)/2-1;       
   
    FreeVector(model->ar,0,llmax);
    FreeVector(model->br,0,llmax);
}

void SumRad(float r, MODEL *model, int cur_mantle)
{
int    lmax,kmax,radfun;
int    k,l,m,lm,mantle,mantle0;
float  f[100],**a,**b,*ar,*br;
  
    radfun=model->radfun;
    mantle=model->mantle;
    if(model->mod==ICANI){
       lmax  =model->lmaxum;
       kmax  =model->kmaxum;
       r = (150.0-r)/30.0;
       printf(" check this!!!\n");
       ChebyshevFun(r,kmax,f);
       a     =model->au;
       b     =model->bu;
       exit(1);
    }else{
       if(mantle==WHOLEMANTLE){
           mantle0=WHOLEMANTLE;
           lmax  =model->lmaxum;
           kmax  =model->kmaxum;
           a     =model->au;
           b     =model->bu;
       }else if(mantle==INNERCORE){
           mantle0=INNERCORE;
           lmax  =model->lmaxum;
           kmax  =model->kmaxum;
           a     =model->au;
           b     =model->bu;
       }else if( mantle==UPPERMANTLE||
           (mantle==UPPERLOWERM && cur_mantle==UPPERMANTLE)){
           mantle0=UPPERMANTLE;
           lmax  =model->lmaxum;
           kmax  =model->kmaxum;
           a     =model->au;
           b     =model->bu;
       }else if( mantle==LOWERMANTLE ||
           (mantle==UPPERLOWERM && cur_mantle==LOWERMANTLE)){
           mantle0=LOWERMANTLE;
           lmax  =model->lmaxlm;
           kmax  =model->kmaxlm;
           a     =model->al;
           b     =model->bl;
       } else {
          printf("error in function SumRad\n");
          exit(1);
       }
       RadFunc(r,f,mantle0,radfun,kmax);     /* calculate radial function */
    }

    ar = model->ar;    /* Upper mantle knot location in case of B-splines */
    br = model->br;    /* Lower mantle knot location in case of B-splines */
    for(l=lm=0;l<=lmax;l++){
        ar[lm]=0.0;
        br[lm++]=0.0;
        for(m=1;m<=l;m++,lm++){
            ar[lm]=0.0;
            br[lm]=0.0;
        }
    } 
    for(k=0;k<=kmax;k++){
        for(l=lm=0;l<=lmax;l++){
            if(model->degmask[l]!='0'){
               ar[lm]+=f[k]*a[k][lm];
            }
            lm++;
            for(m=1;m<=l;m++,lm++){
                if(model->degmask[l]!='0'){
                   ar[lm]+=f[k]*a[k][lm];
                   br[lm]+=f[k]*b[k][lm];
                }
            }
/*	    printf("%d %d %f %f\n", k, lm, ar[lm], br[lm]);  */
        }
    } 
}

void SumMKRad(float r, MODEL model, int cur_mantle, float xi, float vsp2, int choice)
{
int    lmax,kmax,radfun;
int    k,l,m,lm,mantle,mantle0;
float  f[100],**amu,**bmu,**akappa,**bkappa,*ar,*br;
float  fac1,fac2,fac3;
char  s[132];
  
    radfun=model.radfun;
    mantle=model.mantle;

    if(mantle==WHOLEMANTLE){
       mantle0=WHOLEMANTLE;
       lmax  =model.lmaxum;
       printf("model.kmaxum=%d\n",model.kmaxum);
       kmax  =model.kmaxum/2;
       amu   =model.au;
       bmu   =model.bu;
       akappa   =&model.au[kmax+1];
       bkappa   =&model.bu[kmax+1];
    }
    RadFunc(r,f,mantle0,radfun,kmax);     /* calculate radial function */

    ar = model.ar;
    br = model.br;
    for(l=lm=0;l<=lmax;l++){              /* initialize               */
        ar[lm]=0.0;
        br[lm++]=0.0;
        for(m=1;m<=l;m++,lm++){
            ar[lm]=0.0;
            br[lm]=0.0;
        }
    }
    
/*    fac1 = 1.0/(2.0+xi);
    fac2 = 0.5-2.0*vsp2/3.0;
    fac3 = 2.0*vsp2/3.0-0.5*xi*fac1; */
    
    fac1 = 0.5;
    fac2 = 0.5-2.0*vsp2/3.0;
    fac3 = 2.0*vsp2/3.0;
    
/*    if(choice==SMODEL){
       printf("**************temporally set to cal Mu-Fac*Kappa***************\n");
       printf("read in fac\n");
       GetLine(s,MAXSTR);
       sscanf(s,"%f",&fac1);
       printf("fac=%f\n",fac1);
    } */


    for(k=0;k<=kmax;k++){
        for(l=lm=0;l<=lmax;l++){
            if(model.degmask[l]!='0'){
               if      (choice==SMODEL){
                  ar[lm]+=f[k]*fac1*amu[k][lm]; 
              /*      ar[lm]+=amu[k][lm]-fac1*akappa[k][lm]; */
               }else if(choice==PMODEL){
                   ar[lm]+=f[k]*(fac2*akappa[k][lm]+fac3*amu[k][lm]);
               } 
               
            }
            lm++;
            for(m=1;m<=l;m++,lm++){
                if(model.degmask[l]!='0'){
                   if      (choice==SMODEL){
                      ar[lm]+=f[k]*fac1*amu[k][lm];
                       br[lm]+=f[k]*fac1*bmu[k][lm]; 
                   /*    ar[lm]+=amu[k][lm]-fac1*akappa[k][lm];
                       br[lm]+=bmu[k][lm]-fac1*bkappa[k][lm]; */
                   }else if(choice==PMODEL){
                       ar[lm]+=f[k]*(fac2*akappa[k][lm]+fac3*amu[k][lm]);
                       br[lm]+=f[k]*(fac2*bkappa[k][lm]+fac3*bmu[k][lm]);
                   } 
                }
            }
        }
    } 

}

void ShellRad(float r, MODEL model)
{
float depth,**rknot,deptop,depbot,*ar,*br,**a,**b;
int k,l,m,lm,lmax,kmax;

      rknot  = model.rknot;
      ar     = model.ar;
      br     = model.br;
      a      = model.au;
      b      = model.bu;
      lmax   = model.lmaxum;
      kmax   = model.kmaxum;
      
      printf(" kmax =%d\n",kmax);
      printf(" lmax =%d\n",lmax);
      
      depth  = RADIUS-r;
      
      k=0;
      while(1){
         deptop=RADIUS-rknot[0][k];
         depbot=RADIUS-rknot[1][k];
         if(deptop<=depth && depth<depbot)break;
         k++;
         if(k>=(kmax+1))break;
      }

      printf(" depth=%f k=%d\n",depth,k);

      if(k>=(kmax+1)){
         printf(" depth is out of the range.\n");
      } else {
      
         for(l=lm=0;l<=lmax;l++){
             ar[lm]=a[k][lm];
             br[lm]=0.0;
             lm++;
             for(m=1;m<=l;m++,lm++){
                 ar[lm]=a[k][lm];
                 br[lm]=b[k][lm];
             }
         }
      }

}

void  Harvard2Complex(MODEL *model)
{
float *ar,*br;
int    lmax,l,m,lm;
   
   ar  = model->ar;
   br  = model->br;
   lmax= model->lmaxum;
/*
 * convert from harvard format to complex format
 */
   for(l=lm=0;l<=lmax;l++){
       for(m=0;m<=l;m++,lm++){
           if(m!=0){
              ar[lm]*= 0.5;
              br[lm]*=-0.5;
           }
       }
   }
    
}

void  Complex2Harvard(MODEL *model)
{
float *ar,*br;
int    lmax,l,m,lm;
   
   ar  = model->ac;
   br  = model->bc;
   lmax= model->lmaxum;
/*
 * convert from complex format to harvard format
 */
   for(l=lm=0;l<=lmax;l++){
       for(m=0;m<=l;m++,lm++){
           if(m!=0){
              ar[lm]*= 2.0;
              br[lm]*=-2.0;
           }
       }
   }
    
}

void  CST2Harvard(MODEL *model)
{
float *ar,*br;
int    lmax,l,m,lm;
   
   ar  = model->ar;
   br  = model->br;
   lmax= model->lmaxum;
/*
 * convert from CST format to Harvard format
 */
   for(l=lm=0;l<=lmax;l++){
       lm++;
       for(m=1;m<=l;m++,lm++){
           ar[lm]*= 2.0;
           br[lm]*=-2.0;
       }
   }
    
}

void  Harvard2HarvardPub(MODEL *model)
{
float *ar,*br;
int    lmax,l,m,lm;

float fac0=1.0/sqrt(4.0*PI);
float fac1=1.0/sqrt(8.0*PI);
  
   ar  = model->ar;
   br  = model->br;
   lmax= model->lmaxum;
/*
 * convert from harvard format to publication format
 */
   for(l=lm=0;l<=lmax;l++){
       ar[lm++]*=fac0;
       for(m=1;m<=l;m++,lm++){
           ar[lm]*=fac1;
           br[lm]*=fac1;
           if(m%2!=0){
              ar[lm]=-ar[lm];
              br[lm]=-br[lm];
           }
       }
   }
}

void  HarvardPub2Harvard(MODEL *model)
{
float *ar,*br;
int    lmax,l,m,lm;

float fac0=sqrt(4.0*PI);
float fac1=sqrt(8.0*PI);
  
   ar  = model->ar;
   br  = model->br;
   lmax= model->lmaxum;
/*
 * convert from harvard publication format to harvard format
 */
   for(l=lm=0;l<=lmax;l++){
       ar[lm++]*=fac0;
       for(m=1;m<=l;m++,lm++){
           ar[lm]*=fac1;
           br[lm]*=fac1;
           if(m%2!=0){
              ar[lm]=-ar[lm];
              br[lm]=-br[lm];
           }
       }
   }
}

void  PrintSphereModel(FILE *fp, MODEL* model)
{
float *ac, *bc;
int   lmax,l,m,lm;

   ac  = model->ar;
   bc  = model->br;
   lmax= model->lmaxum;

   for(l=lm=0;l<=lmax;l++){
       fprintf(fp,"%3d%3d%12.4e%12.4e\n",l,0,ac[lm],bc[lm]);
       lm++;
       for(m=1;m<=l;m++,lm++)
           fprintf(fp,"%3d%3d%12.4e%12.4e\n",l,m,ac[lm],bc[lm]);
   }
   
} 

/*
 * radial functions
 */
void RadFunc(float r0, float *f, int mantle, int function, int kmax)
{
double r,u,u2,df[100],rtop,rbot;

static double legendre[8]={
    0.7071067811865476,    /* sqrt(0.5)       */
    1.224744871391589,     /* sqrt(1.5)       */
    0.7905694150420949,    /* sqrt(5.0/8.0)   */
    0.9354143466934853,    /* sqrt(7.0/8.0)   */
    0.2651650429449553,    /* sqrt(9.0/128.0) */
    0.29315098498896,
    0.15934435979977,
    0.17116329922036,
};
int i;

   r=(double)r0;
   
   if(mantle==LOWERMANTLE){
      rtop=R670;
      rbot=RCMB;
/*     printf("LOWERMANTLE in sumrad\n"); */
   } else if (mantle==UPPERMANTLE){
      rtop=RMOHO;
      rbot=R670;
/*      printf("UPPERMANTLE in sumrad\n"); */
   } else if (mantle==WHOLEMANTLE){
      rtop=RMOHO;
      rbot=RCMB;
   } else if (mantle==INNERCORE){
      rtop=RICB;
      rbot=0.0;
   }
   if(r>rtop || r<rbot){      /* initialize radial function */
      for(i=0;i<=kmax;i++)f[i]=0.0;
      return;
   }

   if (mantle==INNERCORE){
      u = r/rtop;
   }else{
      u = (r+r-rtop-rbot)/(rtop-rbot);
   }
   
   if(function==LEGENDRE){
      printf(" LEGENDRE\n");
      u2   = u*u;
      if(kmax>7){
         printf(" error in calculating of LEGENDRE polynomial, kmax>7\n");
         exit(1);
      }
      df[0]  =  legendre[0];
      df[1]  =  legendre[1]*u;
      df[2]  =  legendre[2]*(3.0*u2-1.0);
      df[3]  =  legendre[3]*(5.0*u2-3.0)*u;
      df[4]  =  legendre[4]*((35.0*u2-30.0)*u2+3.0);
      df[5]  =  legendre[5]*((63.0*u2-70.0)*u2+15.0)*u;
      df[6]  =  legendre[6]*(((231.0*u2-315.0)*u2+105.0)*u2-5.0);
      df[7]  =  legendre[7]*(((429.0*u2-693.0)*u2+315.0)*u2-35.0)*u;
/* 
 * convert to single precision
 */      
      for(i=0;i<=kmax;i++)f[i]=(float)df[i];
   }else if(function==CHEBYSHEV){
      ChebyshevFun(u,kmax,f);
      if (mantle==INNERCORE){
          for(i=1;i<=kmax;i+=2)f[i]=0.0;
      }
   }
          
}

void ChebyshevFun(float x, int kmax, float *f)
{
float twox;
static float cheby_coef[14]={
    0.70710678118655, 1.2247448713916, 1.0350983390135, 1.0145993123918,
    1.00803225754840, 1.0050890913907, 1.0035149493262, 1.0025740068320,
    1.00196657023780, 1.0015515913133, 1.0012554932754, 1.0010368069141,
    1.00087070107920, 1.0007415648034};
int i;

   if(kmax>13){
      printf(" error in function ChebyshevFun, kmax>13\n");
      exit(1);
   }
   
   f[0] = 1.0;
   f[1] = x;
   twox = 2.0*x;
   for(i=2;i<=kmax;i++)
       f[i] =  twox*f[i-1]-f[i-2];
   for(i=0;i<=kmax;i++)
       f[i]*=cheby_coef[i];

}

void DChebyshevDx(double x, int kmax, double *f, double *df)
{
double twox;
static double cheby_coef[14]={
    0.70710678118655, 1.2247448713916, 1.0350983390135, 1.0145993123918,
    1.00803225754840, 1.0050890913907, 1.0035149493262, 1.0025740068320,
    1.00196657023780, 1.0015515913133, 1.0012554932754, 1.0010368069141,
    1.00087070107920, 1.0007415648034};
int i;

   if(kmax>13){
      printf(" error in function DChebyshevDx, kmax>13\n");
      exit(1);
   }
/*
 * calculate Chebyshev polynomials
 */
   f[0] = 1.0;
   f[1] = x;
   twox = 2.0*x;
   for(i=2;i<=kmax;i++)
       f[i] = twox*f[i-1]-f[i-2];
/*
 * calculate their derivative with respect to x
 */   
   df[0] = 0.0;
   df[1] = 1.0;
   
   for(i=2;i<=kmax;i++)
       df[i] = 2.0*f[i-1]+twox*df[i-1]-df[i-2];
/*
 * normalize
 */
   for(i=0;i<=kmax;i++){
       df[i]*=cheby_coef[i];
        f[i]*=cheby_coef[i];
   }
}

void ComplexPower(FILE *fp, MODEL *model)
{
float *ar,*br,power;
int   lmax,l,m,lm;
   
   ar  = model->ar;
   br  = model->br;
   lmax= model->lmaxum;
/*
 * calculate power for each degree
 */
   for(l=lm=0;l<=lmax;l++){
       power=ar[lm]*ar[lm];
       lm++;
       for(m=1;m<=l;m++,lm++){
           power+=2.0*(ar[lm]*ar[lm]+br[lm]*br[lm]);
       }
       fprintf(fp,"%5d %12.4e\n",l,power);
   }
}

float ComplexPower_RMS(MODEL *model)
{
float *ar,*br,power;
float rms, temp;
int   lmax,l,m,lm;
   
   ar  = model->ar;
   br  = model->br;
   lmax= model->lmaxum;
/*
 * calculate power for each degree
 */
   rms = 0.0;
   for(l=lm=0;l<=lmax;l++){
       power=ar[lm]*ar[lm];
       rms += power;
       lm++;
       for(m=1;m<=l;m++,lm++){
	   temp = 2.0*(ar[lm]*ar[lm]+br[lm]*br[lm]);
           power+=temp;
	   rms += temp;
       }
   }
   rms = sqrt((rms-ar[0]*ar[0]));
   return rms;
}


void HarvardPower(MODEL *model)
{
int    l,m,k,lm,lmax,kmax,mantle;
float  *ar,*br,**a,**b,rtop,rbot,r,depth;
char  s[MAXSTR];
FILE *fp;
/*
 * convert from harvard publication format to complex format
 */
  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]; 
          br[lm]  = b[k][lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]; 
              br[lm]  = b[k][lm];
          }
      }
      Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

  rtop= model->rtop;
  rbot= model->rbot;
  mantle= model->mantle;

  while(1){
        printf("enter depth (%8.2f - %8.2f km), to quit hit return.\n",
                RADIUS-rtop,RADIUS-rbot);
        printf("\ndepth=");
        GetLine(s,MAXSTR);
        if(s[0]=='\n')break;
        sscanf(s,"%f",&depth);
       
        fp=OpenFileWt("output data");         /* open file for output */
           r=RADIUS-depth;
           if(r<=RMOHO && r>=RCMB && mantle==WHOLEMANTLE){
              SumRad(r,model,WHOLEMANTLE);
           }else if(r>=R670 && r<=RMOHO && mantle==UPPERLOWERM){
              printf("upper mantle r=%f\n",r);
              SumRad(r,model,UPPERMANTLE);
           }else if(r>=RCMB && r<=R670  && mantle==UPPERLOWERM){
              printf("lower mantle r=%f\n",r);
              SumRad(r,model,LOWERMANTLE);
           }else{
              printf("error\n");
              exit(1);
          }
          ComplexPower(fp,model);
        fclose(fp);
  }

}

float HarvardPower_RMS(MODEL *model, float depth, int degree)
{
/* this is Yu Gu's implementation to calculate the rms heterogeneity
 *  for a given depth
 */
int    l,m,k,lm,lmax,kmax,mantle;
float  *ar,*br,**a,**b,rtop,rbot,r;
float  rms;
char  s[MAXSTR];
FILE *fp;
/*
 * convert from harvard publication format to complex format
 */
  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]; 
          br[lm]  = b[k][lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]; 
              br[lm]  = b[k][lm];
          }
      }
      Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

  rtop= model->rtop;
  rbot= model->rbot;
  mantle= model->mantle;

  printf("depth=%f\n", depth);
  if(depth<RADIUS-rtop || depth > RADIUS-rbot){
        printf("depth not between (%8.2f - %8.2f km)!  exit...\n",
                RADIUS-rtop,RADIUS-rbot);
	exit(-1);
  }  
  r=RADIUS-depth;
  if(r<=RMOHO && r>=RCMB && mantle==WHOLEMANTLE){
  	SumRad(r,model,WHOLEMANTLE);
  }else if(r>=R670 && r<=RMOHO && mantle==UPPERLOWERM){
        printf("upper mantle r=%f\n",r);
        SumRad(r,model,UPPERMANTLE);
  }else if(r>=RCMB && r<=R670  && mantle==UPPERLOWERM){
        printf("lower mantle r=%f\n",r);
        SumRad(r,model,LOWERMANTLE);
  }else{
        printf("error\n");
        exit(1);
  }
  rms = ComplexPower_RMS(model);
  return rms;

}


void SAF2Complex(MODEL *model)
{
int    l,m,k,lm,lmax,kmax;
float  *ar,*br,**a,**b,rtop,rbot,r0,depth,fac;
char  s[MAXSTR];
FILE *fp;
/*
 * convert from harvard publication format to complex format
 */
  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  rtop= model->rtop;
  rbot= model->rbot;
  
  fac =sqrt(4.0*PI);
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]*fac; 
          br[lm]  = b[k][lm]*fac;
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]*fac; 
              br[lm]  = b[k][lm]*fac;
          }
      }
      HarvardPub2Harvard(model); 
      Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

  while(1){
        printf("enter depth (%8.2f - %8.2f km), to quit hit return.\n",
                RADIUS-rtop,RADIUS-rbot);
        printf("\ndepth=");
        GetLine(s,MAXSTR);
        if(s[0]=='\n')break;
        sscanf(s,"%f",&depth);
       
        fp=OpenFileWt("output data");         /* open file for output */
           r0=RADIUS-depth;
           SumRad(r0,model,WHOLEMANTLE);
           PrintSphereModel(fp,model);
        fclose(fp);
  }

}

void PrintShell(MODEL model)
{
float depth,**rknot,deptop,depbot,*ac,*bc,**a,**b;
int k,l,m,lm,lmax,kmax;
char s[MAXSTR],filename[MAXSTR];
FILE *fp;

      rknot  = model.rknot;
      ac     = model.ar;
      bc     = model.br;
      a      = model.au;
      b      = model.bu;
      lmax   = model.lmaxum;
      kmax   = model.kmaxum;

      while(1){
        printf("\n\n");
        for(k=0;k<=kmax;k++){
            printf("layer = %4d %12.2f%12.2f\n",k+1,RADIUS-rknot[0][k],RADIUS-rknot[1][k]);
        }
        printf("enter depth (%8.2f - %8.2f km), to quit hit return.\n",
                RADIUS-rknot[0][0],RADIUS-rknot[1][kmax]);
        printf("\ndepth=");
        GetLine(s,MAXSTR);
        if(s[0]=='\n')break;
        sscanf(s,"%f",&depth);
                
        k=0;
        while(1){
          deptop=RADIUS-rknot[0][k];
          depbot=RADIUS-rknot[1][k];
          if(deptop<=depth && depth<depbot)break;
          k++;
          if(k>=(kmax+1))break;
        }
        if(k>=(kmax+1)){
           printf(" depth is out of the range.\n");
        } else {
           printf(" layer = %d\n",k);
           fp=OpenFileWt("output data");         /* open file for output */
           
           for(l=lm=0;l<=lmax;l++){
               ac[lm]=a[k][lm];
               bc[lm]=0.0;
               lm++;
               for(m=1;m<=l;m++,lm++){
                  ac[lm]=a[k][lm];
                  bc[lm]=b[k][lm];
               }
           }
           PrintSphereModel(fp,&model);
           fclose(fp);
        }
      }

}


void PrintHarvard(MODEL *model)
{
char  DegMask[50],LayerMask[50];
char  s[MAXSTR];
int   k,l,m,lm,newl,radfun,lmax,kmax;
float **a,**b;
FILE  *fp;


   radfun=model->radfun;
   lmax  =model->lmaxum;
   kmax  =model->kmaxum;
   a     =model->au;
   b     =model->bu;
/*
 * define layer mask and degree mask
 */
   for(l=0;l<=lmax;l++)  DegMask[l]='1';
   DegMask[lmax+1]='\0';
   for(k=2;k<=kmax+2;k++)LayerMask[k]='1';
   LayerMask[0]='0';
   LayerMask[1]='0';
   LayerMask[kmax+3]='\0';
/*
 * open file for output 
 */
   fp=OpenFileWt("output data");
   
   fprintf(fp,"             %2d %s  %2d %s%2d\n",lmax,DegMask,kmax+2+1,LayerMask,radfun);
   for(k=0;k<=kmax;k++){
       printf(" writing layer k=%4d \n",k);
       for(l=lm=0;l<=lmax;l++){
           fprintf(fp,"%12.4e",a[k][lm]);
           lm++;
           newl=1;
           for(m=1;m<=l;m++,lm++){
               fprintf(fp,"%12.4e",a[k][lm]);
               newl++;
               if(newl%11==0)fprintf(fp,"\n");
               fprintf(fp,"%12.4e",b[k][lm]);
               newl++;
               if(newl%11==0)fprintf(fp,"\n");
           }
           if(newl%11!=0)fprintf(fp,"\n");
       }
   }
   
   fclose(fp);
    
}

void MB92_2Complex(MODEL *model)
{
int l,m,k,lm,lmax,kmax;
float *ar,*br,**a,**b;

  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]; 
          br[lm]  = b[k][lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]; 
              br[lm]  = b[k][lm];
          }
      }
      CST2Harvard(model);
      Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

}

void ZT2Complex(MODEL *model)
{
int l,m,k,lm,lmax,kmax;
float *ar,*br,**a,**b,*vavg,vk;

  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  vavg= model->vavg;
  
  for(k=0;k<=kmax;k++){
        a[k][0]-=vavg[k];            /* remove mean velocity   */
  }
  
  for(k=0;k<=kmax;k++){
      vk=vavg[k];
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]/vk; 
          br[lm]  = b[k][lm]/vk;
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]/vk; 
              br[lm]  = b[k][lm]/vk;
          }
      }
      HarvardPub2Harvard(model);
      Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

}

void MRD2ComOrHarvard(MODEL *model, int fmt)
{
int l,m,k,lm,lmax,kmax;
float *ar,*br,**a,**b,r,**rknot,density,fac;
PREM  *par;  

  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  rknot = model->rknot;
  par   = model->par;
  
  
  for(k=0;k<=kmax;k++){
      r=0.5*(rknot[0][k]+rknot[1][k]);         /* radius at the center of layer*/
      PremVelDen(r,&density,par,'d');          /* density from PREM            */
      fac  = 1.0e-5/(rknot[0][k]-rknot[1][k]); /* 1/(layer thickness  in  cm ) */
      fac *= 0.1/density;                      /* delta d /d                   */
      printf(" density = %8.2f\n",density);
/*      fac /= 0.3;                              /* convert to ds / s            */
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]*fac; 
          br[lm]  = b[k][lm]*fac;
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]*fac; 
              br[lm]  = b[k][lm]*fac;
          }
      }
      HarvardPub2Harvard(model);
      if(fmt==CMPFMT) Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

}

void ZT2Harvard(MODEL *model)
{
int l,m,k,lm,lmax,kmax;
float *ar,*br,**a,**b,*vavg,vk;

  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  vavg= model->vavg;
  printf("converting ZT model to harvard format\n");
  
  for(k=0;k<=kmax;k++){
        a[k][0]-=vavg[k];            /* remove mean velocity   */
  }
  
  for(k=0;k<=kmax;k++){
      vk=vavg[k];
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]/vk; 
          br[lm]  = b[k][lm]/vk;
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]/vk; 
              br[lm]  = b[k][lm]/vk;
          }
      }
      HarvardPub2Harvard(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }
  
}


void MDLSH2Complex(MODEL *model)
{
int l,m,k,lm,lmax,kmax;
float *ar,*br,**a,**b;

  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]*0.01; 
          br[lm]  = b[k][lm]*0.01;
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]*0.01; 
              br[lm]  = b[k][lm]*0.01;
          }
      }
      HarvardPub2Harvard(model);
      Harvard2Complex(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

}

void MDLSH2Harvard(MODEL *model)
{
int l,m,k,lm,lmax,kmax;
float *ar,*br,**a,**b;

  lmax= model->lmaxum;
  kmax= model->kmaxum;
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]*0.01; 
          br[lm]  = b[k][lm]*0.01;
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]*0.01; 
              br[lm]  = b[k][lm]*0.01;
          }
      }
      HarvardPub2Harvard(model);

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

}

void  PrintZhang(MODEL model)
{
float *ac, *bc;
int   lmax,l,m,lm,nl;

   ac  = model.ar;
   bc  = model.br;
   lmax= model.lmaxum;

   for(l=lm=nl=0;l<=lmax;l++){
       printf("%13.6e",ac[lm]);
       nl++;
       if((nl%6)==0){
           printf("\n");
           nl=0;
       }
       lm++;
       for(m=1;m<=l;m++,lm++){
           printf("%13.6e",ac[lm]);
           nl++;
           if((nl%6)==0){
               printf("\n");
               nl=0;
           }
           printf("%13.6e",bc[lm]);
           nl++;
           if((nl%6)==0){
               printf("\n");
               nl=0;
           }
       }
   }
   printf("\n");
   
} 

void Harvard2Zhang(MODEL *model)
{
int    l,m,k,lm,lmax,kmax;
float  *ar,*br,**a,**b,r0,v;
int    i,nr=145,nc;
PREM    *par;

static float r[200]={  
    0.0,   15.0,   24.4,   52.2,   80.0,  100.0,  120.0,  140.0,  160.0,  180.0,
  200.0,  220.0,  240.0,  260.0,  280.0,  300.0,  320.0,  340.0,  360.0,  380.0, 
  400.0,  420.0,  440.0,  460.0,  480.0,  500.0,  520.0,  540.0,  560.0,  580.0,
  600.0,  620.0,  640.0,  660.0,  680.0,  700.0,  720.0,  740.0,  760.0,  780.0,
  800.0,  820.0,  840.0,  860.0,  880.0,  900.0,  920.0,  940.0,  960.0,  980.0,
 1000.0, 1020.0, 1040.0, 1060.0, 1080.0, 1100.0, 1120.0, 1140.0, 1160.0, 1180.0,
 1200.0, 1220.0, 1240.0, 1260.0, 1280.0, 1300.0, 1320.0, 1340.0, 1360.0, 1380.0,
 1400.0, 1420.0, 1440.0, 1460.0, 1480.0, 1500.0, 1520.0, 1540.0, 1560.0, 1580.0,
 1600.0, 1620.0, 1640.0, 1660.0, 1680.0, 1700.0, 1720.0, 1740.0, 1760.0, 1780.0,
 1800.0, 1820.0, 1840.0, 1860.0, 1880.0, 1900.0, 1920.0, 1940.0, 1960.0, 1980.0,
 2000.0, 2020.0, 2040.0, 2060.0, 2080.0, 2100.0, 2120.0, 2140.0, 2160.0, 2180.0,
 2200.0, 2220.0, 2240.0, 2260.0, 2280.0, 2300.0, 2320.0, 2340.0, 2360.0, 2380.0,
 2400.0, 2420.0, 2440.0, 2460.0, 2480.0, 2500.0, 2520.0, 2540.0, 2560.0, 2580.0,
 2600.0, 2620.0, 2640.0, 2660.0, 2680.0, 2700.0, 2720.0, 2740.0, 2760.0, 2780.0,
 2800.0, 2820.0, 2840.0, 2860.0, 2880.0, 2890,0};

/*
 * convert from harvard publication format to harvard format
 */
  lmax= model->lmaxum;
  kmax= model->kmaxum;
  nc  = (lmax+2)*(lmax+1)/2*2-(lmax+1);
  ar  = model->ar;
  br  = model->br;
  a   = model->au;
  b   = model->bu;
  
  for(k=0;k<=kmax;k++){
      for(l=lm=0;l<=lmax;l++){
          ar[lm]  = a[k][lm]; 
          br[lm]  = b[k][lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              ar[lm]  = a[k][lm]; 
              br[lm]  = b[k][lm];
          }
      }
      Harvard2HarvardPub(model); 

      for(l=lm=0;l<=lmax;l++){
          a[k][lm] = ar[lm]; 
          b[k][lm] = br[lm];
          lm++;
          for(m=1;m<=l;m++,lm++){
              a[k][lm] = ar[lm]; 
              b[k][lm] = br[lm];
          }
      }
  }

  par = model->par;
/*
 * sum radial functions for different r
 */
   for(i=2;i<nr;i++){
       r0=0.5*(RADIUS-r[i]+RADIUS-r[i+1]);
       PremVelDen(r0,&v,par,'s');             /* S velocity for PREM at r */
       printf("%10.2f%10.2f%10.5f%5d%5d\n",r[i],r[i+1],v,lmax,nc);
       SumRad(r0,model,WHOLEMANTLE);
       PrintZhang(*model);
   }

}

int ChooseFormat(void)
{
int sel;

    printf("---------------------------------------------\n");
    printf("|%2d---- Harvard Format                      |\n",HRVFMT);
    printf("|%2d---- Harvard Publication Format          |\n",HRVPUBFMT);
    printf("|%2d---- L02.56  Format                      |\n",L0256FMT);
    printf("|%2d---- Complex Spherical Format            |\n",CMPFMT);
    printf("|%2d---- Zhang and Tanimoto Format           |\n",ZTFMT);
    printf("|%2d---- CST Format                          |\n",CSTFMT);
    printf("|%2d---- Power                               |\n",POWER);
    printf("|%2d---- Scale to P-model                    |\n",SCALEP);
    printf("|%2d---- Geomap Binary Format	         |\n",GEOMAPFMT);
    printf("|%2d---- Crust thickness                     |\n",CRUST);
    printf("---------------------------------------------\n");

    printf("select output format:");
    sel=CGetInt();

    if     (sel==   HRVFMT)printf("Select Harvard Format\n");
    else if(sel==HRVPUBFMT)printf("Select Harvard Publication Format\n");
    else if(sel== L0256FMT)printf("Select L02.56  Format\n");
    else if(sel==   CMPFMT)printf("Select Complex Format\n");
    else if(sel==    ZTFMT)printf("Select Zhang and Tanimoto Format\n");
    else if(sel==   SCALEP)printf("Scale S-model to P-model\n");
    else if(sel==   CSTFMT)printf("Select CST     Format\n");

    if     (sel==   POWER) printf("Select power\n");
    else if(sel==   CRUST) printf("Select crust thickness\n");

    if(sel>=HRVFMT && sel<= CRUST)return sel;
    else                         return FALSE;

}

void ReadZT36(MODEL *model)
{
float **au,**bu,a,b,**rknot,*vavg,deptop,depbot;
int  l,m,k,l1,m1,lm,lmax,kmax,l0,lm0;
char *degmask;
FILE *fp;

   lmax=model->lmaxum;
   kmax=model->kmaxum;
   au = model->au;
   bu = model->bu;
   rknot  = model->rknot;
   vavg   = model->vavg;
   degmask= model->degmask;
   
   for(l=0;l<=lmax;l++)degmask[l]='1';
   degmask[lmax+1]='\0';

   fp=OpenFl(model->name1);
    
    for(k=0;k<=kmax;k++){
        printf(" reading coefficients for  layer k=%4d\n",k);
        fscanf(fp,"%f %f %f %d %d",&deptop,&depbot,&vavg[k],&l0,&lm0);
        rknot[0][k]=RADIUS-deptop;
        rknot[1][k]=RADIUS-depbot;
        if(l0!=lmax || lm0 != 1369){
           printf(" error in reading Zhang's model\n");
           exit(1);
        }
        for(l=lm=0;l<=lmax;l++){
            fscanf(fp,"%f",&a);
            au[k][lm]  =a;
            bu[k][lm++]=0.0;
            for(m=1;m<=l;m++,lm++){
                fscanf(fp,"%f %f",&a,&b);
                au[k][lm]=a;
                bu[k][lm]=b;
            }
        }
    }
    fclose(fp);

}

/*
 ***************************CMT functions********************************
 */
void OpenCMT(int *fd, int *numeq)
         
                          /* total # of eq in CMT file */
{
int size;
static char *defname="/home/tempo/eq/cmt/allorder.bin";
char *name,buff[MAXSTR];
   
/*
 * Open data file
 */
   printf("Input CMT file name, hit return to use the default file.\n");
   printf("Default file:%s\n",defname);
   
   GetLine(buff,MAXSTR);
   if(buff[0]=='\n'){
      name=defname;
   } else {
      name=buff;
   }
   
   printf("CMT file:%s\n",name);
   
   *fd=SysOpen(name,0);
/*
 * the total # of eqs in the CMT catalog
 */   
   size =sizeof(CMT);
   printf("record length = %d\n",size);
   *numeq=lseek(*fd,0L,2)/size;
   printf("Total of %d earthquakes in the cmt catalog\n",*numeq);
}

int ReadAllCMT(int fd, int numeq, int maxcmt, CMT *cmt)
{
int  size;

   if(maxcmt<numeq){
      printf("error, number of eq exceeds the size of  array cmt %d > %d\n",
              numeq,maxcmt);
      exit(1);
   }
   
   size=sizeof(CMT);

   lseek(fd,0,0);
   read(fd,cmt,size*numeq);
   printf("Total record read = %d\n",numeq);
   return 1;

} 


ISCEQ *ReadISCEq(int *numeq)
{
int fd,size;
static char *defname="/scratch/troika13/su/isc/data/eq.log";
char *name,buff[MAXSTR];
ISCEQ  *isceq;
   
   printf("Input ISC file name, hit return to use the default file.\n");
   printf("Default file:%s\n",defname);
   
   GetLine(buff,MAXSTR);
   if(buff[0]=='\n'){
      name=defname;
   } else {
      name=buff;
   }
   
   printf("ISC file:%s\n",name);
   
   fd=SysOpen(name,0); 
   size =sizeof(ISCEQ);
   printf("record length = %d\n",size);
   *numeq=lseek(fd,0L,2)/size;
   printf("Total of %d earthquakes in the ISC catalog\n",*numeq);
   
   isceq=ISCEqVec(1,*numeq);	/* array to save the eqs */

   lseek(fd,0,0);			/* rewind to the beginning of the file */
   read(fd,isceq,size*(*numeq));
   printf("Total number of earthquakes read = %d\n",*numeq);
   return isceq-1;
} 

ISCST *ReadISCSt(int *numst)
{
int fd,size;
static char *defname="/scratch/troika13/su/isc/data/station.log";
char *name,buff[MAXSTR];
ISCST  *iscst;
   
   printf("Type station file, hit return to use the default file.\n");
   printf("Default file:%s\n",defname);
   
   GetLine(buff,MAXSTR);
   if(buff[0]=='\n'){
      name=defname;
   } else {
      name=buff;
   }
   
   printf("ISC station file:%s\n",name);
   
   fd=SysOpen(name,0); 
   size =sizeof(ISCST);
   printf("record length = %d\n",size);
   *numst=lseek(fd,0L,2)/size;
   printf("Total of %d stations in the ISC catalog\n",*numst);
   
   iscst=ISCStVec(1,*numst);	/* array to save the eqs */

   lseek(fd,0,0);			/* rewind to the beginning of the file */
   read(fd,iscst,size*(*numst));
   printf("Total number of stations read = %d\n",*numst);
   return iscst-1;
} 

int OpenResFile(void)
{
char buf[132],filenm[132];
int fd;

  printf("Travel time residual data file:\n");
  GetLine(buf,132);
  sscanf(buf,"%s",filenm); 
  fd=SysOpen(filenm,0);		/* open for read only   		*/
  printf("total number of residuals =%d\n",lseek(fd,0L,2)/12);
  lseek(fd,0,0);			/* rewind to the beginning of file	*/
  return fd;
}

void UnPackEq(ISCEQ isceq, int *iyear, int *month, int *iday, int *ihour, int *imin, float *sec, float *lat, float *lon, float *dep, int *np, float *xmag1, float *xmag2, int *ibeg, int *nobs, int *ii)
{
int jt;
float dep1,dep2, mag;

  *iyear=isceq.iymag1 >> 8;
  *xmag1= 0.1*(float)(isceq.iymag1-(*iyear)*256);
  
  *month=isceq.mondep1/1000;
  dep1=(float)(isceq.mondep1%1000);
  
  *iday=isceq.daydep2/1000;
  dep2=(float)(isceq.daydep2%1000);
  
  *ihour=isceq.hournp/1000;
  *np=isceq.hournp%1000;
  
  *imin=isceq.minmag2 >> 8;
  *xmag2=0.1*(float)(isceq.minmag2-(*imin)*256);
  
  jt  =isceq.jtsec/1000;
  *sec=0.1*(float)(isceq.jtsec%1000);
  
  *ibeg=isceq.rec;
  
  *lon=0.01*(float)isceq.lon;
  *lat=0.01*(float)isceq.lat;
  *nobs=isceq.nobs;
  
  *ii=isceq.iflag;

  if(jt==1)*dep=dep2;
  else     *dep=dep1;

}


#define recl 12
#define i2to17 131072
#define i2to13 8192
#define i2to20 1048576
#define i2to10 1024
#define i2to16 65536

void PackRes(int *ires, int iflag, int neq, int ist, int iscres, float time, float pres1, float pres2)
{
int itime,itemp;

  if(time<0.0){
	printf("error in subroutine pack_res");
	exit(1);
  }
       
  ires[0]=i2to13*iflag+ist;
  itime = time*10.0+0.5;
  ires[0]=ires[0]*i2to17 + itime;
  ires[1]=(iscres+i2to10)*i2to20+ neq;
  itemp = 100.0*pres1+0.5+i2to13;
  ires[2]=itemp*i2to16;
  itemp = 100.0*pres2+0.5+i2to13;
  ires[2]=ires[2] + itemp;
}


void UnPackRes(int *ires, int *iflag, int *neq, int *ist, int *iscres, float *time, float *pres1, float *pres2)
{
int itemp,ib1;

  ib1=ires[0]>>17;
  *iflag=ib1 >> 13;	  /* first two bits for flag */
  *ist  =ib1-((*iflag) << 13);
  *time = 0.1*(float)(ires[0]%i2to17);
  
  *iscres = ires[1]/i2to20-i2to10;
  *neq    = ires[1]%i2to20;
   itemp  = ires[2]/i2to16-i2to13;
  *pres1  = 0.01*(float)itemp;
   itemp  = ires[2]%i2to16-i2to13;
  *pres2  = 0.01*(float)itemp;

}

#undef recl
#undef i2to17
#undef i2to13
#undef i2to20
#undef i2to10
#undef i2to16

void  WriteIpm(float *ata, float *atd, int numatd)
{
int fd,numata,n_written;
char s[132],flname[132];

  printf("output ipm  file:");
  scanf("%s",flname);
/*  GetLine(s,130);
  sscanf(s,"%s",flname); */

  fd=SysOpen(flname,1);
   
  n_written=write(fd,&numatd,sizeof(int));
  numata=(numatd*(numatd+1))/2;
  n_written=write(fd,ata,numata*sizeof(float));
  printf("n_written=%d\n",n_written);
  n_written=write(fd,atd,numatd*sizeof(float));
  printf("n_written=%d\n",n_written);
  close(fd);
}

void  ReadIpm(IPM *ipm)
{
char name[132];
float *ata,*atd;
int fd,numata,numatd,n_read;
 
  printf("Ipm file:");
  GetLine(name,132);
  sscanf(name,"%s",name);
  printf("input file=%s\n",name);
  
  fd=SysOpen(name,0);
  
  n_read=read(fd,&numatd,sizeof(int));
  numata=(numatd*(numatd+1))/2;
  ipm->ata=ata = Vector(1,numata);
  ipm->atd=atd = Vector(1,numatd);

  n_read=read(fd,ata,numata*sizeof(float));
  if(n_read!=(numata*sizeof(float))){
     printf(" could not read matrix ata\n");
     exit(1);
  }else{
     printf("number of elements in ata=%d\n",numata);
  }
  n_read=read(fd,atd,numatd*sizeof(float));
  if(n_read!=(numatd*sizeof(float))){
     printf(" could not read matrix atd\n");
     exit(1);
  }else{
     ipm->numatd=numatd;
     printf("number of elements in atd=%d\n",numatd);
  }  
}

/*
 **********************num_r functions**************************
 */
void rk4(float *y, float *dydx, int n, float x, float h, float *yout, 
	 void (*derivs)(float,float *,float *))
{
	int i;
	float xh,hh,h6,*dym,*dyt,*yt,*vector(int nl, int nh);
	void free_vector(float *v, int nl, int nh);

	dym=vector(1,n);
	dyt=vector(1,n);
	yt=vector(1,n);
	hh=h*0.5;
	h6=h/6.0;
	xh=x+hh;
	for (i=1;i<=n;i++) yt[i]=y[i]+hh*dydx[i];
	(*derivs)(xh,yt,dyt);
	for (i=1;i<=n;i++) yt[i]=y[i]+hh*dyt[i];
	(*derivs)(xh,yt,dym);
	for (i=1;i<=n;i++) {
		yt[i]=y[i]+h*dym[i];
		dym[i] += dyt[i];
	}
	(*derivs)(x+h,yt,dyt);
	for (i=1;i<=n;i++)
		yout[i]=y[i]+h6*(dydx[i]+dyt[i]+2.0*dym[i]);
	free_vector(yt,1,n);
	free_vector(dyt,1,n);
	free_vector(dym,1,n);
}

void nrerror(char *error_text)
{
/*	void exit(); */

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}



float *vector(int nl, int nh)
{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1+1)*sizeof(float));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

int *ivector(int nl, int nh)
{
	int *v;

	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl;
}

double *dvector(int nl, int nh)
{
	double *v;

	v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl;
}



float **matrix(int nrl, int nrh, int ncl, int nch)
{
	int i;
	float **m;

	m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
		m[i] -= ncl;
	}
	return m;
}

double **dmatrix(int nrl, int nrh, int ncl, int nch)
{
	int i;
	double **m;

	m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!m) nrerror("allocation failure 1 in dmatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
		m[i] -= ncl;
	}
	return m;
}

int **imatrix(int nrl, int nrh, int ncl, int nch)
{
	int i,**m;

	m=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
	if (!m) nrerror("allocation failure 1 in imatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int));
		if (!m[i]) nrerror("allocation failure 2 in imatrix()");
		m[i] -= ncl;
	}
	return m;
}



float **submatrix(float **a, int oldrl, int oldrh, int oldcl, int oldch, int newrl, int newcl)
{
	int i,j;
	float **m;

	m=(float **) malloc((unsigned) (oldrh-oldrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure in submatrix()");
	m -= newrl;

	for(i=oldrl,j=newrl;i<=oldrh;i++,j++) m[j]=a[i]+oldcl-newcl;

	return m;
}



void free_vector(float *v, int nl, int nh)
{
	free((char*) (v+nl));
}

void free_ivector(int *v, int nl, int nh)
{
	free((char*) (v+nl));
}

void free_dvector(double *v, int nl, int nh)
{
	free((char*) (v+nl));
}



void free_matrix(float **m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

void free_dmatrix(double **m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

void free_imatrix(int **m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}



void free_submatrix(float **b, int nrl, int nrh, int ncl, int nch)
{
	free((char*) (b+nrl));
}



float **convert_matrix(float *a, int nrl, int nrh, int ncl, int nch)
{
	int i,j,nrow,ncol;
	float **m;

	nrow=nrh-nrl+1;
	ncol=nch-ncl+1;
	m = (float **) malloc((unsigned) (nrow)*sizeof(float*));
	if (!m) nrerror("allocation failure in convert_matrix()");
	m -= nrl;
	for(i=0,j=nrl;i<=nrow-1;i++,j++) m[j]=a+ncol*i-ncl;
	return m;
}



void free_convert_matrix(float **b, int nrl, int nrh, int ncl, int nch)
{
	free((char*) (b+nrl));
}


#define EPS 3.0e-11

void gauleg(double x1, double x2, double *x, double *w, int n)
{
	int m,j,i;
	double z1,z,xm,xl,pp,p3,p2,p1;

	m=(n+1)/2;
	xm=0.5*(x2+x1);
	xl=0.5*(x2-x1);
	for (i=1;i<=m;i++)  {
		z=cos(3.141592654*(i-0.25)/(n+0.5));
		do {
			p1=1.0;
			p2=0.0;
			for (j=1;j<=n;j++) {
				p3=p2;
				p2=p1;
				p1=((2.0*j-1.0)*z*p2-(j-1.0)*p3)/j;
			}
			pp=n*(z*p1-p2)/(z*z-1.0);
			z1=z;
			z=z1-p1/pp;
		} while (fabs(z-z1) > EPS);
		x[i]=xm-xl*z;
		x[n+1-i]=xm+xl*z;
		w[i]=2.0*xl/((1.0-z*z)*pp*pp);
		w[n+1-i]=w[i];
	}
}

#undef EPS



void SaveGrid2D(float **grid, float bllat, float bllon, float trlat, float trlon, float r, int row, int col)
{
char filenm[132],line[132];
int fd,n_written,i,dim,ver,row0=row,col0=col;
float blat,blon,tlat,tlon,rr;
int j;

 printf(" Type file name to save the grid points [def_file]:");
 GetLine(line,132);
 if(line[0]=='\n'){
    fd=creat("def_file",PMODE);
    fd=SysOpen("def_file",2);
 }else{
    sscanf(line,"%s",filenm);
    fd=creat(filenm,PMODE);
    fd=SysOpen(filenm,2);
 }
/*
 for(i=1;i<row;i++){
 	for(j=1;j<col;j++){
		printf("i=%d j=%d %f\n", i, j, grid[i][j]);
	}
 }
*/
 printf("grid[0][0]=%f, grid[0][1]=%f\n", grid[0][0], grid[0][1]);
 dim=2;
 ver=100;						/* version number */
 write(fd,&ver,sizeof(int));
 write(fd,&dim,sizeof(int));			/* 2-D */
 printf("blat=%f %f %f %f\n",bllat,bllon,trlat,trlon);
 blat=bllat;
 blon=bllon;
 tlat=trlat;
 tlon=trlon;
 rr=r;
 write(fd,&blat,sizeof(float));		/* bottom-left lat */
 write(fd,&blon,sizeof(float));		/* bottom-left lon */
 write(fd,&tlat,sizeof(float));		/* top-right lat   */
 write(fd,&tlon,sizeof(float));		/* top-right lon   */
 write(fd,&rr,sizeof(float));			/* radius          */
 write(fd,&row0,sizeof(int));
 write(fd,&col0,sizeof(int));
 for(i=0;i<row;i++)
     printf("number of bytes written=%d\n",write(fd,grid[i],col*sizeof(float)));
 
 if(line[0]=='\n')printf(" The grid values are saved in file grid.dat\n");
 else	printf(" The grid values are saved in file %s\n",filenm);

}

int GetOptions(char *opt, int argc, char **argv, char **str, char **arg)
{
int i,k,len,narg;
char *cs;

   k=0;
   while(strcmp(str[k],"END")){
      if(!strncmp(opt,str[k],2)){
         cs=strstr(str[k],":");    /* 1st occurence of ":" 	*/
         cs=strstr(&cs[1],":");    /* 2nd occurence		*/
         sscanf(&cs[1],"%d",&narg);
         break;
      }
      k++;
   }
   
   for(i=1;i<argc;i++){		/* loop on every argument */
       if(argv[i][0]=='-'){
          if((len=strlen(argv[i]))>2||narg==0){		/* there is more than one option */
              for(k=1;k<len;k++){				/* no argument is needed */
                  if(argv[i][k]==opt[1])return TRUE;
              }
          }else{		/* there is only one option, it may require one or more arguments */
              if(argv[i][1]==opt[1]){
                 for(k=0;k<narg;k++){
                     strcpy(arg[k],argv[i+k+1]);
                     printf("%s\n",arg[k]);
                 }
                 return TRUE;
              }
          }
       }
   }
   return FALSE;
}

int SetOptions(int argc, char **argv, char **str)
{
int i,j,k,narg,find,len;
char *cs,*ar0;
extern int HelpMessage(char *exec, char **str, char *mess);
/*
 * print out help messages if necessary
 */
   if(argc>1){
      ar0=argv[1];
      if(ar0[0]=='-' && !isalpha(ar0[1]))HelpMessage(argv[0],str,"");
   }
/*
 * check all the required options are there
 */
   k=0;
   while(strcmp(str[k],"END")){
      if(str[k][3]=='r'){		/* for required options */
         find=FALSE;
         for(i=1;i<argc;i++){		/* loop on every argument */
             ar0=argv[i];
             if(ar0[0]=='-'){		/* every option has to begin with '-' */
                if(strchr(ar0,str[k][1])!=NULL){
                   find=TRUE;
                   break;
                }
             }
         }
         if(!find)
            HelpMessage(argv[0],str,"Missing required options");
      }
      k++;
   }
/*   
 * check whether all the options are valid
 */

    for(i=1;i<argc;i++){		/* loop on every argument */
        ar0=argv[i];
        if(ar0[0]=='-'){		/* every option has to begin with '-' */
           len=strlen(ar0);
           if(len==2){			/* one option with/without argument */
              find=FALSE;    
              k=0;
              while(strcmp(str[k],"END")){
                    if(str[k++][1]==ar0[1]){
                       find=TRUE;
                       break;
                    }
              }
              if(!find){
                  HelpMessage(argv[0],str,"unknown options");		
              }else{			/* further check the required argument */
                  cs=strstr(str[k-1],":");  /* 1st occurence of ":" 	*/
                  cs=strstr(&cs[1],":");    /* 2nd occurence		*/
                  if(cs[1]!='0'){	   	  /* options needs arguments */
                     sscanf(&cs[1],"%d",&narg);
                     if((narg+i)>=argc)
                         HelpMessage(argv[0],str,"too few arguments");
                     for(k=i+1;k<(i+1+narg);k++){
                         ar0=argv[k];
                         if(ar0[0]=='-')HelpMessage(argv[0],str,"too few arguments");
                     }
                     i+=narg;
                  }
              }
           }else{				/* several options without argument */
              for(j=1;j<len;j++){	/* check each option			*/
                  find=FALSE;    
                  k=0;
                  while(strcmp(str[k],"END")){
                     if(str[k][1]==ar0[j]){
                        find=TRUE;
                        if(str[k][5]!='0')
                           HelpMessage(argv[0],str,"wrong usage for option which needs argument");	                        
                        break;
                     }
                     k++;
                  }
                  if(!find)HelpMessage(argv[0],str,"unknown options");		
              }
           }
        }else{
           HelpMessage(argv[0],str,"Program option error");		
        }
    }
    return 1;
}


int HelpMessage(char *exec, char **str, char *mess)
{
int i,j,k,narg;
char *cs;

/*
 * print out help
 */
   if(strlen(mess)>0)printf("*************%s*************\n",mess);
   printf("Program usage:\n");
   printf("   %s -",exec);
   j=0;					/* flags without argument	*/
   while(strcmp(str[j],"END")){
         cs=strstr(str[j],":");  	/* 1st occurence of ":" 	*/
         cs=strstr(&cs[1],":");  	/* 2nd occurence			*/
         if(cs[1]=='0')printf("%c",str[j][1]);
               j++;
  }
  j=0;					/* flags with argument */
  while(strcmp(str[j],"END")){
        cs=strstr(str[j],":");  /* 1st occurence of ":" 	*/
        cs=strstr(&cs[1],":");  /* 2nd occurence		*/
        if(cs[1]!='0'){	   /* options needs arguments */
           sscanf(&cs[1],"%d",&narg);
           printf(" -%c ",str[j][1]);
           for(k=0;k<narg;k++)printf("arg%d ",k+1);
        }
        j++;
  }
  printf("\n\n");

  printf("Description\n");
  j=0;
  while(strcmp(str[j],"END")){
        cs=str[j];
        printf("%c%c  ",cs[0],cs[1]);
        if(cs[5]=='0'){
           printf(" 0 argument     ");
        }else if(cs[5]=='1'){
           printf(" 1 argument    ");
        }else{
           sscanf(&cs[5],"%d",&narg);
           printf("%2d arguments    ",narg);
        }
        if(cs[3]=='o')printf("\t\toptional:");
        if(cs[3]=='r')printf("\t\trequired:");
        printf("%s\n",&cs[7]);
        j++;
  }
  exit(0);		/* finished print out help message, donot need to continue */
  return 0;

}


SPHERE2 Transform3(TRANSMATRIX t, SPHERE2 c0)
{
float x,y,z,x1,y1,z1;

  x =sin(c0.theta);
  y =x*sin(c0.phi);
  x*=cos(c0.phi);
  z =cos(c0.theta);
  
  x1=t.x.x*x+t.x.y*y+t.x.z*z;
  y1=t.y.x*x+t.y.y*y+t.y.z*z;
  z1=t.z.x*x+t.z.y*y+t.z.z*z;

  c0.phi=0.0;
  c0.theta=acos(z1);
  if(fabs(c0.theta) > 1.0e-6){
     if(x1!=0.||y1!=0.0)c0.phi=atan2(y1,x1);
     if(c0.phi<0.0)c0.phi+=2.*PI;
  }
  return c0;

}

void FormBSMatrix(float *rker,int irr, float *hker, int *iker,
		int nrad, float dd, float *ata, float *atd, float *arow, int *iar, int nnode)
{
int i,i0,i1,j,k,m,ii,jj;
float rowi;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   for(k=0,i=i0;i<=i1;i++){
      m=(irr-3+i)*nnode;
      for(j=1;j<=iker[0];j++){
          k++;
          arow[k]=rker[i]*hker[j];
          iar[k]=m+iker[j];
      }
   }
   
   for(i=1;i<=k;i++){		/* add to ata and atd */
       ii=iar[i];
       rowi=arow[i];
       atd[ii]+=rowi*dd;
       for(j=1;j<=k;j++){
           jj=iar[j];
           if(jj<=ii)ata[jj+ii*(ii-1)/2]+=arow[j]*rowi;
       }
   }
   
    
}


float CalBSVal_2D(float *coef, float *hker, int *iker,int nnode)
{ /* calculate model value for a 2D B-spline model */
int i,i0,i1,j,k,m,ii,jj;
float val;

   val = 0.0;
   for(j=1;j<=iker[0];j++){
       val += hker[j]*coef[iker[j]];
   }
   return val;
}

float CalBSVal(MODEL model, float *rker,int irr, float *hker, int *iker,
		int nrad, int nnode)
{ /* calculate model value for a continuous B-spline model */
int i,i0,i1,j,k,m,ii,jj;
float rowi;
float val;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   val = 0.0;
   for(i=i0;i<=i1;i++){
      m=(irr-2+i);
      for(j=1;j<=iker[0];j++){
          val +=rker[i]*hker[j]*model.bu[m][iker[j]];
      }
   }
   return val;
}

float CalBSVal_LM(MODEL model, float *rker,int irr, float *hker, int *iker,int nrad, int nnode)
{ /* calculate model value for a Split B-spline model in LM */
int i,i0,i1,j,k,m,ii,jj;
float rowi;
float val;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   val = 0.0;
   for(i=i0;i<=i1;i++){
      m=(irr-2+i);
      for(j=1;j<=iker[0];j++){ /* iker[0] is number of nonzero elements */
          val +=rker[i]*hker[j]*model.bl[m][iker[j]];
      }
   }
   return val;
}

float CalBSVal_UM(MODEL model, float *rker,int irr, float *hker, int *iker,int nrad, int nnode)
{ /* calculate model value for a Split B-spline model in UM */
int i,i0,i1,j,k,m,ii,jj;
float rowi;
float val;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   val = 0.0;
   for(i=i0;i<=i1;i++){
      m=(irr-2+i);
      for(j=1;j<=iker[0];j++){
          val +=rker[i]*hker[j]*model.bu[m][iker[j]];
      }
   }
   return val;
}

float CalBSVal_MULTISPLIT(MODEL model, float *rker,int irr, float *hker, int *iker, int *nbslayer)
{ /* calculate model value for a Multiply Split B-spline model, YG 99. */
int i,i0,i1,j,k,m,ii,jj;
int itemp,nsplit;
float rowi;
float val;

   nsplit=model.nsplit[0]+model.nsplit[1];
/*
   printf("irr=%d  nsplit=%d\n", irr, nsplit);
*/
   i0=1;
   i1=4;
   if(irr==1)i0=2;
   
   for(i=0;i<nsplit;i++){
	if(irr==nbslayer[i]-1){
/*
		printf("set i1=3 due to top layer!\n");
*/
		i1=3;
		break;
   	}
   	if(irr==nbslayer[i]+1){
/*		printf("set i0=2 due to bottom layer!\n"); */
		i0=2;
		break;
   	}
   }
   val = 0.0;
   if(irr>model.kmaxlm) {
	itemp=irr-model.kmaxlm;
   	for(i=i0;i<=i1;i++){
/*		printf("i=%d rker=%f\n", i, rker[i]); */
      		m=(itemp-2+i);
      		for(j=1;j<=iker[0];j++){ /* iker[0] is number of nonzero elements */
          		val +=rker[i]*hker[j]*model.bu[m][iker[j]];
      		}
   	}
   }else{
	itemp=irr;
   	for(i=i0;i<=i1;i++){
/*		printf("i=%d rker=%f\n", i, rker[i]); */
      		m=(itemp-2+i);
      		for(j=1;j<=iker[0];j++){ /* iker[0] is number of nonzero elements */
          		val +=rker[i]*hker[j]*model.bl[m][iker[j]];
      		}
   	}
   }
   return val;
}

void GetGrid(float *xlat, float *xlon, float *dlat,float *dlon,
	float *latmin, float *latmax,float *lonmin,float *lonmax, int *ndata)
{
int dataopt,nlat,nlon,i,j,k;
float lat, lon, lat1, lon1;
FILE *fp;
char s[MAXSTR];
char gridfile[256];

  printf("surface point:\n");
  printf("------------------------------------\n");
  printf("1. enter by keyboard\n");    
  printf("2. enter from a file\n");
  printf("3. enter a range and increment\n");
  printf("enter:\n");
  GetLine(s,MAXSTR);
  sscanf(s,"%d", &dataopt);
  assert(dataopt>0 && dataopt<4);
	
  switch (dataopt) {
	case 1:
		printf("enter longitude and latitude:\n");
		GetLine(s,MAXSTR);
		sscanf(s, "%f %f", &lon,&lat);
		xlat[1]=lat;
		xlon[1]=lon;	
		*ndata=1;
		break;
	case 2:
  		printf("enter longitude--latitude grid file:\n");
		GetLine(gridfile,256);
  		fp=fopen(gridfile, "r");
    		i=1;
    		while(fscanf(fp,"%f %f",&lon,&lat)!=EOF){
       			xlat[i]=lat;
       			xlon[i]=lon;
       			if(i>=100000){
          			printf("too many data points\n");
          			exit(1);
       			}
			i++;
   	 	}
    		*ndata=i-1;
    		printf("total surface node points = %d\n",*ndata);
    
 	 	fclose(fp);
		break;
	case 3:
		printf("latitude range (min, max, # point):\n");
		GetLine(s,MAXSTR);
		sscanf(s,"%f %f %d", latmin,latmax,&nlat);
		printf("colongitude range (min, max, # point):\n");
		GetLine(s,MAXSTR);
		sscanf(s,"%f %f %d", lonmin,lonmax,&nlon);
		assert(*latmax>*latmin && *lonmax>*lonmin);
		k=1;
		lat1=*latmin;
		*dlat=(*latmax-*latmin)/(nlat-1);
		*dlon=(*lonmax-*lonmin)/(nlon-1);
		printf("dlat = %f, dlon = %f\n", *dlat, *dlon);
	 	for(i=1;i<=nlat;i++){
			lon1=*lonmin;
			for(j=1;j<=nlon;j++){
				lon=lon1;
				if(lon1>180.0) lon=lon1-360;
				xlon[k]=lon;
				xlat[k]=lat1;
				k++;
				lon1+=*dlon;
			}
			lat1+=*dlat;
		}
		*ndata=k-1;
		break;
	default:
		break;
  }
}

void ReadDepth(float *rr, int *n)
{ /* read in a file with depths to calculate radial correlation, etc. */
  int i;
  FILE *fp;
  char depfile[MAXSTR];
 
  printf("enter file with depths:\n");
  GetLine(depfile, MAXSTR);
  if((fp=fopen(depfile,"r")) == NULL){
	printf("error opening file  %s ...\n", depfile);
	exit(-1);
  }
  *n=0;
  i = 1;
  while(fscanf(fp, "%f", &rr[i]) != EOF){
	printf("reading %d,  depth = %f\n", i, rr[i]);
	rr[i] = RADIUS - rr[i];
	if(rr[i] >= RMOHO || rr[i] <= RCMB) {
		printf("depth not in mantle, ignore...\n");
		continue;
	}
	i++;
  }
  *n = i - 1;
  printf("total number of depth = %d\n", *n);
  fclose(fp);
}

int find_region(float r, int nrad, MODEL model)
{ /* locate between which knots the given radius fall */
 int i, ireg;
	
 if(model.mod!=SPHBSPL2){
	printf("model inconsistent!  exit...\n");
	exit(-1);
 }
 assert((r+0.01)>=model.ar[1] && (r+0.01)<=model.ar[nrad]);

 ireg = -1;	
 for(i=1;i<=nrad;i++){
	if((r+0.01)>=model.ar[i] && (r+0.01)<=model.ar[i+1]) {
		ireg = i;
		break;
	}
 }
	printf("ireg = %d\n", ireg);
 return ireg;
}

int find_region_split(float r, int nrad, MODEL model, int uplow)
{ /* locate between which knots the given radius fall, split model */
  /* uplow = 0 (lower mantle knots), uplow = 1 (upper mantle) */
 int i, ireg;
	
 ireg = -1;	
 if(model.mod!=BSPSPLT){
	printf("model inconsistent!  exit...\n");
	exit(-1);
 }
 if(uplow ==0) { /* lower mantle */
 	for(i=1;i<=nrad;i++){
		if((r+0.01)>=model.br[i] && (r+0.01)<=model.br[i+1]) {
			ireg = i;
			break;
		}
	}
 }else {	/* upper mantle */
 	for(i=1;i<=nrad;i++){
		if((r+0.01)>=model.ar[i] && (r+0.01)<=model.ar[i+1]) {
			ireg = i;
			break;
		}
	}

 }
 return ireg;
}

int find_region_multisplit(float r, MODEL model, int *layernum)
{ /* locate between which knots the given radius fall, multiply
   * split model, YG, 1999.
   */
 int i,j,k,l,m,ireg;
 int ntotal;
 float **rknot;
 float rtop, rbot;
 int iffound;

 ntotal=model.nsplit[0]+model.nsplit[1];
 printf("ntotal = %d  r = %f\n", ntotal,r);	
 ireg = -1;	
 if(model.mod!=BSMULTISPLIT){
	printf("model inconsistent!  exit...\n");
	exit(-1);
 }
 rknot=model.rknot;
 iffound=0;
 for(k=0,l=1;l<=ntotal;l++){
    	for(j=1;j<=model.nbsperlay[l-1];j++){ /* loop for every region */
		k++;
		rtop=rknot[l][j+1];
		rbot=rknot[l][j];
		if((r+0.001)>=rbot && (r+0.001)<rtop) {
			ireg=j ;
			iffound=1;
			*layernum=l;
			printf("ireg=%d  rbot=%f rtop=%f\n", ireg, rbot,rtop);
			break;
		}
	}
	if(iffound==1){
		for(m=0;m<l-1;m++){ /* irr is accumulative from 1 to 13 */
        		ireg=ireg+model.nbsperlay[m];
		}
		break;
	}
 }

 return ireg;
}
